(* ::Package:: *)

(**********************************************************************************************)
(*                     Stringvacua package, Version 1.0, December 2007                         *)
(*                  James Gray, Yang-Hue He, Anton Ilderton, Andre Lukas                      *)
(**********************************************************************************************)
BeginPackage["Stringvacua`"]

(**********************************************************************************************)
(* Error and warning messages *)
(**********************************************************************************************)

Stringvacua::"no executable"="`1` executable not found. Use SetOptions[Stringvacua,`1`Path->path] to set the path manually. See ?StringvacuaOptions for details.";
Stringvacua::"incomplete install"="Package not fully installed. `1` not found.";
Stringvacua::"model incorrect"="The number of real or imaginary parts of fields is different from the number of fields!";
Stringvacua::"no model"="Model set-up incomplete. Run module CreateModel.";
Stringvacua::"illegal value"="Option `1` not defined properly. Set to default value `2`.";
Stringvacua::"illegal option"="Value for option `1` illegal. See '?`1`.";
Stringvacua::"missing script"="Singular script `1` missing in module `2`. Check ?SingularScriptInfo";
Stringvacua::"illegal variable"="Illegal variables/parameters in module `1`.";
Stringvacua::"illegal ideal"="Ideal is not well-formed in module `1`. See ?IdealKeywords.";
Stringvacua::"not polynomial"="Value of option 'Poly' not polynomial in module `1`. See ?Poly.";
Stringvacua::"no extensions"="Parameters need to be replaced by numbers in module `1`.";
Stringvacua::"wrong fterms"="Integer in option 'Fterms' is not in allowed range. See ?Fterms."
Stringvacua::"no matrix"="Argument provided is not a matrix.";
Stringvacua::"not square"="Matrix provided is not square.";
Stringvacua::"matrix error"="Entries of matrix are not rational.";
Stringvacua::"Kahler metric error"="Inverse Kahler metric not a matrix of the correct size";
Stringvacua::"field char"="The field characteristic is set to the finite value `1`. Use SetOptions[Stringvacua,FieldChar->value] to change to other values.";
Stringvacua::"Singular status"="Singular `1`";
Stringvacua::"not rational"="Value of option `1` does not consist of rational functions or still depends on parameters in module `2`.";
Stringvacua::"test empty"="Nothing to test with Sturm query in module PolySigncond.";
Stringvacua::"Sturm"="Warning: Sturm filtering requires calculation of number of real roots. Option RRoots set to \"Probab\".";
Stringvacua::"NRoots"="Warning: Real root filtering with option 'NRoots' requires calculation of number of real roots. Option RRoots set to \"Probab\".";
Stringvacua::"real roots"="Warning: Real root filtering does not work for finite characteristic or field extensions. Switched off.";
Stringvacua::"no elim"="Warning: Nothing to eliminate. Set option VarPar properly.";
Stringvacua::"model incomplete"="Warning: Entry `1` not properly defined in model list. Ideal `2` cannot be computed.";
Stringvacua::"no ideal"="Warning: Generators of model ideal `1` are not polynomial in variables/parameters!";
Stringvacua::"ideal incomplete"="Warning: Ideal is not well-formed. See ?IdealKeywords.";
Stringvacua::"no I"="No factors of I allowed in module `1`";
Stringvacua::"not varpar"="Warning: Object requested for elimination is neither variable nor parameter.";
Stringvacua::"canceled"="Module SimpleMinimise canceled!";
Stringvacua::"extra matrix"="Only one matrix can be tested for eigenvalues. Used Hesse matrix.";

(**********************************************************************************************)

SingularScripts::usage="List of Singular scripts used by the package with entries
of the form 'scriptname -> script'.";

Begin["`Private`"]

(* define usage help function with clickable more... button, old version, doesn't work for 6.0 *)

(* use[name_,link_,txt_,shorttxt_]:=Module[{},
  If[$Notebooks,
     name::usage=shorttxt<>" "<>"\*ButtonBox[\"More...\",Active->True,ButtonStyle->\"AddOnsLink\",ButtonData:>\""<>ToString[link]<>"\"]",
      name::usage=txt]]; *)

(* define usage help function with clickable more... button, new version *)

use[name_,link_,txt_,shorttxt_]:=Module[{},
  If[$Notebooks,
     name::usage=shorttxt<>" "<>"\*ButtonBox[StyleBox[\"More...\",FontColor->RGBColor[0,0,0.7]],Active->True,ButtonStyle->\"AddOnsLink\",ButtonFrame->None,ButtonFunction :> (FrontEndExecute[ { FrontEnd`HelpBrowserLookup[\"AddOns\",\""<>ToString[link]<>"\"]}]&)]",
      name::usage=txt]];

(* a nicer looking usage function *)

usen[name_,link_,txt_,comm_,arg_,shorttxt_]:=Module[{},
  If[$Notebooks,
     name::usage="\*" <> ToString[StyleBox[comm, FontColor -> RGBColor[0.5, 0, 0], FontSize -> 12]] <> 
                 "\*" <> ToString[StyleBox["\"[\"", FontColor -> RGBColor[0.5, 0, 0], FontSize -> 12]] <>
                 "\*" <> ToString[StyleBox[arg,FontColor -> RGBColor[0.5, 0, 0],FontSlant -> "Italic", FontSize -> 11]] <> 
                 "\*" <> ToString[StyleBox["\"]\"", FontColor -> RGBColor[0.5, 0, 0], FontSize -> 12]]<>" "<>shorttxt<>" "<>"\*ButtonBox[StyleBox[\"More...\",FontColor->RGBColor[0,0,0.7]],Active->True,ButtonStyle->\"AddOnsLink\",ButtonFrame->None,ButtonFunction :> (FrontEndExecute[ { FrontEnd`HelpBrowserLookup[\"AddOns\",\""<>ToString[link]<>"\"]}]&)]",
      name::usage=txt]];

sb[txt_]:="\*"<>ToString[StyleBox[txt, FontColor -> RGBColor[0.5, 0, 0], FontSize -> 11]];
sbi[txt_]:="\*"<>ToString[StyleBox[txt, FontColor -> RGBColor[0.5, 0, 0], FontSize -> 11,FontSlant->Italic]];

End[]

(**********************************************************************************************)
(* text interface help *)
(**********************************************************************************************)
(* General help *)

`Private`use[Stringvacua,Generalities,"This package is designed to determine the vacuum structure of (string derived or inspired) potentials in d=4, N=1 supergravity, using methods of algebraic geometry. It automatically links to the algebraic geometry package 'Singular' for fast polynomial calculations. A protocol of the Singular calculation is normally displayed in an xterm which is automatically opened when necessary. For details about the Mathematica/Singular interface see '?StringvacuaInterface'. A supergravity model is stored as a list of rules which assign certain d=4, N=1 quantities to keywords, such that, for example 'SPot/.model1' produces the superpotential of a model with name 'model1'. A list of keywords for model lists can be obtained by '?ModelKeywords'. Ideals are also stored as a list of rules which assign ideal properties to certain keywords. See '?IdealKeywords' for details. A list of modules in this package and their optional arguments can be obtained by '?StringvacuaModules' and '?ModuleOptions'. Global options of the package are defined in 'Options[Stringvacua]'. See '?StringvacuaOptions' for a detailed description of these variables. The status of the package is stored in global variables listed in '?StringvacuaStatus'. A list of the Singular scripts called by Mathematica is obtained by'?SingularScriptInfo'.",
"This package is designed to determine the vacuum structure of (string derived or inspired) potentials in d=4, N=1 supergravity, using methods of algebraic geometry. It automatically links to the algebraic geometry package "<>`Private`sb["\"Singular\""]<>" for fast polynomial calculations."];

`Private`use[StringvacuaInterface,"Stringvacua Interface","The package is designed to be used exclusively from Mathematica without any need to deal with Singular directly. The relation to Singular is controlled by a number of global options listed under '?StringvacuaOptions'. If option 'SingularTimer' is set to \"on\" (only available on Unix-type systems), the option 'SingularProtocol' offers three choices for the Singular protocol. For the default choice \"Xterm\" a protocol of the Singular calculation is displayed in an xterm which is automatically opened. This provides a convenient way of monitoring and, if appropriate, interrupting long calculations.? Ctrl c in the xterm interrupts the Singular calculation and triggers the return to Mathematica. The package also provides a built-in timer with an upper time limit stored in the global variable 'SingularMaxTime'. When a Singular calculation exceeds this time limit it will be automatically stopped and the process returns to Mathematica. Some modules, such as 'SaturationExpand', perform multiple calls to Singular. Each such call is then timed individually. A ctrl c signal in the xterm will stop the currently running Singular process but not subsequent ones. To terminate such a module completely, perform first a Mathematica interrupt and then a crtl c in the Singular xterm. If 'SingularTimer' is set of \"off\" no timer feature or protocol feature in a separate xterm is available. When using the Mathematica text interface in an xterm, the Singular protocol will then be displayed in the same xterm provided 'SingularProtocol' is switched to 'Xterm'. This feature is not available for the notebook interface. Variables for fields or parameters must be of the form '_Symbol', 'Subscript[_Symbol,_Integer]', 'Subscript[_Symbol,_Symbol]' or '_Symbol[_Integer]' to be recognised by the package and be accurately converted to Singular.","The package is designed to be used exclusively from Mathematica without any need to deal with Singular directly. The relation to Singular is controlled by a number of global options listed under "<>`Private`sb["\"?StringvacuaOptions\""]<>"."];

`Private`use[StringvacuaModules,"Stringvacua Modules","For detailed help on any of the modules use '?modulename' and to obtain the defaults for local optional arguments use 'Options[modulename]'. To change these default options use the Mathematica command 'SetOptions[modulename,option->value]'. Note that the global options (see '?StringvacuaOptions') with defaults 'Options[Stringvacua]' can also be used as optional arguments for most functions. In this case the global defaults are overridden for the specific module call. Here is a list of modules in the Stringvacua package.\n"<>
"* CalcModel         : Calculates ideals in a model list.\n"<>
"* CallSingular      : Low-level call of Singular.\n"<>
"* CreateIdeal       : Creates an ideal list.\n"<>
"* CreateModel       : Creates a model list, calculates scalar potential etc.\n"<>
"* DimIdeal          : Calculates the dimension of an ideal.\n"<>
"* Elimination       : Calculates the ideal obtained by eliminating a set of\n"<>
"                      variables.\n"<>
"* FacGroebner       : Performs the factorising Buchberger algorithm on a set\n"<>
"                      of generators of an ideal.\n"<>
"* NumRoots          : Calculates number of real roots of a zero-dim. ideal.\n"<>
"* PrimDec           : Primary decomposition of an ideal.\n"<>
"* RemoveEntry       : Removes an entry from a list.\n"<>
"* Saturation        : Form an element of the saturation expansion of an ideal.\n"<>
"* SaturationExpand  : Computes the saturation expansion for a model and\n"<>
"                      primary decomposes.\n"<>
"* SaturationPrimDec : Form an element of the saturation expansion of an ideal\n"<>
"                      and primary decompose.\n"<>
"* SimpleMinimise    : Simple interactive minimisation.\n"<>
"* SturmQuery        : Calculates the sign of various objects on the real roots\n"<>
"                      of a zero dimensional ideal.\n"<>
"* TF                : Displays model, ideal or other lists in table form.\n"<>
"* ThomEncode        : Thom encodes the real roots of zero dimensional ideals.",
"For detailed help on any of the modules use "<>`Private`sbi["\"?modulename\""]<>" and to find out the defaults for local optional arguments use "<>`Private`sb["\"Options\""]<>`Private`sbi["\"[modulename]\""]<>"."];

`Private`use[ModuleOptions,"Module Options","Modules in the Stringvacua package accept a number of optional arguments. See '?StringvacuaModules' for a list of these modules and use '?modulename' to obtain a detailed description of a module and its options. Use '?optionname' for a general description of an option. The default settings for these options are specific to the particular module and can be obtained from 'Options[modulename]'. In addition, most modules depend on the global options (see '?StringvacuaOptions') whose defaults are defined globally in 'Options[Stringvacua]'. They can also be provided as arguments in most modules to change the global default. Here is a list of module-specific optional parameters used in the arguments of modules in this package.\n"<>
"CDummies       -> list with entries 'expr -> complexdummyvariable'\n"<>
"DeltaPot       -> non-superysmmetric addition to the potential\n"<>
"Dim            -> (list of) integer(s), keep ideals with these complex dims.\n"<>
"Dummies        -> list with entries 'expr -> realdummyvariable'\n"<>
"FTerms         -> (list of) real field(s) specifying F-Terms of a model\n"<>
"KMetricInverse -> inverse of Kahler metric\n"<>
"Meth           -> a string, specifying a particular method.\n"<>
"NRoots         -> Integer, keep ideal with at most this many real roots.\n"<>
"Par            -> (a list of) parameter(s)\n"<>
"Poly           -> (list of) polynomial(s) or ideal list(s)\n"<>
"RDummies       -> list of replacements\n"<>
"RRoots         -> \"Probab\", \"Determ\" or \"Off\", calculate real roots or not.\n"<>
"SturmLabel     -> string to provide label for 'Sturm' entry in ideal list\n"<>
"SturmPoly      -> (list of) polynomial(s) for filtering with Sturm queries\n"<>
"SturmSigns     -> (list of) sign(s) for filtering with Sturm queries\n"<>
"TestLambda     -> \"yes\" or \"no\" for Sturm query on scalar potential.\n"<>
"TestMatDet     -> (list of) rational matrix(ces), Sturm query\n"<>
"                  on determinant(s)\n"<>
"TestMatEigen   -> rational matrix, Sturm query on eigenvalues\n"<>
"TestRational   -> (list of) rational functions for Sturm query\n"<>
"TestStability  -> \"Hesse\" for stability test, otherwise \"Off\"\n"<>
"Var            -> (a list of) variable(s)\n"<>
"VarPar         -> list of (assignments of) variables/parameters (to values)",
"Modules in the Stringvacua package accept a number of optional arguments. See "<>`Private`sb["\"?StringvacuaModules\""]<>" for a list of these modules and use "<>`Private`sbi["\"?modulename\""]<>" to obtain a detailed description of a module and its options. Use "<>`Private`sbi["\"?optionname\""]<>" for a general description of an option."];

`Private`use[ModelKeywords,"Model Keywords","A SUGRA model is stored as a list of assigments 'keyword -> value'. For a model list called 'model' access a particular entry with 'keyword/.model'. For details on a particular keyword see '?keyword'. Keywords starting with 'I' are assigned to ideal lists. For their structure see '?IdealKeywords'. Here is a list of model keywords.\n"<>
"* CDummies       -> list of replacements expr -> complexdummyvariable\n"<>
"* ComplexToReal  -> list of rules replacing complex with real fields\n"<>
"* DeltaPot       -> non-supersymmetric addition to scalar potential\n"<>
"* DPot           -> list: real field -> derivative of potential\n"<>
"* DSPot          -> list: compl. field -> first derivative of superpotential\n"<>
"* Dummies        -> list of replacements expr -> realdummyvariable\n"<>
"* Fields         -> list of complex fields\n"<>
"* FTerms         -> list: complex field -> corresponding F term\n"<>
"* Hesse          -> Hessian w.r.t. real fields\n"<>
"* IDPot          -> Ideal list for derivatives of scalar potential\n"<>
"* IDSPot         -> ideal list for ideal generated by dW\n"<>
"* IFTerms        -> Ideal list for F terms\n"<>
"* IImSPot        -> Ideal list for imaginary part of superpotential\n"<>
"* ImFields       -> list of imaginary parts of fields\n"<>
"* IPot           -> Ideal list for scalar potential\n"<>
"* IReImDSPot     -> Ideal list for superpotential derivatives\n"<>
"* IReSPot        -> Ideal list for real part of superpotential\n"<>
"* ISPot          -> ideal list for ideal generated by W\n"<>
"* KMetric        -> kahlermetric\n"<>
"* KPot           -> Kahler potential\n"<>
"* Par            -> list of parameters of model\n"<>
"* Pot            -> scalar potential\n"<>
"* RDummies       -> list of rules for all dummy variables for non-poly.\n"<>
"                    expressions\n"<>
"* ReFields       -> list of real parts of fields\n"<>
"* RFields        -> list of real and imaginary parts of fields\n"<>
"* SPot           -> superpotential",
"A SUGRA model is stored as a list of assigments "<>`Private`sb["\"keyword -> value\""]<>". For a model list called 'model' access a particular entry with "<>`Private`sb["\"keyword/.model\""]<>". For details on a particular keyword see "<>`Private`sb["\"?keyword\""]<>"."];

(**********************************************************************************************)
(* Help on model keywords *)

`Private`use[Fields,"Model Keywords","Keyword for complex fields in a model list.",
"Keyword for complex fields in a model list."];
`Private`use[ReFields,"Model Keywords","Keyword for real parts of fields in a model list.",
"Keyword for real parts of fields in a model list."]; 
`Private`use[ImFields,"Model Keywords","Keyword for imaginary parts of fields in a model list.",
"Keyword for imaginary parts of fields in a model list."];
`Private`use[RFields,"Model Keywords","Keyword for all real fields (real and imaginary parts) in a model list.",
"Keyword for all real fields (real and imaginary parts) in a model list."];
`Private`use[Par,"Model Keywords","Keyword for parameters in a model or ideal list. Also option for Stringvacua modules.","Keyword for parameters in a model or ideal list. Also option for Stringvacua modules."];
`Private`use[ComplexToReal,"Model Keywords","Keyword for list of rules converting complex to real fields in a model list.","Keyword for list of rules converting complex to real fields in a model list."];
`Private`use[KPot,"Model Keywords","Keyword for Kahler potential in a model list.",
"Keyword for Kahler potential in a model list."];
`Private`use[SPot,"Model Keywords","Keyword for superpotential in a model list.",
"Keyword for superpotential in a model list."];
`Private`use[ISPot,"Model Keywords","Keyword for ideal list generated by W (complex fields).",
"Keyword for ideal list generated by W (complex fields)."];
`Private`use[KMetric,"Model Keywords","Keyword for Kahler metric in a model list.",
"Keyword for Kahler metric in a model list."];
`Private`use[DSPot,"Model Keywords","Keyword for first derivatives (w.r.t. complex fields) of W in model list.",
"Keyword for first derivatives (w.r.t. complex fields) of W in model list."];
`Private`use[IDSPot,"Model Keywords","Keyword for ideal list generated by dW (complex fields).",
"Keyword for ideal list generated by dW (complex fields)."];
`Private`use[FTerms,"Model Keywords","Keyword for F terms (split into real and imaginary parts)
in a model list or option for Stringvacua modules.","Keyword for F terms (split into real and imaginary parts)
in a model list or option for Stringvacua modules."];
`Private`use[DeltaPot,"Model Keywords","Keyword for non-superysmmetric addition to scalar potential in a model list.",
"Keyword for non-superysmmetric addition to scalar potential in a model list."];
`Private`use[Pot,"Model Keywords","Keyword for scalar potential in a model list.",
"Keyword for scalar potential in a model list."];
`Private`use[DPot,"Model Keywords","Keyword for first derivatives (w.r.t. real fields as in 'RFields')
of potential in a model list.","Keyword for first derivatives (w.r.t. real fields as in 'RFields')
of potential in a model list."];
`Private`use[Hesse,"Model Keywords","Keyword for Hessian matrix (w.r.t. to real fields as in 'RFields')
in a model list.","Keyword for Hessian matrix (w.r.t. to real fields as in 'RFields')
in a model list."];
`Private`use[IReSPot,"Model Keywords","Keyword for ideal list containing the ideal generated by Re(W)
in a model list.","Keyword for ideal list containing the ideal generated by Re(W)
in a model list."];
`Private`use[IImSPot,"Model Keywords","Keyword for ideal list containing the ideal generated by Im(W)
in a model list.","Keyword for ideal list containing the ideal generated by Im(W)
in a model list."];
`Private`use[IReImDSPot,"Model Keywords","Keyword for ideal list containing the ideal generated by
the derivatives of W.","Keyword for ideal list containing the ideal generated by
the derivatives of W."];
`Private`use[IFTerms,"Model Keywords","Keyword for ideal list containing the ideal generated by the F terms.",
"Keyword for ideal list containing the ideal generated by the F terms."];
`Private`use[IPot,"Model Keywords","Keyword for ideal list containing the ideal generated by scalar potential.",
"Keyword for ideal list containing the ideal generated by scalar potential."];
`Private`use[IDPot,"Model Keywords","Keyword for ideal list containing the ideal generated by
derivatives of scalar potential.","Keyword for ideal list containing the ideal generated by
derivatives of scalar potential."]; 
`Private`use[Dummies,"Model Keywords","1) Keyword for model list, containing a list of assignments of
transcendental expressions to (real) dummy variables. 2) Option for module 'CalcModel'.","1) Keyword for model list, containing a list of assignments of transcendental expressions to (real) dummy variables. 2) Option for module "<>`Private`sb["\"CalcModel\""]<>"."];
`Private`use[CDummies,"Model Keywords","1) Keyword for model list, containing a list of assignments of transcendental expressions to (complex) dummy variables. 2) Option for module 'CalcModel'.",
"Keyword for model list, containing a list of assignments of transcendental expressions to (complex) dummy variables. 2) Option for module "<>`Private`sb["\"CalcModel\""]<>"."];
`Private`use[RDummies,"Module Options","Option for module 'CalcModel'.",
"Option for module "<>`Private`sb["\"CalcModel\""]<>"."];

(**********************************************************************************************)
(* help on ideal keywords *)

`Private`use[IdealKeywords,"Ideal Keywords","An ideal is stored as a list of assigments 'keyword -> value'. For an ideal list called 'id' access entries with 'keyword/.id'. For details on a particular keyword see '?keyword'. Here is a list of ideal keywords.\n"<>
"Dim            -> dimension of ideal\n"<>
"Gen            -> list of polynomials generating the ideal\n"<>
"NRoots         -> number of real roots of a zero-dim. ideal\n"<>
"Par            -> parameters of ideal (treated as field extensions)\n"<>
"Sturm          -> result of a Sturm query\n"<>
"Thom           -> Thom polynomials for a 0-dim. ideal\n"<>
"Var            -> variables of ideal (ring variables)",
"An ideal is stored as a list of assignments "<>`Private`sb["\"keyword -> value\""]<>". For an ideal list, "<>`Private`sb["\"id\""]<>", access entries with "<>`Private`sb["\"keyword/.id\""]<>". For details on a particular keyword see "<>`Private`sb["\"?keyword\""]<>"."];

`Private`use[NRoots,"Ideal Keywords","1) Option for some Stringvacua modules to specify the minimal number of real roots required for an ideal to appear in the result. 
2) Keyword for number of real roots in an ideal list for zero-dim. ideal.",
"1) Option for some Stringvacua modules to specify the minimal number of real roots required for an ideal to appear in the result. 
2) Keyword for number of real roots in an ideal list for zero-dim. ideal."];

`Private`use[Dim,"Ideal Keywords","1) Option specifying (list of) complex dimension(s) of ideals
to be kept as output for some Stringvacua modules. 2) Keyword for complex dimension of an ideal in an ideal list.",
"1) Option specifying (list of) complex dimension(s) of ideals
to be kept as output for some Stringvacua modules. 2) Keyword for complex dimension of an ideal in an ideal list."];

`Private`use[Var,"Ideal Keywords","1) Keyword for (ring) variables in an ideal list. 2) Option for Stringvacua modules.",
"1) Keyword for (ring) variables in an ideal list. 2) Option for Stringvacua modules."];

`Private`use[Gen,"Ideal Keywords","Keyword for polynomial generators of ideal in an ideal list.",
"Keyword for polynomial generators of ideal in an ideal list."];

`Private`use[Sturm,"Ideal Keywords","Keyword for the result of a Sturm query in an ideal list.
This entry is a list with a particular structure which depends on whether or not we are testing the signs of the eigenvalues of a matrix (see '?SturmQuery' for details).",
"Keyword for the result of a Sturm query in an ideal list."];

`Private`use[Thom,"Ideal Keywords","Ideal keyword for the Thom encoding polynomials the signs of which uniquely determine the roots of the zero-dimensional ideal (see '?ThomEncode' for more details).",
"Keyword for the Thom encoding polynomials."];

`Private`use[RRoots,"Module Options","Option for Stringvacua modules specifying if number of real roots should be calculated. The number of real roots is calculated if set to \"Probab\" or \"Determ\" and not calculated if set to \"Off\".", "Option for Stringvacua modules specifying if number of real roots should be calculated. The number of real roots is calculated if set to \"Probab\" or \"Determ\" and not calculated if set to \"Off\"."];

(**********************************************************************************************)
(* help on options for modules *)
 
`Private`use[SturmPoly,"Module Options","(List of) polynomial(s) for ideal filtering using Sturm queries. Only ideals with real roots on which specified polynomials assume the signs specified by option 'SturmSigns' are kept.",
"(List of) polynomial(s) for ideal filtering using Sturm queries. Only ideals with real roots on which specified polynomials assume the signs specified by option "<>`Private`sb["\"SturmSigns\""]<>" are kept."];

`Private`use[SturmSigns,"Module Options","(List of) sign(s) (i.e. +1, -1 or 0) for ideal filtering using Sturm queries. The number of signs must be an integer multiple of the number, n,  of polynomials in 'SturmPoly' with the first n signs specifying one pattern of signs, the second n another pattern, etc. Only ideals with real roots on which polynomials specified in option 'SturmPoly' assume one of these patterns of signs are kept.",
"(List of) sign(s) (i.e. +1, -1 or 0) for ideal filtering using Sturm queries."];

`Private`use[TestRational,"Module Options","(list of) rational functions for Sturm query.",
"(list of) rational functions for Sturm query."];

`Private`use[TestMatDet,"Module Options","(list of) rational matrix(ces) for Sturm query on determinant(s).",
"(list of) rational matrix(ces) for Sturm query on determinant(s)."];

`Private`use[TestMatEigen,"Module Options","Rational matrix for Sturm query on eigenvalues.",
"Rational matrix for Sturm query on eigenvalues."];

`Private`use[TestLambda,"Module Options","Takes values \"yes\" or \"no\" to switch Sturm query on scalar potential on or off.","Takes values \"yes\" or \"no\" to switch Sturm query on scalar potential on or off."];

`Private`use[TestStability,"Module Options","If set to \"Hesse\" a Sturm query is performed on the Eigenvalues of the Hesse matrix, no action if set to \"Off\".","If set to \"Hesse\" a Sturm query is performed on the Eigenvalues of the Hesse matrix, no action if set to \"Off\"."];

`Private`use[SturmLabel,"Module Options","Takes values \"Automatic\" or \"None\" or some other string. Can be used to provide a user-defined label for the 'Sturm' entry of an ideal list in modules 'SturmQuery' and 'SaturationExpand'. If set to \"Automatic\" 'SaturationExpand' will autmatically fill in the labels 'Pot' and 'Hessian' when the 'TestLambda' and 'TestStability' options are used.",
"Takes values \"Automatic\" or \"None\" or some other string. Can be used to provide a user-defined label for the "<>`Private`sb["\"Sturm\""]<>" entry of an ideal list in modules "<>`Private`sb["\"SturmQuery\""]<>" and "<>`Private`sb["\"SaturationExpand\""]<>"."];

`Private`use[KMetricInverse,"Module Options","Optional argument to specify inverse Kahler metric in module 'CreateModel'. If not provided the module will attempt to calculate the inverse.",
"Optional argument to specify inverse Kahler metric in module 'CreateModel'. If not provided the module will attempt to calculate the inverse."];

`Private`use[StringvacuaOptions,"Stringvacua Options","The Stringvacua package and its relation to Singular is controlled by a number of global options whose current values can be listed with 'Options[Stringvacua]' and changed with the Mathematica 'SetOptions' command.  See '?optionname' for details on a particular option. Sensible defaults are set by the system. Start-up values for these options can be defined in the file \"StringvacuaOptions.m\" which is included with the package. Here is a list of global options.\n"<>
"FieldChar         -> Prime between 0 (default) and 2147483629.\n"<>
"                     Characteristic of field.\n"<>
"PackageProtocol   -> Integer 0...10, protocol level for package modules.\n"<>
"PlanckInverse     -> Number or symbol, inverse Planck mass.\n"<>
"SingularAccuracy  -> Real, accuracy for converting reals to rationals.\n"<>
"SingularEcho      -> Integer 0...10, level of Singular input echo.\n"<>
"SingularMaxTime   -> Integer, max. time (in sec.) for a single Singular call.\n"<>
"SingularPath      -> Path to the Singular executable.\n"<>
"SingularProtocol  -> \"Xterm\", \"String\" or \"None\", determines form\n"<>
"                     of the Singular protocol.\n"<>
"SingularTimer     -> timer script \"on\" or \"off\"\n"<>
"SingularTrace     -> Integer 0...4, level of the Singular protocol.\n"<>
"XtermGeometry     -> String, specifies the geometry of xterm.\n"<>
"XtermPath         -> Path to the xterm executable.",
"The Stringvacua package and its relation to Singular is controlled by a number of global options whose current values can be listed with "<>`Private`sb["\"Options[Stringvacua]\""]<>" and changed with the Mathematica "<>`Private`sb["\"SetOptions\""]<>" command. See "<>`Private`sbi["\"?optionname\""]<>" for details on a particular option."]; 

(**********************************************************************************************)
(* help on status variables and global options *)

`Private`use[StringvacuaStatus,"Stringvacua Status","A number of global variables contain information about the global status of the Stringvacua package and Singular. Here is a list of these variables.\n"<>
"* SingularStat    : Status of last Singular call.\n"<>
"* SingularVar     : Mapping of variables from Mathematica to Singular\n"<>
"                    from last Singular call.\n"<>
"* SingularProt    : Protocol from last Singular call if option\n"<>
"                    'SingularProtocol' is set to \"String\".\n"<>
"* SingularScripts : A list of singular scripts loaded by the package.",
"A number of global variables contain information about the global status of the Stringvacua package and Singular."];

`Private`use[SingularProtocol,"Stringvacua Options","Determines the protocol mode for Singular calls. Can be set to \"Xterm\", \"String\" or \"None\". If set to \"Xterm\" the Singular protocol is shown in a separate xterm. If set to \"String\" the protocol is stored in 'SingularProt/.StringvacuaVariables'. If the xterm executable is found the default is \"Xterm\", otherwise it is \"None\". This feature will work only for Unix systems and if the global option 'SingularTimer' is set to \"on\".",
"Determines the protcol mode for Singular calls. Can be set to \"Xterm\", \"String\" or \"None\"."];

`Private`use[SingularTimer,"Stringvacua Options","Determines if the timer is used during a Singular call. Can be set to \"on\" (only available on Unix systems) or \"off\". If set to \"off\" no timing or protocal features are available for Singular calls (except when Mathematica is run in text mode in an xterm where setting of 'SingularProtocol' to \"Xterm\" will cause the protocol to appear in this same xterm). No intermediate scripts between Mathematica and Singular are involved in this case, so this may be used on systems where the intermediate timer script causes problems.",
"Takes values \"on\" or \"off\". Determines if the timer is used during a Singular call."];

`Private`use[PlanckInverse,"Stringvacua Options","Value of inverse Planck mass. Default value is 1. Set to 0 in order to obtain the global SUSY potential.",
"Value of inverse Planck mass. Default value is 1."];

`Private`use[FieldChar,"Stringvacua Options","Characteristic of field underlying polynomial ring for Singular calculations. Must be a prime integer between 0 and 2147483629. Default is 0 which corresponds to the rationals.","Characteristic of field underlying polynomial ring for Singular calculations."];

`Private`use[SingularMaxTime,"Stringvacua Options","Time limit for a single call to Singular (in seconds). If the Singular calculation exceeds this limit, the calculation is timed out and returns to Mathematica. The default is 60 seconds. Timing only works if the global option 'SingularTimer' is set to \"on\" (only possible on Unix systems).",
"Time limit for a single call to Singular (in seconds)."];

`Private`use[PackageProtocol,"Stringvacua Options","Ranges from 0 (no protocol) to 10 and determines the level of protocol for package modules which perform multiple Singular calls. Useful to follow the progress of longer calculations. Default is 10.",
"Ranges from 0 (no protocol) to 10 and determines the level of protocol for package modules which perform multiple Singular calls."];

`Private`use[SingularTrace,"Stringvacua Options","Takes values 0,..,4 which determine the level of detail of the Singular protocol. Default is 0.","Takes values 0,..,4 which determine the level of detail of the Singular protocol. Default is 0."];

`Private`use[SingularEcho,"Stringvacua Options","Takes values 0,..,10 which determine the level of input echo of the Singular protocol. Default is 0.",
"Takes values 0,..,10 which determine the level of input echo of the Singular protocol. Default is 0."];

`Private`use[SingularAccuracy,"Stringvacua Options","Accuracy for converting real into rational numbers before a Singular call (default 10^(-5)).",
"Accuracy for converting real into rational numbers before a Singular call (default 10^(-5))."];

`Private`use[XtermGeometry,"Stringvacua Options","A string specifying the geometry (i.e. size and position on screen) of the xterm which is opened when the global option 'SingularProtocol' is set to \"Xterm\" and 'SingularTimer' is \"on\". The format of the string exactly correponds to the argument of the -geometry option of the xterm command (see the xterm man pages). Default is \"50x10-50+0\" which creates an xterm of length 50 and height 10 at horizontal position 50 from the right of the screen and vertical position 0 from the top of the screen.",
"A string specifying the geometry (i.e. size and position on screen) of the xterm which is opened when the global option "<>`Private`sb["\"SingularProtocol\""]<>" is set to \"Xterm\"."];

`Private`use[XtermPath,"Stringvacua Options","Path to the xterm executable.",
"Path to the xterm executable."];

`Private`use[SingularPath,"Stringvacua Options","Path to the Singular executable.",
"Path to the Singular executable."];

(**********************************************************************************************)
(* help on package modules *)

`Private`usen[CreateModel,CreateModel,"* usage   : 'CreateModel[fields,refields,imfields,K,W<,options>]' where\n"<>
"  - fields   : a list of complex fields of the model\n"<>
"  - refields : a list of the real parts of the fields\n"<>
"  - imfields : a list of the imaginary parts of the fields\n"<>
"  - K        : Kahler potential of the model\n"<>
"  - W        : superpotential of the model\n"<>
"* options : Par      -> list of parameters of the model (default {})\n"<>
"            DeltaPot -> non-susy addition to the potential (default 0)\n"<>
"            Meth     -> \"math\" (default), \"hand\" or \"off\"\n"<>
"            KMetricInverse -> inverse Kahler metric\n"<>
"            From global options '?StringvacuaOptions': PlanckInverse\n"<>
"* purpose : Creates a model list from the input data. If option 'Meth' is set to \"math\" the inverse Kahler metric is calculated, if set to \"hand\" it has to be provided using the option 'KMetricInverse', if set to 'off' the inverse Kahler metric is not calculated. In the last case, the model list does not contain entries for the scalar potential and its derivatives.\n"<>
"* output  : Model list, assigning values to the keywords 'Fields', 'ReFields', 'ImFields', 'RFields', 'Par', 'ComplexToReal', 'KPot', 'SPot', 'DSPot', 'KMetric', 'Fterms', 'Pot', 'DPot', 'Hesse', 'DeltaPot'.\n"<>
"* example : 'model=CreateModel[{T,Z},{t,z},{tau,zeta},-3 Log[T+Conjugate[T]]-3 Log[Z+Conjugate[Z]], a T+b Z,Par->{a,b}]' creates a model list 'model' for a simple model with two moduli T,Z, a standard Kahler potential and a simple linear superpotential with parameters a,b.",
"\"CreateModel\"","\"fields,refields,imfields,K,W<,options>\"","creates a model list from basic input data."]; 

(**********************************************************************************************)

`Private`usen[CreateIdeal,CreateIdeal,"* usage  : 'CreateIdeal[ideal<,options>]' with\n"<>
"  - ideal : polynomial, ideal list or list of polynomials and ideal lists\n"<>
"* options : Var -> list of variables or assignments of variables to numbers\n"<>
"            Par -> list of parameters or assignments of parameters to numbers\n"<>
"* purpose : Creates an ideal list by combining all polynomials in the first argument, and all variables and parameters specified in all arguments. Parameters (Variables) assigned to values using the Par (Var) option are replaced by those values and thereby eliminated.\n"<>
"* output  : An ideal list with assignments to the keywords 'Var', 'Par' and 'Gen'. See '?IdealKeywords' for the structure of an ideal list.\n"<>
"* example : 'CreateIdeal[{IDPot/.model,IPot/.model}]' combines the ideals 'IDPot' and 'IPot' of 'model' into one ideal.",
"\"CreateIdeal\"","\"ideal<,options>\"","creates an ideal list from basic input data"];

(**********************************************************************************************)

`Private`usen[CalcModel,CalcModel,"* usage   : 'CalcModel[model<,options>]' where\n"<>
"  - model : a model list as created by 'CreateModel'\n"<>
"* options : Dummies  -> a list with entries 'expr -> realdummyvariable'\n"<>
"            CDummies -> a list with entries 'expr -> complexdummyvariable'\n"<>
"            RDummies -> a list of rules\n"<>
"* purpose : Adds all ideal lists to a model list. The options can be used to replace transcendental functions by dummy variables.\n"<>
"* output  : A model list, with added assigments to all keywords starting with 'I'. (See '?ModelKeywords' for a list of keywords.)\n"<>
"* example : 'model=CalcModel[model]' adds assignments to all keywords starting with 'I' to the model list 'model'.",
"\"CalcModel\"","\"model<,options>\"","adds all ideal lists to a model list."];

(**********************************************************************************************)

Meth::usage="Method option for Stringvacua modules. Usage: 'Meth->\"methodstring\"' in
module call.";

`Private`use[VarPar,"Module Options","Option to specify variables and parameters in Stringvacua modules. Usage: 'VarPar->varparlist', where varparlist is a list of variables/parameters or a list of assigments of variables/parameters to values (depending on module).",
"Option to set values of variables and parameters in Stringvacua modules."];

`Private`usen[Elimination,Elimination,"* usage     : 'Elimination[ideal<,options>]' where\n"<>
"  - ideal   : (list of) ideal list(s), combined into a single ideal by the module\n"<>
"* options   : Meth   -> a string set to \"Basic\", \"HD\" or \"Prod\" (default \"Basic\")\n"<>
"              VarPar -> list of parameters/variables to be eliminated\n"<>
"                        (if omitted all variables will be eliminated)\n"<>
"              All global options in '?StringvacuaOptions'\n"<>
"* purpose : Eliminates parameters/variables from ideal to produce constraints on the remaining parameters/variables. Typically, it is used to eliminate all variables to obtain constraints on the parameters of the model.\n"<>
"* output  : An ideal list, with 'Var' containing the remaining parameters/variables, 'Par' the empty list and 'Gen' containing the constraints.\n"<>
"* example : 'Elimination[{IPot,IDPot}/.model]' calculates the constraints on the parameters of 'model' which follow from imposing the vanishing of the potential and its first derivatives.",
"\"Elimination\"","\"ideal<,options>\"","eliminates parameters/variables from an ideal."];

(**********************************************************************************************)

`Private`use[RemoveEntry,RemoveEntry,"* usage : 'RemoveEntry[list,keyword]' where\n"<>
"  - list    : a model or ideal list\n"<>
"  - keyword : (list of) keyword(s) for model or ideal list\n"<>
"* purpose : Removes entries 'keyword->_' from model or ideal list. (see '?ModelKeywords' and '?IdealKeywords' for a list of keywords)\n"<>
"* output  : Model or ideal list with entries removed.",
"* usage : 'RemoveEntry[list,keyword]', removes entry 'keyword->_' from model or ideal list."];

(**********************************************************************************************)

`Private`usen[CallSingular,CallSingular,"* usage     : 'CallSingular[script,varpara,repl<,options>]' where\n"<>
"  - script  : a string containing a Singular script with placeholders\n"<>
"  - varpara : a list of all variables/parameters\n"<>
"  - repl    : a list of replacements of the form 'placeholder->value'\n"<>
"* options   : All global options in '?StringvacuaOptions'\n"<>
"* purpose   : A low-level call to Singular using the script provided in the first argument with placeholders replaced as indicated in the third argument. All variables/parameters must be provided in the second argument and must be of the form explained in '?StringvacuaInterface' to be properly converted for use in Singular. Singular scripts can be taken from the list 'SingularScripts' of scripts available to the package (see '?SingularScriptInfo' for a list of such scripts) or can be written by the user.\n"<>
"* output    : The raw output from Singular as written to the file ~/.singout or an error message if Singular execution is terminated or fails.\n"<>
"* example   : 'CallSingular[\"SingularDim\"/.SingularScripts,{x,y},{\"@@listofvariables\"->{x,y}, \"@@coefffield\"->{0},\"@@inputideal\"->{x^2+2 y^2,x^3-5 y}}]' calls Singular with the 'SingularDim' script and finds the dimension of a simple ideal in two variables x,y.",
"\"CallSingular\"","\"script,varpara,repl<,options>\"","a low-level call to Singular."];

(**********************************************************************************************)

`Private`usen[PrimDec,PrimDec,"* usage     : 'PrimDec[ideal<,options>]' where\n"<>
"  - ideal   : (list of) ideal list(s), combined into a single ideal by the module\n"<>
"* options   : VarPar     -> (list of) rules assigning par./var. to values\n"<>
"              Dim        -> (list of) complex dimension(s) of ideals to be\n"<>
"                            kept (default 0)\n"<>
"              RRoots     -> \"Probab\", \"Determ\" or \"Off\" (default)\n"<>
"              NRoots     -> min. number of real roots for ideal to be\n"<>
"                            kept (default 0)\n"<>
"              SturmPoly  -> (list of) polynomial(s) for Sturm query filtering\n"<>
"              SturmSigns -> (list of) sign(s) for Sturm query filtering\n"<>
"              All global options in '?StringvacuaOptions'\n"<>
"* purpose   : Calculates the primary decomposition of the given ideal. Parameters/variables of the ideal can be replaced by values using the 'VarPar' option. Parameters which are not replaced are treated as field extensions. To switch calculation of the number of real roots on the 'RRoots' option should be set to one of the two methods \"Probab\" or \"Determ\".\n"<>
"* output    : A list of ideal lists, each corresponding to a primary ideal. Ideals with complex dimensions other than those specified in option 'Dim' are filtered out. If real root calculation is switched on, only ideals with at least as many real roots as specified in option 'NRoots' are calculated. If options 'SturmPoly' and 'SturmSigns' are provided only ideals with real roots on which polynomials in 'SturmPoly' assume the specified patterns of signs given in 'SturmSigns' are kept.\n"<>
"* example   : 'PrimDec[IDPot/.model,VarPar->{a->1,b->1}]' calculates the (zero complex dim.) primary ideals of 'IDPot' for a 'model' with parameters a,b replaced by 1. This call can be equivalently written as 'PrimDec[IDPot/.model/.{a->1,b->1}]'.",
"\"PrimDec\"","\"ideal<,options>\"","calculates the primary decomposition of an ideal."];

(**********************************************************************************************)

`Private`use[Poly,"Module Options","Option for Stringvacua modules to specify polynomials. Usage: 'Poly->(list of) polynomial(s) or ideal list(s)'","Option for Stringvacua modules to specify polynomials."];

`Private`usen[Saturation,Saturation,"* usage   : 'Saturation[ideal<,options>]' where\n"<>
"  - ideal : (list of) ideal list(s), will be combined into a single ideal by the module\n"<>
"* options : Poly       -> (list of) polynomial(s) or ideal list(s) (def. 1)\n"<>
"            VarPar     -> (list of) rule(s) assigning par./var. to value(s)\n"<>
"            RRoots     -> \"Probab\", \"Determ\" or \"Off\" (default \"Probab\")\n"<>
"            NRoots     -> min. number of real roots for ideal to be kept\n"<>
"                         (default 0)\n"<>
"            SturmPoly  -> (list of) polynomial(s) for Sturm query filtering\n"<>
"            SturmSigns -> (list of) sign(s) for Sturm query filtering\n"<>
"            All global options in '?StringvacuaOptions'\n"<>
"* purpose : Calculates the saturation of the ideal w.r.t. the polynomials specified in the 'Poly' option. Parameters can be assigned to values using the 'VarPar' option. All parameters not replaced are treated as field extensions. To switch calculation of the number of real roots on, the 'RRoots' option should be set to one of the two methods \"Probab\" or \"Determ\".\n"<>
"* output : An ideal list containing the saturated ideal. See '?IdealKeywords' for the structure of an ideal list. If real root calculation is switched on, only ideals with at least as many real roots as specified in option 'NRoots' are calculated. If options 'SturmPoly' and 'SturmSigns' are provided, only ideals with real roots on which the polynomials in 'SturmPoly' assume the specified patterns of signs given in 'SturmSigns' are kept.\n"<>
"* example : 'Saturation[IDPot/.model,Poly->t/.Gen/.IFTerms/.model]' calculates the saturation of 'IDPot' w.r.t. the real part of the F term ideal corresponding to variable t, treating all parameters as field extensions.",
"\"Saturation\"","\"ideal<,options>\"","calculates the saturation of an ideal w.r.t. a set of polynomials."];

(**********************************************************************************************)

`Private`usen[DimIdeal,DimIdeal,"* usage     : 'DimIdeal[ideal<,options>]' where\n"<>
"  - ideal   : (list of) ideal list(s), combined into a single ideal by the module\n"<>
"* options   : VarPar -> (list of) rule(s) assigning parameter(s)/variable(s) to value(s)\n"<>
"* purpose   : Computes complex dimension of an ideal. Parameters are replaced if values are assigned to them with the 'VarPar' option, otherwise they are treated as field extensions.\n"<>
"* output    : An ideal list with its dimension assigned to the keyword 'Dim'. See '?IdealKeywords' for the structure of an ideal list.\n"<>
"* example   : 'DimIdeal[IDPot/.model/.{a->1,b->1}]' finds the complex dimension of 'IDPot/.model' for parameter values a=b=1.",
"\"DimIdeal\"","\"ideal<,options>\"","calculates the complex dimension of an ideal."];

(**********************************************************************************************)

`Private`usen[SaturationPrimDec,SaturationPrimDec,"* usage   : 'SaturationPrimDec[ideal<,options>]' where\n"<>
"  - ideal : (list of) ideal list(s), combined into a single ideal by the module\n"<>
"* options : Poly       -> (list of) polynomial(s) or ideal list(s)\n"<>
"                          (default 1)\n"<>
"            VarPar     -> (list of) rule(s) assigning par./var. to value(s)\n"<>
"            Dim        -> (list of) complex dimension(s) of ideals to be kept\n"<>
"                          (default 0)\n"<>
"            RRoots     -> \"Probab\", \"Determ\" or \"Off\" (default \"Off\")\n"<>
"            NRoots     -> minimal number of real roots for ideal to be kept\n"<>
"                         (default 0)\n"<>
"            SturmPoly  -> (list of) polynomial(s) for Sturm query filtering\n"<>
"            SturmSigns -> (list of) sign(s) for Sturm query filtering\n"<>
"            All global options in '?StringvacuaOptions'\n"<>
"* purpose : Calculates the saturation of the ideal w.r.t. the polynomials specified in the 'Poly' option and then performs a primary decomposition. To switch calculation of the number of real roots on, the 'RRoots' option should be set to one of the two methods \"Probab\" or \"Determ\". This combines the modules 'Saturation' and 'PrimDec' at the level of Singular for faster performance.\n"<>
"* output  : A list of ideal lists, each corresponding to a primary ideal. Ideals with complex dimensions other than those specified in option 'Dim' are filtered out. If real root calculation is switched on, only ideals with at least as many real roots as specified in option 'NRoots' are calculated. If options 'SturmPoly' and 'SturmSigns' are provided only ideals with real roots on which the polynomials in 'SturmPoly' assume the specified patterns of signs given in 'SturmSigns' are kept.\n"<>
"* example : 'SaturationPrimDec[IDPot/.model,t/.Gen/.IFTerms/.model]' saturates 'IDPot' w.r.t. the real part of the F term corresponding to the variable t and performs a primary decomposition.",
"\"SaturationPrimDec\"","\"ideal<,options>\"","performs a saturation and primary decomposition for an ideal."]; 

(**********************************************************************************************)

`Private`usen[NumRoots,NumRoots,"* usage   : 'NumRoots[ideal<,options>]' where\n"<>
"  - ideal : (list of) ideal list(s), combined into a single ideal by the module\n"<>
"* options : VarPar -> (list of) rule(s) assigning par./var. to value(s)\n"<>
"            Meth   -> \"Probab\" (default) or \"Determ\".\n"<>
"            All global options in '?StringvacuaOptions'\n"<>
"* purpose : Calculates the number of real roots of a zero-dimensional ideal. Field extensions are not allowed, so all parameters must be replaced by numbers.\n"<>
"* output  : An ideal list with the number of real roots assigned to the keyword 'NRoots'. See '?IdealKeywords' for the structure of an ideal list.\n"<>
"* example : 'NumRoots[IDPot/.model,VarPar->{a->1,b->1}]' calculates the number of real roots of the ideal 'IDPot' for parameter values a=b=1.",
"\"NumRoots\"","\"ideal<,options>\"","calculates the number of real roots of an ideal."];

(**********************************************************************************************)

`Private`usen[SaturationExpand,SaturationExpand,"* usage : 'SaturationExpand[model<,options>]' where\n"<>
"  - model : a model list as created by 'CreateModel' and 'CalcPotential'\n"<>
"* options : VarPar -> (list of) rule(s) assigning par./var. to value(s)\n"<>
"              Poly -> (list of) polynomial(s) or ideal list(s) (def. {})\n"<>
"            FTerms -> integer n, a real field,list of real fields\n"<>
"                      or list of real fields and integers n\n"<>
"               Dim -> (list of) complex dimension(s) of ideals\n"<>
"                       to be kept (default 0)\n"<>
"            RRoots -> \"Probab\" (def.), \"Determ\" or \"Off\"\n"<>
"            NRoots -> min. number of real roots for ideal to be kept (def. 0)\n"<>
"         SturmPoly -> (list of) polynomial(s) for Sturm query filter (def. 1)\n"<>
"        SturmSigns -> (list of) sign(s) for Sturm query filter (def. {})\n"<>
"      TestRational -> (list of) rat. function(s) for Sturm query (def. {1})\n"<>
"        TestMatDet -> (list of) rat. matrices for Sturm query on\n"<>
"                      determinant(s) (def. {{1}})\n"<>
"      TestMatEigen -> (list of) rat. matrices for Sturm query on\n"<>
"                      eigenvalues (def. {{1}})\n"<>
"        TestLambda -> \"yes\" or \"no\" (default) for Sturm query on\n"<>
"                      scalar potential\n"<>
"     TestStability -> \"Hesse\" to test stability, otherwise \"Off\" (default)\n"<>
"        SturmLabel -> \"Automatic\", \"None\", or a string labeling the quantities\n"<>
"		      to be tested by Sturm queries\n"<>
"                      All global options in '?StringvacuaOptions'\n"<>
"* purpose : Calculate terms in the saturation expansion of IDPot/.model which arise from adding a subset of F terms to the ideal and saturating out the complementary set. Each term is then primary decomposed. The module only considers those terms in the saturation expansion where the F terms specified (by their corresponding real fields) in the 'FTerms' option vanish and a number between n_min and n_max of other F terms do not vanish. Additional polynomial constraints which will be imposed on each term in the saturation expansion can be provided in the 'Poly' option. Note that terms with empty ideal list are omitted from the result. To switch calculation of the number of real roots on, the 'RRoots' option should be set to \"Probab\" or \"Determ\". Sturm queries can be performed on each resulting ideal by using the same options as for the 'SturmQuery' module.\n"<>
"* output  : A list with entries of the form {f_1,...,f_k}->{list of ideal lists} where {f_1,...,f_k} is a subset of the real fields indicating the term in the saturation expansion with zero F terms f_1,...,f_k and all other F terms non-zero. In particular, the supersymmetric term is of the form {all real fields}->{list of ideal lists}. Ideals with complex dimensions other than those specified in option 'Dim' are filtered out. If real root calculation is switched on, only ideals with at least as many real roots as specified in option 'NRoots' are calculated. If options 'SturmPoly' and 'SturmSigns' are provided only ideals with real roots on which the polynomials in 'SturmPoly' assume the specified patterns of signs given in 'SturmSigns' are kept.\n"<>
"* example : 'SaturationExpand[model,VarPar->{a->1,b->1}]' finds all terms in the saturation expansion for parameter values a=b=1. 'SaturationExpand[model,VarPar->{a->1,b->1},FTerms->1]' finds all terms in the saturation expansion with a=b=1 and one F term non-zero. 'SaturationExpand[model,VarPar->{a->1,b->1},FTerms->t]' finds all terms in the saturation expansion with a=b=1 and the t F term zero.",
"\"SaturationExpand\"","\"model<,options>\"","calculates the saturation expansion of a model w.r.t. F terms."];

`Private`usen[SturmQuery,SturmQuery,"* usage : 'SturmQuery[ideal<,options>]' where\n"<>
"  - ideal  : (list of) ideal list(s), combined into a single ideal by the module\n"<>
"* options  : TestRational -> (list of) rat. function(s) for Sturm query\n"<>
"                             (def. {1})\n"<>
"             TestMatDet   -> (list of) rat. matrices for Sturm query on\n"<>
"                             determinant(s) (default {{1}})\n"<>
"             TestMatEigen -> rat. Hermitian matrix for Sturm query on\n"<>
"                             eigenvalues (default {{1}})\n"<>
"             VarPar       -> (list of) rule(s) assigning par./var. to\n"<>
"                             value(s) (default {})\n"<>
"             RRoots	   -> \"Probab\", \"Determ\" or \"Off\" (default \"Off\")\n"<>
"             SturmLabel   -> \"None\" or a string labeling the quantity to be\n"<>
"                             tested\n"<>
"             All global options in '?StringvacuaOptions'\n"<>
"* purpose  : Computes the signs of the polynomials in the 'TestRational' option, the sign(s) of the determinant(s) of the matrices in 'TestMatDet' and the signs of the eigenvalues of the matrix in 'TestMatEigen' on the real roots of the ideal. No field extensions are allowed, so parameters have to be replaced by values, for example using the option 'VarPar'.\n"<>
"* output   : An ideal list with a list assigned to the keyword 'Sturm'. The result, as assigned to the keyword 'Sturm', takes two different formats, depending on whether or not the sign of eigenvalues were requested. In the case where eigenvalues are not considered the output is a list of lists. The sublists contain, in order, the rational functions checked, the matrices whose determinants are checked, a series of lists containing the realised sign conditions and a list containing the multiplicities associated to each realised sign condition. In the case where eigenvalues are considered the output is again a list of lists. The sublists in this case contain, the rational functions checked, the matrices whose determinants are checked, the matrix whose eigenvalues are checked, and then one sublist for each real root of the input ideal. Each of these sublists has multiple entries which are, in order, the signs of the rational functions and determinants on that root, the different eigenvalue signs on that root and finally the multiplicities of the signs of eigenvalues on that root (note that degenerate eigenvalues may sometimes result in incorrect multiplicities).\n"<>
"* example    : 'SturmQuery[id,TestRational->x]' finds the signs of the ideal variable x on the real roots of the ideal described by the ideal list id.",
"\"SturmQuery\"","\"ideal<,options>\"","performs Sturm queries on the real roots of a zero-dim. ideal."];

`Private`usen[FacGroebner,FacGroebner,"* usage : 'FacGrobner[ideal<,options>]', where\n"<>
" - ideal  : (list of) ideal list(s), combined into a single ideal by the module\n"<>
"* options : VarPar -> (list of) rule(s) assigning par./var. to value(s)\n"<>
"               Dim -> (list of) complex dimension(s) of ideals\n"<>
"                      to be kept (default 0)\n"<>
"            RRoots -> \"Probab\" (def.), \"Determ\" or \"Off\"\n"<>
"            NRoots -> min. number of real roots for ideal to be kept (def. 0)\n"<>
"            SturmPoly -> (list of) polynomial(s) for Sturm query filter (def. 1)\n"<>
"            SturmSigns -> (list of) sign(s) for Sturm query filter (def. {})\n"<>
"             All global options in '?StringvacuaOptions'\n"<>
"* purpose : Calculates the factorising Groebner basis for a given ideal in list ideal. Parameters/variables of the ideal can be replaced by values using the VarPar option.  Parameters which are not replaced are treated as extensions of the base number field. To switch calculation of the number of real roots on, the RRoots option should be set to one of the two methods \"Probab\" or \"Determ\".\n"<>
"* output : A list of ideal lists. Ideals with complex dimensions other than those specified in option Dim are filtered out. If real root calculation is switched on, only ideals with at least as many real roots as  specified in option NRoots are calculated. If the options SturmPoly and SturmSigns are provided only ideals with real roots on which polynomials in SturmPoly assume the specified patterns of signs given in SturmSigns are kept.\n"<>
"* example: 'FacGroebner[{Var->{x,y},Gen->{x^2-y^2}}]' performs a factorising Groebner basis computation on the ideal generated by x^2-y^2.",
"\"FacGroebner\"","\"ideal<,options>\"","calculates the factorised Groebner basis for the ideal given in list ideal."];

`Private`usen[ThomEncode,ThomEncode,"* usage : 'ThomEncode[ideal<,options>]', where\n"<>
" - ideal  : (list of) ideal list(s), combined to single ideal by the module\n"<>
"* options : VarPar -> (list of) rule(s) assigning par./var. to value(s)\n"<>
"            All global options in '?StringvacuaOptions'\n"<>
"* purpose : Takes a zero-dimensional ideal, breaks it up into triangular systems, and Thom encodes each of them in turn.\n"<>
"* output  : A series of zero-dimensional ideals and a set of polynomials, the signs of which uniquely determine the roots on each triangular system.\n"<>
"* example : 'ThomEncode[id]' will break the ideal 'id' up into triangular systems and Thom encode the resulting ideals.",
"\"ThomEncode\"","\"ideal<,options>\"","takes a zero-dimensional ideal, break it up into triangular systems, and Thom encodes each of them in turn."];

`Private`usen[SimpleMinimise,SimpleMinimise,"* usage   : 'SimpleMinimise[]'\n"<>
"* purpose : This is a simple module for interactive minimisation",
"\"SimpleMinimise\"","\" \"","a simple module for interactive minimisation."];

`Private`usen[TF,TF,"* usage : 'TF[list]', displays a (model, ideal or other) list in table form (available only in notebook interface)",
"\"TF\"","\"list\"","displays a (model, ideal or other) list in table form (available only in notebook interface)"];

(**********************************************************************************************)
(* Initialise the package *)
(**********************************************************************************************)

(* load system dependent part of package: Unix or Windows *)

If[StringPosition[$System,"Windows"]==={},Get[$UserBaseDirectory<>"/Applications/Stringvacualib/StringvacuaUnix.m"],
                                          Get[$UserBaseDirectory<>"\\Applications\\Stringvacualib\\StringvacuaWindows.m"]];

Begin["`Private`"]

(* find paths and Singular scripts *)

FindPaths[];
FindSingularScripts[];

(* turn error off, so that multiple replacements work *)

Off[ReplaceAll::reps];

(* fix Subsets for Mathematica < 5.2 *)

If[$VersionNumber<5.2,<< DiscreteMath`Combinatorica`;Subsets2[a_,b_List] := Select[DiscreteMath`Combinatorica`Subsets[a], Length[#] >= b[[1]] && Length[#] <= b[[2]] &],Subsets2[a_,b_List]:=Subsets[a,b]];

(* print package header *)

StylePrint["Stringvacua package, Version 1.0, December 2007\nJames Gray, Yang-Hui He, Anton Ilderton and Andre Lukas", "Text", FontSize -> 13, CellFrame -> True, FontWeight -> 
"Bold", FontColor -> RGBColor[0.5, 0, 0], TextAlignment -> Center, Background 
-> RGBColor[1., 1., 0.9],FontFamily->"Times"];

(* print initial help message, very old version, does not work for 6.0 *)

(* If[$Notebooks,StylePrint["Execute 'SimpleMinimise[]' for a simple interactive calculation or click \*ButtonBox[\"here\",Active->True,ButtonStyle->\"AddOnsLink\",ButtonData:>\"Generalities\"] to open the Help Browser.",TextAlignment->Center,FontSize->10,FontColor->RGBColor[0.5,0,0]],
Print["Execute 'SimpleMinimise[]' for a simple interactive calculation\nor '?Stringvacua' for help."]]; *)

(* print initial help message, old version *)

(* If[$Notebooks,StylePrint["Execute \*Button[\"SimpleMinimise[]\",SimpleMinimise[]] for a simple interactive calculation or click \*ButtonBox[\"here\",Active->True,ButtonFrame->\"DialogBox\",ButtonFunction:>(FrontEndExecute[ { FrontEnd`HelpBrowserLookup[ \"AddOns\", \"Generalities\"]}]&)] to open the Help Browser.", TextAlignment -> Center, FontSize -> 12, FontColor -> RGBColor[0.5, 0, 0],FontFamily->"Times"],Print["Execute 'SimpleMinimise[]' for a simple interactive calculation\nor '?Stringvacua' for help."]]; *)

(* print initial help message, old version two *)

(*
If[$Notebooks,StylePrint["Click \*ButtonBox[StyleBox[\"here\",FontColor->RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[{FrontEnd`NotebookLocate[{\""<>$UserBaseDirectory<>"/Applications/Stringvacualib/examples/Examples_Overview.nb"<> "\",\"\"}]}]&)] for example notebooks, execute \*ButtonBox[StyleBox[\"SimpleMinimise[]\",FontColor->RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonEvaluator->\"Automatic\",ButtonFunction:>(SimpleMinimise[]&)] for a simple interactive calculation or click \*ButtonBox[StyleBox[\"here\",FontColor->RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { FrontEnd`HelpBrowserLookup[ \"AddOns\", \"Generalities\"]}]&)] to open the Help Browser.", TextAlignment -> Center, FontSize -> 12, FontColor -> RGBColor[0.6, 0.3, 0],FontFamily->"Times"],Print["Execute 'SimpleMinimise[]' for a simple interactive calculation\nor '?Stringvacua' for help."]]; *)

If[$Notebooks,StylePrint["Execute \*ButtonBox[StyleBox[\"SimpleMinimise[]\",FontColor->RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonEvaluator->\"Automatic\",ButtonFunction:>(SimpleMinimise[]&)] for a simple interactive calculation or click \*ButtonBox[StyleBox[\"here\",FontColor->RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { FrontEnd`HelpBrowserLookup[ \"AddOns\", \"Generalities\"]}]&)] to open the Help Browser, which contains \*ButtonBox[StyleBox[\"examples\",FontColor->RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { FrontEnd`HelpBrowserLookup[ \"AddOns\", \"Examples\", \"Examples_Overview.nb\"]}]&)].", TextAlignment -> Center, FontSize -> 12, FontColor -> RGBColor[0.6, 0.3, 0],FontFamily->"Times"],Print["Execute 'SimpleMinimise[]' for a simple interactive calculation\nor '?Stringvacua' for help."]];

End[]

(**********************************************************************************************)
(* initialise local options *)

LocalOptionsToDefault[]:=Module[{},

(* StringvacuaStatus *)

SingularStat="undefined";
SingularVar={};
SingularProt="";

(* Options for individual modules *)

Options[CreateModel]={Par->{},DeltaPot->0,Meth->"math",KMetricInverse->{}};
Options[CalcModel]={Dummies->{},CDummies->{},RDummies->{}};
Options[CreateIdeal]={Var->{},Par->{}};
Options[Elimination]={Meth->"Basic",VarPar->{}};
Options[PrimDec]={Dim->0,RRoots->"Off",NRoots->0,VarPar->{},SturmPoly->1,SturmSigns->{}};
Options[Saturation]={Poly->{1},VarPar->{},RRoots->"Off",NRoots->0,SturmPoly->1,SturmSigns->{}};
Options[DimIdeal]={VarPar->{}};
Options[SaturationPrimDec]={Poly->{1},VarPar->{},Dim->0,RRoots->"Off",NRoots->0,
SturmPoly->1,SturmSigns->{}};
Options[NumRoots]={VarPar->{},Meth->"Off"};
Options[SaturationExpand]={VarPar->{},Poly->{},FTerms->{},Dim->0,RRoots->"Off",NRoots->0,
                           SturmPoly->1,SturmSigns->{},Meth->"Basic",TestRational->{1},TestMatDet->{{1}},
                           TestMatEigen->{{1}},TestLambda->"no",TestStability->"Off",Thom->"no",SturmLabel->"Automatic"};
Options[SturmQuery]={TestRational->{1},TestMatDet->{{1}},TestMatEigen->{{1}},VarPar->{},RRoots->"Off",SturmLabel->"None"};
Options[FacGroebner]={Dim->0,RRoots->"Off",NRoots->0,VarPar->{},SturmPoly->1,SturmSigns->{}};
Options[ThomEncode]={VarPar->{}};

(* Warning messages in relation to options settings *)
If[(FieldChar/.Options[Stringvacua]) > 0,Message[Stringvacua::"field char",FieldChar/.Options[Stringvacua]]]];

(* Initialise global and local options *)

(**********************************************************************************************)

OptionsToDefault[];
LocalOptionsToDefault[];
SingularScriptInfo::usage="Singular scripts used by the package are collected in the list "<>`Private`sb["\"SingularScripts\""]<>" and are automatically read in from files with extension "<>`Private`sb["\".mathsing\""]<>" in "<>`Private`sb["\"$UserBaseDirectory<>/Applications/Stringvacualib\""]<>". Use "<>`Private`sb["\"scriptname/.SingularScripts\""]<>" to access an individual script. Currently available : \n"<>`Private`SingularScriptHelp;

Begin["`Private`"]

(**********************************************************************************************)
(* Internal modules *)
(**********************************************************************************************)

(* Check common local options and create Singular replacements. *)

CommonOptions[opt_,def_,var_,repl_:{},coef0_:{}]:=Module[{repllist,stat},

repllist={};

(* Check option Dim *)

If[MemberQ[def,Dim->_],Dim1=Flatten[{Dim/.opt/.def}];
If[!IsInteger[Dim1,0,Infinity],Message[Stringvacua::"illegal value","Dim",0]; Dim1={0}];
repllist=Append[repllist,"@@compdimkeep"->Dim1]];

(* Check option NRoots *)

If[MemberQ[def,NRoots->_],NRoots1=NRoots/.opt/.def;
If[!IsInteger[{NRoots1},0,Infinity],
Message[Stringvacua::"illegal value","NRoots",0]; NRoots1=0];
repllist=Append[repllist,"@@rootnumkeep"->NRoots1]];

(* Check option SturmPoly *)

If[MemberQ[def,SturmPoly->_],SturmPoly1=ExtractPol[Flatten[{SturmPoly/.opt/.def}]/.repl];
If[!IsPolynomial[SturmPoly1,var],
Message[Stringvacua::"illegal value","SturmPoly",1]; SturmPoly1={1}];
repllist=Append[repllist,"@@polysignfilter"->Numerator[SturmPoly1]]];

(* Check option SturmSigns *)

If[MemberQ[def,SturmSigns->_],SturmSigns1=Flatten[{SturmSigns/.opt/.def}];
If[Length[SturmSigns1]>0,
If[(Mod[Length[SturmSigns1],Length[SturmPoly1]]!=0) || (!IsInteger[SturmSigns1,-1,1]),
Message[Stringvacua::"illegal value","SturmSigns","{}"]; SturmSigns1={}]];
repllist=Append[repllist,"@@signfiltersigns"->SturmSigns1]];

(* Check option RRoots *)

If[MemberQ[def,RRoots->_],RRoots1=RRoots/.opt/.def;
  If[!MemberQ[{"Probab","Determ","Off"},RRoots1],Message[Stringvacua::"illegal value","RRoots","Off"]; RRoots1="Off"];
  stat={FieldChar1>0,Length[coef0]>1,RRoots1=!="Off",NRoots1>0,SturmPoly1 =!= {1} && SturmSigns1 =!= {}};
  If[stat[[1]] || stat[[2]],                        (* if finite fields or field extension *)
     If[stat[[3]] || stat[[4]] || stat[[5]],        (* if any real root filtering is switch on... *)
       RRoots1="Off"; NRoots1=0; SturmPoly1={1}; SturmSigns1={};      (* switch if off ... *)
       Message[Stringvacua::"real roots"]],                           (* and display warning to that effect *)
                                                                      (* no finite fields or field extension *)
     If[!stat[[3]] && stat[[4]],RRoots1="Probab";Message[Stringvacua::"NRoots"]];  (* for NRoots>0 switch RRoots on *)
     If[!stat[[3]] && stat[[5]],RRoots1="Probab";Message[Stringvacua::"Sturm"]]];   (* for Sturm on switch RRoots on *)    


  If[(FieldChar1 > 0) && (RRoots1 =!= "Off"),Message[Stringvacua::"finite char"];RRoots1="Off"];
  If[(Length[coef0] > 1) && (RRoots1 =!= "Off"),Message[Stringvacua::"field extension"];RRoots1="off"];
  If[SturmPoly1 =!= {1} && SturmSigns1 =!= {} && RRoots1 === "Off" && FieldChar1 == 0
     && Length[coef0] <=1,RRoots1="Probab";Message[Stringvacua::"Sturm"]];
  If[NRoots1>0 && RRoots1==="Off" && FieldChar1 == 0 && Length[coef0] <=1 ,RRoots1="Probab";Message[Stringvacua::"NRoots"]];
repllist=Append[repllist,"@@realrootsonly"->RRoots1]];

(* return replacement list *)

repllist];

(**********************************************************************************************)
(* Check if central global options exist and have sensible values. If not set to defaults. *)

CheckOptions1[opt___]:=Module[{opt1},

opt1=Flatten[{opt}];

(* determine values of global options from defaults and local options *)

SingularProtocol1=SingularProtocol/.opt1/.Options[Stringvacua];
SingularMaxTime1=SingularMaxTime/.opt1/.Options[Stringvacua];
SingularTrace1=SingularTrace/.opt1/.Options[Stringvacua];
SingularEcho1=SingularEcho/.opt1/.Options[Stringvacua];
PackageProtocol1=PackageProtocol/.opt1/.Options[Stringvacua];
XtermGeometry1=XtermGeometry/.opt1/.Options[Stringvacua];
XtermPath1=XtermPath/.opt1/.Options[Stringvacua];
SingularPath1=SingularPath/.opt1/.Options[Stringvacua];
SingularTimer1=SingularTimer/.opt1/.Options[Stringvacua];

(* Check variable SingularTimer *)

If[!MemberQ[{"on","off"},SingularTimer1],
Message[Stringvacua::"illegal value","SingularTimer","on"];
SingularTimer1="on"];

If[SingularTimer1 === "on",

(* Check variable SingularProtocol *)

If[!MemberQ[{"Xterm","String","None"},SingularProtocol1],
   SingularProtocol1="None";
   Message[Stringvacua::"illegal value","SingularProtocol","None"]];

(* Check variable SingularMaxTime *)

If[!IsInteger[{SingularMaxTime1},0],
   Message[Stringvacua::"illegal value","SingularMaxTime",60];
   SingularMaxTime1=60]];

(* Check variable SingularTrace *)

If[!IsInteger[{SingularTrace1},0,4],
   Message[Stringvacua::"illegal value","SingularTrace",0];
   SingularTrace=0];

(* Check variable SingularEcho *)

If[!IsInteger[{SingularEcho1},0,10],
   Message[Stringvacua::"illegal value","SingularEcho",0];
   SingularEcho1=0];

(* Set variable SingularStat to undefined *)

SingularStat="undefined"];
(**********************************************************************************************)
(* Check if non-central global variables exist and if they have sensible values. If not set to defaults. *)

CheckOptions0[opt___]:=Module[{opt1},

opt1=Flatten[{opt}];

(* determine values of global options from defaults and local options *)

SingularAccuracy1=SingularAccuracy/.opt1/.Options[Stringvacua];
PackageProtocol1=PackageProtocol/.opt1/.Options[Stringvacua];
PlanckInverse1=PlanckInverse/.opt1/.Options[Stringvacua];
FieldChar1=FieldChar/.opt1/.Options[Stringvacua];

(* check variable PackageProtocol *)

If[!IsInteger[{PackageProtocol1},0,10],
   Message[Stringvacua::"illegal value","PackageProtocol",10];
   PackageProtocol1=10];

(* check variable FieldChar *)

If[(!IsInteger[{FieldChar1},0,2147483629]) || ((FieldChar1 > 0) && !PrimeQ[FieldChar1]),
    Message[Stringvacua::"illegal value","FieldChar",0];
    FieldChar1=0];

(* check variable SingularAccuracy *)

If[!NumericQ[SingularAccuracy1],Message[Stringvacua::"illegal value","SingularAccuracy",10^(-5)];
   SingularAccuracy1=10^(-5),
   If[(SingularAccuracy1 <= 0) || (SingularAccuracy1 > 1),
      Message[Stringvacua::"illegal value","SingularAccuracy",10^(-5)];
      SingularAccuracy1=10^(-5)]]];

(**********************************************************************************************)
(* Check if one list is subset of another *)

IsSubset[list1_List,list2_List]:=Module[{},

Apply[And,Table[MemberQ[list2,list1[[i]]],{i,1,Length[list1]}]]];

(**********************************************************************************************)
(* Test if list consists of polynomials in variables over real numbers *)

IsPolynomial[pollist_,varlist_]:=Module[{poly,flatlist,indvar,polvar,dummyvariable},

poly=(Length[pollist]>0);
flatlist=Flatten[pollist];
indvar=Union[Cases[pollist, Subscript[X_Symbol, Y_Integer] | X_Symbol[Y_Integer] | 
                   Subscript[X_Symbol, Y_Symbol], Infinity]];
repllist=flatlist/.Table[indvar[[i]]->dummyvariable[i],{i,1,Length[indvar]}];
polvar=Join[indvar,Union[Cases[repllist,_Symbol?(! NumericQ[#] &), Infinity]]];
poly=poly&&Apply[And,Table[PolynomialQ[flatlist[[i]],polvar],{i,1,Length[flatlist]}]];
poly=poly&&Apply[And,Table[FreeQ[flatlist[[i]],List],{i,1,Length[flatlist]}]];
poly=poly&&(Length[Complement[polvar,varlist]]==0)&&FreeQ[flatlist,_String,Infinity];
poly];

(**********************************************************************************************)
(* Test if list consists of integers in specified range *)

IsInteger[ilist_,min_,max_:Infinity]:=Module[{},
If[Length[ilist]>0,
  Apply[And,Table[IntegerQ[ilist[[i]]]&&(ilist[[i]]>=min)&&(ilist[[i]]<=max),{i,1,Length[ilist]}]],
False]];

(**********************************************************************************************)
(* Convert variable into Singular form *)

ReplaceVar[var_]:= StringReplace[Apply[StringJoin,Map[ToString,Map[FullForm,var /.
                  {Subscript[X_Symbol, Y_Integer] -> {X,"_",Y},
                   X_Symbol[Y_Integer] -> {X,"(",Y,")"},Subscript[X_Symbol,Y_Symbol]->{X,"_",Y},
                   X_Symbol -> {X}}]]],{"\\["->"","]"->"_","\"_\""->"_","\""->""}];

(**********************************************************************************************)
(* Test if expression has correct form to be a Singular variable *)

IsVar[varlist_]:=Count[varlist, Subscript[X_Symbol, Y_Integer] | X_Symbol[Y_Integer] |
                      Subscript[X_Symbol,Y_Symbol] | X_Symbol]==Length[varlist];

(**********************************************************************************************)
(* Extract polynomials from a more complicated list with sub-list, make coefficients integer *)

ExtractPol[list_]:=Module[{},
 Flatten[Together[Rationalize[N[Flatten[list/.{___,Gen->X_,___}->Flatten[{X}]]/.(Z_->T_)->T,100],SingularAccuracy1]]]];

(**********************************************************************************************)
(* Extract variables from an ideal or list of ideals *)

ExtractVar[list_]:=Module[{},
 Cases[Union[Flatten[Cases[list,(Var->X_)->X,Infinity]]],Y_/;(!NumericQ[Y])]
];

(**********************************************************************************************)
(* Extract parameters from an ideal or list of ideals *)

ExtractPar[list_]:=Module[{},
 Cases[Union[Flatten[Cases[list,(Par->X_)->X,Infinity]]],Y_/;(!NumericQ[Y])]
];

(**********************************************************************************************)
(* remove entry with a particular keyword from a list *)

RemoveEntry[list_,keyword_]:=Module[{k,i,l},

k=Flatten[{keyword}];
l=list;
Do[l=Delete[l,Position[l,k[[i]]->_]],{i,1,Length[k]}];
l];

(**********************************************************************************************)
(* write protocol entry *)

WriteProt[expr_,s_]:=Module[{p},

p=PackageProtocol1;
If[p>0,
  ProtCounter=ProtCounter-1;
  ProtNum=ProtNum+1;
  If[ProtCounter<=0,
    If[$Notebooks,
      NotebookWrite[EvalNote,RowBox[{Cell[BoxData[ToBoxes[{ProtNum,expr,s}]],FontSize->10,FontWeight->"Plain",TextAlignment->-1]}]],
WriteString["stdout",ToString[ProtNum]<>", "]];ProtCounter=11-p]]];

(**********************************************************************************************)
(* initialise protocol *)

InitProt[s_]:=Module[{nbl},

ProtNum=0;
ProtCounter=1;
If[PackageProtocol1 > 0,
  If[$Notebooks,
     nbl=Notebooks["Stringvacua protocol"];
     If[Length[nbl]>0,EvalNote=nbl[[1]],
        EvalNote=NotebookCreate[WindowTitle -> "Stringvacua protocol", WindowSize -> {320, 150}]];
     NotebookWrite[EvalNote,Cell[s,FontSize -> 10,FontFamily->"Times",FontColor -> RGBColor[0.6, 0.3, 0],FontWeight -> "Plain"]],
     WriteString["stdout",s]]]];


(**********************************************************************************************)
(* remove protocol *)

CleanProt[s_]:=Module[{},
 If[PackageProtocol1 > 0,
  If[$Notebooks,
     SelectionMove[EvalNote,After,Cell];
     NotebookWrite[EvalNote,Cell[BoxData[s],FontFamily->"Times",FontSize -> 10, FontColor -> RGBColor[0.6, 0.3, 0],FontWeight -> "Plain"]]]]];

(**********************************************************************************************)
(* Extract ideal lists from Singular output *)

ExtractIdeals[sout_,var_,para_]:=Module[{breaklist,i,j,outlist,id},

kwlist={Gen,Dim,NRoots};
breaklist=Join[Position[sout,{}],Position[sout,{Null}]];
If[Length[breaklist]==0 || Length[sout]==0,{},
  breaklist=Flatten[Prepend[breaklist,{0}]];
  outlist=DeleteCases[Table[Thread[Rule[Take[kwlist, breaklist[[i+1]]-breaklist[[i]] - 1],
          Take[sout,{breaklist[[i]]+1,breaklist[[i+1]]-1}]]],{i,1,Length[breaklist]-1}],{}];
  outlist=Map[Join[#,{Var->var,Par->para}]&,outlist];
outlist]];

(**********************************************************************************************)

TCell[s_]:=Cell[BoxData[ToBoxes[s]],FontSize->10,FontColor -> RGBColor[0.5,0,0]];
FCell[f_]:=Cell[BoxData[ToBoxes[f]],FontSize->12];

(**********************************************************************************************)
(* module to generate all replacements of transcendental functions, only Exp for now *)

translist[repl_List]:=Module[{allrepl,i,trans,dum,X,Y,Z},
allrepl=repl;

(* Add additional replacements for Exp *)

allrepl=Join[allrepl,Cases[repl,(Exp[-X_]->Y_)->(Exp[n_ X]->Y^(-n))]];
allrepl=Join[allrepl,Cases[repl,(Exp[-X_]->Y_)->(Exp[X]->1/Y)]];
allrepl=Join[allrepl,Cases[repl,(Exp[X_]->Y_)->(Exp[n_ X]->Y^n)]];
allrepl=Join[allrepl,Cases[repl,(Exp[X_]->Y_)->(Exp[X+Z_]->Y Exp[Z])]];
allrepl=Join[allrepl,Cases[repl,(Exp[-I X_]->Y_)->(Cos[X]->(Y+1/Y)/2)]];
allrepl=Join[allrepl,Cases[repl,(Exp[-I X_]->Y_)->(Sin[X]->I (Y-1/Y)/2)]];

(* Add additional replacements for Sqrt *)

allrepl=Join[allrepl,Cases[repl, (Sqrt[X_] -> Y_) -> (Power[X, Rational[n_, 2]] -> Y^n)]];
allrepl=Join[allrepl,Cases[repl, (Sqrt[X_] -> Y_) -> (X -> Y^2)]];
allrepl=Join[allrepl,Cases[repl, (Sqrt[X_] -> Y_) -> (Power[X,n_] -> Y^(2 n))]];
allrepl=Join[allrepl,Cases[repl, (Sqrt[X_] -> Y_) -> (Power[Power[X,m_],Rational[n_, k_]] -> Y^(2 n m/k))]];

allrepl];

(**********************************************************************************************)
(* module to generate polynomials for ideals *)

ToPol[expr_,repl_List]:=Module[{},

Map[Replace[#,{I X_->X,-I X_->X},{0}]&,Numerator[Together[Expand[TrigExpand[Flatten[{expr}]]//.repl]]]]];


(**********************************************************************************************)
(* Main package modules *)
(**********************************************************************************************)
(* module to create ideal list *)

CreateIdeal[inideal_List,opt___]:=Module[
{var1,par1,varrepl,parrepl,replide,outide,var,par,opt1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
var=Var/.opt1/.Options[CreateIdeal];
par=Par/.opt1/.Options[CreateIdeal];
CheckOptions0[opt1];

(* built ideal list *)

 varrepl=Cases[Flatten[{var}],(X_ -> Y_) -> (X -> Y)];
 parrepl=Cases[Flatten[{par}],(X_ -> Y_) -> (X -> Y)];
 replide=inideal/.Join[varrepl,parrepl];
 outide=ExtractPol[replide];
 var1=Union[Join[ExtractVar[replide],Complement[Flatten[{var}],varrepl]]];
 par1=Union[Join[ExtractPar[replide],Complement[Flatten[{par}],parrepl]]];
 If[!IsPolynomial[outide,Join[var1,par1]] || Length[var1]==0,Message[Stringvacua::"ideal incomplete"]];
{Var->var1,Par->par1,Gen->outide}];

(**********************************************************************************************)
(* module to create a model list *)

CreateModel[f_List, fR_List, fI_List,k_,w_,opt___]:=Module[
{opt1,par,deltav,n,Mpi,replf,fa,dk,dkb,g,dw,dwb,dwr,dwbr,F,Fb,Fr,Fbr,res,matdim,gi,ek,wb,wr,wbr,X,Y},

(* Options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
par=Par/.opt1/.Options[CreateModel];
deltav=DeltaPot/.opt1/.Options[CreateModel];
gih=KMetricInverse/.opt1/.Options[CreateModel];
meth=Meth/.opt1/.Options[CreateModel];
CheckOptions0[opt1];

n=Length[f];               (* number of complex fields *)
Mpi=PlanckInverse1;        (* inverse Planck mass *)

(* error checking *)

If[Length[fR]!=n || Length[fI]!=n,Message[Stringvacua::"model incorrect"],
If[!MemberQ[{"math","hand","off"},meth],Message[Stringvacua::"illegal value","Meth","math"],

(* replacement rules for complex with real fields *)

replf=Join[Table[f[[i]] -> fR[[i]] + I fI[[i]], {i, 1, n}],Table[Conjugate[f[[i]]] -> fR[[i]] - I fI[[i]],
      {i, 1, n}]];  
fa=Join[fR,fI];    

(* first derivatives of Kahler potential *)

dk=Simplify[Table[D[k,f[[i]]]/.Conjugate'[f[[i]]]->0,{i,1,n}]/.replf];
dkb=Simplify[Table[D[k,Conjugate[f[[i]]]],{i,1,n}]/.replf];        

(* Kahler metric *)

g=Simplify[Table[Table[D[k,f[[i]],Conjugate[f[[j]]]]/.Conjugate'[f[[i]]]->0,{j,1,n}],{i,1,n}]/.replf]; 

(* different forms of the superpotential *)

wb=w/.Table[f[[i]]->Conjugate[f[[i]]],{i,1,n}]/.Complex[X_,Y_]->Complex[X,-Y];       (* cc of superpotential *)
wr=ComplexExpand[w/.replf];                                         (* superpotential in terms of real fields *)
wbr=ComplexExpand[wb/.replf];                                       (* cc superpotential in terms of real fields *)

(* first derivatives of superpotential *)

dw=Table[D[w,f[[i]]],{i,1,n}];                                      (* first derivatives of superpotential, dW *)
dwb=Table[D[wb,Conjugate[f[[i]]]],{i,1,n}];                         (* first derivatives of cc superpotential *)
dwr=ComplexExpand[dw/.replf];                                       (* dW in terms of real fields *)
dwbr=ComplexExpand[dwb/.replf];                                     (* cc of dW in terms of real fields *)

(* F terms *)

F=Simplify[Table[dw[[i]]+Mpi^2 dk[[i]] w,{i,1,n}]];             (* F terms *)
Fb=Simplify[Table[dwb[[i]]+Mpi^2 dkb[[i]] wb,{i,1,n}]];         (* cc F terms *)
Fr=Simplify[Table[dwr[[i]]+Mpi^2 dk[[i]] wr,{i,1,n}]];          (* F terms in terms of real fields *)
Fbr=Simplify[Table[dwbr[[i]]+Mpi^2 dkb[[i]] wbr,{i,1,n}]];      (* cc F terms in terms of real fields *)

(* collect the results so far *)

res={Fields -> f, ReFields -> fR,ImFields -> fI,RFields->fa,Par->par,ComplexToReal -> replf,
     KPot->k,SPot->w,KMetric->g,DSPot->Table[f[[i]]->dw[[i]],{i,1,n}],FTerms->Table[f[[i]]->F[[i]],{i,1,n}],
     DeltaPot->deltav};

If[meth==="off",res,                                   (* finish if calculation of inverse is switched off *)
If[meth==="hand",gi=gih,gi=Simplify[Inverse[g]]];      (* determine inverse metric *)

(* some error checking on inverse metric *)

If[!MatrixQ[gi],Message[Stringvacua::"Kahler metric error"];res,
   matdim=Dimensions[gi];
   If[(matdim[[1]] != n) || (matdim[[2]] != n),Message[Stringvacua::"Kahler metric error"];res,

(* Compute exp (K) *)

ek=Simplify[Exp[Mpi^2 k]/.replf];

(* Calculate scalar potential *)

v=Simplify[Expand[ek (Sum[Sum[gi[[i,j]] Fbr[[i]] Fr[[j]],{j,1,n}],{i,1,n}]-3 Mpi^2 wr wbr)]]
         +deltav;

(* accuracy *)

acc=10^(-Accuracy[{k,w}]+2);
v=v/.(X_Real/;(Abs[X]<acc)->0)/.(X_Complex/;(Abs[X]<acc)->0)/.(X_Complex/;(Abs[Im[X]]<acc)->Re[X])/.(X_Complex/;(Abs[Re[X]]<acc)->Im[X]);

(* first derivatives of potential *)

dv=Table[Simplify[D[v,fa[[i]]]],{i,1,2 n}]; 

(* Hesse matrix *)

h=Table[Table[Simplify[D[v,fa[[i]],fa[[j]]]],{j,1,2 n}],{i,1,2 n}];

(* return final result *)

Join[res,{Pot->v,DPot->Table[fa[[i]]->dv[[i]],{i,1,2 n}],Hesse->h}]]]]]]];

(**********************************************************************************************)
(* module to calculate ideals in a model list *)

CalcModel[model_List,opt___]:=Module[
{res,f,fa,par,n,w,dw,Idw,replf,wr,Rew,Imw,IRew,IImw,ftr,X,Y,ft,Ift,
 Reft,Imft,Redw,Imdw,Idwr,v,Iv,dv,Idv,cvarpar,varpar,opt1,dum,cdum,repldum,fdum,fadum,du,cdu,repldu},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
du=Cases[Flatten[{Dummies/.opt1/.Options[CalcModel]}],(X_->Y_)->(X->Y)];
cdu=Cases[Flatten[{CDummies/.opt1/.Options[CalcModel]}],(X_->Y_)->(X->Y)];
repldum=Cases[Flatten[{RDummies/.opt1/.Options[CalcModel]}],(X_->Y_)->(X->Y)];
dumvar=Cases[du,(X_->Y_)->Y];
cdumvar=Cases[cdu,(X_->Y_)->Y];
dum=translist[Join[du,repldum]];
cdum=translist[Join[cdu,repldum]];

res=model;

(* add entries for dummies *)

res=RemoveEntry[res,{Dummies,CDummies}];
res=Join[res,{Dummies->du,CDummies->cdu,RDummies->repldum}];

(* check if basic entries in model list are present *)

If[!MemberQ[model,Fields->_List] || !MemberQ[model,Par->_List] || !MemberQ[model,RFields->_List] ||
   !MemberQ[model,ComplexToReal->_List],Message[Stringvacua::"no model"],

(* extract basic entries *)

f=Fields/.model;
fdum=Join[f,cdumvar];
n=Length[f];
fa=RFields/.model;
fadum=Join[fa,dumvar];
par=Par/.model;
replf=ComplexToReal/.model;
cvarpar=Join[fdum,par];
varpar=Join[fadum,par];

(* superpotential ideals *)

If[!MemberQ[model,SPot->_],Message[Stringvacua::"model incomplete","Spot","ISPot"],

w=SPot/.model;
Iw=ToPol[w,cdum];
wr=SPot/.model/.replf;
Rew=ComplexExpand[Re[wr]];
Imw=ComplexExpand[Im[wr]];
IRew=ToPol[Rew,dum];
IImw=ToPol[Imw,dum];

If[!IsPolynomial[Iw,cvarpar],Message[Stringvacua::"no ideal","ISPot"]];
If[!IsPolynomial[IRew,varpar],Message[Stringvacua::"no ideal","IReSPot"]];
If[!IsPolynomial[IImw,varpar],Message[Stringvacua::"no ideal","IImSPot"]];

res=RemoveEntry[res,{ISPot,IReSPot,IImSPot}];
res=Join[res,{ISPot->{Var->fdum,Par->par,Gen->Iw},IReSPot->{Var->fadum,Par->par,Gen->IRew},
         IImSPot->{Var->fadum,Par->par,Gen->IImw}}]];

(* derivatives of superpotential ideals *)

If[!MemberQ[model,DSPot->_],Message[Stringvacua::"model incomplete","DSpot","IDSPot"],

dw=Cases[DSPot/.model,(X_->Y_)->Y];
Idw=ToPol[dw,cdum];
Redw=ComplexExpand[Re[dw/.replf]];
Imdw=ComplexExpand[Im[dw/.replf]];
Idwr=Join[ToPol[Redw,dum],ToPol[Imdw,dum]];  

If[!IsPolynomial[Idw,cvarpar],Message[Stringvacua::"no ideal","IDSPot"]];
If[!IsPolynomial[Idwr,varpar],Message[Stringvacua::"no ideal","IReImDSPot"]];

res=RemoveEntry[res,{IDSPot,IReImDSPot}];   
res=Join[res,{IDSPot->{Var->fdum,Par->par,Gen->Idw},
              IReImDSPot->{Var->fadum,Par->par,Gen->Table[fa[[i]]->Idwr[[i]],{i,1,2 n}]}}]];

(* ideal for F terms *)

If[!MemberQ[model,FTerms->_List],Message[Stringvacua::"model incomplete","FTerms","IFTerms"],

ftr=Cases[FTerms/.model,(X_->Y_)->Y]/.replf;
If[Length[ftr] != n,Message[Stringvacua::"model incomplete","FTerms","IFTerms"],

Reft=ComplexExpand[Re[ftr]];
Imft=ComplexExpand[Im[ftr]];
Ift=Join[ToPol[Reft,dum],ToPol[Imft,dum]];

If[!IsPolynomial[Ift,varpar],Message[Stringvacua::"no ideal","IFTerms"]];

res=RemoveEntry[res,IFTerms];
res=Append[res,IFTerms->{Var->fadum,Par->par,Gen->Table[fa[[i]]->Ift[[i]],{i,1,2 n}]}]]];

(* ideal for potential *)

If[!MemberQ[model,Pot->_],Message[Stringvacua::"model incomplete","Pot","IPot"],

v=Pot/.model;
Iv=ToPol[v,dum];

If[!IsPolynomial[Iv,varpar],Message[Stringvacua::"no ideal","IPot"]];

res=RemoveEntry[res,IPot];
res=Append[res,IPot->{Var->fadum,Par->par,Gen->Iv}]];

(* ideal for derivatives of potential *)

If[!MemberQ[model,DPot->_],Message[Stringvacua::"model incomplete","DPot","IDPot"],

dv=Cases[DPot/.model,(X_->Y_)->Y];
If[Length[dv] != (2 n),Message[Stringvacua::"model incomplete","DPot","IDPot"],

Idv=ToPol[dv,dum];

If[!IsPolynomial[Idv,varpar],Message[Stringvacua::"no ideal","IDPot"]];

res=RemoveEntry[res,IDPot];
res=Append[res,IDPot->{Var->fadum,Par->par,Gen->Table[fa[[i]]->Idv[[i]],{i,1,2 n}]}]]];
 
res]];

(**********************************************************************************************)
(* Elimination ordering, using script 'SingularElimination' *)

Elimination[ideallist_List,opt___]:=
Module[{var,varcoef,inide,elim,elimprod,singout,meth,elimvar,coef0,opt1,par},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
meth=Meth/.opt1/.Options[Elimination];
elimvar=Flatten[{VarPar/.opt1/.Options[Elimination]}];
CheckOptions0[opt1];

(* Prepare input for Singular *)

var=ExtractVar[ideallist];
par=ExtractPar[ideallist];
varcoef=Union[var,par];
If[Length[elimvar]==0,elim=var,elim=Intersection[Flatten[elimvar],varcoef];
  If[Sort[elim] =!= Sort[elimvar],Message[Stringvacua::"not varpar"]]];
inide=ExtractPol[ideallist];
elimprod=Product[elim[[i]],{i,1,Length[elim]}];

(* Test if inputideal consists of polynomials and if so call Singular *)

If[!IsPolynomial[inide,varcoef] || Length[var]==0,Message[Stringvacua::"illegal ideal","Elimination"],
  If[!IsVar[varcoef],Message[Stringvacua::"illegal variable","Elimination"],
    If[elim==={},Message[Stringvacua::"no elim"];{Var->var,Par->par,Gen->inide},
      If[!MemberQ[SingularScripts,"SingularElimination"->_],Message[Stringvacua::"missing script",
                                                                    "SingularElimination","Elimination"],

If[!MemberQ[{"Basic","HD","Prod"},meth],Message[Stringvacua::"illegal value","Meth","Basic"];meth="Basic"];

If[FreeQ[inide,Complex[_,_]],coef0=FieldChar1,coef0={FieldChar1,I}];

      singout=CallSingular["SingularElimination"/.SingularScripts,varcoef,{"@@listofvariables"->varcoef,
                           "@@inputideal"->Numerator[inide],"@@coefffield"->coef0,
                           "@@eliminationproduct"->elimprod,"@@method"->meth},opt1];

If[SingularStat === "ok",
  {Var->Complement[varcoef,elim],Par->{},Gen->Flatten[singout]},
{SingularStat}]]]]]];

(**********************************************************************************************)
(* Primary decomposition, using script 'SingularPrimDec' *)

PrimDec[ideallist_List,opt___]:=
Module[{idelist,var,coef,coef0,varcoef,inide,singout,meth,repl,singrepl,opt1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
meth=Meth/.opt1/.Options[PrimDec];
repl=VarPar/.opt1/.Options[PrimDec];
CheckOptions0[opt1];

(* Prepare input for Singular *)

idelist=ideallist/.repl;
var=ExtractVar[idelist];
coef=ExtractPar[idelist];
varcoef=Join[var,coef];
inide=ExtractPol[idelist];

(* Test for errors and call Singular *)

If[!IsPolynomial[inide,varcoef] || Length[var]==0,
Message[Stringvacua::"illegal ideal","PrimDec"],
If[!IsVar[varcoef] || Length[Intersection[var,coef]]>0,
Message[Stringvacua::"illegal variable","PrimDec"],
If[!MemberQ[SingularScripts,"SingularPrimDec"->_],
Message[Stringvacua::"missing script","SingularPrimDec","PrimDec"],

If[FreeQ[inide,Complex[_,_]],coef0=Prepend[coef,FieldChar1],coef0=Join[{FieldChar1,I},coef]];
 
  singrepl=CommonOptions[opt1,Options[PrimDec],var,repl,coef0];
  singrepl=Join[singrepl,{"@@listofvariables"->var,"@@inputideal"->Numerator[inide],"@@coefffield"->coef0}];
  
  singout=CallSingular["SingularPrimDec"/.SingularScripts,varcoef,singrepl,opt1]; 

(* output result *)          

If[SingularStat === "ok",

ExtractIdeals[singout,var,coef],

Append[ExtractIdeals[singout,var,coef],SingularStat]]]]]];

(**********************************************************************************************)
(* Form an element of a saturation expansion for an ideal, using script 'SingularSaturation' *)

Saturation[ideallist_List,opt___]:=Module[
{idelist,var,coef,coef0,varcoef,inide,poly,singout,polylist,repl,singrepl,opt1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
polylist=Poly/.opt1/.Options[Saturation];
repl=VarPar/.opt1/.Options[Saturation];
CheckOptions0[opt1];

(* Prepare input for Singular *)

idelist=ideallist/.repl;
var=ExtractVar[idelist];
coef=ExtractPar[idelist];
varcoef=Join[var,coef];
inide=ExtractPol[idelist];
poly=ExtractPol[{polylist/.repl}];

(* Test for errors and call Singular *)

If[!IsPolynomial[inide,varcoef] || Length[var]==0,
Message[Stringvacua::"illegal ideal","Saturation"],
If[!IsPolynomial[poly,varcoef],
Message[Stringvacua::"not polynomial","Saturation"],
If[!IsVar[varcoef] || Length[Intersection[var,coef]]>0,
Message[Stringvacua::"illegal variable","Saturation"],
If[!MemberQ[SingularScripts,"SingularSaturation"->_],
Message[Stringvacua::"missing script","SingularSaturation","Saturation"],

If[FreeQ[inide,Complex[_,_]]&&FreeQ[poly,Complex[_,_]],coef0=Prepend[coef,FieldChar1],coef0=Join[{FieldChar1,I},coef]];

singrepl=CommonOptions[opt1,Options[Saturation],var,repl,coef0];
singrepl=Join[singrepl,{"@@listofvariables"->var,"@@inputideal"->Numerator[inide],
                        "@@coefffield"->coef0,"@@polylist"->Numerator[poly]}];
singout=CallSingular["SingularSaturation"/.SingularScripts,varcoef,singrepl,opt1];

If[SingularStat === "ok",

Flatten[ExtractIdeals[singout,var,coef]],

{SingularStat}]]]]]];

(**********************************************************************************************)
(* Determine the dimension of an ideal, using script SingularDim *)

DimIdeal[ideallist_List,opt___]:=Module[
{idelist,var,coef,coef0,varcoef,inide,singout,repl,opt1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
repl=VarPar/.opt1/.Options[DimIdeal];
CheckOptions0[opt1];

(* Prepare input for Singular *)

idelist=ideallist/.repl;
var=ExtractVar[idelist];
coef=ExtractPar[idelist];
varcoef=Join[var,coef];
inide=ExtractPol[idelist];

(* Test for errors and call Singular *)

If[!IsPolynomial[inide,varcoef] || Length[var]==0,
Message[Stringvacua::"illegal ideal","DimIdeal"],
If[!IsVar[varcoef] || Length[Intersection[var,coef]]>0,
Message[Stringvacua::"illegal variable","DimIdeal"],
If[!MemberQ[SingularScripts,"SingularDim"->_],
Message[Stringvacua::"missing script","SingularDimIdeal","DimIDeal"],

If[FreeQ[inide,Complex[_,_]],coef0=Prepend[coef,FieldChar1],coef0=Join[{FieldChar1,I},coef]];

singout=CallSingular["SingularDim"/.SingularScripts,varcoef,{"@@listofvariables"->var,
                     "@@inputideal"->Numerator[inide],"@@coefffield"->coef0},opt1];

If[SingularStat === "ok",
{Var->var,Par->coef,Gen->inide,Dim->singout[[1]]},{SingularStat}]]]]];

(**********************************************************************************************)
(* Form an element of a saturation expansion for an ideal and primary decompose *)
(* using script 'SingularSaturationPrimDec' *)

SaturationPrimDec[ideallist_List,opt___]:=Module[
{idelist,var,coef,coef0,varcoef,inide,poly,singout,polylist,repl,meth,singrepl,opt1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
polylist=Poly/.opt1/.Options[SaturationPrimDec];
repl=VarPar/.opt1/.Options[SaturationPrimDec];
meth=Meth/.opt1/.Options[SaturationPrimDec];
CheckOptions0[opt1];

(* Prepare input for Singular *)

idelist=ideallist/.repl;
var=ExtractVar[idelist];
coef=ExtractPar[idelist];
varcoef=Join[var,coef];
inide=ExtractPol[idelist];
poly=ExtractPol[{polylist/.repl}];

(* Test for errors and call Singular *)

If[!IsPolynomial[inide,varcoef] || Length[var]==0,
Message[Stringvacua::"illegal ideal","SaturationPrimDec"],
If[!IsPolynomial[poly,varcoef],
Message[Stringvacua::"not polynomial","SaturationPrimDec"],
If[!IsVar[varcoef] || Length[Intersection[var,coef]]>0,
Message[Stringvacua::"illegal variable","SaturationPrimDec"],
If[!MemberQ[SingularScripts,"SingularSaturationPrimDec"->_],
Message[Stringvacua::"missing script","SingularSaturationPrimDec","SaturationPrimDec"],

If[FreeQ[inide,Complex[_,_]]&&FreeQ[poly,Complex[_,_]],coef0=Prepend[coef,FieldChar1],coef0=Join[{FieldChar1,I},coef]];

singrepl=CommonOptions[opt1,Options[SaturationPrimDec],var,repl,coef0];
singrepl=Join[singrepl,{"@@listofvariables"->var,"@@inputideal"->Numerator[inide],
                        "@@coefffield"->coef0,"@@polylist"->Numerator[poly]}];
singout=CallSingular["SingularSaturationPrimDec"/.SingularScripts,varcoef,singrepl,opt1];

If[SingularStat === "ok",

ExtractIdeals[singout,var,coef],

Append[ExtractIdeals[singout,var,coef],SingularStat]]]]]]];

(**********************************************************************************************)
(* Find the number of real roots of a zero-dimensional ideal, using script 'SingularNumRoots' *)

NumRoots[ideallist_List,opt___]:=Module[
{idelist,var,coef,inide,singout,repl,meth,opt1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
repl=VarPar/.opt1/.Options[NumRoots];
meth=Meth/.opt1/.Options[NumRoots];
CheckOptions0[opt1];

(* Prepare input for Singular *)

idelist=ideallist/.repl;
var=ExtractVar[idelist];
coef=ExtractPar[idelist];
inide=ExtractPol[idelist];

If[Length[coef]>0,Message[Stringvacua::"no extensions","NumRoots"],
If[!IsVar[var],Message[Stringvacua::"illegal variable","NumRoots"],
If[!IsPolynomial[inide,var] || Length[var]==0,Message[Stringvacua::"illegal ideal","NumRoots"],
If[!MemberQ[SingularScripts,"SingularNumRoots"->_],
Message[Stringvacua::"missing script","SingularNumRoots","NumRoots"],
If[!FreeQ[inide,Complex[_,_]],Message[Stringvacua::"no I","NumRoots"],

singout=CallSingular["SingularNumRoots"/.SingularScripts,var,{"@@listofvariables"->var,
                     "@@inputideal"->Numerator[inide],"@@coefffield"->0,"@@method"->meth},opt1];

If[SingularStat === "ok",
{Var->var,Par->{},Gen->inide,Dim->{0},NRoots->singout[[1]]},
{SingularStat}]]]]]]];

(**********************************************************************************************)
(* Saturation expansion and primary decomposition for a model *)
(* using module 'SaturationPrimDec' based on script 'SingularSaturationPrimDec' *)

SaturationExpand[model_List,opt___]:=Module[
{X,fnum,fterms0,n,ftermssat,satlist,res,inide,nsat,ftermszero,i,polylist,fterms1,
resid,nterms,nt,repl,polyl,flist,Dim1,RRoots1,NRoots1,SturmPoly1,SturmSigns1,Meth1,nmin,nmax,fsat,
TestRational1,TestMatDet1,TestMatEigen1,TestStability1,TestLambda1,VarPar1,Thom1,TestAtAll,opt1,opt2,
ftermslist,SturmLabel1,SturmLabel2,NoExtraTest},

(* options for PrimDec and SaturationPrimDec *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
polyl=Poly/.opt1/.Options[SaturationExpand];
flist=FTerms/.opt1/.Options[SaturationExpand];
Dim1=Dim/.opt1/.Options[SaturationExpand];
RRoots1=RRoots/.opt1/.Options[SaturationExpand];
NRoots1=NRoots/.opt1/.Options[SaturationExpand];
SturmPoly1=SturmPoly/.opt1/.Options[SaturationExpand];
SturmSigns1=SturmSigns/.opt1/.Options[SaturationExpand];
SturmLabel1=SturmLabel/.opt1/.Options[SaturationExpand];
Meth1=Meth/.opt1/.Options[SaturationExpand];
CheckOptions0[opt1];
opt2=Delete[Flatten[{opt1}],Position[Flatten[{opt1}],Poly->_]];
opt2=Join[opt2,{Dim->Dim1,RRoots->RRoots1,NRoots->NRoots1,SturmPoly->SturmPoly1,
                SturmSigns->SturmSigns1,Meth->Meth1}];

(* options for SturmQuery *)

VarPar1=VarPar/.opt1/.Options[SaturationExpand];
Thom1=Thom/.opt1/.Options[SaturationExpand];
TestRational1=TestRational/.opt1/.Options[SaturationExpand];
TestMatDet1=TestMatDet/.opt1/.Options[SaturationExpand];
TestMatEigen1=TestMatEigen/.opt1/.Options[SaturationExpand];
TestStability1=TestStability/.opt1/.Options[SaturationExpand];
TestLambda1=TestLambda/.opt1/.Options[SaturationExpand];
NoExtraTest= (TestRational1 === {1}) && (TestMatDet1 === {{1}});
If[TestStability1 === "Hesse",
If[TestMatEigen1 === {{1}},TestMatEigen1=Hesse/.model,
   Message[Stringvacua::"extra matrix"];TestMatEigen1=Hesse/.model]];

If[TestLambda1 === "yes",
   If[TestRational1 === {1},TestRational1=Pot/.model,TestRational1=Flatten[Prepend[{TestRational1},Pot/.model]]]];

TestAtAll= (TestRational1 =!= {1}) || (TestMatDet1 =!= {{1}}) || (TestMatEigen1 =!= {{1}});

(* deal with SturmLabel option *)

If[SturmLabel1==="Automatic",
  If[NoExtraTest,SturmLabel2={}; If[TestLambda1 === "yes",SturmLabel2=Append[SturmLabel2,"Pot"]];
                                 If[TestStability1 === "Hesse",SturmLabel2=Append[SturmLabel2,"Hessian"]],
     SturmLabel2="None"],SturmLabel2=SturmLabel1];

(* check if model list has required entries and other error checking *)

If[!IsSubset[{RFields->_,Par->_,IFTerms->_,IDPot->_},model],Message[Stringvacua::"model incomplete"],

(* prepare input *)

fa=RFields/.model;
para=Par/.model;
fnum=Cases[Flatten[{flist}],X_Integer->X];
ftermszero=Intersection[Complement[Flatten[{flist}],fnum],fa];
ftermssat=Complement[fa,ftermszero];
nsat=Length[ftermssat];
If[Length[fnum]==0,nmin=0;nmax=nsat,
  If[Length[fnum]==1,nmin=fnum[[1]];nmax=nmin,
                     nmin=Min[fnum];nmax=Max[fnum]]];
If[!IsInteger[{nmin},0,nsat] || !IsInteger[{nmax},0,nsat],Message[Stringvacua::"wrong fterms"],
satlist=Sort[Subsets2[ftermssat,{nmin,nmax}]];
res={};
nterms=Length[satlist];
ftermslist={};

(* init protocol settings *)

InitProt["Calculate "<>ToString[nterms]<>" saturation terms : "];

(* Loop over the terms in the saturation expansion *)

CheckAbort[
Do[
   fterms1=satlist[[i]];
   fterms0=Join[ftermszero,Complement[ftermssat,fterms1]];
   inide={IDPot/.model,fterms0/.Gen/.IFTerms/.model,polyl};
   polylist=fterms1/.Gen/.IFTerms/.model;
   If[Length[polylist]==0,resid=PrimDec[{IFTerms/.model,polyl},opt2],
         resid=SaturationPrimDec[inide,Poly->polylist,opt2]];
   If[Length[Flatten[{resid}]]>0,
      If[TestAtAll,resid1={};
         Do[resid1=Append[resid1,SturmQuery[resid[[k]],TestRational->TestRational1,VarPar->VarPar1,Thom->Thom1,
            TestMatDet->TestMatDet1,TestMatEigen->TestMatEigen1,RRoots->RRoots1,SturmLabel->SturmLabel2]],{k,1,Length[resid]}],
         resid1=resid];
   res=Append[res,fterms1->resid1]];
   WriteProt[fterms1,SingularStat];
   ftermslist=Append[ftermslist,fterms1],
{i,1,nterms}];

(* in case of normal finish *)

CleanProt["Finished!"];
res,

(* in case of abort *)

CleanProt["Aborted!"];
Append[res,fterms1->{$Aborted}]]]]];

(**********************************************************************************************)
(* Sturm query on real roots of ideal w.r.t. a set of polyomials, with optional Thom encoding *)

SturmQuery[ideallist_List,opt___]:=Module[{ratlist,detlist,eigenlist,ratflag,detflag,eigenflag,detdim,eigendim,detdim2,eigendim2,ratl,detl,eigenl,
repl,idelist,var,inide,poly,singrepl,rat,len,nroots,opt1,res,pos,errlist,SturmLabel1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
repl=VarPar/.opt1/.Options[SturmQuery];
ratlist=TestRational/.opt1/.Options[SturmQuery]/.repl;
If[ratlist =!= {1},ratflag="yes",ratflag="no"];
detlist=TestMatDet/.opt1/.Options[SturmQuery]/.repl;
If[MatrixQ[detlist],detlist={detlist}];
If[detlist =!= {{{1}}},detflag="yes",detflag="no"];
eigenlist=TestMatEigen/.opt1/.Options[SturmQuery]/.repl;
If[eigenlist =!= {{1}},eigenflag="yes",eigenflag="no"];
SturmLabel1=SturmLabel/.opt1/.Options[SturmQuery];
CheckOptions0[opt1];

(* test if detlist and eigenlist are lists of quadratic matrices and determine their size *)

If[!Apply[And,Map[MatrixQ,detlist]],Message[Stringvacua::"illegal option","TestMatDet"],
If[!MatrixQ[eigenlist],Message[Stringvacua::"illegal option","TestMatEigen"],
detdim=Dimensions[#][[1]]& /@ detlist;
detdim2=Dimensions[#][[2]]& /@ detlist;
If[detdim != detdim2,Message[Stringvacua::"illegal option","TestMatDet"],
eigendim=Dimensions[eigenlist][[1]];
eigendim2=Dimensions[eigenlist][[2]];
If[eigendim != eigendim2,Message[Stringvacua::"illegal option","TestMatEigen"],

(* Prepare input for Singular *)

idelist=ideallist/.repl;
var=ExtractVar[idelist];
inide=ExtractPol[idelist];
ratl=ExtractPol[Flatten[Join[Numerator[Together[{ratlist}]],Denominator[Together[{ratlist}]]]]];
detl=ExtractPol[Flatten[{Numerator[#],Denominator[#]}& /@ Together[detlist]]];
eigenl=ExtractPol[Join[Flatten[Numerator[eigenlist]], Flatten[Denominator[eigenlist]]]];

(* Test for errors and call Singular *)

errlist=Append[idelist,Sturm->{"error: not tested"}];

If[(ratflag === "no") && (detflag === "no") && (eigenflag === "no"),
   Message[Stringvacua::"test empty"];errlist,
If[!IsPolynomial[inide,var] || Length[var]==0,
   Message[Stringvacua::"illegal ideal","SturmQuery"];errlist,
If[!IsPolynomial[ratl,var],
   Message[Stringvacua::"not rational","TestRational","SturmQuery"];errlist,
If[!IsPolynomial[detl,var],
   Message[Stringvacua::"not rational","TestMatDet","SturmQuery"];errlist,
If[!IsPolynomial[eigenl,var],
   Message[Stringvacua::"not rational","TestMatEigen","SturmQuery"];errlist,
If[!IsVar[var],
   Message[Stringvacua::"illegal variable","SturmQuery"];errlist,
If[!MemberQ[SingularScripts,"SingularSturmQuery"->_],
   Message[Stringvacua::"missing script","SingularSturmQuery","SturmQuery"];errlist,
   If[!FreeQ[inide,Complex[_,_]],Message[Stringvacua::"no I","SturmQuery"];errlist,

singrepl=CommonOptions[opt1,Options[SturmQuery],var,repl];
singrepl=Join[singrepl,{"@@listofvariables"->var,"@@inputideal"->Numerator[inide],
                        "@@polylist"->Numerator[ratl],"@@rationals"->ratflag,
                        "@@detlist"->Numerator[detl],"@@dets"->detflag,"@@dimdetlist"->detdim,
                        "@@eigenlist"->Numerator[eigenl],"@@eigens"->eigenflag,"@@dimeigenlist"->eigendim}];

singout=CallSingular["SingularSturmQuery"/.SingularScripts,var,singrepl,opt1];

If[SingularStat === "ok",

nroots=singout[[1,1]];
singout=Drop[singout,1];
res={};

If[Length[singout]==0,res={},
  If[eigenflag==="no",res=singout,
    pos=Prepend[Position[singout,{Null}],{0}];
    res=Table[Take[singout,{pos[[i,1]]+1,pos[[i+1,1]]-1}],{i,1,Length[pos]-1}];
    If[SturmLabel1 === "None",res=Prepend[res,eigenlist]]];
  If[detflag==="yes" && SturmLabel1 === "None",res=Prepend[res,detlist]];
  If[ratflag==="yes" && SturmLabel1 === "None",res=Prepend[res,ratlist]]];
  If[SturmLabel1 =!= "None",res=Prepend[res,SturmLabel1]];

{Var->var,Par->{},Gen->inide,Dim->{0},NRoots->{nroots},Sturm->res},

   {Var->var,Par->{},Gen->inide,Sturm->SingularStat}]]]]]]]]]]]]]];

(**********************************************************************************************)
(* facstd module *)

FacGroebner[ideallist_List,opt___]:=Module[{idelist,var,coef,coef0,varcoef,inide,singout,repl,singrepl,opt1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
repl=VarPar/.opt1/.Options[FacGroebner];
CheckOptions0[opt1];

(* Prepare input for Singular *)

idelist=ideallist/.repl;
var=ExtractVar[idelist];
coef=ExtractPar[idelist];
varcoef=Join[var,coef];
inide=ExtractPol[idelist];

(* Test for errors and call Singular *)

If[!IsPolynomial[inide,varcoef] || Length[var]==0,
Message[Stringvacua::"illegal ideal","FacGroebner"],
If[!IsVar[varcoef] || Length[Intersection[var,coef]]>0,
Message[Stringvacua::"illegal variable","FacGroebner"],
If[!MemberQ[SingularScripts,"SingularFacStd"->_],
Message[Stringvacua::"missing script","SingularFacStd","FacGroebner"],

If[FreeQ[inide,Complex[_,_]],coef0=Prepend[coef,FieldChar1],coef0=Join[{FieldChar1,I},coef]];
 
  singrepl=CommonOptions[opt1,Options[FacGroebner],var,repl,coef0];
  singrepl=Join[singrepl,{"@@listofvariables"->var,"@@inputideal"->Numerator[inide],"@@coefffield"->coef0}];
  
  singout=CallSingular["SingularFacStd"/.SingularScripts,varcoef,singrepl,opt1]; 

(* output result *)          

If[SingularStat === "ok",

ExtractIdeals[singout,var,coef],

Append[ExtractIdeals[singout,var,coef],SingularStat]]]]]];

(**********************************************************************************************)
(* Thom encoding module *)

ThomEncode[ideallist_List,opt___]:=Module[{idelist,var,coef,coef0,varcoef,inide,singout,repl,singrepl,opt1},

(* options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
repl=VarPar/.opt1/.Options[ThomEncode];
CheckOptions0[opt1];

(* Prepare input for Singular *)

idelist=ideallist/.repl;
var=ExtractVar[idelist];
coef=ExtractPar[idelist];
varcoef=Join[var,coef];
inide=ExtractPol[idelist];

(* Test for errors and call Singular *)

If[!IsPolynomial[inide,varcoef] || Length[var]==0,
Message[Stringvacua::"illegal ideal","ThomEncode"],
If[!IsVar[varcoef] || Length[Intersection[var,coef]]>0,
Message[Stringvacua::"illegal variable","ThomEncode"],
If[!MemberQ[SingularScripts,"SingularFacStd"->_],
Message[Stringvacua::"missing script","SingularThomEncode","ThomEncode"],

If[FreeQ[inide,Complex[_,_]],coef0=Prepend[coef,FieldChar1],coef0=Join[{FieldChar1,I},coef]];
 
   singrepl={"@@listofvariables"->var,"@@inputideal"->Numerator[inide],"@@coefffield"->coef0};

  singout=CallSingular["SingularThomEncode"/.SingularScripts,varcoef,singrepl,opt1]; 

(* output result *)          

If[SingularStat === "ok",

singout=DeleteCases[singout,{Null}];
Table[{Var->var,Par->coef,Gen->singout[[2 i-1]],Thom->singout[[2 i]]},{i,1,Length[singout]/2}],
{SingularStat}]]]]];

(**********************************************************************************************)
(* A simple interactive minimisation module *)

SimpleMinimise[]:=Module[{f,fR,fI,k,w,maxtime,model,res,par,cellcont},

(* interactive input *)

f=Input["List of complex fields? (example: {T}) "]; If[f===Null,f={Global`T},f=Flatten[{f}]];
If[f==={$Canceled},Message[Stringvacua::"canceled"],
fR=Input["List of real parts of fields? (example: {t}) "];If[fR===Null,fR={Global`t},fR=Flatten[{fR}]];
If[fR==={$Canceled},Message[Stringvacua::"canceled"],
fI=Input["List of imaginary parts of fields? (example: {tau}) "];If[fI===Null,fI={Global`\[Tau]},fI=Flatten[{fI}]];
If[fI==={$Canceled},Message[Stringvacua::"canceled"],
k=Input["Kahler potential? (example: -3 Log[T+Conjugate[T]]) "];If[k===Null,k=-3 Log[Global`T+Conjugate[Global`T]]];
If[k===$Canceled,Message[Stringvacua::"canceled"],
w=Input["Superpotential? (example: 1+Power[T,4]) "];If[w===Null,w=1+Global`T^4];
If[w===$Canceled,Message[Stringvacua::"canceled"],
par=Input["Parameters? (example: {} for no parameters) "];If[par===Null,par={},par=Flatten[{par}]];
If[par==={$Canceled},Message[Stringvacua::"canceled"],
maxtime=10;
If[StringPosition[$System,"Windows"]==={},maxtime=Input["How many seconds at most for a Singular calculation? (example: 10) "];If[maxtime === Null,maxtime=10],maxtime=10];
If[maxtime===$Canceled,Message[Stringvacua::"canceled"],

(* calculation *)

model=CreateModel[f,fR,fI,k,w,Par->par];
model=CalcModel[model];
res=SaturationExpand[model,PackageProtocol->0, SingularMaxTime->maxtime];

(* formatted output *)

If[$Notebooks,

(* notebook output *)

StylePrint[GridBox[{{TCell["complex fields"],FCell[f]},
                   {TCell["real fields"],FCell[fR]},
                   {TCell["imaginary fields"],FCell[fI]},
                   {TCell["Kahler potential"],FCell[k]},
                   {TCell["superpotential"],FCell[w]},
                   {TCell["scalar potential"],FCell[Pot/.model]}},
                   ColumnAlignments -> Left,RowSpacings->0,ColumnLines->True,RowLines->True]
                  // DisplayForm,CellDingbat->"Model"];

resgrid={{TCell["non-zero F terms"],TCell["0-dim. ideals"]}};
Do[Do[If[MemberQ[res[[i,2,j]],Gen->_],cellcont=Gen/.res[[i,2,j]],cellcont=res[[i,2,j]]];
resgrid=Append[resgrid,{FCell[res[[i,1]]],FCell[cellcont]}],
{j,1,Length[res[[i,2]]]}],{i,1,Length[res]}];

StylePrint[
GridBox[resgrid,ColumnAlignments -> Left,RowSpacings->0,ColumnLines->True,RowLines->True]//DisplayForm,
CellDingbat->"Result"];

(* StylePrint["Now check out modules '?CreateModel', '?CalcModel' and '?SaturationExpand' to perform such a calculation \"by hand\". Set the option 'Dim' (type '?Dim') if you want more than 0-dim. ideals. See '?ModuleOptions' for details on other options.","Text",FontSize->12,FontColor->RGBColor[0.5,0,0]] *)

StylePrint["Now check out modules \*ButtonBox[StyleBox[\"CreateModel\",FontColor\[Rule]RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[{FrontEnd`HelpBrowserLookup[\"AddOns\
\",\"CreateModel\"]}]&)], \*ButtonBox[StyleBox[\"CalcModel\",FontColor->RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[{FrontEnd`HelpBrowserLookup[\"AddOns\",\"CalcModel\"]}]&)] and \*ButtonBox[StyleBox[\"SaturationExpand\",FontColor->RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->
None,ButtonFunction:>(FrontEndExecute[{FrontEnd`HelpBrowserLookup[\"AddOns\",\"SaturationExpand\"]}]&)] to perform such a calculation \"by hand\". Set the option Dim if you want more than 0-dimensional ideals. See \*ButtonBox[StyleBox[\"here\",FontColor\[Rule]RGBColor[0,0,0.7],FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[{FrontEnd`HelpBrowserLookup[\"AddOns\",\"Module Options\"]}]&)] for details on module options.", 
TextAlignment -> Center, FontSize -> 12,FontColor -> RGBColor[0.6, 0.3, 0], FontFamily -> "Times"],

(* text interface output *)
Print["\nModel"];
Print["complex fields   : ",f];
Print["real fields      : ",fR];
Print["imaginary fields : ",fI];
Print["Kahler potential : ",k];
Print["superpotential   : ",w];
Print["scalar potential : ",Pot/.model];
Print["\nResult in the form {zero F terms} -> {0-dim. ideal}"];
Do[Do[Print[res[[i,1]],"  ->  ",Gen/.res[[i,2,j]]],{j,1,Length[res[[i,2]]]}],{i,1,Length[res]}]]

]]]]]]]];

(**********************************************************************************************)
(* Keyword translation table *)

KeyTrans={Gen->"Generators",Var->"Variables",Par->"Parameters",Fields->"\[Phi]",ReFields->"Re(\[Phi])",
          ImFields->"Im(\[Phi])",RFields->"Im(\[Phi]),Re(\[Phi])",ComplexToReal->"\[Phi]->Re(\[Phi])+iIm(\[Phi])",
          KPot->"K",SPot->"W",
          ISPot->"I(W)",KMetric->"g",DSPot->"\[PartialD]W",IDSPot->"I(\[PartialD]W)",FTerms->"F",
          DeltaPot->"\[Delta]V",Pot->"V",DPot->"\[PartialD]V",Hesse->"Hesse",IReSPot->"I(Re(W))",IImSPot->"I(Im(W))",
          IReImDSPot->"I(Re(\[PartialD]W),Im(\[PartialD]W))",IFTerms->"I(F)",IPot->"I(V)",IDPot->"I(\[PartialD]V)",
          Dim->"compl.dim.",NRoots->"# real roots",Sturm->"Sturm query",0->" ",Dummy->"real replacements",
          CDummy->"compl. replacements"};

(**********************************************************************************************)
(* display a model, ideal or other list in table form *)

TF[l_List]:=Module[{la,lc,i,j,colk,nc,vkey,vval,ord,colwid,s,leaf},
If[$Notebooks && (Length[l]>0),

(* Make all entries in list assignments *)

la=Cases[l, (X_ -> Y_) -> (X -> Y)];
lc=Complement[l,la];
la=Join[la,Table[i->lc[[i]],{i,1,Length[lc]}]];

(* if an assigned value is a list of a list split *)

lc={};
Do[
   If[VectorQ[la[[i,2]],(Head[#] === List) &] && !MemberQ[KeyTrans,la[[i,1]]->_],
     lc=Join[lc,Table[la[[i,1]]->la[[i,2,j]],{j,1,Length[la[[i,2]]]}]],lc=Append[lc,la[[i]]]],
{i,1,Length[la]}];

(* bring assigned values into standard form *)

la={};
colk={};
Do[
   v=lc[[i,2]];
   If[VectorQ[v] && (Count[v,X_/;MemberQ[KeyTrans,X->_]->Y_]>0),
     va=Sort[Cases[v,(X_ -> Y_) -> (X -> Y)]];
     vc=Complement[v,va];
     vkey=Join[Table[j,{j,1,Length[vc]}],Table[va[[j,1]],{j,1,Length[va]}]];
     vval=Join[vc,Table[va[[j,2]],{j,1,Length[va]}]];
     la=Append[la,lc[[i,1]]->vval];
     colk=Append[colk,vkey],
     la=Append[la,lc[[i,1]]->{v}];
     colk=Append[colk,{""}]],
{i,1,Length[lc]}];

(* order according to type of row *)

ord=Ordering[colk];
colk=Sort[colk];
lc=Table[0,{Length[la]}];
lc=Table[la[[ord[[i]]]],{i,1,Length[la]}];
nc=Max[Map[Length,colk]];

(* construct gridbox *)

resgrid={};
colwid=Table[1,{nc+1}];
Do[row={};

  (* take care of keyword rows *)
 
  If[colk[[i]]=!=colk[[i-1]],
     row=Append[row,TCell[" "]];
     Do[If[MemberQ[KeyTrans,colk[[i,j]]->_],
        s=ToString[colk[[i,j]]]<>"("<>(colk[[i,j]]/.KeyTrans)<>")",
        s=ToString[colk[[i,j]]]];
        If[StringLength[s]>colwid[[j+1]],colwid[[j+1]]=StringLength[s]];
        row=Append[row,TCell[s]],{j,1,Length[colk[[i]]]}];
     Do[row=Append[row,TCell[" "]],{j,Length[colk[[i]]]+1,nc}];
     resgrid=Append[resgrid,row]];
    
  (* take care of data boxes *) 
  
  If[MemberQ[KeyTrans,lc[[i,1]]->_],s=ToString[lc[[i,1]]]<>"("<>(lc[[i,1]]/.KeyTrans)<>")";leaf=StringLenght[s],
                                    s=lc[[i,1]];leaf=LeafCount[s]];
  row={TCell[s]};
  If[leaf>colwid[[1]],colwid[[1]]=leaf];

  Do[row=Append[row,FCell[lc[[i,2,j]]]];
     leaf=LeafCount[lc[[i,2,j]]];
     If[leaf>colwid[[j+1]],colwid[[j+1]]=leaf],{j,1,Length[colk[[i]]]}];
  Do[row=Append[row,FCell[" "]],{j,Length[colk[[i]]]+1,nc}];
  resgrid=Append[resgrid,row],
{i,1,Length[lc]}];

colwid=IntegerPart[colwid/1.4];
colwid=colwid/.X_/;(X>45)->45;

(* styleprint the gridbox *)
 
StylePrint[
GridBox[resgrid,ColumnAlignments -> Left,RowSpacings->0,ColumnLines->True,RowLines->True,
        ColumnWidths->Automatic]//DisplayForm],

l]];

End[];
EndPackage[];










