(**********************************************************************************************)
(* System-dependent modules for Stringvacua package: Unix version *)
(**********************************************************************************************)
Begin["`Private`"]
(**********************************************************************************************)
(* Module to find singular scripts *)

FindSingularScripts[]:=Module[{ffn,dn,fn,inlist,instring,str},
SingularScriptFiles=FileNames["*.mathsing",$UserBaseDirectory<>"/Applications/Stringvacualib"];
SingularScripts={};
SingularScriptHelp="";
Do[ffn=SingularScriptFiles[[i]];
   dn=DirectoryName[ffn];
   fn=StringTake[ffn,{StringLength[dn]+1,StringLength[ffn]-9}];
   If[StringTake[fn,1]=!=".",
     str=OpenRead[ffn];
     inlist=fn<>" : "<>StringReplace[Read[str,String],"//"->""]<>"\n";
     Close[str];
     SingularScriptHelp=SingularScriptHelp<>inlist;
     SingularScripts=Append[SingularScripts,fn->Import[ffn,"Text"]]],
{i,1,Length[SingularScriptFiles]}];
]; 

(**********************************************************************************************)
(* Find paths to singular, xterm, timerscript and wrapperscript *)

FindPaths[]:=Module[{pathlist,timerpath,wrapperpath,fp,n,vers,mess},

(* create standard list of unix paths where xterm and Singular will normally be found *)

pathlist={"/usr/local/sbin","/usr/sbin","/sbin","/usr/local/bin","/bin","/usr/bin","/usr/X11R6/bin","/sw/bin",$HomeDirectory<>"/bin",$HomeDirectory<>"/usr/bin"};

(* merge this list of paths with the one in the StringvacuaPaths.m set-up file *)

If[Length[FileNames["StringvacuaPaths.m",$UserBaseDirectory<>"/Applications/Stringvacualib"]]>0,
   pathlist=Union[pathlist,Flatten[{Get[$UserBaseDirectory<>"/Applications/Stringvacualib/StringvacuaPaths.m"]}]]];

(* find paths to Singular and xterm executables *)

singpathlist=FileNames["Singular*",pathlist];
singpathlist=Select[singpathlist,(FileType[#] === File)&];
xtermpathlist=FileNames["xterm",pathlist];

(* deal with Singular *)

If[Length[singpathlist]==0,Message[Stringvacua::"no executable","Singular"]; singpath="",
   If[Length[singpathlist]==1,singpath=singpathlist[[1]],
     mess="The following versions of Singular were found:";
     Do[mess=StringJoin[mess,"\n",ToString[n],".) ",singpathlist[[n]]],{n,1,Length[singpathlist]}];
     StylePrint[mess,FontSize->10,FontColor->RGBColor[0.5,0,0]];
     vers=Input["Which version of Singular? (1 - "<>ToString[Length[singpathlist]]<>") "];
     If[IntegerQ[vers] && (vers > 0) && (vers <= Length[singpathlist]),
        singpath=singpathlist[[vers]];
        StylePrint["Ok. Chose "<>singpath<>". To change use 'SetOptions[Stringvacua,SingularPath->\"/path/to/Singular\"]'. See '?StringvacuaOptions' for details.",FontSize->10,FontColor->RGBColor[0.5,0,0]],
        singpath=singpathlist[[1]];
        StylePrint["Incorrect number! Chose "<>singpath<>". To change use 'SetOptions[Stringvacua,SingularPath->\"/path/to/Singular\"]'. See '?StringvacuaOptions' for details.",FontSize->10,FontColor->RGBColor[0.5,0,0]]]]];

(* deal with xterm *)

If[Length[xtermpathlist]==0,Message[Stringvacua::"no executable","Xterm"]; XtermFound=False; xtermpath="",
   xtermpath=xtermpathlist[[1]]; XtermFound=True];

(* find paths to timer and wrapper scripts *)

timerpath=FileNames["singulartimer",$UserBaseDirectory<>"/Applications/Stringvacualib"];
If[Length[timerpath]>0,SingularTimerPath=timerpath[[1]],
  Message[Stringvacua::"incomplete install","singulartimer"]];
wrapperpath=FileNames["singularwrapper",$UserBaseDirectory<>"/Applications/Stringvacualib"];
If[Length[wrapperpath]>0,SingularWrapperPath=wrapperpath[[1]],
  Message[Stringvacua::"incomplete install","singularwrapper"]]
];

(**********************************************************************************************)
(* Completes a singular template, calls Singular and returns the Singular output *)

CallSingular[singin_String,variables_List,repllist_List,opt___]:=Module[{subsMS,subsSM,singoutlist,fp,singout,timercommand,singprot,timerexit,singerr,subsB,singinrepl,repll,opt1},

(* Check global options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
CheckOptions1[opt1];

(* Variable substitutions between Mathematica and singular *)

subsMS=Table[ToString[variables[[i]],InputForm]->ReplaceVar[variables[[i]]],{i,1,Length[variables]}];
Stringvacua`SingularVar=subsMS;
subsSM=Sort[Table[subsMS[[i,2]]->subsMS[[i,1]],{i,1,Length[subsMS]}],OrderedQ[{#2, #1}] &];
subsB={"{"->"","}"->""};
subsMS=Join[subsMS,subsB];

(* Complete template *)

singinrepl=StringReplace[singin,{"@@tracevalue"->ToString[SingularTrace1],
           "@@echovalue"->ToString[SingularEcho1],
           "@@stringvacualib"->$UserBaseDirectory<>"/Applications/Stringvacualib",
           "@@writepath"->$HomeDirectory<>"/"}]; 
repll=Table[repllist[[i,1]]->StringReplace[ToString[repllist[[i,2]],InputForm],subsMS],{i,1,Length[repllist]}];
singinrepl=StringReplace[singinrepl,repll];
 
(* Write template into Singular input file .singin *)

fp=OpenWrite[$HomeDirectory<>"/.singin"];
WriteString[fp,singinrepl];
Close[fp];
If[Length[FileNames[".singerr",$HomeDirectory]]>0,Run["rm ~/.singerr"]];

If[SingularTimer1 === "on",

(* Compose timer command and call timer *)

timercommand=SingularTimerPath<>" "<>ToString[SingularMaxTime1]<>" "
             <>ToString[SingularProtocol1]<>" "
             <>ToString[SingularWrapperPath]<>" "<>XtermGeometry1<>" "
             <>ToString[XtermPath1]<>" "<>ToString[SingularPath1];
If[Length[FileNames[".singout",$HomeDirectory]]>0,Run["rm ~/.singout"]];
timerexit=Run[timercommand];

(* Read Singular protocol from file if appropriate *)

If[SingularProtocol1 == "String",
   If[Length[FileNames[".singprot",$HomeDirectory]]>0,
   singprotlist=ReadList[$HomeDirectory<>"/.singprot",String];
   singprot="";
   Do[singprot=singprot<>singprotlist[[i]]<>"\n",{i,1,Length[singprotlist]}];
   Stringvacua`SingularProt=singprot;
   Run["rm ~/.singprot"],
   Stringvacua`SingularProt={},Stringvacua`SingularProt="Error: No protocol!"]],

(* if no timerscripts are being used call Singular directly *)   
 
   If[SingularProtocol1==="Xterm",timerexit=Run[SingularPath1<>" -q --no-warn ~/.singin"],
                                  timerexit=Run[SingularPath1<>" -q --no-warn ~/.singin > /dev/null"]]];

(* Read Singular results *)

If[Length[FileNames[".singout",$HomeDirectory]]>0,
    singoutlist=ReadList[$HomeDirectory<>"/.singout",String];
    singoutlist=Table["{"<>StringReplace[singoutlist[[j]],subsSM]<>"}",{j,1,Length[singoutlist]}];
     (* singoutlist=Select[singoutlist,SyntaxQ[#]&]; *)
    singout=Map[ToExpression[#,InputForm]&,singoutlist];
    Stringvacua`SingularStat="ok",
    Stringvacua`SingularStat="No Singular output";singout={}];

(* handle errors *)

If[timerexit==256,Stringvacua`SingularStat="timed out after "<>ToString[SingularMaxTime1]<>" seconds"];
If[timerexit==512,Stringvacua`SingularStat="interrupted"];
If[timerexit>512,Stringvacua`SingularStat="fatal error"];
If[Length[FileNames[".singerr",$HomeDirectory]]>0,
    singerr=ReadList[$HomeDirectory<>"/.singerr",String];
    Stringvacua`SingularStat="Singular error : "<> singerr];
singout];

(* old output sequence of CallSingular, keep for reference *)

(* If[timerexit<256,
   If[Length[FileNames[".singout",$HomeDirectory]]>0,
    singoutlist=ReadList[$HomeDirectory<>"/.singout",String];
    singout=Table[ToExpression["{"<>StringReplace[singoutlist[[j]],subsSM]<>"}",InputForm],
            {j,1,Length[singoutlist]}];
    SingularStat="ok",
    SingularStat="No Singular output"],
  If[timerexit==256,
    SingularStat="timed out after "<>ToString[SingularMaxTime1]<>" seconds",
    SingularStat="interrupted"]];
  If[Length[FileNames[".singerr",$HomeDirectory]]>0,
    singerr=ReadList[$HomeDirectory<>"/.singerr",String];
    SingularStat="Singular error : "<> singerr];
singout]; *)
End[]

(**********************************************************************************************)
(* Set StringvacuaVariables to default values *)

OptionsToDefault[]:=Module[{},

(* set StringvacuaOptions *)

Options[Stringvacua]={PlanckInverse->1,PackageProtocol->10,FieldChar->0,
                      SingularProtocol->"Xterm",SingularTimer->"on",
                      SingularMaxTime->60,SingularTrace->0,SingularEcho->0,
                      SingularAccuracy->10^(-5),XtermGeometry->"50x10-50+0",
                      XtermPath->Stringvacua`Private`xtermpath,
                      SingularPath->Stringvacua`Private`singpath};

(* check if options are specified in default file *)

If[Length[FileNames["StringvacuaOptions.m",$UserBaseDirectory<>"/Applications/Stringvacualib"]]>0,
   Map[Function[SetOptions[Stringvacua,#]],Flatten[{Get[$UserBaseDirectory<>"/Applications/Stringvacualib/StringvacuaOptions.m"]}]]];

(* Check for Xterm in SingularProtocol options *)

If[! `Private`XtermFound,SetOptions[Stringvacua,SingularProtocol->"None"]];


];

