Begin["`Private`"]
(**********************************************************************************************)

(* Module to find singular scripts *)

FindSingularScripts[]:=Module[{ffn,dn,fn,inlist,instring,str},
SingularScriptFiles=FileNames["*.mathsing",$UserBaseDirectory<>"\\Applications\\Stringvacualib"];
SingularScripts={};
SingularScriptHelp="";
Do[ffn=SingularScriptFiles[[i]];
   dn=DirectoryName[ffn];
   fn=StringTake[ffn,{StringLength[dn]+1,StringLength[ffn]-9}];
   If[StringTake[fn,1]=!=".",
     str=OpenRead[ffn];
     inlist=fn<>" : "<>StringReplace[Read[str,String],"//"->""]<>"\n";
     Close[str];
     SingularScriptHelp=SingularScriptHelp<>inlist;
     SingularScripts=Append[SingularScripts,fn->Import[ffn,"Text"]]],
{i,1,Length[SingularScriptFiles]}];
]; 

(***************************************************************)
(* Find paths to cygwin and singular: windows version *)

Stringvacua::"no cgywin"="Warning: Cygwin not found!";

FindPaths[]:=Module[{},
{

(* A user-writable directory to store .singin and .singout *)
userwrite = ReadList["!cygpath -d /tmp",String];


(* userbase in unix format *)
userwriteunix = ReadList["!cygpath -u \"" <> userwrite <> "\"", String][[1]];

(* Is Cygwin Installed? *)
If[Length[userwriteunix]=={}, Message[Stringvacua::"no cygwin"]];

(* Find Singular Path in Unix Format *)
(* 'which' is more basic to bash than 'where' *)
singexe = ReadList["!which Singular",String][[1]];
If[Length[singexe]=={}, Message[Stringvacua::"no executable","Singular"]];
}
];


(* userwrite and singexe are global variables *)

(*  OLD:
Convert Windows path format to unix/cygwin path format
Win2Unix[p_] := Module[{cmd},
cmd = "!cygpath -u \"" <> p <> "\" > cygpath.tmp";
ReadList[cmd,String];
Return[ReadList["cygpath.tmp", String][[1]]];
ReadList["!rm -f cygpath.tmp",String];
];
*)


(**********************************************************************************************)
(* Completes a singular template, calls Singular and returns the Singular output *)

CallSingular[singin_String,variables_List,repllist_List,opt___]:=Module[{subsMS,subsSM,singoutlist,fp,singout,timercommand,singprot,timerexit,singerr,subsB,singinrepl,repll,opt1},


(* Check global options *)

opt1=Cases[Flatten[{opt}],(X_->Y_)->(X->Y)];
CheckOptions1[opt1];

(* Variable substitutions between Mathematica and singular *)

subsMS=Table[ToString[variables[[i]],InputForm]->ReplaceVar[variables[[i]]],{i,1,Length[variables]}];
SingularVar=subsMS;
subsSM=Sort[Table[subsMS[[i,2]]->subsMS[[i,1]],{i,1,Length[subsMS]}],OrderedQ[{#2, #1}] &];
subsB={"{"->"","}"->""};
subsMS=Join[subsMS,subsB];

(* Complete template *)

singinrepl=StringReplace[singin,{"@@tracevalue"->ToString[SingularTrace1],
           "@@echovalue"->ToString[SingularEcho1],
           "@@stringvacualib"-> ReadList["!cygpath -u \"" <> $UserBaseDirectory <> "\"", String][[1]] <>"/Applications/Stringvacualib",
           "@@writepath"->userwriteunix<>"/"}];
repll=Table[repllist[[i,1]]->StringReplace[ToString[repllist[[i,2]],InputForm],subsMS],{i,1,Length[repllist]}];
singinrepl=StringReplace[singinrepl,repll];

(* Write template into Singular input file .singin *)

fp=OpenWrite[userwrite<>"\\.singin"];
WriteString[fp,singinrepl];
Close[fp];

(* Clearing up: remove .singout and .singerr *)
If[Length[FileNames[".singerr",userwrite]]>0,
   ReadList["!rm "<>userwriteunix<>"/.singerr",String]];
If[Length[FileNames[".singout",userwrite]]>0,
   ReadList["!rm "<>userwriteunix<>"/.singout",String]];

(* Find Process IDs of any existing Singular running *)
     (* Cygwin ps should be standard format *)
existingsing = #[[1]] & /@ ReadList["!ps -s | grep -i singular", {Number, String}];


(* 
cmd = "!bash " <> singexe <> " /.singin"; 
timerexit = ReadList[cmd,String];
cmd = "start /min \"Running\" bash "<> singexe<> " -q --no-warn" <> " " <> userwriteunix <> "/.singin";
If[SingularTimer1 === "on",
  timerexit = CheckAbort[TimeConstrained[ReadList[cmd,String],SingularMaxTime1],-100]
*)

cmd = "!nice "<> singexe<> " -q --no-warn" <> " " <> userwriteunix <> "/.singin";
timerexit = CheckAbort[ReadList[cmd,String],-100];

(* Read Singular results *)
If[Length[FileNames[".singout",userwrite]]>0,
    singoutlist=ReadList[userwrite<>"\\.singout",String];
    singoutlist=Table["{"<>StringReplace[singoutlist[[j]],subsSM]<>"}",
           {j,1,Length[singoutlist]}];
    singoutlist=Select[singoutlist,SyntaxQ[#]&];
    singout=Map[ToExpression[#,InputForm]&,singoutlist];
    Stringvacua`SingularStat="ok",
    (* Otherwise, no .singout *)
    Stringvacua`SingularStat="No Singular output";singout={}
];



(* handle errors *)
(*
If[timerexit==256,Stringvacua`SingularStat="timed out after "<>ToString[SingularMaxTime1]<>" seconds"];
If[timerexit==512,Stringvacua`SingularStat="interrupted"];
If[timerexit>512,Stringvacua`SingularStat="fatal error"];
If[Length[FileNames[".singerr",userwrite]]>0,
    singerr=ReadList[userwrite<>"\\.singerr",String];
    Stringvacua`SingularStat="Singular error : "<> singerr];
*)

currentsing = #[[1]] & /@ ReadList["!ps -s | grep -i singular", {Number, String}];

If[timerexit==-100,
   Stringvacua`SingularStat="interrupted";
   ReadList["!kill -9 "<>ToString[#],String]&/@Complement[currentsing,existingsing];
];
If[Length[FileNames[".singerr",userwrite]]>0,
   singerr=ReadList[userwrite<>"\\.singerr",String];
   Stringvacua`SingularStat="Singular error : "<> singerr;
];

singout//Return
];

End[]

(**********************************************************************************************)
(* Set StringvacuaVariables to default values *)

OptionsToDefault[]:=Module[{},

(* set StringvacuaOptions *)

(* need to find Singular Execution path inside this module *)
singexe = ReadList["!which Singular",String][[1]];

Options[Stringvacua]={PlanckInverse->1,PackageProtocol->10,FieldChar->0,
                      SingularProtocol->"Xterm",SingularTimer->"on",
                      SingularMaxTime->60,SingularTrace->0,SingularEcho->0,
                      SingularAccuracy->10^(-5),
		      XtermGeometry->"50x10-50+0",
		      XtermPath->Stringvacua`Private`xtermpath,
                      SingularPath->singexe};


(* check if options are specified in default file *)

If[Length[FileNames["StringvacuaOptions.m",$UserBaseDirectory<>"/Applications/Stringvacualib"]]>0,
   Map[Function[SetOptions[Stringvacua,#]],Flatten[{Get[$UserBaseDirectory<>"/Applications/Stringvacualib/StringvacuaOptions.m"]}]]];

(* Check for Xterm in SingularProtocol options *)

If[! `Private`XtermFound,SetOptions[Stringvacua,SingularProtocol->"None"]];

];
