(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     27808,        781]*)
(*NotebookOutlinePosition[     28535,        806]*)
(*  CellTagsIndexPosition[     28491,        802]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["CalcModel",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Adds all ideal lists to a ",
    FontFamily->"Arial"],
  StyleBox[ButtonBox["model list",
    ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "AddOns", "Model Keywords"]}]&),
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial"],
  StyleBox[".\nUsage: ",
    FontFamily->"Arial"],
  StyleBox["CalcModel[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["model<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n\n",
    FontFamily->"Helvetica"],
  StyleBox["model ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[": A model list created with the module ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["CreateModel",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[". This list should be of the form explained in",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["Model Keywords",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\n\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  Cell[BoxData[GridBox[{
          {
            StyleBox["Dummies",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], "\[Rule]", 
            RowBox[{
              RowBox[{
                StyleBox["a",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["list",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["of",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["replacements",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["of",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["the",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["form",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["expression",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"]}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain"], 
              StyleBox["\[Rule]",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain"], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain"], 
              RowBox[{
                StyleBox["real",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox["dummy",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox["variable",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox[
                  RowBox[{
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontSlant->"Italic"], 
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12]}]], 
                RowBox[{
                  StyleBox["(",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["default",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[\({}\),
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain"]}], 
                  StyleBox[")",
                    FontFamily->"Helvetica",
                    FontSize->12]}], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12]}]}]},
          {
            StyleBox["CDummies",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], "\[Rule]", 
            RowBox[{
              RowBox[{
                StyleBox["a",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["list",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["of",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["replacements",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["of",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["the",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["form",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["expression",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"]}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain"], 
              StyleBox["\[Rule]",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain"], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain"], 
              RowBox[{
                StyleBox["complex",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox["dummy",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox["variable",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic"], 
                RowBox[{
                  StyleBox["(",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["default",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[\({}\),
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain"]}], 
                  StyleBox[")",
                    FontFamily->"Helvetica",
                    FontSize->12]}]}]}]},
          {
            StyleBox["RDummies",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], "\[Rule]", 
            RowBox[{
              StyleBox["a",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["list",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["of",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["replacements",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontSlant->"Italic"], 
              RowBox[{
                StyleBox["(",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["default",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[\({}\),
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain"]}], 
                StyleBox[")",
                  FontFamily->"Helvetica",
                  FontSize->12]}]}]}
          }]],
    GridBoxOptions->{ColumnAlignments->{Left}}],
  "\n\n",
  StyleBox["No relevant global options from ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[". ",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox["   Adds all ideal lists to a model list. For perturbative \
examples, everything is automatic and no options need to be specified.  For \
non-perturbative examples the options ",
    FontSize->12],
  StyleBox["Dummies",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["CDummies",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" can be used to introduce dummy variables in order to make the \
ideal generators polynomial. The option ",
    FontSize->12],
  StyleBox["RDummies",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" can be used to add additional replacements if this should be \
necessary to eliminate all transcendental functions.\n\n",
    FontSize->12],
  "Input:",
  StyleBox[" Typically, the output of the module ",
    FontSize->12],
  StyleBox[ButtonBox["CreateModel",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[".\n\n",
    FontSize->12],
  "Output:",
  StyleBox[" A",
    FontSize->12],
  StyleBox[" model list, with added assignments to all keywords starting with \
'I' (see ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["Model Keywords",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["  for a description of these keyword",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["s).\n\n",
    FontSize->12],
  "Example: ",
  StyleBox["model=CalcModel[model]", "Input",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["adds assignments to all keywords starting with 'I' to the model \
list ",
    FontSize->12],
  StyleBox["model",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[".",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Example",
  FontSize->18]], "Section",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["Given a simple theory with ", "Subsection",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["\n", "Subsection",
    FontSize->12],
  StyleBox["\t  ",
    FontSize->12],
  StyleBox["2 complex moduli fields: ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + i \[Tau]",
    FontFamily->"Helvetica",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontFamily->"Helvetica",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Z = z + i \[Zeta]",
    FontFamily->"Helvetica",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\t",
    FontFamily->"Helvetica",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Kahler potential:  ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["- 3 Log[T+Conjugate[T]] - 3 Log[Z+Conjugate[Z]]",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\t  ",
    FontSize->12,
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S",
    FontSize->12,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["uperpotential: ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a T + b Z",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  with two parameters ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a, b",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[TextData[{
  StyleBox["First use ",
    FontFamily->"Arial"],
  StyleBox["CreateModel[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[" ", "Subsubsection",
    FontFamily->"Arial"],
  StyleBox["to create the model from the above input data.",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    StyleBox[\(model\  = \ 
          CreateModel[{T, Z}, {t, 
              z}, {\[Tau], \[Zeta]}, \ \(-3\)\ Log[T + Conjugate[T]]\  - \ 
              3\ Log[Z + Conjugate[Z]], \ a\ T\  + \ b\ Z, \ 
            Par -> {a, b}];\),
      "Input",
      FontSize->12,
      FontVariations->{"CompatibilityType"->0}]], "Input",
  CellLabel->"In[2]:="],

Cell[TextData[{
  StyleBox["Next, use ",
    FontFamily->"Arial"],
  StyleBox["CalcModel[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[" to add the ideal structures associated to this model to the \
model list. We store the result in ",
    FontFamily->"Arial"],
  StyleBox["fullmodel",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", which then contains all of the relevant model keywords for this \
example.",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fullmodel = CalcModel[model]\)], "Input",
  CellLabel->"In[3]:="],

Cell[BoxData[
    \({Fields \[Rule] {T, Z}, ReFields \[Rule] {t, z}, 
      ImFields \[Rule] {\[Tau], \[Zeta]}, 
      RFields \[Rule] {t, z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
      ComplexToReal \[Rule] {T \[Rule] t + \[ImaginaryI]\ \[Tau], 
          Z \[Rule] z + \[ImaginaryI]\ \[Zeta], 
          Conjugate[T] \[Rule] t - \[ImaginaryI]\ \[Tau], 
          Conjugate[Z] \[Rule] z - \[ImaginaryI]\ \[Zeta]}, 
      KPot \[Rule] \(-3\)\ Log[T + Conjugate[T]] - 3\ Log[Z + Conjugate[Z]], 
      SPot \[Rule] a\ T + b\ Z, 
      KMetric \[Rule] {{3\/\(4\ t\^2\), 0}, {0, 3\/\(4\ z\^2\)}}, 
      DSPot \[Rule] {T \[Rule] a, Z \[Rule] b}, 
      FTerms \[Rule] {T \[Rule] a - \(3\ \((a\ T + b\ Z)\)\)\/\(2\ t\), 
          Z \[Rule] b - \(3\ \((a\ T + b\ Z)\)\)\/\(2\ z\)}, 
      DeltaPot \[Rule] 0, 
      Pot \[Rule] \(b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\) - 6\ a\ b\ \((t\ z - 3\ \
\[Zeta]\ \[Tau])\) + a\^2\ \((t\^2 + 9\ \[Tau]\^2)\)\)\/\(192\ t\^3\ z\^3\), 
      DPot \[Rule] {t \[Rule] \(-\(\(3\ b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\) - 
                    6\ a\ b\ \((2\ t\ z - 9\ \[Zeta]\ \[Tau])\) + 
                    a\^2\ \((t\^2 + 
                          27\ \[Tau]\^2)\)\)\/\(192\ t\^4\ z\^3\)\)\), 
          z \[Rule] \(-\(\(b\^2\ \((z\^2 + 27\ \[Zeta]\^2)\) - 
                    6\ a\ b\ \((2\ t\ z - 9\ \[Zeta]\ \[Tau])\) + 
                    3\ a\^2\ \((t\^2 + 
                          9\ \[Tau]\^2)\)\)\/\(192\ t\^3\ z\^4\)\)\), \[Tau] \
\[Rule] \(3\ a\ \((b\ \[Zeta] + a\ \[Tau])\)\)\/\(32\ t\^3\ z\^3\), \[Zeta] \
\[Rule] \(3\ b\ \((b\ \[Zeta] + a\ \[Tau])\)\)\/\(32\ t\^3\ z\^3\)}, 
      Hesse \[Rule] {{\(6\ b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\) - 18\ a\ b\ \((t\ \
z - 6\ \[Zeta]\ \[Tau])\) + a\^2\ \((t\^2 + 54\ \[Tau]\^2)\)\)\/\(96\ t\^5\ z\
\^3\), \(b\^2\ \((z\^2 + 27\ \[Zeta]\^2)\) + a\ b\ \((\(-8\)\ t\ z + 54\ \
\[Zeta]\ \[Tau])\) + a\^2\ \((t\^2 + 27\ \[Tau]\^2)\)\)\/\(64\ t\^4\ z\^4\), \
\(-\(\(9\ a\ \((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^4\ z\^3\)\)\), \(-\(\(9\ b\ \
\((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^4\ z\^3\)\)\)}, {\(b\^2\ \
\((z\^2 + 27\ \[Zeta]\^2)\) + a\ b\ \((\(-8\)\ t\ z + 54\ \[Zeta]\ \[Tau])\) \
+ a\^2\ \((t\^2 + 27\ \[Tau]\^2)\)\)\/\(64\ t\^4\ z\^4\), \(b\^2\ \((z\^2 + \
54\ \[Zeta]\^2)\) - 18\ a\ b\ \((t\ z - 6\ \[Zeta]\ \[Tau])\) + 6\ a\^2\ \((t\
\^2 + 9\ \[Tau]\^2)\)\)\/\(96\ t\^3\ z\^5\), \(-\(\(9\ a\ \((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^3\ z\^4\)\)\), \(-\(\(9\ b\ \
\((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^3\ z\^4\)\)\)}, {\(-\(\(9\ a\
\ \((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^4\ z\^3\)\)\), \(-\(\(9\ a\ \
\((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^3\ z\^4\)\)\), \(3\ a\^2\)\/\
\(32\ t\^3\ z\^3\), \(3\ a\ b\)\/\(32\ t\^3\ z\^3\)}, {\(-\(\(9\ b\ \((b\ \
\[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^4\ z\^3\)\)\), \(-\(\(9\ b\ \
\((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^3\ z\^4\)\)\), \(3\ a\ b\)\/\
\(32\ t\^3\ z\^3\), \(3\ b\^2\)\/\(32\ t\^3\ z\^3\)}}, Dummies \[Rule] {}, 
      CDummies \[Rule] {}, 
      ISPot \[Rule] {Var \[Rule] {T, Z}, Par \[Rule] {a, b}, 
          Gen \[Rule] {a\ T + b\ Z}}, 
      IReSPot \[Rule] {Var \[Rule] {t, z, \[Tau], \[Zeta]}, 
          Par \[Rule] {a, b}, Gen \[Rule] {a\ t + b\ z}}, 
      IImSPot \[Rule] {Var \[Rule] {t, z, \[Tau], \[Zeta]}, 
          Par \[Rule] {a, b}, Gen \[Rule] {b\ \[Zeta] + a\ \[Tau]}}, 
      IDSPot \[Rule] {Var \[Rule] {T, Z}, Par \[Rule] {a, b}, 
          Gen \[Rule] {a, b}}, 
      IReImDSPot \[Rule] {Var \[Rule] {t, z, \[Tau], \[Zeta]}, 
          Par \[Rule] {a, b}, 
          Gen \[Rule] {t \[Rule] a, 
              z \[Rule] b, \[Tau] \[Rule] 0, \[Zeta] \[Rule] 0}}, 
      IFTerms \[Rule] {Var \[Rule] {t, z, \[Tau], \[Zeta]}, 
          Par \[Rule] {a, b}, 
          Gen \[Rule] {t \[Rule] \(-a\)\ t - 3\ b\ z, 
              z \[Rule] \(-3\)\ a\ t - 
                  b\ z, \[Tau] \[Rule] \(-3\)\ \((b\ \[Zeta] + 
                      a\ \[Tau])\), \[Zeta] \[Rule] \(-3\)\ \((b\ \[Zeta] + 
                      a\ \[Tau])\)}}, 
      IPot \[Rule] {Var \[Rule] {t, z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
          Gen \[Rule] {a\^2\ t\^2 - 6\ a\ b\ t\ z + b\^2\ z\^2 + 
                9\ b\^2\ \[Zeta]\^2 + 18\ a\ b\ \[Zeta]\ \[Tau] + 
                9\ a\^2\ \[Tau]\^2}}, 
      IDPot \[Rule] {Var \[Rule] {t, z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
          Gen \[Rule] {t \[Rule] \(-a\^2\)\ t\^2 + 12\ a\ b\ t\ z - 
                  3\ b\^2\ z\^2 - 27\ b\^2\ \[Zeta]\^2 - 
                  54\ a\ b\ \[Zeta]\ \[Tau] - 27\ a\^2\ \[Tau]\^2, 
              z \[Rule] \(-3\)\ a\^2\ t\^2 + 12\ a\ b\ t\ z - b\^2\ z\^2 - 
                  27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
                  27\ a\^2\ \[Tau]\^2, \[Tau] \[Rule] 
                3\ \((a\ b\ \[Zeta] + a\^2\ \[Tau])\), \[Zeta] \[Rule] 
                3\ \((b\^2\ \[Zeta] + a\ b\ \[Tau])\)}}}\)], "Output",
  CellLabel->"Out[3]="]
}, Open  ]],

Cell[TextData[StyleBox["It is easy to extract information from the resulting \
list. For example, to obtain the scalar potential (in terms of the component \
fields), one evaluates the following.", "Subsection",
  FontSize->12,
  FontWeight->"Plain"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Pot /. fullmodel\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(\(b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\) - 6\ a\ b\ \((t\ z - 3\ \[Zeta]\ \
\[Tau])\) + a\^2\ \((t\^2 + 9\ \[Tau]\^2)\)\)\/\(192\ t\^3\ z\^3\)\)], \
"Output",
  CellLabel->"Out[24]="]
}, Open  ]],

Cell[TextData[StyleBox["To obtain the ideal formed from the derivatives of \
the scalar potential with respect to the component fields is equally \
simple.", "Subsection",
  FontSize->12,
  FontWeight->"Plain"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IDPot\  /. \ fullmodel\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \({Var \[Rule] {t, z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
      Gen \[Rule] {t \[Rule] \(-a\^2\)\ t\^2 + 12\ a\ b\ t\ z - 
              3\ b\^2\ z\^2 - 27\ b\^2\ \[Zeta]\^2 - 
              54\ a\ b\ \[Zeta]\ \[Tau] - 27\ a\^2\ \[Tau]\^2, 
          z \[Rule] \(-3\)\ a\^2\ t\^2 + 12\ a\ b\ t\ z - b\^2\ z\^2 - 
              27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
              27\ a\^2\ \[Tau]\^2, \[Tau] \[Rule] 
            3\ \((a\ b\ \[Zeta] + a\^2\ \[Tau])\), \[Zeta] \[Rule] 
            3\ \((b\^2\ \[Zeta] + a\ b\ \[Tau])\)}}\)], "Output",
  CellLabel->"Out[25]="],

Cell[TextData[StyleBox["We may obtain the generators of this ideal from its \
ideal list by evaluating another substitution.", "Subsection",
  FontSize->12,
  FontWeight->"Plain"]], "Text"],

Cell[BoxData[
    \(\(Gen /. IDPot\) /. fullmodel\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \({t \[Rule] \(-a\^2\)\ t\^2 + 12\ a\ b\ t\ z - 3\ b\^2\ z\^2 - 
          27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
          27\ a\^2\ \[Tau]\^2, 
      z \[Rule] \(-3\)\ a\^2\ t\^2 + 12\ a\ b\ t\ z - b\^2\ z\^2 - 
          27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
          27\ a\^2\ \[Tau]\^2, \[Tau] \[Rule] 
        3\ \((a\ b\ \[Zeta] + a\^2\ \[Tau])\), \[Zeta] \[Rule] 
        3\ \((b\^2\ \[Zeta] + a\ b\ \[Tau])\)}\)], "Output",
  CellLabel->"Out[50]=",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{912, 749},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 282, 10, 76, "Title"],
Cell[2012, 64, 766, 23, 72, "Subsection"],
Cell[2781, 89, 12267, 352, 243, "Subsection"],
Cell[15051, 443, 1827, 63, 210, "Subsection"]
}, Open  ]],
Cell[16893, 509, 92, 2, 63, "Section"],
Cell[16988, 513, 2119, 68, 84, "Text"],
Cell[19110, 583, 287, 9, 27, "Text"],
Cell[19400, 594, 363, 9, 45, "Input"],
Cell[19766, 605, 469, 14, 46, "Text"],

Cell[CellGroupData[{
Cell[20260, 623, 85, 2, 45, "Input"],
Cell[20348, 627, 5078, 85, 547, "Output"]
}, Open  ]],
Cell[25441, 715, 260, 4, 46, "Text"],

Cell[CellGroupData[{
Cell[25726, 723, 74, 2, 45, "Input"],
Cell[25803, 727, 197, 4, 61, "Output"]
}, Open  ]],
Cell[26015, 734, 220, 4, 27, "Text"],

Cell[CellGroupData[{
Cell[26260, 742, 80, 2, 45, "Input"],
Cell[26343, 746, 609, 10, 63, "Output"],
Cell[26955, 758, 189, 3, 27, "Text"],
Cell[27147, 763, 87, 2, 45, "Input"],
Cell[27237, 767, 555, 11, 63, "Output"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

