(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     17573,        539]*)
(*NotebookOutlinePosition[     18303,        564]*)
(*  CellTagsIndexPosition[     18259,        560]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["CallSingular",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Module performing a low-level call to Singular.\nUsage: ",
    FontFamily->"Arial"],
  StyleBox["CallSingular[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["script,varpara,repl<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n\n",
    FontFamily->"Helvetica"],
  Cell[BoxData[GridBox[{
          {
            StyleBox[\(\(script\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontSlant->"Italic",
              FontVariations->{"CompatibilityType"->0}], ":", 
            RowBox[{
              StyleBox["A",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["string",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["containing",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["a",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["Singular",
                FontFamily->"Helvetica",
                FontSize->12,
                FontSlant->"Italic"], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["script",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["with",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["placeholders",
                FontFamily->"Helvetica",
                FontSize->12]}]},
          {
            StyleBox["varpara",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontSlant->"Italic",
              FontVariations->{"CompatibilityType"->0}], ":", 
            StyleBox[\(A\ list\ of\ all\ variables/parameters\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox[\(\(repl\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontSlant->"Italic",
              FontVariations->{"CompatibilityType"->0}], ":", 
            RowBox[{
              RowBox[{
                StyleBox["A",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["list",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["of",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["replacements",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["of",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["the",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["form",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["placeholder",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontVariations->{"CompatibilityType"->0}]}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontSlant->"Italic",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox["\[Rule]",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontSlant->"Italic",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox["value",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontSlant->"Italic",
                FontVariations->{"CompatibilityType"->0}]}]}
          }]],
    GridBoxOptions->{ColumnAlignments->{Left}}],
  StyleBox["\n",
    FontFamily->"Helvetica",
    GridBoxOptions->{ColumnAlignments->{Left}}],
  StyleBox["\nGlobal options from ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" A low-level call to ",
    FontSize->12],
  StyleBox["Singular ",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox["using the script provided in the first argument with placeholders \
replaced as indicated in the third argument. All variables/parameters must be \
provided in the second argument and must be of the form explained in ",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Interface",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" to be properly converted for use in ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[". \n",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" scripts can be taken from the list of scripts available to the \
package, which is stored in the global variable ",
    FontSize->12],
  StyleBox["SingularScripts",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" (execute ",
    FontSize->12],
  StyleBox["?SingularScriptInfo",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" in a notebook for a list and brief description of these \
scripts), or can be written by the user.\n\n",
    FontSize->12],
  "Output:",
  StyleBox[" The raw output from ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" as written to the file ",
    FontSize->12],
  StyleBox["~/.singout,",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or an error message if ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" execution is terminated or fails.\n\n",
    FontSize->12],
  "Example: ",
  StyleBox["CallSingular[\"SingularDim\"/.SingularScripts,{x,y},{\"@@\
listofvariables\"->{x,y}, \"@@coefffield\"->{0},\"@@inputideal\"->{x^2+2 \
y^2,x^3-5 y}}]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["calls ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" with the ",
    FontSize->12],
  StyleBox["SingularDim",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" script (which underlies the ",
    FontSize->12],
  StyleBox[ButtonBox["DimIdeal",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" module) and finds the dimension of a simple ideal in two \
variables ",
    FontSize->12],
  StyleBox["x, y",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". As a standard, we preface our internal variables with ",
    FontSize->12],
  StyleBox["@@", "Input",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[".",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Example",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[StyleBox["\nConsider the above ideal id=(x^2 + 2 y^2, x^3-5y) \
in two variables, we can determine its complex dimension by",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"id", "=", 
        RowBox[{"{", 
          RowBox[{\(Var \[Rule] {x, y}\), ",", 
            RowBox[{"Gen", "\[Rule]", 
              RowBox[{"{", 
                StyleBox[\(x^2\  + \ 2\ y^2, \ x^3 - 5  y\),
                  FontFamily->"Arial"], "}"}]}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", \(Dim /. DimIdeal[id]\)}], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \({0}\)], "Output",
  CellLabel->"Out[10]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["In the module ",
    FontFamily->"Arial"],
  StyleBox["DimIdeal[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[", the template script ",
    FontFamily->"Arial"],
  StyleBox["\"SingularDim\"/.SingularScripts", "Input",
    FontFamily->"Arial"],
  StyleBox[" is called:",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    StyleBox[\("\<SingularDim\>" /. SingularScripts\),
      "Input",
      FontFamily->"Arial"]], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \("// Singular script to calculate the dimension of an ideal.\n//    \
Labels: listofvariables -> Ring variables.\n//\t      coefffield -> A list of \
\"p,a,b,...\" where p is the characteristic of, \n//                          \
and a,b,... etc are the parameters in the extensions of, \n//\t\t\t    the \
rings coefficient field.\n//\t      inputideal -> Ideal whose dimension is \
desired.\n//\t      writepath -> Path to the working directory of the \
package.\n//\t      tracevalue -> Sets Singular debugging level.\n//\t      \
echovalue -> Sets echo level for Singular protocol window.\n//    Output: An \
integer - The dimension of the input ideal into file '.singout'.\n\n\
option(prot);\nring @R@=(@@coefffield),(@@listofvariables),dp;\n\
if(charstr(basering)==\"0,I\")\n{\nminpoly=I^2+1;\n}\nshort=0;\n\
TRACE=@@tracevalue;\necho=@@echovalue;\nideal @input@=@@inputideal;\nideal \
@theideal@ = simplify(@input@,1+2+4+8);\nint @thedim@ = dim(std(@theideal@));\
\nwrite(\":w @@writepath.singout\",@thedim@);\nquit;"\)], "Output",
  CellLabel->"Out[8]="]
}, Open  ]],

Cell[TextData[StyleBox["The above is a universal script for any \
ideal\[Dash]dimension calculation. The variables (parameters) are input by \
the replacement of keywords, indicated in the script by a @@ preface, with \
the case\[Dash]specific variables (parameters). In our example, we have:",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"{", 
        RowBox[{\("\<@@listofvariables\>" \[Rule] {x, y}\), ",", 
          RowBox[{"\"\<@@inputideal\>\"", "\[Rule]", 
            RowBox[{"{", 
              StyleBox[\(x^2\  + \ 2\ y^2, \ x^3 - 5  y\),
                FontFamily->"Arial"], "}"}]}], 
          ",", \("\<@@coefffield\>" \[Rule] {0}\)}], "}"}], ";"}]], "Input",
  CellLabel->"In[12]:="],

Cell[TextData[{
  StyleBox["Subsequently, we can, instead of using ",
    FontFamily->"Arial"],
  StyleBox["DimIdeal[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[" as above, perform the equivalent command, which is the the basic \
Singular call:",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CallSingular", "[", 
      StyleBox[" ",
        FontFamily->"Arial"], 
      RowBox[{
        StyleBox[\("\<SingularDim\>" /. SingularScripts\),
          "Input",
          FontFamily->"Arial"], 
        StyleBox[",",
          "Input",
          FontFamily->"Arial"], 
        StyleBox[\({x, y}\),
          "Input",
          FontFamily->"Arial"], 
        StyleBox[",",
          "Input",
          FontFamily->"Arial"], 
        RowBox[{"{", 
          RowBox[{\("\<@@listofvariables\>" \[Rule] {x, y}\), ",", 
            RowBox[{"\"\<@@inputideal\>\"", "\[Rule]", 
              RowBox[{"{", 
                StyleBox[\(x^2\  + \ 2\ y^2, \ x^3 - 5  y\),
                  FontFamily->"Arial"], "}"}]}], 
            ",", \("\<@@coefffield\>" \[Rule] {0}\)}], "}"}]}], 
      "]"}]], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \({{0}}\)], "Output",
  CellLabel->"Out[14]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["Similarly, we can, instead of using ",
    FontFamily->"Arial"],
  StyleBox["NumRoots[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[", evoke the basic script call",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"CallSingular", "[", 
      StyleBox[" ",
        FontFamily->"Arial"], 
      RowBox[{
        StyleBox[\("\<SingularNumRoots\>" /. SingularScripts\),
          "Input",
          FontFamily->"Arial"], 
        StyleBox[",",
          "Input",
          FontFamily->"Arial"], 
        StyleBox[\({x, y}\),
          "Input",
          FontFamily->"Arial"], 
        StyleBox[",",
          "Input",
          FontFamily->"Arial"], 
        RowBox[{"{", 
          RowBox[{\("\<@@listofvariables\>" \[Rule] {x, y}\), ",", 
            RowBox[{"\"\<@@inputideal\>\"", "\[Rule]", 
              RowBox[{"{", 
                StyleBox[\(x^2\  + \ 2\ y^2, \ x^3 - 5  y\),
                  FontFamily->"Arial"], "}"}]}], 
            ",", \("\<@@coefffield\>" \[Rule] {0}\), 
            ",", \("\<@@method\>" -> "\<Probab\>"\)}], "}"}]}], 
      "]"}]], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \({{1}}\)], "Output",
  CellLabel->"Out[26]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["We see that, using the replacement ",
    FontFamily->"Arial"],
  "\"@@method\"->\"Probab\"",
  StyleBox[", i.e., employing a probabilistic algorithm for root-finding, \
there is a single real root for this zero-dimensional ideal.",
    FontFamily->"Arial"]
}], "Text"]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{6, Automatic}, {Automatic, -17}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 285, 10, 76, "Title"],
Cell[2015, 64, 560, 16, 72, "Subsection"],
Cell[2578, 82, 5938, 169, 171, "Subsection"],
Cell[8519, 253, 3094, 102, 226, "Subsection"]
}, Open  ]],
Cell[11628, 358, 115, 3, 31, "Subtitle"],
Cell[11746, 363, 172, 2, 46, "Text"],

Cell[CellGroupData[{
Cell[11943, 369, 408, 10, 65, "Input"],
Cell[12354, 381, 62, 2, 44, "Output"]
}, Open  ]],
Cell[12431, 386, 339, 11, 27, "Text"],

Cell[CellGroupData[{
Cell[12795, 401, 145, 4, 44, "Input"],
Cell[12943, 407, 1081, 15, 500, "Output"]
}, Open  ]],
Cell[14039, 425, 327, 4, 46, "Text"],
Cell[14369, 431, 401, 9, 45, "Input"],
Cell[14773, 442, 290, 8, 27, "Text"],

Cell[CellGroupData[{
Cell[15088, 454, 851, 25, 65, "Input"],
Cell[15942, 481, 64, 2, 44, "Output"]
}, Open  ]],
Cell[16021, 486, 234, 7, 27, "Text"],

Cell[CellGroupData[{
Cell[16280, 497, 910, 26, 65, "Input"],
Cell[17193, 525, 64, 2, 44, "Output"]
}, Open  ]],
Cell[17272, 530, 297, 7, 27, "Text"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

