(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     12129,        392]*)
(*NotebookOutlinePosition[     12857,        417]*)
(*  CellTagsIndexPosition[     12813,        413]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["CreateIdeal",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Creates an ideal list from basic input data.\nUsage: ",
    FontFamily->"Arial"],
  StyleBox["CreateIdeal[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ideal<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  "\n",
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A polynomial, an ideal list or a list of polynomials and ideal \
lists\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\n",
    FontFamily->"Helvetica"],
  StyleBox["Var  \[RightArrow]",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" a list of variables or assignments of variables to numbers\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Par",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" \[Rule]  a list of parameters or assignments of parameters to \
numbers",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox["     Creates an ideal list by combining all polynomials in the \
first argument, and all variables and parameters specified in all arguments. \
Parameters (Variables) assigned to values using the ",
    FontSize->12],
  StyleBox["Par",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" (",
    FontSize->12],
  StyleBox["Var",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[") option are replaced by those values and thereby eliminated.\n\n\
",
    FontSize->12],
  "Output:",
  StyleBox["         An ideal list with assignments to the keywords ",
    FontSize->12],
  StyleBox["Var",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", ",
    FontSize->12],
  StyleBox["Par",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["Gen",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". See ",
    FontSize->12],
  StyleBox[ButtonBox["Ideal Keywords",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for the structure of an ideal list.\n\n",
    FontSize->12],
  "Example:",
  StyleBox["     ",
    FontSize->12],
  StyleBox["CreateIdeal[{IDPot/.model,IPot/.model}]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" combines the ideals ",
    FontSize->12],
  StyleBox["IDPot",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["IPot",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" of a ",
    FontSize->12],
  StyleBox[ButtonBox["model list",
    ButtonData:>"Model Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" called ",
    FontSize->12],
  StyleBox["model ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["into one ideal.",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Examples",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nTo create a simple ideal \[LeftAngleBracket] ",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`x\^2 - 1\)]],
  "\[RightAngleBracket] ",
  StyleBox["with a single generator and in one variable ",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ":"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CreateIdeal[{x^2 - 1}, Var \[Rule] x]\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \({Var \[Rule] {x}, Par \[Rule] {}, 
      Gen \[Rule] {\(-1\) + x\^2}}\)], "Output",
  CellLabel->"Out[16]="]
}, Open  ]],

Cell[TextData[{
  "\n",
  StyleBox["For a physical example, consider a SuGRA model specified by\n",
    FontFamily->"Arial"],
  StyleBox["\t",
    FontFamily->"Helvetica",
    FontWeight->"Bold"],
  StyleBox["2 complex fields: ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + i \[Tau]",
    FontFamily->"Helvetica",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Z = z + i \[Zeta]",
    FontFamily->"Helvetica",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\tKahler potential: ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["-3 Log[T+Conjugate[T]]-3 Log[Z+Conjugate[Z]]\n\t  ",
    FontSize->12,
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S",
    FontSize->12,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["uperpotential: ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a T + b Z",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  with two parameters ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a, b\n\n",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["We can obtain a full list of keywords for this model by using ",
    FontFamily->"Arial"],
  StyleBox["CreateModel[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[" and then ",
    FontFamily->"Arial"],
  StyleBox["CalcModel[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[". The ideal generated by  the scalar potential is then given by ",
    
    FontFamily->"Arial"],
  StyleBox["IPot/.model", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and that generated by the derivatives of the potential by ",
    FontFamily->"Arial"],
  StyleBox["IDPot/.model", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". Below we place these two in a list, ",
    FontFamily->"Arial"],
  StyleBox["PotDPot", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[".",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(model = 
        CreateModel[{T, Z}, {t, 
            z}, {\[Tau], \[Zeta]}, \(-3\)\ Log[T + Conjugate[T]]\  - \ 
            3\ Log[Z + Conjugate[Z]], \ a\ T\  + \ b\ Z, \ 
          Par -> {a, b}];\)\), "\[IndentingNewLine]", 
    \(\(model\  = \ CalcModel[model];\)\), "\[IndentingNewLine]", 
    \(PotDPot = {IDPot /. model, IPot /. model}\)}], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \({{Var \[Rule] {t, z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
        Gen \[Rule] {t \[Rule] \(-a\^2\)\ t\^2 + 12\ a\ b\ t\ z - 
                3\ b\^2\ z\^2 - 27\ b\^2\ \[Zeta]\^2 - 
                54\ a\ b\ \[Zeta]\ \[Tau] - 27\ a\^2\ \[Tau]\^2, 
            z \[Rule] \(-3\)\ a\^2\ t\^2 + 12\ a\ b\ t\ z - b\^2\ z\^2 - 
                27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
                27\ a\^2\ \[Tau]\^2, \[Tau] \[Rule] 
              3\ \((a\ b\ \[Zeta] + a\^2\ \[Tau])\), \[Zeta] \[Rule] 
              3\ \((b\^2\ \[Zeta] + a\ b\ \[Tau])\)}}, {Var \[Rule] {t, 
            z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
        Gen \[Rule] {a\^2\ t\^2 - 6\ a\ b\ t\ z + b\^2\ z\^2 + 
              9\ b\^2\ \[Zeta]\^2 + 18\ a\ b\ \[Zeta]\ \[Tau] + 
              9\ a\^2\ \[Tau]\^2}}}\)], "Output",
  CellLabel->"Out[19]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["Subsequently, we can combine them into a single ideal  with ",
    FontFamily->"Arial"],
  StyleBox["CreateIdeal[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[".",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CreateIdeal[PotDPot]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \({Var \[Rule] {t, z, \[Zeta], \[Tau]}, Par \[Rule] {a, b}, 
      Gen \[Rule] {\(-a\^2\)\ t\^2 + 12\ a\ b\ t\ z - 3\ b\^2\ z\^2 - 
            27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
            27\ a\^2\ \[Tau]\^2, \(-3\)\ a\^2\ t\^2 + 12\ a\ b\ t\ z - 
            b\^2\ z\^2 - 27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
            27\ a\^2\ \[Tau]\^2, 3\ \((a\ b\ \[Zeta] + a\^2\ \[Tau])\), 
          3\ \((b\^2\ \[Zeta] + a\ b\ \[Tau])\), 
          a\^2\ t\^2 - 6\ a\ b\ t\ z + b\^2\ z\^2 + 9\ b\^2\ \[Zeta]\^2 + 
            18\ a\ b\ \[Zeta]\ \[Tau] + 9\ a\^2\ \[Tau]\^2}}\)], "Output",
  CellLabel->"Out[20]="]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 284, 10, 76, "Title"],
Cell[2014, 64, 542, 16, 72, "Subsection"],
Cell[2559, 82, 1488, 51, 194, "Subsection"],
Cell[4050, 135, 2314, 82, 141, "Subsection"]
}, Open  ]],
Cell[6379, 220, 116, 3, 31, "Subtitle"],
Cell[6498, 225, 336, 11, 46, "Text"],

Cell[CellGroupData[{
Cell[6859, 240, 95, 2, 45, "Input"],
Cell[6957, 244, 128, 3, 44, "Output"]
}, Open  ]],
Cell[7100, 250, 2702, 83, 160, "Text"],

Cell[CellGroupData[{
Cell[9827, 337, 405, 8, 85, "Input"],
Cell[10235, 347, 864, 14, 82, "Output"]
}, Open  ]],
Cell[11114, 364, 233, 7, 27, "Text"],

Cell[CellGroupData[{
Cell[11372, 375, 78, 2, 45, "Input"],
Cell[11453, 379, 660, 10, 82, "Output"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

