(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     18796,        573]*)
(*NotebookOutlinePosition[     19559,        599]*)
(*  CellTagsIndexPosition[     19515,        595]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["CreateModel",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Creates a model list from basic input data.\nUsage: ",
    FontFamily->"Arial"],
  StyleBox["CreateModel[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["fields,refields,imfields,K,W<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n\n",
    FontFamily->"Arial"],
  Cell[BoxData[GridBox[{
          {
            StyleBox["fields",
              FontSize->12,
              FontWeight->"Plain",
              FontSlant->"Italic"], 
            StyleBox[":",
              FontSize->12], 
            StyleBox[\(A\ list\ of\ complex\ fields\ of\ the\ model\),
              FontSize->12]},
          {
            StyleBox["refields",
              FontSize->12,
              FontWeight->"Plain",
              FontSlant->"Italic"], 
            StyleBox[":",
              FontSize->12], 
            StyleBox[\(A\ list\ of\ the\ real\ parts\ of\ the\ fields\),
              FontSize->12]},
          {
            StyleBox["imfields",
              FontSize->12,
              FontWeight->"Plain",
              FontSlant->"Italic"], 
            StyleBox[":",
              FontSize->12], 
            StyleBox[\(A\ list\ of\ the\ imaginary\ parts\ of\ the\ fields\),
              FontSize->12]},
          {
            StyleBox["K",
              FontSize->12,
              FontWeight->"Plain",
              FontSlant->"Italic"], 
            StyleBox[":",
              FontSize->12], 
            StyleBox[\(Kahler\ potential\ of\ the\ model\),
              FontSize->12]},
          {
            StyleBox["W",
              FontSize->12,
              FontWeight->"Plain",
              FontSlant->"Italic"], 
            StyleBox[":",
              FontSize->12], 
            StyleBox[\(Superpotential\ of\ the\ model\),
              FontSize->12]}
          },
        ColumnAlignments->{Left}]],
    FontFamily->"Arial"],
  StyleBox["\n\nOptional arguments ",
    FontFamily->"Arial"],
  StyleBox["(see ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for details)\n\n",
    FontFamily->"Arial",
    FontSize->12],
  Cell[BoxData[GridBox[{
          {
            StyleBox["Par",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontSize->12], 
            StyleBox[
              RowBox[{
              "a", " ", "list", " ", "of", " ", "parameters", " ", "of", " ", 
                "the", " ", "model", " ", 
                RowBox[{"(", 
                  RowBox[{"default", 
                    StyleBox[\({}\),
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}]}], ")"}]}],
              FontSize->12]},
          {
            StyleBox["DeltaPot",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontSize->12], 
            StyleBox[
              RowBox[{\(a\ non\), "-", 
                RowBox[{
                "supersymmetric", " ", "addition", " ", "to", " ", "the", " ",
                   "potential", " ", 
                  RowBox[{"(", 
                    StyleBox[
                      RowBox[{"default", 
                        StyleBox["0",
                          FontWeight->"Plain",
                          FontVariations->{"CompatibilityType"->0}]}]], 
                    ")"}]}]}],
              FontSize->12]},
          {
            StyleBox["Meth",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontSize->12], 
            RowBox[{
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontSlant->"Italic"], 
              RowBox[{
                RowBox[{
                  StyleBox["math",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontSlant->"Italic"], 
                  StyleBox[" ",
                    FontSize->12], 
                  StyleBox[\((default)\),
                    FontSize->12]}], 
                StyleBox[",",
                  FontSize->12], 
                StyleBox[" ",
                  FontSize->12], 
                StyleBox[
                  RowBox[{
                    StyleBox["hand",
                      FontWeight->"Plain",
                      FontSlant->"Italic"], " ", "or", " ", 
                    StyleBox["off",
                      FontWeight->"Plain",
                      FontSlant->"Italic"]}],
                  FontSize->12]}]}]},
          {
            StyleBox["KMetricInverse",
              FontSize->12,
              FontWeight->"Plain"], 
            StyleBox["\[Rule]",
              FontSize->12], 
            StyleBox[\(the\ inverse\ Kahler\ metric\),
              FontSize->12]}
          },
        ColumnAlignments->{Left}]],
    FontFamily->"Arial"],
  StyleBox["\n",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\n",
    FontFamily->"Arial"],
  StyleBox[" The only relevant global option from ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" is ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["PlanckInverse",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". ",
    FontFamily->"Arial",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox["     Creates a model list from the input data. If option ",
    FontSize->12],
  StyleBox["Meth ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["is set to",
    FontSize->12],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["\"math\"  ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["the inverse Kahler metric is calculated, if set to ",
    FontSize->12],
  StyleBox["\"hand\" ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[" it has",
    FontSize->12],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["to be provided using the optional argument ",
    FontSize->12],
  StyleBox["KMetricInverse",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", if set to ",
    FontSize->12],
  StyleBox["\"off\"  ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["the inverse Kahler metric is not calculated. In the last case, \
the resulting model list does not contain entries for the scalar potential \
and its derivatives. \n\n",
    FontSize->12],
  "Input:",
  StyleBox["     Complex field names (as well as names for their real and \
imaginary parts), the Kahler and Superpotentials.\n\n",
    FontSize->12],
  "Output:",
  StyleBox["     A model list, with assignments to the keywords ",
    FontSize->12],
  StyleBox["Fields, ReFields, ImFields, RFields,  Par, ComplexToReal, KPot, \
SPot, DSPot, KMetric, Fterms, Pot, DPot, Hesse, DeltaPot",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" (see ",
    FontSize->12],
  StyleBox[ButtonBox["Model Keywords",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for a description of these keywords).\n\n",
    FontSize->12],
  "Example: ",
  StyleBox["model=CreateModel[{T,Z},{t,z},{tau,zeta},-3 Log[T+Conjugate[T]]-3 \
Log[Z+Conjugate[Z]], a T+b Z,Par->{a,b}]", "Input",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["creates a model list, ",
    FontSize->12],
  StyleBox["model,",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" for a simple model with two moduli ",
    FontSize->12],
  StyleBox["T,Z, ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["a standard Kahler potential and a simple linear superpotential \
with parameters ",
    FontSize->12],
  StyleBox["a,b",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[".",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Example",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None,
  LineSpacing->{1, 0}],

Cell[TextData[{
  StyleBox["\nA simple model with",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" \n\t ",
    FontSize->12],
  StyleBox["2 complex moduli fields: ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + i \[Tau]",
    FontFamily->"Helvetica",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontFamily->"Helvetica",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Z = z + i \[Zeta]",
    FontFamily->"Helvetica",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\t",
    FontFamily->"Helvetica",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Kahler potential:  ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["- 3 Log[T+Conjugate[T]] - 3 Log[Z+Conjugate[Z]]",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\t  ",
    FontSize->12,
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S",
    FontSize->12,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["uperpotential: ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a T + b Z",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  with two parameters ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a, b",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}]
}], "Text",
  CellDingbat->None,
  LineSpacing->{1, 3}],

Cell[CellGroupData[{

Cell[BoxData[
    StyleBox[\(model\  = \ 
        CreateModel[{T, Z}, {t, 
            z}, {\[Tau], \[Zeta]}, \ \(-3\)\ Log[T + Conjugate[T]]\  - \ 
            3\ Log[Z + Conjugate[Z]], \ a\ T\  + \ b\ Z, \ Par -> {a, b}]\),
      "Input",
      FontSize->12,
      FontVariations->{"CompatibilityType"->0}]], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \({Fields \[Rule] {T, Z}, ReFields \[Rule] {t, z}, 
      ImFields \[Rule] {\[Tau], \[Zeta]}, 
      RFields \[Rule] {t, z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
      ComplexToReal \[Rule] {T \[Rule] t + \[ImaginaryI]\ \[Tau], 
          Z \[Rule] z + \[ImaginaryI]\ \[Zeta], 
          Conjugate[T] \[Rule] t - \[ImaginaryI]\ \[Tau], 
          Conjugate[Z] \[Rule] z - \[ImaginaryI]\ \[Zeta]}, 
      KPot \[Rule] \(-3\)\ Log[T + Conjugate[T]] - 3\ Log[Z + Conjugate[Z]], 
      SPot \[Rule] a\ T + b\ Z, 
      KMetric \[Rule] {{3\/\(4\ t\^2\), 0}, {0, 3\/\(4\ z\^2\)}}, 
      DSPot \[Rule] {T \[Rule] a, Z \[Rule] b}, 
      FTerms \[Rule] {T \[Rule] a - \(3\ \((a\ T + b\ Z)\)\)\/\(2\ t\), 
          Z \[Rule] b - \(3\ \((a\ T + b\ Z)\)\)\/\(2\ z\)}, 
      DeltaPot \[Rule] 0, 
      Pot \[Rule] \(b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\) - 6\ a\ b\ \((t\ z - 3\ \
\[Zeta]\ \[Tau])\) + a\^2\ \((t\^2 + 9\ \[Tau]\^2)\)\)\/\(192\ t\^3\ z\^3\), 
      DPot \[Rule] {t \[Rule] \(-\(\(3\ b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\) - 
                    6\ a\ b\ \((2\ t\ z - 9\ \[Zeta]\ \[Tau])\) + 
                    a\^2\ \((t\^2 + 
                          27\ \[Tau]\^2)\)\)\/\(192\ t\^4\ z\^3\)\)\), 
          z \[Rule] \(-\(\(b\^2\ \((z\^2 + 27\ \[Zeta]\^2)\) - 
                    6\ a\ b\ \((2\ t\ z - 9\ \[Zeta]\ \[Tau])\) + 
                    3\ a\^2\ \((t\^2 + 
                          9\ \[Tau]\^2)\)\)\/\(192\ t\^3\ z\^4\)\)\), \[Tau] \
\[Rule] \(3\ a\ \((b\ \[Zeta] + a\ \[Tau])\)\)\/\(32\ t\^3\ z\^3\), \[Zeta] \
\[Rule] \(3\ b\ \((b\ \[Zeta] + a\ \[Tau])\)\)\/\(32\ t\^3\ z\^3\)}, 
      Hesse \[Rule] {{\(6\ b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\) - 18\ a\ b\ \((t\ \
z - 6\ \[Zeta]\ \[Tau])\) + a\^2\ \((t\^2 + 54\ \[Tau]\^2)\)\)\/\(96\ t\^5\ z\
\^3\), \(b\^2\ \((z\^2 + 27\ \[Zeta]\^2)\) + a\ b\ \((\(-8\)\ t\ z + 54\ \
\[Zeta]\ \[Tau])\) + a\^2\ \((t\^2 + 27\ \[Tau]\^2)\)\)\/\(64\ t\^4\ z\^4\), \
\(-\(\(9\ a\ \((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^4\ z\^3\)\)\), \(-\(\(9\ b\ \
\((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^4\ z\^3\)\)\)}, {\(b\^2\ \
\((z\^2 + 27\ \[Zeta]\^2)\) + a\ b\ \((\(-8\)\ t\ z + 54\ \[Zeta]\ \[Tau])\) \
+ a\^2\ \((t\^2 + 27\ \[Tau]\^2)\)\)\/\(64\ t\^4\ z\^4\), \(b\^2\ \((z\^2 + \
54\ \[Zeta]\^2)\) - 18\ a\ b\ \((t\ z - 6\ \[Zeta]\ \[Tau])\) + 6\ a\^2\ \((t\
\^2 + 9\ \[Tau]\^2)\)\)\/\(96\ t\^3\ z\^5\), \(-\(\(9\ a\ \((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^3\ z\^4\)\)\), \(-\(\(9\ b\ \
\((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^3\ z\^4\)\)\)}, {\(-\(\(9\ a\
\ \((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^4\ z\^3\)\)\), \(-\(\(9\ a\ \
\((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^3\ z\^4\)\)\), \(3\ a\^2\)\/\
\(32\ t\^3\ z\^3\), \(3\ a\ b\)\/\(32\ t\^3\ z\^3\)}, {\(-\(\(9\ b\ \((b\ \
\[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^4\ z\^3\)\)\), \(-\(\(9\ b\ \
\((b\ \[Zeta] + 
                        a\ \[Tau])\)\)\/\(32\ t\^3\ z\^4\)\)\), \(3\ a\ b\)\/\
\(32\ t\^3\ z\^3\), \(3\ b\^2\)\/\(32\ t\^3\ z\^3\)}}}\)], "Output",
  CellLabel->"Out[27]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["Once we have a model list such as the output above, specific \
quantities can be obtained by substitution of keywords. For example, to \
obtain the Kahler Metric  ",
    FontSize->12],
  Cell[BoxData[
      \(TraditionalForm\`K\_\(A  B\&_\)\)],
    FontSize->12],
  StyleBox[" = ",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\(\[PartialD]\_A\)\(\ \)\),
            "TraditionalForm"], \(\[PartialD]\_B\&_\ K\)}], TraditionalForm]],
    
    FontSize->12],
  StyleBox[", one evaluates the following.",
    FontSize->12]
}], "Text",
  FontFamily->"Arial",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
    \(KMetric /. model\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \({{3\/\(4\ t\^2\), 0}, {0, 3\/\(4\ z\^2\)}}\)], "Output",
  CellLabel->"Out[28]="]
}, Open  ]],

Cell[BoxData[
    StyleBox[\(\(\t\)\(To\ obtain\ the\ set\ of\ substitution\ rules\ for\ \
replacing\ the\ Complex\ field\ variables\ \((T, \ 
          Z)\)\ with\ the\ real\ components\ \((t, \ 
          z, \ \[Tau], \ \[Zeta])\)\ we\ \(\(evaluate\)\(:\)\)\)\),
      FontSize->12]], "Text",
  FontFamily->"Arial",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
    \(ComplexToReal\  /. \ model\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \({T \[Rule] t + \[ImaginaryI]\ \[Tau], 
      Z \[Rule] z + \[ImaginaryI]\ \[Zeta], 
      Conjugate[T] \[Rule] t - \[ImaginaryI]\ \[Tau], 
      Conjugate[Z] \[Rule] z - \[ImaginaryI]\ \[Zeta]}\)], "Output",
  CellLabel->"Out[30]="]
}, Open  ]],

Cell[BoxData[
    RowBox[{"\t", 
      StyleBox[\(The\ F\[Dash]terms, \ as\ another\ example, \ 
        can\ be\ obtained\ \(\(by\)\(:\)\)\),
        FontSize->12]}]], "Text",
  FontFamily->"Arial",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
    \(FTerms /. model\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \({T \[Rule] a - \(3\ \((a\ T + b\ Z)\)\)\/\(2\ t\), 
      Z \[Rule] b - \(3\ \((a\ T + b\ Z)\)\)\/\(2\ z\)}\)], "Output",
  CellLabel->"Out[29]="]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
ScreenStyleEnvironment->"Working",
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 284, 10, 76, "Title"],
Cell[2014, 64, 564, 16, 72, "Subsection"],
Cell[2581, 82, 5712, 177, 342, "Subsection"],
Cell[8296, 261, 2630, 89, 227, "Subsection"]
}, Open  ]],
Cell[10941, 353, 138, 4, 31, "Subtitle"],
Cell[11082, 359, 2095, 68, 103, "Text"],

Cell[CellGroupData[{
Cell[13202, 431, 344, 8, 45, "Input"],
Cell[13549, 441, 3163, 52, 331, "Output"]
}, Open  ]],
Cell[16727, 496, 641, 21, 47, "Text"],

Cell[CellGroupData[{
Cell[17393, 521, 74, 2, 45, "Input"],
Cell[17470, 525, 101, 2, 58, "Output"]
}, Open  ]],
Cell[17586, 530, 333, 7, 25, "Text"],

Cell[CellGroupData[{
Cell[17944, 541, 84, 2, 45, "Input"],
Cell[18031, 545, 252, 5, 44, "Output"]
}, Open  ]],
Cell[18298, 553, 215, 6, 26, "Text"],

Cell[CellGroupData[{
Cell[18538, 563, 73, 2, 45, "Input"],
Cell[18614, 567, 166, 3, 58, "Output"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

