(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      8733,        298]*)
(*NotebookOutlinePosition[      9462,        323]*)
(*  CellTagsIndexPosition[      9418,        319]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["DimIdeal",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Calculates the complex dimension of an ideal.\nUsage: ",
    FontFamily->"Arial"],
  StyleBox["DimIdeal[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ideal<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  "\n",
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A (list of) ideal list(s) of the form explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[", combined into a single ideal by the module\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\n",
    FontFamily->"Helvetica"],
  StyleBox["VarPar",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" \[Rule] (list of) rule(s) assigning parameter(s) or variable(s) \
to value(s)",
    FontFamily->"Helvetica",
    FontSize->12],
  "\n",
  StyleBox["\nGlobal options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" Computes the complex dimension of an ideal. Parameters are \
replaced by any values assigned to them with the ",
    FontSize->12],
  StyleBox["VarPar",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option, otherwise they are treated as field extensions.\n\n",
    FontSize->12],
  "Output:",
  StyleBox[" An ideal list with its dimension assigned to the keyword ",
    FontSize->12],
  StyleBox["Dim",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". See ",
    FontSize->12],
  StyleBox[ButtonBox["Ideal Keywords",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for the structure of an ideal list.\n\n",
    FontSize->12],
  "Example:",
  StyleBox[" ",
    FontSize->12],
  StyleBox["DimIdeal[IDPot/.model/.{a->1,b->1}]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["finds the complex dimension of an ideal ",
    FontSize->12],
  StyleBox["IDPot/.model", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" for parameter values ",
    FontSize->12],
  StyleBox["a=b=1",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Examples",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nIn the following we see that the complex dimension of the ideal \
generated by ",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`\(\((x\^2 + y\^2, \ x*y)\)\(\ \)\)\)],
    FontFamily->"Arial"],
  StyleBox["in two complex variables ",
    FontFamily->"Arial"],
  StyleBox["x",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" and ",
    FontFamily->"Arial"],
  StyleBox["y",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" is 0.",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dim /. 
      DimIdeal[{Var \[Rule] {x, y}, Gen \[Rule] {x^2 + y^2, x*y}}]\)], "Input",\

  CellLabel->"In[31]:="],

Cell[BoxData[
    \({0}\)], "Output",
  CellLabel->"Out[31]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["As a more complicated example, let us consider the ideal dV, \
formed from the numerators of the partial derivatives, with respect to the \
fields, of the scalar potential V for the model,\n\t",
    FontFamily->"Arial"],
  StyleBox["2 complex moduli fields: ",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + i \[Tau]",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Z = z + i \[Zeta]\n\t",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Kahler potential:  ",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["- 3 Log[T+Conjugate[T]] - 3 Log[Z+Conjugate[Z]]",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\tS",
    FontFamily->"Arial",
    FontSize->12,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["uperpotential: a T + b Z  with two parameters a, b",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dim /. 
      DimIdeal[IDPot /. 
          CalcModel[
            CreateModel[{T, Z}, {t, 
                z}, {\[Tau], \[Zeta]}, \ \(-3\)\ Log[T + Conjugate[T]]\  - \ 
                3\ Log[Z + Conjugate[Z]], \ a\ T\  + \ b\ Z, \ 
              Par -> {a, b}]], FieldChar \[Rule] \ 101]\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \({1}\)], "Output",
  CellLabel->"Out[32]="]
}, Open  ]],

Cell[TextData[StyleBox["We performed the above calculation in a number field \
of characteristic 101 and find that the resulting complex dimension is 1.",
  FontFamily->"Arial"]], "Text"]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{889, 622},
WindowMargins->{{Automatic, 89}, {49, Automatic}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 281, 10, 76, "Title"],
Cell[2011, 64, 540, 16, 72, "Subsection"],
Cell[2554, 82, 1776, 63, 211, "Subsection"],
Cell[4333, 147, 1543, 51, 158, "Subsection"]
}, Open  ]],
Cell[5891, 201, 116, 3, 31, "Subtitle"],
Cell[6010, 206, 547, 19, 46, "Text"],

Cell[CellGroupData[{
Cell[6582, 229, 134, 4, 45, "Input"],
Cell[6719, 235, 62, 2, 44, "Output"]
}, Open  ]],
Cell[6796, 240, 1295, 35, 103, "Text"],

Cell[CellGroupData[{
Cell[8116, 279, 346, 8, 105, "Input"],
Cell[8465, 289, 62, 2, 44, "Output"]
}, Open  ]],
Cell[8542, 294, 187, 2, 27, "Text"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

