(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     14064,        464]*)
(*NotebookOutlinePosition[     14792,        489]*)
(*  CellTagsIndexPosition[     14748,        485]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["Elimination",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Eliminates parameters/variables from an ideal to produce \
constraints on the remaining parameters/variables.\nUsage: ",
    FontFamily->"Arial"],
  StyleBox["Elimination[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ideal<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  "\n",
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A (list of)  ideal list(s) of the form explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[", combined into a single ideal by the module",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\n\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Meth",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["    \[Rule]  a string set to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox["Basic",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[", ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox["HD",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" or ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox["Prod",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" (default ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\"Basic\" ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[")\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["VarPar",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" \[Rule]  list of parameters/variables to be eliminated (if \
omitted all variables will be eliminated)\n\nAll global options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" Eliminates parameters/variables from an ideal to produce \
constraints on the remaining parameters/variables. Typically, it is used to \
eliminate all of the variables to obtain constraints on the parameters of the \
model which are necessary for the existence of a vacuum of a given type.\n\n",
    
    FontSize->12],
  "Output:",
  StyleBox["  An ",
    FontSize->12],
  StyleBox[ButtonBox["ideal list",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[", with ",
    FontSize->12],
  StyleBox["Var",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" containing the remaining parameters/variables, ",
    FontSize->12],
  StyleBox["Par",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" the empty list and ",
    FontSize->12],
  StyleBox["Gen",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" containing the constraints.\n\n",
    FontSize->12],
  "Example:",
  StyleBox[" ",
    FontSize->12],
  StyleBox["Elimination[{IPot,IDPot}/.model]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" calculates the constraints on the parameters of ",
    FontSize->12],
  StyleBox["model",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" which follow from imposing the vanishing of the potential and \
its first derivatives.",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Example",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  "\n",
  StyleBox["For a model specified by\n",
    FontFamily->"Arial"],
  StyleBox["\t",
    FontFamily->"Helvetica",
    FontWeight->"Bold"],
  StyleBox["2 complex fields: ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + i \[Tau]",
    FontFamily->"Helvetica",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Z = z + i \[Zeta]",
    FontFamily->"Helvetica",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\tKahler potential: ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["-3 Log[T+Conjugate[T]]-3 Log[Z+Conjugate[Z]]\n\t  ",
    FontSize->12,
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S",
    FontSize->12,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["uperpotential: ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a T + b Z",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  with two parameters ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a, b\n",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["we can obtain a full list of keywords by using ",
    FontFamily->"Arial"],
  StyleBox["CreateModel[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[" and then ",
    FontFamily->"Arial"],
  StyleBox["CalcModel[ ]", "Input",
    FontFamily->"Arial"],
  StyleBox[". The ideal generated by  the scalar potential is then given by ",
    
    FontFamily->"Arial"],
  StyleBox["IPot/.model", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and that generated by the derivatives of the potential, by ",
    FontFamily->"Arial"],
  StyleBox["IDPot/.model", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". We place these two ideals in a list and combine them into a \
single ideal by evaluating ",
    FontFamily->"Arial"],
  StyleBox["CreateIdeal[PotDPot]", "Input",
    FontFamily->"Arial"],
  StyleBox[".",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(model = 
        CreateModel[{T, Z}, {t, 
            z}, {\[Tau], \[Zeta]}, \(-3\)\ Log[T + Conjugate[T]]\  - \ 
            3\ Log[Z + Conjugate[Z]], \ a\ T\  + \ b\ Z, \ 
          Par -> {a, b}];\)\), "\[IndentingNewLine]", 
    \(\(model\  = \ CalcModel[model];\)\), "\[IndentingNewLine]", 
    \(PotDPot = CreateIdeal[{IDPot /. model, IPot /. model}]\)}], "Input",
  CellLabel->"In[33]:="],

Cell[BoxData[
    \({Var \[Rule] {t, z, \[Zeta], \[Tau]}, Par \[Rule] {a, b}, 
      Gen \[Rule] {\(-a\^2\)\ t\^2 + 12\ a\ b\ t\ z - 3\ b\^2\ z\^2 - 
            27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
            27\ a\^2\ \[Tau]\^2, \(-3\)\ a\^2\ t\^2 + 12\ a\ b\ t\ z - 
            b\^2\ z\^2 - 27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
            27\ a\^2\ \[Tau]\^2, 3\ \((a\ b\ \[Zeta] + a\^2\ \[Tau])\), 
          3\ \((b\^2\ \[Zeta] + a\ b\ \[Tau])\), 
          a\^2\ t\^2 - 6\ a\ b\ t\ z + b\^2\ z\^2 + 9\ b\^2\ \[Zeta]\^2 + 
            18\ a\ b\ \[Zeta]\ \[Tau] + 9\ a\^2\ \[Tau]\^2}}\)], "Output",
  CellLabel->"Out[35]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["In the ideal ",
    FontFamily->"Arial"],
  StyleBox["PotDPot", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[", the variables are ",
    FontFamily->"Arial"],
  "{t,z,\[Tau],\[Zeta]}",
  StyleBox[" and the parameters are {a,b}. ",
    FontFamily->"Arial"],
  StyleBox["Elimination[PotDPot]", "Input",
    FontFamily->"Arial"],
  StyleBox[" will eliminate the variables by simply returning any constraints \
on the parameters which are created by imposing the vanishing of ",
    FontFamily->"Arial"],
  StyleBox["Pot", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontFamily->"Arial"],
  StyleBox["DPot", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[".",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Elimination[PotDPot]\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \({Var \[Rule] {a, b}, Par \[Rule] {}, Gen \[Rule] {0}}\)], "Output",
  CellLabel->"Out[45]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["The above means that the no constraints are placed on the \
parameters by requiring the existence of a Minkowski turning point to the \
potential. However, if we wish to eliminate just some of the variables, say \
{t, z}, then conditions will be placed on the remaining variables/parameters \
{\[Tau], \[Zeta], a, b}. These may be read off by demanding the vanishing of \
the polynomials associated with the keyword ",
    FontFamily->"Arial"],
  StyleBox["Gen", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[" in the following result.",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Elimination[PotDPot, Meth -> "\<Prod\>", 
      VarPar \[Rule] {t, z}]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \({Var \[Rule] {a, b, \[Zeta], \[Tau]}, Par \[Rule] {}, 
      Gen \[Rule] {b\^2\ \[Zeta] + a\ b\ \[Tau], 
          a\ b\ \[Zeta] + a\^2\ \[Tau]}}\)], "Output",
  CellLabel->"Out[36]="]
}, Open  ]],

Cell[TextData[{
  "We note that ",
  StyleBox["Elimination[ ]", "Input"],
  " reproduces the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " command ",
  StyleBox["GrobnerBasis[ ]", "Input"],
  " in eliminating variables; however for large sets of polynomials, it is \
much faster. Here we have also set the method option (",
  StyleBox["Meth",
    CellFrame->True,
    Background->GrayLevel[0.849989]],
  ")so as to make use of product orderings which can further speed up \
calculations."
}], "Text",
  FontFamily->"Arial"]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 284, 10, 76, "Title"],
Cell[2014, 64, 607, 17, 72, "Subsection"],
Cell[2624, 83, 3206, 121, 227, "Subsection"],
Cell[5833, 206, 1700, 54, 158, "Subsection"]
}, Open  ]],
Cell[7548, 263, 115, 3, 31, "Subtitle"],
Cell[7666, 268, 2692, 83, 141, "Text"],

Cell[CellGroupData[{
Cell[10383, 355, 418, 8, 85, "Input"],
Cell[10804, 365, 660, 10, 82, "Output"]
}, Open  ]],
Cell[11479, 378, 810, 26, 46, "Text"],

Cell[CellGroupData[{
Cell[12314, 408, 78, 2, 45, "Input"],
Cell[12395, 412, 112, 2, 44, "Output"]
}, Open  ]],
Cell[12522, 417, 627, 13, 65, "Text"],

Cell[CellGroupData[{
Cell[13174, 434, 128, 3, 45, "Input"],
Cell[13305, 439, 204, 4, 44, "Output"]
}, Open  ]],
Cell[13524, 446, 536, 16, 47, "Text"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

