(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     15509,        450]*)
(*NotebookOutlinePosition[     16258,        475]*)
(*  CellTagsIndexPosition[     16214,        471]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Stringvacua package: General Information",
  FontSize->24]], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontFamily->"Arial",
  FontSize->18],

Cell[TextData[{
  StyleBox["\n",
    FontSize->18],
  StyleBox["Help Overview",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  "\n\n",
  StyleBox["Based on methods detailed in [1,2], this package is designed to \
determine the vacuum structure of (string derived or inspired) potentials in \
d=4, N=1 supergravity, using methods of computational algebraic geometry. The \
package automatically links to the popular algebraic geometry programme '", 
    "Subsection",
    FontSize->12],
  StyleBox["Singular", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox["'  [3] for fast polynomial calculations. It realises an embedding \
of ", "Subsection",
    FontSize->12],
  StyleBox["Singular", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" into ", "Subsection",
    FontSize->12],
  StyleBox["Mathematica", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" and provides a range of modules, from low to high level, for \
convenient access of ", "Subsection",
    FontSize->12],
  StyleBox["Singular", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" from within ", "Subsection",
    FontSize->12],
  StyleBox["Mathematica", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[".  This hierarchical structure also allows  for applications to \
other problems which require fast polynomial calculations.\n\nInstallation of \
the package is simple. Details can be found", "Subsection",
    FontSize->12],
  StyleBox[" ", "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>Installation,
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[".\n\nIf this is your first encounter with this package a good \
place to start may be the tutorial provided with this distribution; more \
involved examples can be found ", "Subsection",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Examples Overview",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[".  Alternatively, for an interactive \"guided tour\" which \
demonstrates a simple application to string theory execute: ", "Subsection",
    FontSize->12],
  StyleBox["SimpleMinimise[ ]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".", "Subsubsection",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n", "Input",
    FontSize->12],
  StyleBox["\nA protocol of the ", "Subsection",
    FontSize->12],
  StyleBox["Singular", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculation is normally displayed in an xterm which is \
automatically opened when necessary. For details about the ", "Subsection",
    FontSize->12],
  StyleBox["Mathematica/Singular", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" interface involving xterms, text versus notebook, and operating \
systems, etc., see ", "Subsection",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Interface",
    ButtonData:>"Stringvacua Interface",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[". Alternatively, there is a basic implementation, on operating \
systems which are not too versatile, where ", "Subsection",
    FontSize->12],
  StyleBox["Mathematica", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calls ", "Subsection",
    FontSize->12],
  StyleBox["Singular", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" silently.\n\nA supergravity model is stored as a list of rules \
which assign certain d=4, N=1 quantities to keywords, such that, for example \
", "Subsection",
    FontSize->12],
  StyleBox["SPot/.model1", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" produces the superpotential of a model with name ", "Subsection",
    
    FontSize->12],
  StyleBox["model1", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". A list of keywords for models can be obtained from ", 
    "Subsection",
    FontSize->12],
  StyleBox[ButtonBox["Model Keywords",
    ButtonStyle->"AddOnsLink"], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[".  Polynomial ideals are also stored as a list of rules which \
assign ideal properties to certain keywords. See ", "Subsection",
    FontSize->12],
  StyleBox[ButtonBox["Ideal Keywords",
    ButtonStyle->"AddOnsLink"], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for details.\n\nA list of modules in this package and their \
optional arguments can be obtained from", "Subsection",
    FontSize->12],
  StyleBox[" ", "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Modules",
    ButtonData:>"Stringvacua Modules",
    Active->True,
    ButtonStyle->"AddOnsLink"], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" and ", "Subsection",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[". Global options of the package are defined in ", "Subsection",
    FontSize->12],
  StyleBox["Options[Stringvacua]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". See ", "Subsection",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for a detailed description of these variables.\n\nThe status of \
the package is stored in global variables listed in ", "Subsection",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Status",
    ButtonStyle->"AddOnsLink"], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[". A list of the ", "Subsection",
    FontSize->12],
  StyleBox["Singular", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" scripts called by the package is obtained by typing ", 
    "Subsection",
    FontSize->12],
  StyleBox["?SingularScriptInfo", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ", "Subsection",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["in a notebook", "Subsection",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".", "Subsection",
    FontSize->12]
}], "Text",
  CellFrame->{{0, 0}, {2, 0}},
  FontFamily->"Arial",
  FontSize->18],

Cell[TextData[{
  StyleBox["\n\n",
    FontSize->18],
  StyleBox["What this package can and cannot do",
    FontSize->18,
    FontWeight->"Bold"],
  StyleBox["\n ",
    FontSize->18],
  StyleBox[" \n", "Text",
    FontSize->18],
  StyleBox["If your problem is one of extremisation of a given polynomial \
potential, especially one arising from string and supergravity inspired \
models, subject to the constraint of vanishing or non-vanishing of a set of \
polynomials, then this package is ideal. In fact, if you wish to perform \
manipulations on a system of high degree polynomials in many variables, such \
as finding the dimension of the space of solutions, whether there are real \
roots, is the solution space connected, etc., this is often already beyond \
the usual computational capabilities of ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontTracking->"Extended"],
  StyleBox["Mathematica,", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontTracking->"Extended"],
  StyleBox[" and you may wish to try this package.\n\nMoreover, it was \
pointed out in [2] that if you have transcendental functions, as are \
frequently encountered in non-perturbative potentials, then all that is \
needed is to introduce dummy variables which replace the transcendence and \
manipulate the system as if it were polynomial. One can simplify the system \
of equations as much as possible using this package and then re-substitute \
the transcendental functions, such as exponentials, at the end. We have \
implemented this possibility in the package as well.\n\nNote that many of the \
routines used by ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontTracking->"Extended"],
  StyleBox["Singular", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontTracking->"Extended"],
  StyleBox[" require that one works in the field of rational numbers (or over \
a finite field) rather than with the real numbers. The package will therefore \
automatically approximate any non-rational numbers it encounters before \
calling ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontTracking->"Extended"],
  StyleBox["Singular", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontTracking->"Extended"],
  StyleBox[". See the", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontTracking->"Extended"],
  StyleBox[" ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12],
  StyleBox[ButtonBox["Numerical Approximations",
    ButtonStyle->"AddOnsLink"], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" example and ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Stringvacua Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData], "Subsection",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    ButtonBoxOptions->{ButtonData:>Times[ Stringvacua, Options],
    Active->True,
    ButtonStyle->"Hyperlink"}],
  StyleBox["for further information. \n", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12]
}], "Subtitle",
  CellFrame->{{0, 0}, {2, 0}},
  FontFamily->"Arial",
  FontSize->18],

Cell[TextData[{
  "\n",
  StyleBox["Disclaimer",
    FontSize->18,
    FontWeight->"Bold"],
  StyleBox["\n\n",
    FontSize->18],
  StyleBox["The authors take no responsibility in the correctness of the \
research conducted with the aid of this package; all results must be \
carefully checked under the user's own initiative. Details of questions about \
and problems with ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontTracking->"Extended"],
  StyleBox["Singular", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontTracking->"Extended"],
  StyleBox[" can be found in [3].\n\nFurthermore, although ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontTracking->"Extended"],
  StyleBox["Singular ", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontTracking->"Extended"],
  StyleBox["performs polynomial manipulations more efficiently than ", 
    "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontTracking->"Extended"],
  StyleBox["Mathematica,", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontTracking->"Extended"],
  StyleBox[" the computation time of a given system is unpredictable and may \
be exponentially long. Therefore it is up to the user's discretion to wait or \
to terminate a process.", "Subsection",
    LineSpacing->{1, 2},
    FontSize->12,
    FontWeight->"Bold",
    FontTracking->"Extended"]
}], "Subtitle",
  CellFrame->{{0, 0}, {2, 0}},
  FontFamily->"Arial",
  FontSize->18],

Cell[TextData[{
  StyleBox["\n",
    FontSize->18],
  StyleBox["References",
    FontSize->18,
    FontWeight->"Bold"],
  StyleBox["\n\n",
    FontSize->18],
  StyleBox["To cite this package, please refer to the companion paper in \
[4].", "Text",
    FontSize->14,
    FontColor->RGBColor[1, 0, 0]],
  StyleBox["\n", "Text",
    FontSize->14,
    FontColor->RGBColor[1, 0, 1]],
  StyleBox["\n",
    FontSize->18],
  StyleBox["1.   J. Gray, Y.-H. He and A. Lukas, ``Algorithmic algebraic \
geometry and flux vacua,''\n  \t JHEP 0609, 031 (2006) \
[arXiv:hep-th/0606122].\n\n2.   J. Gray, Y.-H. He, A. Ilderton and A. Lukas, \
``A new method for finding vacua in string phenomenology,''\n\t JHEP 0707, \
023 (2007)  [arXiv:hep-th/0703249].\n\n3.   G.-M. Greuel, G. Pfister, and H. \
Schonemann,  ''Singular: A Computer Algebra System for Polynomial \
Computations,'' \n\tCentre for Computer Algebra, University of Kaiserslautern \
(2001). \n\tAvailable at http://www.singular.uni-kl.de/\n\n4.  J. Gray, Y.-H. \
He, A. Ilderton and A. Lukas, \"STRINGVACUA, A ", "Subsection",
    FontSize->12],
  StyleBox["Mathematica", "Subsection",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" Package for Studying Vacuum Configurations in String \
Phenomenology,\"\n\tarXiv:0801.1508 [hep-th].", "Subsection",
    FontSize->12]
}], "Subtitle",
  CellFrame->{{0, 0}, {2, 0}},
  FontFamily->"Arial",
  FontSize->18]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 777}},
WindowToolbars->{"RulerBar", "EditBar"},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{51, Automatic}, {32, Automatic}},
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 163, 4, 76, "Title"],
Cell[1942, 59, 6905, 197, 530, "Text"],
Cell[8850, 258, 3510, 95, 346, "Subtitle"],
Cell[12363, 355, 1717, 54, 168, "Subtitle"],
Cell[14083, 411, 1410, 36, 304, "Subtitle"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

