(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     20675,        690]*)
(*NotebookOutlinePosition[     22272,        747]*)
(*  CellTagsIndexPosition[     22059,        735]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Ideal Keywords",
  FontSize->24]], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[StyleBox["Overview",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["An ideal is stored as a list of assignments of keywords to \
values. For an ideal list, ",
    FontFamily->"Arial"],
  StyleBox["id",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", a particular entry can then be accessed with ",
    FontFamily->"Arial"],
  StyleBox["keyword ", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["/. ", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["id", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". For a list of keywords see below.",
    FontFamily->"Arial"]
}], "Text",
  CellDingbat->None,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"],

Cell[TextData[{
  StyleBox["List of ideal keywords ",
    FontFamily->"Arial",
    FontSize->18],
  "\n\n",
  Cell[BoxData[GridBox[{
          {
            StyleBox[
              RowBox[{
                ButtonBox["Dim",
                  ButtonData:>"Dim",
                  ButtonStyle->"Hyperlink"], " "}],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[\(dimension\ of\ ideal\),
              GridBoxOptions->{ColumnAlignments->{Left}}]},
          {
            StyleBox[
              RowBox[{
                ButtonBox["Gen",
                  ButtonData:>"Gen",
                  ButtonStyle->"Hyperlink"], " "}],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[\(list\ of\ polynomials\ generating\ the\ ideal\),
              GridBoxOptions->{ColumnAlignments->{Left}}]},
          {
            StyleBox[
              RowBox[{
                ButtonBox["NRoots",
                  ButtonData:>"NRoots",
                  ButtonStyle->"Hyperlink"], " "}],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            
            StyleBox[\(number\ of\ real\ roots\ 
                of\ a\ zero\[Dash]dimensional\ ideal\),
              GridBoxOptions->{ColumnAlignments->{Left}}]},
          {
            StyleBox[
              RowBox[{
                ButtonBox["Par",
                  ButtonData:>"Par",
                  ButtonStyle->"Hyperlink"], " "}],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            
            StyleBox[\(parameters\ of\ ideal\ \((treated\ as\ extensions\ to\ \
base\ number\ field)\)\),
              GridBoxOptions->{ColumnAlignments->{Left}}]},
          {
            StyleBox[
              RowBox[{
                ButtonBox["Sturm",
                  ButtonData:>"Sturm",
                  ButtonStyle->"Hyperlink"], " "}],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            RowBox[{
              StyleBox["result",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["of",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["a",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["Sturm",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["query",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              RowBox[{
                StyleBox["(",
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                RowBox[{
                  StyleBox["see",
                    GridBoxOptions->{ColumnAlignments->{Left}}], 
                  StyleBox[" ",
                    GridBoxOptions->{ColumnAlignments->{Left}}], 
                  StyleBox[
                    ButtonBox["SturmQuery",
                      ButtonStyle->"AddOnsLink"],
                    FontFamily->"Arial",
                    GridBoxOptions->{ColumnAlignments->{Left}}]}], 
                StyleBox[")",
                  GridBoxOptions->{ColumnAlignments->{Left}}]}]}]},
          {
            StyleBox[
              RowBox[{
                ButtonBox["Thom",
                  ButtonData:>"Sturm",
                  ButtonStyle->"Hyperlink"], " "}],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              GridBoxOptions->{
              ColumnAlignments->{
                Left}}], \(Thom\ polynomials\ for\ zero\[Dash]dimensional\ \
ideal\)},
          {
            StyleBox[
              RowBox[{
                ButtonBox["Var",
                  ButtonData:>"Var",
                  ButtonStyle->"Hyperlink"], " "}],
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            
            StyleBox[\(variables\ of\ ideal\ \((variables\ of\ the\ \
polynomial\ ring)\)\),
              GridBoxOptions->{ColumnAlignments->{Left}}]}
          }]],
    FontFamily->"Arial",
    GridBoxOptions->{ColumnAlignments->{Left}}]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Detailed Description of ideal keywords",
  FontSize->18]], "Title",
  FontSize->9],

Cell[TextData[{
  StyleBox["\nNB.\n",
    FontWeight->"Bold"],
  StyleBox["The module ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["CreateIdeal",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" automatically incorporates the first 3 keywords ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["Var",
    ButtonData:>"Var",
    ButtonStyle->"Hyperlink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" ",
    FontFamily->"Arial"],
  StyleBox[", ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["Par",
    ButtonData:>"Par",
    ButtonStyle->"Hyperlink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["  ",
    FontFamily->"Arial"],
  StyleBox["and ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["Gen",
    ButtonData:>"Gen",
    ButtonStyle->"Hyperlink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["  ",
    FontFamily->"Arial"],
  StyleBox["while the keywords\n",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["Dim",
    ButtonData:>"Dim",
    ButtonStyle->"Hyperlink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["NRoots",
    ButtonData:>"NRoots",
    ButtonStyle->"Hyperlink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" ",
    FontFamily->"Arial"],
  StyleBox["and ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["Sturm",
    ButtonData:>"Sturm",
    ButtonStyle->"Hyperlink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" ",
    FontFamily->"Arial"],
  StyleBox["can be obtained by using the modules ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["DimIdeal",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["NumRoots",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", respectively.",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["\n\nIn the following, to illustrate the keywords, we consider the \
following example of an ideal\n",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[Cell[BoxData[
      \(TraditionalForm\`Id\)], "Text",
    FontSlant->"Italic"], "Text"],
  StyleBox["  = ", "Text",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[Cell[BoxData[
      \(TraditionalForm\`\(\(<\)\(a\ x\^2 + b\ y\^2\)\), \ c\ x\ y\)], "Text",
    
    FontSlant->"Italic"], "Text"],
  StyleBox[" ", "Text",
    FontSlant->"Italic"],
  StyleBox[">  \[Subset]  ", "Text",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[Cell[BoxData[
      \(TraditionalForm\`C\ [x, \ y]\)], "Text",
    FontSlant->"Italic"], "Text"],
  "\n",
  StyleBox["The base number field here is just C, the field of complex \
numbers; the polynomial ring is  ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  Cell[BoxData[
      \(TraditionalForm\`C[x, y]\)],
    FontFamily->"Arial"],
  StyleBox[", viz., the polynomials in 2 variables ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  Cell[BoxData[
      FormBox[
        StyleBox["x",
          FontWeight->"Plain"], TraditionalForm]],
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  Cell[BoxData[
      \(TraditionalForm\`y\)],
    FontFamily->"Arial"],
  ";  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a, b, c)\)\(\ \ \)\)\)]],
  StyleBox["are complex parameters",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[".",
    FontFamily->"Arial"]
}], "Text",
  FontSize->12],

Cell[TextData[{
  StyleBox["Keyword ",
    FontSize->14],
  StyleBox["Var",
    FontSize->14,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->14],
  StyleBox["Keyword for (ring) variables in an ideal list. In the above ideal \
",
    FontSize->12],
  StyleBox["Id", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  ",    ",
  StyleBox["Var \[Rule] {x,y}", "Input"]
}], "Section",
  CellFrame->True,
  FontSize->12,
  Background->GrayLevel[0.833326],
  CellTags->"Var"],

Cell[TextData[{
  StyleBox["Keyword ",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox["Par",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontFamily->"Arial",
    FontSize->16],
  StyleBox["Keyword for parameters in an ideal list. Also keyword for ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["model lists",
    ButtonData:>"Model Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial"],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["option for some stringvacua modules",
    ButtonData:>"Module Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial"],
  StyleBox[". In the above ideal ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Id", "Input",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[",   ",
    FontFamily->"Arial"],
  StyleBox["Par \[Rule] {a,b,c}", "Input",
    FontFamily->"Arial"]
}], "Section",
  CellFrame->True,
  FontSize->12,
  Background->GrayLevel[0.833326],
  CellTags->"Par"],

Cell[TextData[{
  StyleBox["Keyword ",
    FontSize->14],
  StyleBox["Gen",
    FontSize->14,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Keyword for polynomial generators in an ideal list. This is the \
crucial part in the description of the ideal. In the above ideal ",
    FontSize->12],
  StyleBox["Id", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  ", ",
  StyleBox["Gen ",
    FontSize->12],
  Cell[BoxData[
      \(TraditionalForm\` \[Rule] \)]],
  "  ",
  Cell[BoxData[
      \(TraditionalForm\`{a\ x\^2 + b\ y\^2, \ c\ x\ y}\)]]
}], "Section",
  CellFrame->True,
  FontSize->12,
  Background->GrayLevel[0.833326],
  CellTags->"Gen"],

Cell[TextData[{
  StyleBox["Keyword ",
    FontSize->14],
  StyleBox["Dim",
    FontSize->14,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Keyword for complex dimension of an ideal in an ideal list. We \
can determine the complex dimension of the ideal with the command ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["DimIdeal",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial"],
  StyleBox[". First, we must write ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Id ",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[" in our standard form as shown below.",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\n\n",
    FontSize->12],
  StyleBox["Id={Var\[Rule]{x,y},Par\[Rule]{a,b,c},Gen\[Rule]{a x^2+b y^2,c x \
y}};\n(*Alternatively one can write fully Id=CreateIdeal[{Var\[Rule]{x,y},Par\
\[Rule]{a,b,c}, Gen\[Rule]{a x^2+b y^2,c x y}}]*)\nDimIdeal[Id]\n\ngives:\n\n\
", "Input"],
  Cell[BoxData[
      StyleBox[\({Var \[Rule] {x, y}, Par \[Rule] {a, b, c}, 
          Gen \[Rule] {a\ x\^2 + b\ y\^2, c\ x\ y}, Dim \[Rule] {0}}\),
        "Text"]],
    FontWeight->"Plain"],
  StyleBox["\n", "Input",
    FontWeight->"Plain"],
  StyleBox["\n", "Input"],
  StyleBox["Thus the ideal ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["Id",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[" is 0 complex dimensional, as can be seen by evaluating  ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["Dim /. DimIdeal[Id]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"]
}], "Section",
  CellFrame->True,
  FontSize->12,
  Background->GrayLevel[0.833326],
  CellTags->"Dim"],

Cell[TextData[{
  StyleBox["Keyword ",
    FontSize->14],
  StyleBox["NRoots",
    FontSize->14,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Keyword for number of real roots in an ideal list for \
zero-dimension ideal. In our ",
    FontSize->12],
  StyleBox["Id",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[" example, since it is indeed of complex dimension 0, we can find \
the number of real roots. Note that we need to specify the parameters ",
    FontSize->12],
  StyleBox["(a,b,c)",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" with numerical values.  For the above example of the ideal ",
    FontSize->12],
  StyleBox["Id",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[",",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox["\n\n",
    FontSize->12],
  StyleBox["NRoots/.NumRoots[Id/.{a\[Rule]1,b\[Rule]1,c\[Rule]1}]\n\ngives:\n\
\n", "Input",
    FontSize->14],
  StyleBox["{1}", "Output",
    FontSize->14,
    FontWeight->"Plain"]
}], "Section",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"NRoots"],

Cell[TextData[{
  StyleBox["Keyword ",
    FontSize->14],
  StyleBox["Sturm",
    FontSize->14,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for the result of a Sturm query in an ideal list. This \
entry is a list with a particular structure, which depends on whether or not \
we are testing the signs of eigenvalues of a matrix (see",
    FontSize->12],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["for more details). For the above example of the ideal ",
    FontSize->12],
  StyleBox["Id",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[",\n\n",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox["Sturm/.SturmQuery[Id/.{a\[Rule]1,b\[Rule]1,c\[Rule]1},\
TestRational\[Rule]{x+y},RRoots\[Rule]\"Probab\"]\n\ngives:\n\n", "Input",
    FontSize->14],
  StyleBox["{{x+y},{0},{1}}\n\n",
    FontSize->14,
    FontWeight->"Plain"],
  StyleBox["This indicates that the polynomial {x+y} takes the value 0 on the \
single real root of ",
    FontSize->12],
  StyleBox["Id",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox[".",
    FontSize->12]
}], "Section",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Sturm"],

Cell[TextData[{
  StyleBox["Keyword ",
    FontSize->14],
  StyleBox["Thom",
    FontSize->14,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Keyword for the Thom encoding polynomials, the signs of which \
uniquely determine the roots of the zero-dimensional ideal (see ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["ThomEncode",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for more details).",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\nFor example, \n\n",
    FontSize->12],
  Cell[BoxData[
      RowBox[{"Thom", "/.", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"ThomEncode", "[", 
              
              StyleBox[\({Var \[Rule] {x, y}, Par \[Rule] {a, b, c}, 
                  Gen \[Rule] {a\ x\^2 + b\ y\^2, c\ x\ y}, 
                  Dim \[Rule] {0}}\),
                FontWeight->"Plain"], "]"}], "[", \([1]\), "]"}], ")"}]}]], 
    "Text",
    FontSize->14],
  "\n\n",
  StyleBox["gives:\n\n",
    FontSize->12],
  Cell[BoxData[
      \({y, x\^2, 2\ x}\)],
    FontSize->14,
    FontWeight->"Plain"]
}], "Section",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->{"NRoots", "Thom"}]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
ScreenStyleEnvironment->"Working",
WindowSize->{752, 653},
WindowMargins->{{0, Automatic}, {0, Automatic}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Var"->{
    Cell[12322, 386, 554, 22, 92, "Section",
      CellTags->"Var"]},
  "Par"->{
    Cell[12879, 410, 1250, 46, 108, "Section",
      CellTags->"Par"]},
  "Gen"->{
    Cell[14132, 458, 751, 28, 109, "Section",
      CellTags->"Gen"]},
  "Dim"->{
    Cell[14886, 488, 1825, 58, 301, "Section",
      CellTags->"Dim"]},
  "NRoots"->{
    Cell[16714, 548, 1198, 42, 233, "Section",
      CellTags->"NRoots"],
    Cell[19368, 643, 1291, 44, 252, "Section",
      CellTags->{"NRoots", "Thom"}]},
  "Sturm"->{
    Cell[17915, 592, 1450, 49, 267, "Section",
      CellTags->"Sturm"]},
  "Thom"->{
    Cell[19368, 643, 1291, 44, 252, "Section",
      CellTags->{"NRoots", "Thom"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Var", 21359, 708},
  {"Par", 21440, 711},
  {"Gen", 21523, 714},
  {"Dim", 21605, 717},
  {"NRoots", 21691, 720},
  {"Sturm", 21863, 725},
  {"Thom", 21949, 728}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 114, 3, 76, "Title"],
Cell[1844, 57, 85, 2, 31, "Subtitle"],
Cell[1932, 61, 976, 31, 48, "Text"],
Cell[2911, 94, 5190, 133, 283, "Subsection"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8138, 232, 106, 2, 66, "Title"],
Cell[8247, 236, 4072, 148, 179, "Text"],
Cell[12322, 386, 554, 22, 92, "Section",
  CellTags->"Var"],
Cell[12879, 410, 1250, 46, 108, "Section",
  CellTags->"Par"],
Cell[14132, 458, 751, 28, 109, "Section",
  CellTags->"Gen"],
Cell[14886, 488, 1825, 58, 301, "Section",
  CellTags->"Dim"],
Cell[16714, 548, 1198, 42, 233, "Section",
  CellTags->"NRoots"],
Cell[17915, 592, 1450, 49, 267, "Section",
  CellTags->"Sturm"],
Cell[19368, 643, 1291, 44, 252, "Section",
  CellTags->{"NRoots", "Thom"}]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

