(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     55380,       1775]*)
(*NotebookOutlinePosition[     59602,       1912]*)
(*  CellTagsIndexPosition[     58845,       1881]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Model Keywords",
  FontSize->24]], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[StyleBox["Overview",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["A SUGRA model is stored as a list of assignments of the form ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["keyword",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  \[Rule] ",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["value",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".",
    FontFamily->"Arial",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["For a model list called ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["model",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" a particular quantity is accessed with ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["keyword ", "Input",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["/. model", "Input",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". ", "Subsubsection",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["For a list of keywords and their interpretation see below. For \
some keywords (keywords starting with a capital I) the associated quantity is \
an ideal list. See ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["Ideal Keywords",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for the structure of such a list.",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  CellDingbat->None,
  FontFamily->"Helvetica",
  FontSize->14,
  FontWeight->"Bold"],

Cell[TextData[{
  StyleBox["List of model Keywords \n\n",
    FontFamily->"Arial",
    FontSize->18],
  Cell[BoxData[GridBox[{
          {
            StyleBox[
              ButtonBox["CDummies",
                ButtonData:>"CDummies",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(list\ of\ rules\ for\ creating\ complex\ dummy\ \
variables\ for\ non\[Dash]polynomial\ expressions\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["ComplexToReal",
                  FontFamily->"Arial"],
                ButtonData:>"ComplexToReal",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(list\ of\ rules\ replacing\ complex\ with\ real\ \
fields\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["DeltaPot",
                  FontFamily->"Arial"],
                ButtonData:>"DeltaPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(non\[Dash]supersymmetric\ addition\ to\ scalar\ \
potential\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["DPot",
                  FontFamily->"Arial"],
                ButtonData:>"DPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(list\ of\ first\ derivatives\ of\ scalar\ potential\ \
with\ respect\ to\ real\ fields\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["DSPot",
                  FontFamily->"Arial"],
                ButtonData:>"DSPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(list\ of\ first\ derivatives\ of\ superpotential\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["Dummies",
                  FontFamily->"Arial"],
                ButtonData:>"Dummies",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(list\ of\ rules\ for\ creating\ real\ dummy\ variables\
\ for\ non\[Dash]polynomial\ expressions\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["Fields",
                  FontFamily->"Arial"],
                ButtonData:>"Fields",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(list\ of\ complex\ fields\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["FTerms",
                  FontFamily->"Arial"],
                ButtonData:>"FTerms",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[
              RowBox[{
                RowBox[{
                "list", " ", "of", " ", "assignments", " ", "of", " ", "the", 
                  " ", "form", " ", 
                  StyleBox["complex",
                    FontWeight->"Plain",
                    FontSlant->"Italic",
                    FontTracking->"Plain",
                    FontVariations->{"Underline"->False,
                    "Outline"->False,
                    "Shadow"->False,
                    "StrikeThrough"->False,
                    "Masked"->False,
                    "CompatibilityType"->0,
                    "RotationAngle"->0}], 
                  StyleBox[" ",
                    FontWeight->"Plain",
                    FontSlant->"Italic",
                    FontTracking->"Plain",
                    FontVariations->{"Underline"->False,
                    "Outline"->False,
                    "Shadow"->False,
                    "StrikeThrough"->False,
                    "Masked"->False,
                    "CompatibilityType"->0,
                    "RotationAngle"->0}], 
                  StyleBox["field",
                    FontWeight->"Plain",
                    FontSlant->"Italic",
                    FontTracking->"Plain",
                    FontVariations->{"Underline"->False,
                    "Outline"->False,
                    "Shadow"->False,
                    "StrikeThrough"->False,
                    "Masked"->False,
                    "CompatibilityType"->0,
                    "RotationAngle"->0}]}], 
                StyleBox["\[Rule]",
                  FontWeight->"Plain",
                  FontSlant->"Plain",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}], 
                StyleBox[\(corresponding\ F\ term\),
                  FontWeight->"Plain",
                  FontSlant->"Italic",
                  FontTracking->"Plain",
                  FontVariations->{"Underline"->False,
                  "Outline"->False,
                  "Shadow"->False,
                  "StrikeThrough"->False,
                  "Masked"->False,
                  "CompatibilityType"->0,
                  "RotationAngle"->0}]}],
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["Hesse",
                  FontFamily->"Arial"],
                ButtonData:>"Hesse",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(\(\ \)\(Hessian\ matrix\ calculated\ with\ respect\ to\
\ real\ fields\)\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["IDPot",
                  FontFamily->"Arial"],
                ButtonData:>"IDPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(Ideal\ list\ for\ derivatives\ of\ scalar\ potential\),
              
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["IDSPot",
                  FontFamily->"Arial"],
                ButtonData:>"IDSPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(ideal\ list\ for\ derivatives\ of\ superpotential\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["IFTerms",
                  FontFamily->"Arial"],
                ButtonData:>"IFTerms",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(Ideal\ list\ of\ F\ terms\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["IImSPot",
                  FontFamily->"Arial"],
                ButtonData:>"IImSPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(Ideal\ list\ of\ imaginary\ part\ of\ superpotential\),
              
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["ImFields",
                  FontFamily->"Arial"],
                ButtonData:>"ImFields",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(list\ of\ imaginary\ parts\ of\ fields\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["IPot",
                  FontFamily->"Arial"],
                ButtonData:>"IPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(Ideal\ list\ for\ scalar\ potential\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["IReImDSPot",
                  FontFamily->"Arial"],
                ButtonData:>"IReIMDSPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(Ideal\ list\ for\ superpotential\ derivatives\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["IReSPot",
                  FontFamily->"Arial"],
                ButtonData:>"IReSPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(Ideal\ list\ for\ real\ part\ of\ superpotential\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["ISPot",
                  FontFamily->"Arial"],
                ButtonData:>"ISPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(ideal\ list\ for\ superpotential\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["KMetric",
                  FontFamily->"Arial"],
                ButtonData:>"KMetric",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(Kahler\ metric\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["KPot",
                  FontFamily->"Arial"],
                ButtonData:>"KPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(Kahler\ potential\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["Par",
                  FontFamily->"Arial"],
                ButtonData:>"Par",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(list\ of\ parameters\ in\ the\ model\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["Pot",
                  FontFamily->"Arial"],
                ButtonData:>"Pot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(scalar\ potential\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["RDummies",
                  FontFamily->"Arial"],
                ButtonData:>"RDummies",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            
            StyleBox[\(list\ of\ rules\ for\ all\ dummy\ variables\ for\ non\
\[Dash]polynomial\ expressions\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["ReFields",
                  FontFamily->"Arial"],
                ButtonData:>"ReFields",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(list\ of\ real\ parts\ of\ fields\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["RFields",
                  FontFamily->"Arial"],
                ButtonData:>"RFields",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontSize->14,
              FontWeight->"Bold"], 
            StyleBox[\(list\ of\ real\ and\ imaginary\ parts\ of\ fields\),
              FontFamily->"Arial",
              FontSize->14,
              FontWeight->"Bold"]},
          {
            StyleBox[
              ButtonBox[
                StyleBox["SPot",
                  FontFamily->"Arial"],
                ButtonData:>"SPot",
                ButtonExpandable->False,
                ButtonStyle->"Hyperlink"],
              FontFamily->"Arial"], 
            StyleBox["\[Rule]",
              FontWeight->"Bold"], 
            StyleBox["superpotential",
              FontFamily->"Arial",
              FontWeight->"Bold"]}
          }]],
    GridBoxOptions->{ColumnAlignments->{Left}}]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Detailed description of model keywords",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nWe exemplify the keywords for the following model, hereafter \
referred to as ",
    FontFamily->"Helvetica",
    FontWeight->"Bold"],
  StyleBox["M1",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[":\n\t",
    FontFamily->"Helvetica",
    FontWeight->"Bold"],
  StyleBox["2 complex fields: ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + i \[Tau]",
    FontFamily->"Helvetica",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Z = z + i \[Zeta]",
    FontFamily->"Helvetica",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\tKahler potential: ",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["-3 Log[T+Conjugate[T]]-3 Log[Z+Conjugate[Z]]\n\t  ",
    FontSize->12,
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S",
    FontSize->12,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["uperpotential: ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a T + b Z",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  with two parameters ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["a, b\n\t  \n",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["This model may be created by evaluating ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[BoxData[
    \(\(M1 = 
        CalcModel[
          CreateModel[{T, Z}, {t, 
              z}, {\[Tau], \[Zeta]}, \(-3\) Log[T + Conjugate[T]] - 
              3  Log[Z + Conjugate[Z]], a\ T + b\ Z, 
            Par \[Rule] {a, b}]];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Regular keywords:",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  "Keyword ",
  StyleBox["CDummies",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\nKeyword for list of replacement rules for complex dummy \
variables introduced to take care of non-polynomial (transcendental) \
expressions.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->{"ComplexToReal", "CDummies"}],

Cell[TextData[{
  "Keyword ",
  StyleBox["ComplexToReal",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\nKeyword for list of rules converting complex to real fields in \
a model list.\nFor M1, this is given as \n",
    FontSize->12],
  StyleBox["{T\[Rule]t + \[ImaginaryI] \[Tau],  Z\[Rule] z + \[ImaginaryI] \
\[Zeta],  Conjugate[T]\[Rule] t - \[ImaginaryI] \[Tau], Conjugate[Z]\[Rule] z \
- \[ImaginaryI] \[Zeta]}",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"ComplexToReal"],

Cell[TextData[{
  "Keyword ",
  StyleBox["DeltaPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for non-superysmmetric addition to scalar potential in a \
model list.\nThe default for this is 0",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"DeltaPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["DPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for list of first derivatives (with respect to the real \
fields as in ",
    FontSize->12],
  StyleBox["RFields",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[") of the potential in a model list.\nFor M1, this contains the \
derivatives of ",
    FontSize->12],
  StyleBox["Pot",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", by ",
    FontSize->12],
  StyleBox["t, \[Tau], z",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["\[Zeta] ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["and is given by\n",
    FontSize->12],
  Cell[BoxData[
      StyleBox[\({t \[Rule] \(-\(\(1\/\(192\ t\^4\ z\^3\)\) \((a\^2\ \((t\^2 \
+ 27\ \[Tau]\^2)\) - 6\ a\ b\ \((2\ t\ z - 9\ \[Tau]\ \[Zeta])\) + 
                    3\ b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\))\)\)\), 
          z \[Rule] \(-\(1\/\(192\ t\^3\ z\^4\)\)\) \((3\ a\^2\ \((t\^2 + 
                        9\ \[Tau]\^2)\) - 
                  6\ a\ b\ \((2\ t\ z - 9\ \[Tau]\ \[Zeta])\) + 
                  b\^2\ \((z\^2 + 
                        27\ \[Zeta]\^2)\))\), \[Tau] \[Rule] \(3\ a\ \((a\ \
\[Tau] + b\ \[Zeta])\)\)\/\(32\ t\^3\ z\^3\), \[Zeta] \[Rule] \(3\ b\ \((a\ \
\[Tau] + b\ \[Zeta])\)\)\/\(32\ t\^3\ z\^3\)}\),
        FontSize->16]],
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"DPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["DSPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Keyword for list of first derivatives (with respect to complex \
fields) of W in model list. \nFor M1, this is given as ",
    FontSize->12],
  StyleBox["{T\[Rule]a, Z\[Rule]b}",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", \ni.e., ",
    FontSize->12],
  Cell[BoxData[
      \(TraditionalForm\`\[PartialD]\_T\((a\ T\  + \ b\ Z)\)\  = \ a\)],
    FontWeight->"Plain"],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[PartialD]\_Z\((a\ T\  + \ b\ Z)\)\  = \ b\)],
    FontWeight->"Plain"],
  " "
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"DSPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["Dummies",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\nKeyword for list of replacement rules for real dummy variables \
introduced to take care of non-polynomial (transcendental) expressions. ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->{"ComplexToReal", "Dummies"}],

Cell[TextData[{
  "Keyword ",
  StyleBox["Fields",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Keyword for list of complex fields in a model list.\nFor M1, this \
is given as ",
    FontSize->12],
  StyleBox["{T, Z}",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Fields"],

Cell[BoxData[{
    RowBox[{
      StyleBox["Keyword",
        FontFamily->"Arial"], 
      StyleBox[" ",
        FontFamily->"Arial"], 
      StyleBox["FTerms",
        FontFamily->"Arial",
        FontWeight->"Plain"]}], "\[IndentingNewLine]", 
    StyleBox[
      RowBox[{
        RowBox[{
        "Keyword", " ", "for", " ", "list", " ", "of", " ", "F", " ", "terms",
           " ", \((split\ into\ real\ and\ imaginary\ parts)\), " ", "in", 
          " ", "a", " ", "model", " ", 
          RowBox[{"list", ".", "    ", "\[IndentingNewLine]", 
            RowBox[{"(", 
              RowBox[{
                
                RowBox[{\(NB . \ This\), " ", "is", " ", "also", " ", "an", 
                  " ", "option", " ", "for", " ", "some", " ", 
                  ButtonBox[\(Stringvacua\ modules\),
                    ButtonStyle->"AddOnsLink"]}], ",", " ", 
                RowBox[{"see", " ", 
                  RowBox[{
                    ButtonBox[\(Module\ Options\),
                      ButtonStyle->"AddOnsLink"], "."}]}]}], ")"}]}]}], " "}],
      
      FontFamily->"Arial",
      FontSize->12], "\[IndentingNewLine]", 
    RowBox[{
      StyleBox[" ",
        FontFamily->"Arial",
        FontSize->12], 
      RowBox[{
        StyleBox[
          StyleBox[
            RowBox[{
              StyleBox["R",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["ecall",
                FontFamily->"Arial",
                FontSize->12]}]],
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[",",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[\(for\ SuGRA\ theories\),
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[",",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Arial",
          FontSize->12], 
        RowBox[{
          StyleBox[
            RowBox[{
              StyleBox["the",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["F\[Dash]term",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["  ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["associated",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["with",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["field",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["T",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["for",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["Kahler",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["potential",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["K",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["and",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["superpotential",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["W",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["is",
                FontFamily->"Arial"], 
              StyleBox[" ",
                FontFamily->"Arial"], 
              StyleBox[\(F\_T\),
                FontFamily->"Arial"]}],
            FontFamily->"Arial",
            FontSize->12], 
          StyleBox["=",
            FontFamily->"Arial",
            FontSize->12], 
          RowBox[{
            StyleBox[\([\[PartialD]\_T\(+\((\[PartialD]\_T K)\)\)]\),
              FontFamily->"Arial",
              FontSize->12], 
            RowBox[{
              StyleBox["W",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox[".",
                FontFamily->"Arial"], 
              StyleBox["\[IndentingNewLine]",
                FontFamily->"Arial",
                FontSize->12], 
              StyleBox["For",
                FontFamily->"Arial",
                FontSize->12]}], 
            StyleBox[" ",
              FontFamily->"Arial",
              FontSize->12], 
            StyleBox["M1",
              FontFamily->"Arial",
              FontSize->12]}]}], 
        StyleBox[",",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Arial",
          FontSize->12], 
        RowBox[{
          StyleBox["this",
            FontFamily->"Arial",
            FontSize->12], 
          StyleBox[" ",
            FontFamily->"Arial",
            FontSize->12], 
          StyleBox["is",
            FontFamily->"Arial",
            FontSize->12], 
          StyleBox[" ",
            FontFamily->"Arial",
            FontSize->12], 
          StyleBox["given",
            FontFamily->"Arial",
            FontSize->12], 
          StyleBox[" ",
            FontFamily->"Arial",
            FontSize->12], 
          StyleBox["by",
            FontFamily->"Arial",
            FontSize->12], 
          StyleBox[" ",
            FontFamily->"Arial",
            FontSize->12], 
          
          StyleBox[\({T \[Rule] a - \(3\ \((a\ T + b\ Z)\)\)\/\(2\ t\), 
              Z \[Rule] b - \(3\ \((a\ T + b\ Z)\)\)\/\(2\ z\)}\),
            FontFamily->"Arial",
            FontSize->12,
            FontWeight->"Plain"]}]}]}], "\[IndentingNewLine]", 
    RowBox[{"\t\t"}]}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"FTerms"],

Cell[TextData[{
  "Keyword ",
  StyleBox["Hesse",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for matrix which contains Hessian (with respect to real \
fields as in ",
    FontSize->12],
  StyleBox["RFields",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[") in a model list.\nFor M1, this is 4 x 4 matrix of second \
derivatives of ",
    FontSize->12],
  StyleBox["Pot",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", with respect to ",
    FontSize->12],
  StyleBox["t, \[Tau], z",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["\[Zeta] ",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Hesse"],

Cell[TextData[{
  "Keyword ",
  StyleBox["ImFields",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for list of imaginary parts of fields in a model list.\n\
For M1, this is given as ",
    FontSize->12],
  StyleBox["{\[Tau], \[Zeta]}",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"ImFields"],

Cell[TextData[{
  "Keyword ",
  StyleBox["KMetric",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for the Kahler metric in a model list. Recall that the \
standard SuGRA Kahler metric is given as  ",
    FontSize->12],
  Cell[BoxData[
      \(TraditionalForm\`K\_\(A  B\&_\)\)]],
  "= ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[PartialD]\_A\),
            "TraditionalForm"], \(\[PartialD]\_B\&_ K\)}], 
        TraditionalForm]]],
  ".\n",
  StyleBox["For M1, this is given by ",
    FontSize->12],
  Cell[BoxData[
      \({{3\/\(\(\((T\  + \ Conjugate[T])\)\^2\)\(\ \)\), 0}, {0, 
          3\/\(\(\((Z\  + \ Conjugate[Z])\)\^2\)\(\ \)\)}}\)]],
  " ",
  StyleBox["= ",
    FontSize->12],
  Cell[BoxData[
      \({{3\/\(4\ t\^2\), 0}, {0, 3\/\(4\ z\^2\)}}\)]]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"KMetric"],

Cell[TextData[{
  "Keyword ",
  StyleBox["KPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for Kahler potential in a model list.\nFor M1, this is \
given by ",
    FontSize->12],
  StyleBox["-3 Log[T+Conjugate[T]] - 3 Log[Z+Conjugate[Z]]",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"KPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["Par",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Keyword for list of parameters in a model list. Also one of the ",
    
    FontFamily->"Arial",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[ButtonBox["Ideal Keywords",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[ButtonBox["option for some Stringvacua modules",
    ButtonData:>"Module Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Arial",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\nFor M1, this is given by ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["{a, b}",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["\t",
    FontFamily->"Arial",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Par"],

Cell[TextData[{
  "Keyword ",
  StyleBox["Pot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for scalar potential in a model list.\nRecall for SuGRA \
theories, the standard scalar potential is obtained as\n\tV = ",
    FontSize->12],
  Cell[BoxData[
      \(TraditionalForm\`\(e\^W\)\ [\ \(K\^\(A  B\&_\)\) \(D\_A\) 
              W\ \(D\_B\&_\) W\  - \ 3 | W\( | \^2\)]\)],
    FontSize->12],
  StyleBox[" where ",
    FontSize->12],
  Cell[BoxData[
      \(TraditionalForm\`K\^\(A  B\&_\)\)],
    FontSize->12],
  StyleBox[" is the inverse of the Kahler metric ",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[PartialD]\_A\),
            "TraditionalForm"], \(\[PartialD]\_B\&_ K\)}], 
        TraditionalForm]]],
  StyleBox["\nFor M1, this is given by ",
    FontSize->12],
  Cell[BoxData[
      \(\(a\^2\ \((t\^2 + 9\ \[Tau]\^2)\) - \ 6\ a\ b\ \((t\ z - 3\ \[Tau]\ \
\[Zeta])\) + \ b\^2\ \((z\^2 + 9\ \[Zeta]\^2)\)\)\/\(192\ t\^3\ z\^3\)\)],
    FontSize->18,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Pot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["RDummies",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\nKeyword for list of replacement rules for all dummy variables \
introduced to take care of non-polynomial (transcendental) expressions. ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->{"ComplexToReal", "RDummies"}],

Cell[TextData[{
  "Keyword ",
  StyleBox["ReFields",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for list of real parts of fields in a model list.\nFor \
M1, this is given by ",
    FontSize->12],
  StyleBox["{t, z}",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"ReFields"],

Cell[TextData[{
  "Keyword ",
  StyleBox["RFields",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for list of all real fields (real and imaginary parts) in \
a model list.\nFor M1, this is given by ",
    FontSize->12],
  StyleBox["{t, z, \[Tau], \[Zeta]}",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["\t",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"RFields"],

Cell[TextData[{
  "Keyword ",
  StyleBox["SPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for superpotential in a model list.\nFor M1, this is \
given by ",
    FontSize->12],
  StyleBox["a T + b Z",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SPot"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Keywords assigned to ideal lists:",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  "Keyword ",
  StyleBox["IDPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for ideal list containing the ideal generated by \
derivatives of scalar potential.\nFor M1, comparing with ",
    FontSize->12],
  StyleBox["DPot",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", this is given by \t  \n\t ",
    FontSize->12],
  Cell[BoxData[
      \({Var \[Rule] {t, z, \[Tau], \[Zeta]}, 
        Par \[Rule] {a, 
            b}, \[IndentingNewLine]Gen \[Rule] {t \[Rule] \(-a\^2\)\ t\^2 + 
                12\ a\ b\ t\ z - 3\ b\^2\ z\^2 - 27\ b\^2\ \[Zeta]\^2 - 
                54\ a\ b\ \[Zeta]\ \[Tau] - 27\ a\^2\ \[Tau]\^2, 
            z \[Rule] \(\(-3\)\ a\^2\ t\^2 + 12\ a\ b\ t\ z - b\^2\ z\^2 - 
                27\ b\^2\ \[Zeta]\^2 - 54\ a\ b\ \[Zeta]\ \[Tau] - 
                27\ a\^2\ \(\[Tau]\^2\)\^\(\(\[IndentingNewLine]\)\(,\)\)\
\[IndentingNewLine]
              \[Tau] \[Rule] 
                3\ \((a\ b\ \[Zeta] + a\^2\ \[Tau])\)\), \[Zeta] \[Rule] 
              3\ \((b\^2\ \[Zeta] + a\ b\ \[Tau])\)}}\)],
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"IDPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["IDSPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for ideal list generated by dW (complex fields).\nFor \
M1,comparing with ",
    FontSize->12],
  StyleBox["DSPot",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[",  this is given by ",
    FontSize->12],
  StyleBox["{Var\[Rule]{T,Z}, Par\[Rule]{a,b}, Gen\[Rule]{a, b}}\n",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["i.e., the polynomial ideal associated with dW, the partial \
derivatives of the superpotential, has complex variables ",
    FontSize->12],
  StyleBox["{T, Z} ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["with generators ",
    FontSize->12],
  StyleBox["dW = {a, b}",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" and parameters ",
    FontSize->12],
  StyleBox["{a, b}",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"IDSPot"],

Cell[BoxData[{
    RowBox[{
      StyleBox["Keyword",
        FontFamily->"Arial"], 
      StyleBox[" ",
        FontFamily->"Arial"], 
      StyleBox["IFTerms",
        FontFamily->"Arial",
        FontWeight->"Plain"]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        StyleBox["Keyword",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox["for",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox["ideal",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox["list",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox["generated",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox["by",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox["the",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox["F",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox["terms",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontSize->12], 
        StyleBox[
          RowBox[{
            RowBox[{
              StyleBox["(",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["with",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["respect",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["to",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["the",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["  ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["real",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["field",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["components",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["in",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["RFields",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain"]}], 
              StyleBox[")",
                FontFamily->"Helvetica",
                FontSize->12]}], 
            StyleBox[".",
              FontFamily->"Helvetica",
              FontSize->12], 
            StyleBox[" ",
              FontFamily->"Helvetica",
              FontSize->12], 
            StyleBox["\[IndentingNewLine]",
              FontFamily->"Helvetica",
              FontSize->12], "For"}],
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox["M1",
          FontFamily->"Arial",
          FontSize->12]}], 
      StyleBox[",",
        FontFamily->"Arial",
        FontSize->12], 
      StyleBox[\(comparing\ with\ FTerms\),
        FontFamily->"Arial",
        FontSize->12], 
      StyleBox[",",
        FontFamily->"Arial",
        FontSize->12], 
      StyleBox[" ",
        FontFamily->"Arial",
        FontSize->12], 
      StyleBox[\(this\ is\ given\ by\),
        FontFamily->"Arial",
        FontSize->12]}], "\[IndentingNewLine]", 
    StyleBox[\({Var \[Rule] {t, z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
        Gen \[Rule] {t \[Rule] \(-a\)\ t - 3\ b\ z, 
            z \[Rule] \(-3\)\ a\ t - 
                b\ z, \[Tau] \[Rule] \(-3\)\ \((a\ \[Tau] + 
                    b\ \[Zeta])\), \[Zeta] \[Rule] \(-3\)\ \((a\ \[Tau] + 
                    b\ \[Zeta])\)}}\),
      FontSize->12,
      FontWeight->"Plain"], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
      "There", " ", "are", " ", "4", " ", "field", " ", "components", " ", 
        StyleBox[\({t, z, \[Tau], \[Zeta]}\),
          FontWeight->"Plain"], 
        StyleBox[" ",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox["with",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox["parameters",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[" ",
          FontFamily->"Arial",
          FontSize->12], 
        StyleBox[\({a, b}\),
          FontFamily->"Arial",
          FontSize->12,
          FontWeight->"Plain"]}], 
      StyleBox[",",
        FontFamily->"Arial",
        FontSize->12,
        FontWeight->"Plain"], 
      StyleBox[\(with\ the\ vanishing\ of\ each\ of\ the\ 2\ Fterms\ above\ \
giving\ 2\ generators\ for\ the\ real\ and\ imaginary\ parts\),
        FontFamily->"Arial",
        FontSize->12]}]}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"IFTerms"],

Cell[TextData[{
  "Keyword ",
  StyleBox["IImSPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for ideal list containing the ideal generated by Im(W) in \
a model list.\nFor M1, this is given by ",
    FontSize->12],
  StyleBox["{Var\[Rule]{t,z,\[Tau],\[Zeta]}, Par\[Rule]{a,b}, Gen\[Rule]a \
\[Tau]+b \[Zeta]} ",
    FontWeight->"Plain"],
  StyleBox["since",
    FontSize->12],
  " ",
  StyleBox["Im(W) = a \[Tau] + b \[Zeta]",
    FontWeight->"Plain"],
  StyleBox["\t",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"IImSPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["IPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for ideal list containing the ideal generated by scalar \
potential.\nFor M1, comparing with ",
    FontSize->12],
  StyleBox["Pot",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", this is given by\n",
    FontSize->12],
  StyleBox["\t",
    FontSize->12,
    FontWeight->"Plain"],
  Cell[BoxData[
      \({Var \[Rule] {t, z, \[Tau], \[Zeta]}, Par \[Rule] {a, b}, 
        Gen \[Rule] 
          a\^2\ t\^2 - 6\ a\ b\ t\ z + b\^2\ z\^2 + 9\ b\^2\ \[Zeta]\^2 + 
            18\ a\ b\ \[Zeta]\ \[Tau] + 9\ a\^2\ \[Tau]\^2}\)],
    FontWeight->"Plain"],
  "\n",
  StyleBox["we see the generator takes only the numerator as we do not allow \
field values to tend to infinity, which correspond to runaway behaviour",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"IPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["IReImDSPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for ideal list containing the ideal generated by the \
derivatives of W.\nFor M1, this is given by ",
    FontSize->12],
  StyleBox[" {Var\[Rule]{t,z,\[Tau],\[Zeta]}, Par\[Rule]{a,b}, Gen\[Rule]{t\
\[Rule]a, z\[Rule]b, \[Tau]\[Rule]0, \[Zeta]\[Rule]0}}\n",
    FontWeight->"Plain"],
  StyleBox["The derivatives are taken with respect to the",
    FontSize->12],
  StyleBox[" RFields {t,z,\[Tau],\[Zeta]} ",
    FontWeight->"Plain"],
  StyleBox["and the real part is set to vanish",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"IReIMDSPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["IReSPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Keyword for ideal list containing the ideal generated by Re(W) in \
a model list.\nFor M1, this is given by ",
    FontSize->12],
  StyleBox["{Var\[Rule]{t,z,\[Tau],\[Zeta]}, Par\[Rule]{a,b}, Gen\[Rule]a t+b \
z} ",
    FontWeight->"Plain"],
  StyleBox["since",
    FontSize->12],
  " ",
  StyleBox["Re(W) = a t + b z",
    FontWeight->"Plain"],
  StyleBox["\t",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"IReSPot"],

Cell[TextData[{
  "Keyword ",
  StyleBox["ISPot",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\nKeyword for ideal list generated by the superpotential W \
(complex fields).\nFor M1, comparing with ",
    FontSize->12],
  StyleBox["SPot",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", this is given by ",
    FontSize->12],
  StyleBox["{Var\[Rule]{T,Z}, Par\[Rule]{a,b}, Gen\[Rule]{a T+b Z}}\n",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["i.e., the polynomial ideal associated with W has complex \
variables ",
    FontSize->12],
  StyleBox["{T, Z} ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["with a single generator ",
    FontSize->12],
  StyleBox["{W = a T + b Z}",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["  and parameters ",
    FontSize->12],
  StyleBox["{a, b}",
    FontSize->12,
    FontWeight->"Plain"]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"ISPot"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
ScreenStyleEnvironment->"Working",
WindowToolbars->"EditBar",
WindowSize->{879, 683},
WindowMargins->{{Automatic, 58}, {Automatic, -24}},
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "ComplexToReal"->{
    Cell[24129, 728, 415, 12, 104, "Subsection",
      CellTags->{"ComplexToReal", "CDummies"}],
    Cell[24547, 742, 600, 16, 121, "Subsection",
      CellTags->"ComplexToReal"],
    Cell[27842, 850, 409, 11, 104, "Subsection",
      CellTags->{"ComplexToReal", "Dummies"}],
    Cell[40777, 1286, 410, 11, 104, "Subsection",
      CellTags->{"ComplexToReal", "RDummies"}]},
  "CDummies"->{
    Cell[24129, 728, 415, 12, 104, "Subsection",
      CellTags->{"ComplexToReal", "CDummies"}]},
  "DeltaPot"->{
    Cell[25150, 760, 366, 12, 104, "Subsection",
      CellTags->"DeltaPot"]},
  "DPot"->{
    Cell[25519, 774, 1541, 46, 177, "Subsection",
      CellTags->"DPot"]},
  "DSPot"->{
    Cell[27063, 822, 776, 26, 122, "Subsection",
      CellTags->"DSPot"]},
  "Dummies"->{
    Cell[27842, 850, 409, 11, 104, "Subsection",
      CellTags->{"ComplexToReal", "Dummies"}]},
  "Fields"->{
    Cell[28254, 863, 430, 16, 104, "Subsection",
      CellTags->"Fields"]},
  "FTerms"->{
    Cell[28687, 881, 6969, 215, 186, "Subsection",
      CellTags->"FTerms"]},
  "Hesse"->{
    Cell[35659, 1098, 847, 32, 104, "Subsection",
      CellTags->"Hesse"]},
  "ImFields"->{
    Cell[36509, 1132, 428, 15, 104, "Subsection",
      CellTags->"ImFields"]},
  "KMetric"->{
    Cell[36940, 1149, 939, 32, 112, "Subsection",
      CellTags->"KMetric"]},
  "KPot"->{
    Cell[37882, 1183, 432, 15, 104, "Subsection",
      CellTags->"KPot"]},
  "Par"->{
    Cell[38317, 1200, 1276, 46, 104, "Subsection",
      CellTags->"Par"]},
  "Pot"->{
    Cell[39596, 1248, 1178, 36, 152, "Subsection",
      CellTags->"Pot"]},
  "RDummies"->{
    Cell[40777, 1286, 410, 11, 104, "Subsection",
      CellTags->{"ComplexToReal", "RDummies"}]},
  "ReFields"->{
    Cell[41190, 1299, 412, 15, 104, "Subsection",
      CellTags->"ReFields"]},
  "RFields"->{
    Cell[41605, 1316, 485, 17, 104, "Subsection",
      CellTags->"RFields"]},
  "SPot"->{
    Cell[42093, 1335, 393, 15, 104, "Subsection",
      CellTags->"SPot"]},
  "IDPot"->{
    Cell[42636, 1359, 1227, 31, 225, "Subsection",
      CellTags->"IDPot"]},
  "IDSPot"->{
    Cell[43866, 1392, 1023, 36, 138, "Subsection",
      CellTags->"IDSPot"]},
  "IFTerms"->{
    Cell[44892, 1430, 6454, 201, 170, "Subsection",
      CellTags->"IFTerms"]},
  "IImSPot"->{
    Cell[51349, 1633, 641, 22, 105, "Subsection",
      CellTags->"IImSPot"]},
  "IPot"->{
    Cell[51993, 1657, 980, 30, 155, "Subsection",
      CellTags->"IPot"]},
  "IReIMDSPot"->{
    Cell[52976, 1689, 752, 21, 123, "Subsection",
      CellTags->"IReIMDSPot"]},
  "IReSPot"->{
    Cell[53731, 1712, 619, 22, 105, "Subsection",
      CellTags->"IReSPot"]},
  "ISPot"->{
    Cell[54353, 1736, 999, 35, 121, "Subsection",
      CellTags->"ISPot"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"ComplexToReal", 56082, 1793},
  {"CDummies", 56473, 1802},
  {"DeltaPot", 56587, 1805},
  {"DPot", 56678, 1808},
  {"DSPot", 56767, 1811},
  {"Dummies", 56858, 1814},
  {"Fields", 56969, 1817},
  {"FTerms", 57060, 1820},
  {"Hesse", 57152, 1823},
  {"ImFields", 57245, 1826},
  {"KMetric", 57340, 1829},
  {"KPot", 57431, 1832},
  {"Par", 57518, 1835},
  {"Pot", 57605, 1838},
  {"RDummies", 57697, 1841},
  {"ReFields", 57812, 1844},
  {"RFields", 57907, 1847},
  {"SPot", 57998, 1850},
  {"IDPot", 58087, 1853},
  {"IDSPot", 58179, 1856},
  {"IFTerms", 58273, 1859},
  {"IImSPot", 58369, 1862},
  {"IPot", 58460, 1865},
  {"IReIMDSPot", 58554, 1868},
  {"IReSPot", 58651, 1871},
  {"ISPot", 58743, 1874}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 114, 3, 76, "Title"],
Cell[1844, 57, 85, 2, 31, "Subtitle"],
Cell[1932, 61, 2154, 73, 65, "Text"],
Cell[4089, 136, 17425, 502, 795, "Subsection"],

Cell[CellGroupData[{
Cell[21539, 642, 137, 3, 31, "Subtitle"],
Cell[21679, 647, 2060, 64, 141, "Text"],
Cell[23742, 713, 253, 6, 65, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24032, 724, 94, 2, 31, "Subtitle"],
Cell[24129, 728, 415, 12, 104, "Subsection",
  CellTags->{"ComplexToReal", "CDummies"}],
Cell[24547, 742, 600, 16, 121, "Subsection",
  CellTags->"ComplexToReal"],
Cell[25150, 760, 366, 12, 104, "Subsection",
  CellTags->"DeltaPot"],
Cell[25519, 774, 1541, 46, 177, "Subsection",
  CellTags->"DPot"],
Cell[27063, 822, 776, 26, 122, "Subsection",
  CellTags->"DSPot"],
Cell[27842, 850, 409, 11, 104, "Subsection",
  CellTags->{"ComplexToReal", "Dummies"}],
Cell[28254, 863, 430, 16, 104, "Subsection",
  CellTags->"Fields"],
Cell[28687, 881, 6969, 215, 186, "Subsection",
  CellTags->"FTerms"],
Cell[35659, 1098, 847, 32, 104, "Subsection",
  CellTags->"Hesse"],
Cell[36509, 1132, 428, 15, 104, "Subsection",
  CellTags->"ImFields"],
Cell[36940, 1149, 939, 32, 112, "Subsection",
  CellTags->"KMetric"],
Cell[37882, 1183, 432, 15, 104, "Subsection",
  CellTags->"KPot"],
Cell[38317, 1200, 1276, 46, 104, "Subsection",
  CellTags->"Par"],
Cell[39596, 1248, 1178, 36, 152, "Subsection",
  CellTags->"Pot"],
Cell[40777, 1286, 410, 11, 104, "Subsection",
  CellTags->{"ComplexToReal", "RDummies"}],
Cell[41190, 1299, 412, 15, 104, "Subsection",
  CellTags->"ReFields"],
Cell[41605, 1316, 485, 17, 104, "Subsection",
  CellTags->"RFields"],
Cell[42093, 1335, 393, 15, 104, "Subsection",
  CellTags->"SPot"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42523, 1355, 110, 2, 31, "Subtitle"],
Cell[42636, 1359, 1227, 31, 225, "Subsection",
  CellTags->"IDPot"],
Cell[43866, 1392, 1023, 36, 138, "Subsection",
  CellTags->"IDSPot"],
Cell[44892, 1430, 6454, 201, 170, "Subsection",
  CellTags->"IFTerms"],
Cell[51349, 1633, 641, 22, 105, "Subsection",
  CellTags->"IImSPot"],
Cell[51993, 1657, 980, 30, 155, "Subsection",
  CellTags->"IPot"],
Cell[52976, 1689, 752, 21, 123, "Subsection",
  CellTags->"IReIMDSPot"],
Cell[53731, 1712, 619, 22, 105, "Subsection",
  CellTags->"IReSPot"],
Cell[54353, 1736, 999, 35, 121, "Subsection",
  CellTags->"ISPot"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

