(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     50704,       1696]*)
(*NotebookOutlinePosition[     53933,       1803]*)
(*  CellTagsIndexPosition[     53311,       1778]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Module Options",
  FontSize->24]], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[StyleBox["Overview",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox[ButtonBox["Stringvacua Modules",
    ButtonData:>"Stringvacua Modules",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" accept a number of module-specific optional arguments which are \
provided in the usual way by adding an assignment ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["optionname \[Rule] ",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["value",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  to the module call. The default settings for these options are \
specific to the particular module and can be obtained from ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Options[", "Input",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["modulename", "Input",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". In addition, all global  ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonData:>"Stringvacua Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" can be provided as arguments in module calls to change their \
setting temporarily for the specific call.  The following appear as  \
module-specific optional parameters used in the arguments of modules in this \
package.",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  CellDingbat->None,
  FontFamily->"Helvetica",
  FontSize->14,
  FontWeight->"Bold"],

Cell[TextData[{
  StyleBox["List of module-specific Options ",
    FontFamily->"Arial",
    FontSize->18],
  StyleBox["\n\n",
    FontSize->18],
  Cell[BoxData[GridBox[{
          {
            RowBox[{
              ButtonBox["CDummies",
                ButtonData:>"Dummies",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            
            StyleBox[\(list\ with\ entries' expr \[Rule] 
                complexdummyvariable'\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["DeltaPot",
              ButtonData:>DeltaPot,
              Active->True,
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            
            StyleBox[\(non\[Dash]supersymmetric\ addition\ to\ the\ potential\
\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["Dim",
                ButtonData:>"Dim",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            
            StyleBox[\(\((list\ of)\)\ integer \((s)\), 
              ideals\ with\ these\ complex\ dimensions\ are\ kept\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["Dummies",
                ButtonData:>"Dummies",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            
            StyleBox[\(list\ with\ entries' expr \[Rule] realdummyvariable'\),
              
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["FTerms",
                ButtonData:>"FTerms",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            
            StyleBox[\(\((list\ of)\)\ real\ field \((s)\)\ specifying\ \
FTerms\ of\ a\ model\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["KMetricInverse",
              ButtonData:>KMetricInverse,
              Active->True,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            
            StyleBox[\(user\ specification\ of\ the\ inverse\ Kahler\ \
metric\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["Meth",
              ButtonData:>Meth,
              Active->True,
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            StyleBox[\(a\ string\ specifying\ a\ particular\ method\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["NRoots",
                ButtonData:>"NRoots",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            
            StyleBox[\(Integer, 
              minimal\ number\ of\ real\ roots\ for\ an\ ideal\ to\ be\ \
kept\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["Par",
                ButtonData:>"Par",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            StyleBox[\(\((a\ list\ of)\)\ parameter \((s)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["Poly",
                ButtonData:>"Poly",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            
            StyleBox[\(\((list\ of)\)\ polynomial \((s)\)\ or\ ideal\ list \
\((s)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["RDummies",
              ButtonData:>"Dummies",
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            StyleBox[\(list\ of\ replacements\ of\ all\ dummy\ variables\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["RRoots",
                ButtonData:>"RRoots",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            StyleBox[
              RowBox[{
                RowBox[{
                  RowBox[{
                    StyleBox["\"\<\>",
                      ShowStringCharacters->True], 
                    StyleBox["Probab",
                      ShowStringCharacters->True,
                      FontWeight->"Plain"], 
                    StyleBox["\"\<\>",
                      ShowStringCharacters->True]}], 
                  StyleBox[",",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True]}], 
                StyleBox["Determ",
                  ShowStringCharacters->True,
                  FontWeight->"Plain"], 
                StyleBox["\"\< or \>\"",
                  ShowStringCharacters->True], 
                StyleBox["Off",
                  ShowStringCharacters->True,
                  FontWeight->"Plain"], 
                StyleBox["\"\<, calculate real roots or not\>",
                  ShowStringCharacters->True]}],
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox[\(\(SturmLabel\)\(\ \ \)\),
              ButtonData:>"SturmLabel",
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            
            StyleBox[\(a\ string\ labelling\ the\ output\ of\ various\ sign\ \
\(\(tests\)\(.\)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["SturmPoly",
              ButtonData:>"SturmPoly",
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            
            StyleBox[\(\((list\ of)\)\ polynomial \((s)\)\ for\ filtering\ \
with\ Sturm\ queries\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["SturmSigns",
              ButtonData:>"SturmSigns",
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            
            StyleBox[\(\((list\ of)\)\ sign \((s)\)\ for\ filtering\ with\ \
Sturm\ queries\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox[\(\(TestLambda\)\(\ \ \)\),
                ButtonData:>"TestLambda",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            StyleBox[
              RowBox[{
                RowBox[{
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True], 
                  StyleBox["yes",
                    ShowStringCharacters->True,
                    FontWeight->"Plain"], 
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["or",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True]}], 
                StyleBox["no",
                  ShowStringCharacters->True,
                  FontWeight->"Plain"], 
                
                StyleBox[
                  "\"\< to switch Sturm query on scalar potential on or \
off\>",
                  ShowStringCharacters->True]}],
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["TestMatDet",
              ButtonData:>"TestMatDet",
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            
            StyleBox[\(\((list\ of)\)\ rational\ matrix \((ces)\)\ for\ Sturm\
\ query\ on\ determinant \((s)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["TestMatEigen",
              ButtonData:>"TestMatEigen",
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            
            StyleBox[\(rational\ matrix\ with\ real\ eigenvalues\ for\ Sturm\ \
query\ on\ Eigenvalues\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["TestRational",
                ButtonData:>"TestRational",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            
            StyleBox[\(\((list\ of)\)\ rational\ functions\ for\ Sturm\ query\
\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox[\(\(TestStability\)\(\ \ \)\),
              ButtonData:>"TestStability",
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            StyleBox[
              RowBox[{
                RowBox[{
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True], 
                  StyleBox["Hesse",
                    ShowStringCharacters->True,
                    FontWeight->"Plain"], 
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["to",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["perform",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["stability",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["test",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True]}], 
                StyleBox["Off",
                  ShowStringCharacters->True,
                  FontWeight->"Plain"], 
                StyleBox["\"\<to not to do so\>",
                  ShowStringCharacters->True]}],
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            ButtonBox["Var",
              ButtonData:>Var,
              Active->True,
              ButtonExpandable->False,
              ButtonStyle->"Hyperlink"], "\[Rule]", 
            StyleBox[\(\((a\ list\ of)\)\ variable \((s)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            RowBox[{
              ButtonBox["VarPar",
                ButtonData:>"VarPar",
                ButtonStyle->"Hyperlink"], " "}], "\[Rule]", 
            
            StyleBox[\(list\ of\ \((assignments\ of)\)\ variables/
                  parameters\ \((to\ values)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]}
          }]],
    TextJustification->0,
    FontFamily->"Arial",
    GridBoxOptions->{ColumnAlignments->{Left}}]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[StyleBox["Detailed description of module specific options",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  "Option ",
  StyleBox["Dummies, CDummies, RDummies",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Option (see module ",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        ButtonBox["CalcModel",
          ButtonStyle->"AddOnsLink"], TraditionalForm]]],
  StyleBox[") for replacement rules when introducing dummy variables for \
non-perturbative (i.e., non-polynomial) theories.\n(see also the ",
    FontSize->12],
  ButtonBox["Non Perturbative",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" example from the Examples tab).\nIn a theory with a model list \
called model, which has two fields\n",
    FontSize->12],
  Cell[BoxData[
      \(TraditionalForm\`T = \ \ t\  + \ \[ImaginaryI]\ \[Tau]\ , \ 
      S = s + \[ImaginaryI]\ \[Sigma]\)],
    FontFamily->"Arial"],
  "\n",
  StyleBox["and a non-perturbative superpotential\n",
    FontSize->12],
  StyleBox["W = ",
    FontFamily->"Arial",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox["T",
            "TraditionalForm"], "+", \(Exp[\(-S\)]\)}], TraditionalForm]],
    FontFamily->"Arial",
    FontSize->12],
  ",",
  StyleBox["\n",
    FontSize->10],
  StyleBox["we can specify the dummy variables as follows:",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Dummies"],

Cell[BoxData[
    \(\(CalcModel[model, 
        Dummies \[Rule] {Exp[\(-s\)] \[Rule] x, 
            Exp[\(-I\) \[Sigma]] \[Rule] y}, 
        CDummies \[Rule] {Exp[\(-S\)] \[Rule] X, 
            Exp[\(-Conjugate[S]\)] \[Rule] Y}];\)\)], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["DeltaPot",
    FontWeight->"Plain"],
  "\n",
  StyleBox["Specifies a non-supersymmetric addition to the supergravity \
potential, such as a 'raising term'. \nUsage: ",
    FontSize->12],
  StyleBox["CreateModel[{T},{t},{\[Tau]},-3Log[T+Conjugate[T]],1+2 \
T,DeltaPot->{1/t^9}] ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["creates a model list for a simple supergravity theory and adds ",
    FontSize->12],
  StyleBox["t^(-9) ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["to the scalar potential",
    FontSize->12],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[".",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"DeltaPot"],

Cell[TextData[{
  "Option ",
  StyleBox["Dim",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Option specifying a (list of) complex dimension(s) of ideals to \
be kept as output for some Stringvacua modules. \nUsage: ",
    FontSize->12],
  StyleBox["Dim \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["dimlist ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["dimlist ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is a single integer or a list of integers (see modules  ",
    FontSize->12],
  ButtonBox["PrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["Saturation",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["SaturationPrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Dim"],

Cell[TextData[{
  StyleBox["For example, if we performed a primary decomposition of some \
ideal called ",
    FontSize->14],
  StyleBox["Id",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox[" and wished to keep only dimension 0,1 and 2 pieces:",
    FontSize->14]
}], "Text"],

Cell[BoxData[
    \(PrimDec[Id, Dim \[Rule] {0, 1, 2}]\)], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["FTerms",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Option to specify which F terms are saturated out in ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". \nUsage: ",
    FontSize->12],
  StyleBox["FTerms \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ftermlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["ftermlist ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is a single integer ",
    FontSize->12],
  StyleBox["n",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" , a real field, a list with a subset of the real fields or a \
list with a subset of the real fields and integers",
    FontSize->12],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" (see module ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"FTerms"],

Cell[TextData[{
  StyleBox["For example, if we had some supergravity model list called ",
    FontSize->14],
  StyleBox["model ",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox["which has a modulus field called ",
    FontSize->14],
  StyleBox["t",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox[", and we wished to set the F\[Dash]term corresponding to t to 0 \
in a saturation expansion, we can do",
    FontSize->14]
}], "Text"],

Cell[BoxData[
    StyleBox[\(SaturationExpand[model, FTerms \[Rule] \ t]\),
      "Input",
      FontSize->12,
      FontVariations->{"CompatibilityType"->0}]], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["KMetricInverse",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Optional argument to specify inverse Kahler metric in module ",
    FontSize->12],
  ButtonBox["CreateModel",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". \nIf not provided the module will attempt to calculate the \
inverse of the Kahler metric itself. \nUsage: ",
    FontSize->12],
  StyleBox["KMetricInverse \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["mat",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["mat",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  is a matrix which is the inverse Kahler metric.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"KMetricInverse"],

Cell[TextData[{
  "Option ",
  StyleBox["Meth",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["An option to specify the particular method or algorithm to be \
used by Singular.\nUsage: ",
    FontSize->12],
  StyleBox["Meth \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["string ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["string ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is a string whose possible values depend on the module (see \
modules  ",
    FontSize->12],
  ButtonBox["Elimination",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",   ",
    FontSize->12],
  ButtonBox["CreateModel",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", and   ",
    FontSize->12],
  ButtonBox["NumRoots",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" ).",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Meth"],

Cell[TextData[{
  StyleBox["For example, if we wished to find the number of real roots to a \
given ideal ",
    FontSize->14],
  StyleBox["Id",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox[", there are two methods: probabilistic and deterministic, and we \
can choose amongst the two:",
    FontSize->14]
}], "Text"],

Cell[BoxData[
    \(NumRoots[Id, Meth \[Rule] "\<Probab\>"]\)], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["NRoots",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Option to specify the minimal number of real roots required for \
an zero dimensional ideal to appear in a result. \nUsage: ",
    FontSize->12],
  StyleBox["NRoots \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["int ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where  ",
    FontSize->12],
  StyleBox["int ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is an integer (see modules  ",
    FontSize->12],
  ButtonBox["PrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["Saturation",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["SaturationPrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"NRoots"],

Cell[TextData[StyleBox["For example, if we wished to primary decompose an \
ideal Id, and we wanted to keep only those primary ideals which had a number \
of real roots larger than 1, we could evaluate the following.",
  FontSize->14]], "Text"],

Cell[BoxData[
    \(PrimDec[Id, NRoots \[Rule] \ 1]\)], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["Par",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Specifies a list of parameters. \nUsage: ",
    FontSize->12],
  StyleBox["Par \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["parlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["parlist ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is a parameter or a list of parameters (see modules ",
    FontSize->12],
  ButtonBox["CreateModel",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["CreateIdeal",
    ButtonStyle->"AddOnsLink"],
  StyleBox[").",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Par"],

Cell[TextData[StyleBox["For example, we can create an ideal \
\[LeftAngleBracket]x^2-a\[RightAngleBracket] which has a variable x and a \
parameter a as follows.",
  FontSize->14]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CreateIdeal[{x^2 - a}, Var \[Rule] {x}, Par \[Rule] {a}]\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \({Var \[Rule] {x}, Par \[Rule] {a}, 
      Gen \[Rule] {\(-a\) + x\^2}}\)], "Output",
  CellLabel->"Out[54]="]
}, Open  ]],

Cell[TextData[{
  "Option ",
  StyleBox["Poly",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Option to specify polynomials for various, module dependent, \
purposes. \nUsage: ",
    FontSize->12],
  StyleBox["Poly \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["polylist ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["polylist ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is a (list of) polynomial(s) or a (list of ) ideal list(s) (see \
modules ",
    FontSize->12],
  ButtonBox["Saturation",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ",
    FontSize->12],
  ButtonBox["SaturationPrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox[").",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Poly"],

Cell[TextData[{
  StyleBox["For example, if we wished to saturate the ideal \
\[LeftAngleBracket]x^2 - 1\[RightAngleBracket] by the condition x \[NotEqual] \
-1, then we simply set ",
    FontSize->14],
  StyleBox["Poly ",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox["to x +1 ",
    FontSize->14]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Saturation[CreateIdeal[{x^2 - 1}, Var \[Rule] {x}], 
      Poly \[Rule] {x + 1}]\)], "Input",
  CellLabel->"In[58]:="],

Cell[BoxData[
    \({Gen \[Rule] {\(-1\) + x}, Dim \[Rule] {0}, Var \[Rule] {x}, 
      Par \[Rule] {}}\)], "Output",
  CellLabel->"Out[58]="]
}, Open  ]],

Cell[TextData[{
  "Option ",
  StyleBox["RRoots",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Option for Stringvacua modules specifying if the number of real \
roots of zero dimensional ideals should be calculated. \nUsage: ",
    FontSize->12],
  StyleBox["RRoots \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["str",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where the string ",
    FontSize->12],
  StyleBox["str ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" takes the values ",
    FontSize->12],
  StyleBox["\"Probab\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\"Determ\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\"Off\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". The values ",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\"Probab\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\"Determ\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" refer to two different methods for determining the number of \
real roots of zero dimensional ideals, and imply that the number of real \
roots is calculated. ",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\"Off\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" means that the number of real roots is not calculated ",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["(see modules  ",
    FontSize->12],
  ButtonBox["PrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["Saturation",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["SaturationPrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"RRoots"],

Cell[TextData[StyleBox["For example, if we wished to primary decompose an \
ideal Id, and wished to use a deterministic method to find the number of real \
roots of any zero dimensional ideals in the output, we could write the \
following.",
  FontSize->14]], "Text"],

Cell[BoxData[
    \(PrimDec[Id, RRoots \[Rule] \ "\<Determ\>"]\)], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["SturmLabel",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Option for labelling output in modules ",
    FontSize->12],
  ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
  StyleBox["  and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" .\nFor ",
    FontSize->12],
  ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" this option can be set to",
    FontSize->12],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"None\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" whereupon the output will include the complete expressions for \
the objects whose signs are tested by the module. If set to any other string, \
this string will label the first entry in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Sturm",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["\[Rule]... instead of the full expressions for the tested \
quantities. ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\nFor ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" , i",
    FontSize->12],
  StyleBox["f set to ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"Automatic\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", it will handle labelling the test results from other options \
such as ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["TestLambda",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["TestStability",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["  automatically, filling in the labels ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"Pot\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"Hessian\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" (or a list of the two) and making this the first entry in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Sturm",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["\[Rule].... If any other tests are performed in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"Automatic\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" mode, the output will contain the full expressions for the \
quantities whose signs are tested. If set to ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"None\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", the output is as in SturmQuery. If set to any other string, \
this will label the first entry in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Sturm",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["\[Rule]... instead of the full expressions for the tested \
quantities. ",
    FontFamily->"Arial",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SturmLabel"],

Cell[TextData[{
  "Option ",
  StyleBox["SturmPoly",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Option to specify (list of) polynomial(s) for ideal filtering \
using Sturm queries. \nUsage: ",
    FontSize->12],
  StyleBox["SturmPoly \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["polylist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["polylist ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is a polynomial or a list of polynomials. Only ideals with real \
roots on which the polynomials in ",
    FontSize->12],
  StyleBox["polylist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  assume the signs specified by the option ",
    FontSize->12],
  StyleBox[ButtonBox["SturmSigns",
    ButtonData:>"SturmSigns",
    ButtonStyle->"Hyperlink"],
    FontSize->12],
  StyleBox[" ",
    FontSize->10],
  StyleBox["are kept in filtering process (see modules  ",
    FontSize->12],
  ButtonBox["PrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["Saturation",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["SaturationPrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SturmPoly"],

Cell[TextData[{
  "Option ",
  StyleBox["SturmSigns",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Options to specify (list of) sign(s) for ideal filtering using \
Sturm queries. \nUsage: ",
    FontSize->12],
  StyleBox["SturmSigns \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["signlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["signlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  is a single sign, -1, 1, or 0, or a list of such signs. The \
number of signs must be an integer multiple of the number, ",
    FontSize->12],
  StyleBox["n",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[",  of polynomials in ",
    FontSize->12],
  StyleBox[ButtonBox["SturmPoly",
    ButtonData:>"SturmPoly",
    ButtonStyle->"Hyperlink"],
    FontSize->12],
  StyleBox[" with the first ",
    FontSize->12],
  StyleBox["n ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" signs specifying one allowed pattern of signs, the second ",
    FontSize->12],
  StyleBox["n ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" another allowed pattern, etc. Only ideals with real roots on \
which the polynomials specified in option",
    FontSize->12],
  StyleBox[" ",
    FontSize->10],
  StyleBox[ButtonBox["SturmPoly",
    ButtonData:>"SturmPoly",
    ButtonStyle->"Hyperlink"],
    FontSize->12],
  StyleBox[" assume one of the specified patterns of signs are kept (see \
modules  ",
    FontSize->12],
  ButtonBox["PrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["Saturation",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ",
    FontSize->12],
  ButtonBox["SaturationPrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SturmSigns"],

Cell[TextData[StyleBox["For example, if we wished to primary decompose an \
ideal Id, and we wished to only keep the primary pieces which have real roots \
where x is positive, we would evaluate the following.",
  FontSize->14]], "Text"],

Cell[BoxData[
    \(PrimDec[Id, \ SturmPoly \[Rule] {x}, SturmSigns \[Rule] {1}]\)], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["TestLambda",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Option to perform a Sturm query on the scalar potential of a \
model, in order to find the sign of the cosmological constant for the real \
roots of a zero dimensional ideal. \nUsage: ",
    FontSize->12],
  StyleBox["TestLambda \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["str",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["str ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" takes the values ",
    FontSize->12],
  StyleBox["\"yes\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12],
  StyleBox["\"no\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". (see module ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"TestLambda"],

Cell[TextData[{
  "Option ",
  StyleBox["TestMatDet",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Option to specify rational matrices for a SturmQuery on their \
determinants. \nUsage: ",
    FontSize->12],
  StyleBox["TestMatDet \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["matlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["matlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  is a list of square matrices with rational entries (see modules \
",
    FontSize->12],
  ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
  " ",
  StyleBox["and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"TestMatDet"],

Cell[TextData[{
  StyleBox["If we wish to test the sign of the determinant of some square \
matrix, ",
    FontSize->14],
  StyleBox["mat, ",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox["on the real roots of the ideal ",
    FontSize->14],
  StyleBox["Id ",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox["we simply evaluate the following.",
    FontSize->14]
}], "Text"],

Cell[BoxData[
    \(SturmQuery[id, TestMatDet \[Rule] {mat}]\)], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["TestMatEigen",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Option to specify rational matrices for a SturmQuery on their \
Eigenvalues. \nUsage: ",
    FontSize->12],
  StyleBox["TestMatDet \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["matlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["matlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  is a list of square matrices with rational entries and real \
eigenvalues (see modules ",
    FontSize->12],
  ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
  " ",
  StyleBox["and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"TestMatEigen"],

Cell[TextData[{
  "Option ",
  StyleBox["TestRational",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Option to specify rational functions for SturmQuery. \nUsage: ",
    FontSize->12],
  StyleBox["TestRational \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["polylist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["polylist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  is a rational function or a list of rational functions on which \
a SturmQuery is to be carried out (see module ",
    FontSize->12],
  ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox["). ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"TestRational"],

Cell[TextData[{
  StyleBox["For example, if we wished to test the signs of the rational \
functions {1, x, y} on the real roots of some ideal ",
    FontSize->14],
  StyleBox["Id, ",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox["we would simply evaluate the following.",
    FontSize->14]
}], "Text"],

Cell[BoxData[
    \(SturmQuery[id, TestRational \[Rule] {1, x, y}]\)], "Input"],

Cell[TextData[{
  "Option ",
  StyleBox["TestStability",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontSize->16],
  StyleBox["Option to perform a stability test based on Sturm queries for the \
real roots of a zero dimensional ideal. \nUsage: ",
    FontSize->12],
  StyleBox["TestStability \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["str",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["str ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" takes the values ",
    FontSize->12],
  StyleBox["\"Hesse\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" for a stability test based on the Eigenvalues of the Hesse \
matrix of the model, or ",
    FontSize->12],
  StyleBox["\"Off\" ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["to switch this test off (see module  ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox[")",
    FontSize->12],
  "."
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"TestStability"],

Cell[TextData[{
  "Option ",
  StyleBox["Var",
    FontWeight->"Plain"],
  "\n",
  StyleBox["Specifies a list of variables. \nUsage: ",
    FontSize->12],
  StyleBox["Var \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["varlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["varlist ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is a variable or a list of variables (see module ",
    FontSize->12],
  ButtonBox["CreateIdeal",
    ButtonStyle->"AddOnsLink"],
  StyleBox[").",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"Var"],

Cell[TextData[{
  "Option ",
  StyleBox["VarPar",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Option to specify variables and parameters in Stringvacua \
modules. \nUsage: ",
    FontSize->12],
  StyleBox["VarPar \[Rule] ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["varparlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", where ",
    FontSize->12],
  StyleBox["varparlist",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  is a list of variables/parameters or a list of assignments of \
variables/parameters to values, depending on module (see modules ",
    FontSize->12],
  ButtonBox["Elimination",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ",
    FontSize->12],
  ButtonBox["PrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ",
    FontSize->12],
  ButtonBox["Saturation",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ",
    FontSize->12],
  ButtonBox["DimIdeal",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",",
    FontSize->12],
  ButtonBox["FacGroebner",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ",
    FontSize->12],
  ButtonBox["SaturationPrimDec",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ",
    FontSize->12],
  ButtonBox["NumRoots",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ",
    FontSize->12],
  ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ",
    FontSize->12],
  ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and ",
    FontSize->12],
  ButtonBox["ThomEncode",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". ",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"VarPar"],

Cell[TextData[StyleBox["For example we could set the parameter a to 1 in a \
primary decomposition as follows.",
  FontSize->14]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[CreateIdeal[{x^2 - a}, Var \[Rule] {x}, Par \[Rule] {a}], 
      VarPar \[Rule] {a \[Rule] 1}]\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \({{Gen \[Rule] {1 + x}, Dim \[Rule] {0}, Var \[Rule] {x}, 
        Par \[Rule] {}}, {Gen \[Rule] {\(-1\) + x}, Dim \[Rule] {0}, 
        Var \[Rule] {x}, Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[55]="]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Dummies"->{
    Cell[15453, 422, 1417, 46, 190, "Subsection",
      CellTags->"Dummies"]},
  "DeltaPot"->{
    Cell[17123, 477, 774, 27, 121, "Subsection",
      CellTags->"DeltaPot"]},
  "Dim"->{
    Cell[17900, 506, 1319, 47, 123, "Subsection",
      CellTags->"Dim"]},
  "FTerms"->{
    Cell[19578, 569, 1500, 53, 123, "Subsection",
      CellTags->"FTerms"]},
  "KMetricInverse"->{
    Cell[21703, 646, 1054, 34, 122, "Subsection",
      CellTags->"KMetricInverse"]},
  "Meth"->{
    Cell[22760, 682, 1173, 43, 105, "Subsection",
      CellTags->"Meth"]},
  "NRoots"->{
    Cell[24341, 742, 1266, 46, 105, "Subsection",
      CellTags->"NRoots"]},
  "Par"->{
    Cell[25924, 798, 1005, 37, 105, "Subsection",
      CellTags->"Par"]},
  "Poly"->{
    Cell[27407, 854, 1207, 44, 123, "Subsection",
      CellTags->"Poly"]},
  "RRoots"->{
    Cell[29259, 925, 2751, 93, 139, "Subsection",
      CellTags->"RRoots"]},
  "SturmLabel"->{
    Cell[32361, 1029, 3408, 122, 192, "Subsection",
      CellTags->"SturmLabel"]},
  "SturmPoly"->{
    Cell[35772, 1153, 1768, 62, 140, "Subsection",
      CellTags->"SturmPoly"]},
  "SturmSigns"->{
    Cell[37543, 1217, 2443, 84, 157, "Subsection",
      CellTags->"SturmSigns"]},
  "TestLambda"->{
    Cell[40325, 1311, 1371, 48, 122, "Subsection",
      CellTags->"TestLambda"]},
  "TestMatDet"->{
    Cell[41699, 1361, 1097, 40, 106, "Subsection",
      CellTags->"TestMatDet"]},
  "TestMatEigen"->{
    Cell[43268, 1422, 1149, 41, 124, "Subsection",
      CellTags->"TestMatEigen"]},
  "TestRational"->{
    Cell[44420, 1465, 1120, 38, 122, "Subsection",
      CellTags->"TestRational"]},
  "TestStability"->{
    Cell[45938, 1519, 1427, 49, 123, "Subsection",
      CellTags->"TestStability"]},
  "Var"->{
    Cell[47368, 1570, 857, 32, 105, "Subsection",
      CellTags->"Var"]},
  "VarPar"->{
    Cell[48228, 1604, 1891, 71, 140, "Subsection",
      CellTags->"VarPar"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Dummies", 51380, 1714},
  {"DeltaPot", 51475, 1717},
  {"Dim", 51565, 1720},
  {"FTerms", 51654, 1723},
  {"KMetricInverse", 51754, 1726},
  {"Meth", 51852, 1729},
  {"NRoots", 51942, 1732},
  {"Par", 52031, 1735},
  {"Poly", 52118, 1738},
  {"RRoots", 52208, 1741},
  {"SturmLabel", 52304, 1744},
  {"SturmPoly", 52405, 1747},
  {"SturmSigns", 52505, 1750},
  {"TestLambda", 52606, 1753},
  {"TestMatDet", 52707, 1756},
  {"TestMatEigen", 52810, 1759},
  {"TestRational", 52915, 1762},
  {"TestStability", 53021, 1765},
  {"Var", 53118, 1768},
  {"VarPar", 53207, 1771}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 114, 3, 76, "Title"],
Cell[1844, 57, 85, 2, 31, "Subtitle"],
Cell[1932, 61, 2024, 63, 84, "Text"],
Cell[3959, 126, 11364, 290, 684, "Subsection"],
Cell[15326, 418, 124, 2, 31, "Subtitle"],
Cell[15453, 422, 1417, 46, 190, "Subsection",
  CellTags->"Dummies"],
Cell[16873, 470, 247, 5, 45, "Input"],
Cell[17123, 477, 774, 27, 121, "Subsection",
  CellTags->"DeltaPot"],
Cell[17900, 506, 1319, 47, 123, "Subsection",
  CellTags->"Dim"],
Cell[19222, 555, 283, 9, 28, "Text"],
Cell[19508, 566, 67, 1, 45, "Input"],
Cell[19578, 569, 1500, 53, 123, "Subsection",
  CellTags->"FTerms"],
Cell[21081, 624, 447, 14, 48, "Text"],
Cell[21531, 640, 169, 4, 45, "Input"],
Cell[21703, 646, 1054, 34, 122, "Subsection",
  CellTags->"KMetricInverse"],
Cell[22760, 682, 1173, 43, 105, "Subsection",
  CellTags->"Meth"],
Cell[23936, 727, 327, 10, 48, "Text"],
Cell[24266, 739, 72, 1, 45, "Input"],
Cell[24341, 742, 1266, 46, 105, "Subsection",
  CellTags->"NRoots"],
Cell[25610, 790, 244, 3, 48, "Text"],
Cell[25857, 795, 64, 1, 45, "Input"],
Cell[25924, 798, 1005, 37, 105, "Subsection",
  CellTags->"Par"],
Cell[26932, 837, 189, 3, 28, "Text"],

Cell[CellGroupData[{
Cell[27146, 844, 114, 2, 45, "Input"],
Cell[27263, 848, 129, 3, 44, "Output"]
}, Open  ]],
Cell[27407, 854, 1207, 44, 123, "Subsection",
  CellTags->"Poly"],
Cell[28617, 900, 319, 10, 28, "Text"],

Cell[CellGroupData[{
Cell[28961, 914, 138, 3, 45, "Input"],
Cell[29102, 919, 142, 3, 44, "Output"]
}, Open  ]],
Cell[29259, 925, 2751, 93, 139, "Subsection",
  CellTags->"RRoots"],
Cell[32013, 1020, 267, 4, 48, "Text"],
Cell[32283, 1026, 75, 1, 45, "Input"],
Cell[32361, 1029, 3408, 122, 192, "Subsection",
  CellTags->"SturmLabel"],
Cell[35772, 1153, 1768, 62, 140, "Subsection",
  CellTags->"SturmPoly"],
Cell[37543, 1217, 2443, 84, 157, "Subsection",
  CellTags->"SturmSigns"],
Cell[39989, 1303, 237, 3, 48, "Text"],
Cell[40229, 1308, 93, 1, 45, "Input"],
Cell[40325, 1311, 1371, 48, 122, "Subsection",
  CellTags->"TestLambda"],
Cell[41699, 1361, 1097, 40, 106, "Subsection",
  CellTags->"TestMatDet"],
Cell[42799, 1403, 390, 14, 28, "Text"],
Cell[43192, 1419, 73, 1, 45, "Input"],
Cell[43268, 1422, 1149, 41, 124, "Subsection",
  CellTags->"TestMatEigen"],
Cell[44420, 1465, 1120, 38, 122, "Subsection",
  CellTags->"TestRational"],
Cell[45543, 1505, 310, 9, 28, "Text"],
Cell[45856, 1516, 79, 1, 45, "Input"],
Cell[45938, 1519, 1427, 49, 123, "Subsection",
  CellTags->"TestStability"],
Cell[47368, 1570, 857, 32, 105, "Subsection",
  CellTags->"Var"],
Cell[48228, 1604, 1891, 71, 140, "Subsection",
  CellTags->"VarPar"],
Cell[50122, 1677, 138, 2, 28, "Text"],

Cell[CellGroupData[{
Cell[50285, 1683, 160, 3, 45, "Input"],
Cell[50448, 1688, 228, 4, 44, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

