(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     12319,        429]*)
(*NotebookOutlinePosition[     13025,        453]*)
(*  CellTagsIndexPosition[     12981,        449]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["NumRoots",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Calculates the number of real roots of a zero-dimensional ideal.\n\
Usage: ",
    FontFamily->"Arial"],
  StyleBox["NumRoots[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ideal<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  "\n",
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A (list of) ideal list(s) of the form explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[", combined into a single ideal by the module\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\n",
    FontFamily->"Helvetica"],
  StyleBox["VarPar",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" \[Rule]  (list of) rule(s) assigning parameter(s)/variable(s) to \
value(s)\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Meth",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["    \[Rule]  a string set to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\"Probab\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\"Determ\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". Default is ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\"Probab\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["\nGlobal options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" Calculates the number of real roots of a zero-dimensional ideal. \
Field extensions (i.e., parameters) are not allowed, so all parameters must \
be replaced by actual numbers. Also, the imaginary unit ",
    FontSize->12],
  StyleBox["I", "Input",
    FontSize->12],
  StyleBox[" is forbidden in the input ideal.\n\n",
    FontSize->12],
  "Output:",
  StyleBox[" An ",
    FontSize->12],
  ButtonBox["ideal list",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
  StyleBox[" with the number of real roots assigned to the keyword ",
    FontSize->12],
  StyleBox["NRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".\n\n",
    FontSize->12],
  "Example:",
  StyleBox[" ",
    FontSize->12],
  StyleBox["NumRoots[IDPot/.model,VarPar->{a->1,b->1}]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" calculates the number of real roots of the ideal ",
    FontSize->12],
  StyleBox["IDPot/.model", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" for parameter values ",
    FontSize->12],
  StyleBox["a=b=1",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".",
    FontSize->12]
}], "Subsection"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Detailed Example",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[BoxData[{
    StyleBox[\(Consider\ the\ following\ example\ of\ an\ ideal\),
      FontFamily->"Arial"], "\n", 
    RowBox[{
      StyleBox[
        FormBox["Id",
          "TraditionalForm"],
        FontWeight->"Bold"], 
      StyleBox["  ",
        FontFamily->"Arial",
        FontWeight->"Bold"], 
      StyleBox["=",
        FontFamily->"Arial",
        FontWeight->"Bold"], 
      StyleBox[" ",
        FontFamily->"Arial",
        FontWeight->"Bold"], 
      RowBox[{
        StyleBox[
          RowBox[{
            FormBox[\(\(\(<\)\(x\^2 + \ y\^2\)\), \ x\ y\),
              "TraditionalForm"], " ", 
            StyleBox[">",
              FontFamily->"Arial",
              FontWeight->"Bold"]}],
          FontWeight->"Bold"], 
        StyleBox["  ",
          FontFamily->"Arial",
          FontWeight->"Bold"], 
        StyleBox["\[Subset]",
          FontFamily->"Arial",
          FontWeight->"Bold"], 
        StyleBox["  ",
          FontFamily->"Arial",
          FontWeight->"Bold"], 
        StyleBox[
          FormBox[\(C\ [x, \ y]\),
            "TraditionalForm"],
          FontWeight->"Bold"]}]}], "\n", 
    RowBox[{
      StyleBox[\(The\ base\ number\ field\ here\ is\ just\ C\),
        FontFamily->"Arial"], 
      StyleBox[",",
        FontFamily->"Arial"], 
      StyleBox[" ",
        FontFamily->"Arial"], 
      RowBox[{
        StyleBox[\(the\ field\ of\ complex\ numbers\),
          FontFamily->"Arial"], 
        StyleBox[";",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        RowBox[{
          StyleBox["the",
            FontFamily->"Arial"], 
          StyleBox[" ",
            FontFamily->"Arial"], 
          StyleBox["polynomial",
            FontFamily->"Arial"], 
          StyleBox[" ",
            FontFamily->"Arial"], 
          StyleBox["ring",
            FontFamily->"Arial"], 
          StyleBox[" ",
            FontFamily->"Arial"], 
          StyleBox["is",
            FontFamily->"Arial"], 
          StyleBox[
            RowBox[{
              StyleBox[" ",
                FontFamily->"Arial",
                FontWeight->"Bold",
                FontSlant->"Italic"], 
              StyleBox[" ",
                FontFamily->"Arial",
                FontSlant->"Italic"]}]], 
          StyleBox[\(C[x, y]\),
            FontFamily->"Arial",
            FontSlant->"Italic"]}]}], 
      StyleBox[",",
        FontFamily->"Arial",
        FontSlant->"Italic"], 
      StyleBox[" ",
        FontFamily->"Arial",
        FontSlant->"Italic"], 
      StyleBox[\(\(viz\)\(.\)\),
        FontFamily->"Arial",
        FontSlant->"Italic"], 
      StyleBox[",",
        FontFamily->"Arial",
        FontSlant->"Italic"], 
      StyleBox[" ",
        FontFamily->"Arial",
        FontWeight->"Bold"], 
      RowBox[{
        StyleBox["the",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["polynomials",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["in",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["2",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["variables",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        FormBox["x",
          "TraditionalForm"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["and",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        RowBox[{
          FormBox["y",
            "TraditionalForm"], ".", 
          StyleBox["\[IndentingNewLine]",
            FontFamily->"Arial",
            FontWeight->"Bold",
            FontSlant->"Italic"], 
          StyleBox["This",
            FontFamily->"Arial"]}], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["is",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["an",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["ideal",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["of",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["complex",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["dimension",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["0.",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["Hence",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["we",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["can",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["perform",
          FontFamily->"Arial"]}]}]}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NRoots /. 
      NumRoots[{Var \[Rule] {x, y}, Gen \[Rule] {\ x\^2 + \ y\^2, x\ y}, 
          Dim \[Rule] {0}}]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \({1}\)], "Output",
  CellLabel->"Out[37]="]
}, Open  ]],

Cell[TextData[StyleBox["Indeed there is a single real root at (0,0), as may \
be seen by inspection.",
  FontFamily->"Arial"]], "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 281, 10, 76, "Title"],
Cell[2011, 64, 561, 17, 72, "Subsection"],
Cell[2575, 83, 2581, 92, 228, "Subsection"],
Cell[5159, 177, 1443, 47, 143, "Subsection"],

Cell[CellGroupData[{
Cell[6627, 228, 93, 2, 31, "Subtitle"],
Cell[6723, 232, 5158, 176, 82, "Text"],

Cell[CellGroupData[{
Cell[11906, 412, 170, 4, 46, "Input"],
Cell[12079, 418, 62, 2, 44, "Output"]
}, Open  ]],
Cell[12156, 423, 135, 2, 27, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

