(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     20223,        610]*)
(*NotebookOutlinePosition[     20951,        635]*)
(*  CellTagsIndexPosition[     20907,        631]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["PrimDec",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Calculates the primary decomposition of an ideal. \nUsage: ",
    FontFamily->"Arial"],
  StyleBox["PrimDec[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ideal<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  "\n",
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A (list of) ideal list(s) of the form explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[", combined into a single ideal by the module",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\n\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[Cell[BoxData[GridBox[{
          {
            StyleBox["VarPar",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox[\(\(\[Rule]\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ rules\ assigning\ parameters\ \
\((variables)\)\ to\ values\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["Dim",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox[\(\(\[Rule]\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ complex\ dimension \((s)\)\ of\ ideals\
\ to\ be\ kept\ \((default\ 0)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["RRoots",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox[\(\(\[Rule]\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12], 
            StyleBox[
              RowBox[{
                RowBox[{
                  StyleBox["\"\<Probab\>\"",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontVariations->{"CompatibilityType"->0}], 
                  StyleBox[",",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["\"\<Determ\>\"",
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}], 
                    StyleBox[" ",
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox["or",
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[" ",
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox["\"\<Off\>\"",
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}], 
                    StyleBox[" ",
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    RowBox[{
                      StyleBox["(",
                        ShowStringCharacters->True,
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox["default",
                        ShowStringCharacters->True,
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox[" ",
                        ShowStringCharacters->True,
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox["\"\<\>",
                        ShowStringCharacters->True,
                        FontFamily->"Helvetica",
                        FontSize->12,
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["Off",
                        ShowStringCharacters->True,
                        FontFamily->"Helvetica",
                        FontSize->12,
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}], 
                StyleBox["\"\<\>",
                  ShowStringCharacters->True,
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[")",
                  ShowStringCharacters->True,
                  FontFamily->"Helvetica",
                  FontSize->12]}],
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            StyleBox["NRoots",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox[\(\(\[Rule]\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(minimal\ number\ of\ real\ roots\ for\ ideal\ to\ be\ \
kept\ \((default\ 0)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["SturmPoly",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox[\(\(\[Rule]\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ polynomial \((s)\)\ for\ Sturm\ query\ \
filtering\ of\ ideals\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["SturmSigns",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox[\(\(\[Rule]\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ sign \((s)\)\ for\ Sturm\ query\ \
filtering\ of\ ideals\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              FontFamily->"Helvetica",
              FontSize->12]}
          }]], "Text",
    GridBoxOptions->{ColumnAlignments->{Left}}], "Text"],
  StyleBox[" ", "Text"],
  StyleBox["\n",
    FontFamily->"Helvetica"],
  StyleBox["\nAll global options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options\n",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["NB. We advise against setting the FieldChar option (a global \
option) to a number field of finite characteristic here as it will cause \
spurious roots to emerge.",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  StyleBox["Purpose:",
    FontSize->16],
  StyleBox[" Calculates the primary decomposition of the given ideal in list \
",
    FontSize->12],
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[". The parameters/variables of the ideal can be replaced by values \
using the ",
    FontSize->12],
  StyleBox["VarPar",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option. Parameters which are not replaced are treated as \
extensions of the base number field. To switch calculation of the number of \
real roots on, the ",
    FontSize->12],
  StyleBox["RRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option should be set to one of the two methods ",
    FontSize->12],
  StyleBox["\"Probab\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12],
  StyleBox["\"Determ\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".\n\n",
    FontSize->12],
  StyleBox["Output:",
    FontSize->16],
  StyleBox[" A list of ideal lists, each corresponding to a primary ideal in \
the primary decomposition of the input. Ideals with complex dimensions other \
than those specified in option ",
    FontSize->12],
  StyleBox["Dim",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are filtered out. If real root calculation is switched on, only \
ideals with at least as many real roots as  specified in option ",
    FontSize->12],
  StyleBox["NRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are calculated. If the options ",
    FontSize->12],
  StyleBox["SturmPoly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["SturmSigns",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are provided only ideals with real roots on which the \
polynomials in ",
    FontSize->12],
  StyleBox["SturmPoly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" assume the specified patterns of signs given in ",
    FontSize->12],
  StyleBox["SturmSigns",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are kept.\n\n",
    FontSize->12],
  "Example:",
  StyleBox[" ",
    FontSize->12],
  StyleBox["PrimDec[IDPot/.model,VarPar->{a->1,b->1}]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["calculates the (zero complex dimensional) primary ideals of ",
    FontSize->12],
  StyleBox["IDPot",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" for a ",
    FontSize->12],
  StyleBox["model",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" with parameters ",
    FontSize->12],
  StyleBox["a,b",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" replaced by ",
    FontSize->12],
  StyleBox["1",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". This call can be equivalently written as ",
    FontSize->12],
  StyleBox["PrimDec[IDPot/.model/.{a->1,b->1}]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".", "Text",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Examples",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["\nCreate a simple model with ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["\n\t",
    FontWeight->"Plain"],
  StyleBox["one modulus field ",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + i \[Tau]",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[", \n\tstandard Kahler potential ",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["K = - 3  ",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Log[",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T + ",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Conjugate[",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["]]\n\tsuperpotential ",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["W = a + b T^8 ",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["with parameters",
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" (a,b)",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1]]
}], "Text",
  CellDingbat->None,
  FontFamily->"Arial"],

Cell[BoxData[
    \(\(model = 
        CreateModel[{T}, {t}, {\[Tau]}, \(-3\)\ \ Log[T + Conjugate[T]], 
          a + b\ T^8, Par \[Rule] {a, b}];\)\)], "Input",
  CellLabel->"In[39]:="],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Calculate the scalar potential etc. for this model",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    \(\(model = CalcModel[model];\)\)], "Input",
  CellLabel->"In[40]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Calculate the primary decomposition of the ideal dV, generated by \
the numerator of the derivatives with respect to all the component fields; \
this is stored in the keyword ",
    FontFamily->"Arial"],
  StyleBox["IDPot", "Input",
    FontFamily->"Arial"],
  StyleBox[". Parameters can be set to numerical values either by explicit \
substitution into the ideal or by providing these substitutions in the \
optional VarPar argument of the ",
    FontFamily->"Arial"],
  StyleBox["PrimDec[ ]", "Input"],
  StyleBox[" module.",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[IDPot /. model, VarPar \[Rule] {a \[Rule] 1, b \[Rule] 1}, 
      RRoots \[Rule] "\<Probab\>", NRoots \[Rule] 1]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], \(-3\) + 13\ t\^8}, Dim \[Rule] {0}, 
        NRoots \[Rule] {2}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}, {Gen \[Rule] {\[Tau], t}, Dim \[Rule] {0}, 
        NRoots \[Rule] {1}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}, {Gen \[Rule] {11\ t\^6 - 49\ t\^4\ \[Tau]\^2 + 
              21\ t\^2\ \[Tau]\^4 + \[Tau]\^6, 
            363 + 65268\ t\^4\ \[Tau]\^4 - 41304\ t\^2\ \[Tau]\^6 - 
              1772\ \[Tau]\^8, 
            179487\ t\^2 - 685947\ \[Tau]\^2 + 15564096\ t\^2\ \[Tau]\^8 + 
              414656\ \[Tau]\^10, 
            24075711\ t\^4 - 107887770\ t\^2\ \[Tau]\^2 + 
              49067691\ \[Tau]\^4 + 20000000\ \[Tau]\^12}, Dim \[Rule] {0}, 
        NRoots \[Rule] {4}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[41]="]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell["\<\
We see that, dV breaks up into the intersection of the primary \
ideals given above (the NRoots\[Rule]1 option throws out primary ideals which \
have less than 1 real root)\
\>", "Text",
  FontFamily->"Arial"],

Cell[TextData[StyleBox["Note that the option Dim is by default set to 0, \
which means only 0-dimensional ideals are returned. For example, consider the \
ideal {x^2+y^2} in two variables",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[{Var \[Rule] {x, y}, Gen \[Rule] {x^2 - y^2}}]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \({}\)], "Output",
  CellLabel->"Out[42]="]
}, Open  ]],

Cell[TextData[StyleBox["Although nothing is returned in the above, if we set \
Dim so as to keep other dimensionalities, a non-trivial result may be \
obtained.",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[{Var \[Rule] {x, y}, Gen \[Rule] {x^2 - y^2}}, 
      Dim \[Rule] {0, 1, 2}]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \({{Gen \[Rule] {x + y}, Dim \[Rule] {1}, Var \[Rule] {x, y}, 
        Par \[Rule] {}}, {Gen \[Rule] {x - y}, Dim \[Rule] {1}, 
        Var \[Rule] {x, y}, Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[43]="]
}, Open  ]],

Cell[TextData[StyleBox["In the above we now find two 1-dimensional prime \
ideals.",
  FontFamily->"Arial"]], "Text"]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 280, 10, 76, "Title"],
Cell[2010, 64, 544, 16, 72, "Subsection"],
Cell[2557, 82, 8538, 225, 313, "Subsection"],
Cell[11098, 309, 4044, 132, 234, "Subsection"]
}, Open  ]],
Cell[15157, 444, 116, 3, 31, "Subtitle"],

Cell[CellGroupData[{
Cell[15298, 451, 1403, 50, 103, "Text"],
Cell[16704, 503, 187, 4, 45, "Input"],

Cell[CellGroupData[{
Cell[16916, 511, 109, 1, 27, "Text"],
Cell[17028, 514, 87, 2, 45, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17152, 521, 592, 14, 66, "Text"],

Cell[CellGroupData[{
Cell[17769, 539, 178, 3, 45, "Input"],
Cell[17950, 544, 844, 15, 82, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[18833, 564, 219, 5, 27, "Text"],
Cell[19055, 571, 222, 3, 27, "Text"],

Cell[CellGroupData[{
Cell[19302, 578, 112, 2, 45, "Input"],
Cell[19417, 582, 61, 2, 44, "Output"]
}, Open  ]],
Cell[19493, 587, 195, 3, 27, "Text"],

Cell[CellGroupData[{
Cell[19713, 594, 142, 3, 45, "Input"],
Cell[19858, 599, 229, 4, 44, "Output"]
}, Open  ]],
Cell[20102, 606, 117, 2, 27, "Text"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

