(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      6442,        212]*)
(*NotebookOutlinePosition[      7183,        237]*)
(*  CellTagsIndexPosition[      7139,        233]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["RemoveEntry",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Removes entry from a (model or ideal) list.\nUsage:",
    FontFamily->"Arial"],
  StyleBox[" ", "Text",
    FontFamily->"Arial"],
  StyleBox["RemoveEntry[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["list,keyword", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  StyleBox["\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["list ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["            :  a ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["model list",
    ButtonData:>"Model Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" or an ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["ideal list",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["keyword   ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[": a (list of) keyword(s) for model or ideal list",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" Removes the entries ",
    FontSize->12],
  StyleBox["keyword  \[Rule]",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" _",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  from the model or ideal list.\n\n",
    FontSize->12],
  "Output:",
  StyleBox[" Model or ideal list with entry removed.",
    FontSize->12]
}], "Subsection"],

Cell[CellGroupData[{

Cell["Example", "Subsection"],

Cell["Take a model generated as", "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    StyleBox[\(model\  = 
        CreateModel[{T}, {t}, {\[Tau]}, \ \(-3\)\ Log[T + Conjugate[T]]\ , \ 
          T]\),
      "Input",
      FontSize->12,
      FontVariations->{"CompatibilityType"->0}]], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \({Fields \[Rule] {T}, ReFields \[Rule] {t}, ImFields \[Rule] {\[Tau]}, 
      RFields \[Rule] {t, \[Tau]}, Par \[Rule] {}, 
      ComplexToReal \[Rule] {T \[Rule] t + \[ImaginaryI]\ \[Tau], 
          Conjugate[T] \[Rule] t - \[ImaginaryI]\ \[Tau]}, 
      KPot \[Rule] \(-3\)\ Log[T + Conjugate[T]], SPot \[Rule] T, 
      KMetric \[Rule] {{3\/\(4\ t\^2\)}}, DSPot \[Rule] {T \[Rule] 1}, 
      FTerms \[Rule] {T \[Rule] 1 - \(3\ T\)\/\(2\ t\)}, DeltaPot \[Rule] 0, 
      Pot \[Rule] \(-\(1\/\(3\ t\)\)\), 
      DPot \[Rule] {t \[Rule] 1\/\(3\ t\^2\), \[Tau] \[Rule] 0}, 
      Hesse \[Rule] {{\(-\(2\/\(3\ t\^3\)\)\), 0}, {0, 0}}}\)], "Output",
  CellLabel->"Out[44]="]
}, Open  ]],

Cell["Now remove the keyword FTerms ", "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(RemoveEntry[model, FTerms]\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \({Fields \[Rule] {T}, ReFields \[Rule] {t}, ImFields \[Rule] {\[Tau]}, 
      RFields \[Rule] {t, \[Tau]}, Par \[Rule] {}, 
      ComplexToReal \[Rule] {T \[Rule] t + \[ImaginaryI]\ \[Tau], 
          Conjugate[T] \[Rule] t - \[ImaginaryI]\ \[Tau]}, 
      KPot \[Rule] \(-3\)\ Log[T + Conjugate[T]], SPot \[Rule] T, 
      KMetric \[Rule] {{3\/\(4\ t\^2\)}}, DSPot \[Rule] {T \[Rule] 1}, 
      DeltaPot \[Rule] 0, Pot \[Rule] \(-\(1\/\(3\ t\)\)\), 
      DPot \[Rule] {t \[Rule] 1\/\(3\ t\^2\), \[Tau] \[Rule] 0}, 
      Hesse \[Rule] {{\(-\(2\/\(3\ t\^3\)\)\), 0}, {0, 0}}}\)], "Output",
  CellLabel->"Out[45]="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
ScreenStyleEnvironment->"Working",
WindowSize->{1144, 683},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 284, 10, 76, "Title"],
Cell[2014, 64, 587, 18, 74, "Subsection"],
Cell[2604, 84, 1362, 50, 121, "Subsection"],
Cell[3969, 136, 510, 18, 89, "Subsection"],

Cell[CellGroupData[{
Cell[4504, 158, 29, 0, 54, "Subsection"],
Cell[4536, 160, 64, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[4625, 165, 252, 7, 45, "Input"],
Cell[4880, 174, 692, 11, 116, "Output"]
}, Open  ]],
Cell[5587, 188, 69, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[5681, 193, 84, 2, 45, "Input"],
Cell[5768, 197, 634, 10, 116, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

