(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     24611,        684]*)
(*NotebookOutlinePosition[     25339,        709]*)
(*  CellTagsIndexPosition[     25295,        705]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["Saturation",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Calculates the saturation of an ideal with respect to a set of \
polynomials.\nUsage: ",
    FontFamily->"Arial"],
  StyleBox["Saturation[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ideal<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  "\n",
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A (list of) ideal list(s) of the form explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[", combined into a single ideal by the module",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\n\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n",
    FontFamily->"Helvetica",
    FontSize->12],
  " ",
  StyleBox["\n",
    FontFamily->"Helvetica"],
  Cell[BoxData[GridBox[{
          {
            StyleBox[
              RowBox[{
                StyleBox["Poly",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12]}],
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True], 
            StyleBox["\[Rule]",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12], 
            StyleBox[
              RowBox[{
                RowBox[{
                  StyleBox[\((list\ of)\),
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["polynomial",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[\((s)\),
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["or",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["ideal",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["list",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[\((s)\),
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["(",
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    RowBox[{
                      StyleBox["default",
                        ShowStringCharacters->True,
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox[" ",
                        ShowStringCharacters->True,
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox["1",
                        ShowStringCharacters->True,
                        FontFamily->"Helvetica",
                        FontSize->12,
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}], 
                    StyleBox[")",
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12]}]}], 
                StyleBox["\[IndentingNewLine]",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                
                StyleBox[\(NB . \ 
                      setting\ the\ polynomials\ to\ be\ functions\ of\ \
parameters\ only\ will\ return\ the\ original\ \(\(ideal\)\(.\)\(\ \)\)\),
                  FontFamily->"Helvetica",
                  FontSize->12]}],
              AutoStyleOptions->{"UnmatchedBracketStyle"->None}]},
          {
            StyleBox["VarPar",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ rules\ assigning\ parameters/
                  variables\ to\ values\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["RRoots",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox["\"\<Probab\>\"",
                AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                ShowStringCharacters->True,
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[",",
                AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                ShowStringCharacters->True,
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                ShowStringCharacters->True,
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["\"\<Determ\>\"",
                  AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                  ShowStringCharacters->True,
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[" ",
                  AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                  ShowStringCharacters->True,
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["or",
                  AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                  ShowStringCharacters->True,
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                  ShowStringCharacters->True,
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["\"\<Off\>\"",
                  AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                  ShowStringCharacters->True,
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox["  ",
                  AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                  ShowStringCharacters->True,
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["(",
                    AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["default",
                      AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[
                      StyleBox[
                        RowBox[{
                          StyleBox[" ",
                            ShowStringCharacters->True,
                            FontFamily->"Helvetica",
                            FontSize->12], 
                          StyleBox[" ",
                            ShowStringCharacters->True,
                            FontFamily->"Helvetica",
                            FontSize->12,
                            FontWeight->"Plain",
                            FontVariations->{"CompatibilityType"->0}]}]],
                      AutoStyleOptions->{"UnmatchedBracketStyle"->None}], 
                    StyleBox["\"\<\>",
                      AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}], 
                    StyleBox["Probab",
                      AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}], 
                    StyleBox["\"",
                      AutoStyleOptions->{"UnmatchedBracketStyle"->None},
                      ShowStringCharacters->True,
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}]}], 
                  StyleBox[")",
                    ShowStringCharacters->True,
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontVariations->{"CompatibilityType"->0}]}]}]}]},
          {
            StyleBox[
              RowBox[{
                StyleBox["NRoots",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12]}],
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True], 
            StyleBox["\[Rule]",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(minimal\ number\ of\ real\ roots\ for\ ideal\ to\ be\ \
kept\ \((default\ 0)\)\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["SturmPoly",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ polynomial \((s)\)\ for\ Sturm\ query\ \
filtering\ of\ ideals\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["SturmSigns",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ sign \((s)\)\ for\ Sturm\ query\ \
filtering\ of\ ideals\),
              AutoStyleOptions->{"UnmatchedBracketStyle"->None},
              ShowStringCharacters->True,
              FontFamily->"Helvetica",
              FontSize->12]}
          },
        ColumnAlignments->{Left}]]],
  StyleBox["\n",
    FontFamily->"Helvetica"],
  StyleBox["\nGlobal options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" Calculates the saturation of the ideal with respect to the \
polynomials specified in the ",
    FontSize->12],
  StyleBox["Poly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option. Parameters can be replaced by values using the ",
    FontSize->12],
  StyleBox["VarPar ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["option. All parameters not replaced are treated as field \
extensions. To switch calculation of the number of real roots on, the ",
    FontSize->12],
  StyleBox["RRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option should be set to one of the two methods ",
    FontSize->12],
  StyleBox["\"Probab\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12],
  StyleBox["\"Determ\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".\n\n",
    FontSize->12],
  "Output:",
  StyleBox[" An ",
    FontSize->12],
  StyleBox[ButtonBox["ideal list",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" containing the saturated ideal. If real root calculation is \
switched on, only ideals with at least as many real roots as specified in \
option ",
    FontSize->12],
  StyleBox["NRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are calculated. If options ",
    FontSize->12],
  StyleBox["SturmPoly ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["and ",
    FontSize->12],
  StyleBox["SturmSigns",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are provided filtering based on Sturm queries is carried out. In \
this case, only ideals with real roots on which polynomials in ",
    FontSize->12],
  StyleBox["SturmPoly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" assume the specified patterns of signs given in ",
    FontSize->12],
  StyleBox["SturmSigns",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are kept.\n\n",
    FontSize->12],
  "Example:",
  StyleBox[" ",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        StyleBox[\(Saturation[{Var -> {x, y}, x^2 + y^2}, Poly -> {x - 3}]\),
          "Input",
          FontWeight->"Plain"], TraditionalForm]],
    FontSize->12],
  " ",
  StyleBox["computes the saturation of the ideal ",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        RowBox[{"{", 
          StyleBox[\(x\^2 + y\^2\),
            "Text"], 
          StyleBox["}",
            "Text"]}], TraditionalForm]], "Text",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" with respect to the ideal ",
    FontSize->12],
  StyleBox["x-3",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", i.e., it finds the sub-ideal of  ",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["{",
            FontWeight->"Plain"], 
          StyleBox[\(x\^2 + y\^2\),
            "Text",
            FontWeight->"Plain"], 
          StyleBox["}",
            "Text",
            FontWeight->"Plain"]}], TraditionalForm]],
    FontSize->12],
  StyleBox[" where",
    FontSize->12],
  StyleBox[" ", "Text",
    FontSize->12],
  StyleBox["x-3",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" does not vanish.",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Example",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nConsider a theory given by \n\t",
    FontFamily->"Arial"],
  StyleBox["a single field T = t + i \[Tau] \n\tKahler potential K = -3 Log[ \
T + Conjugate[T] ]\n\tSuperpotential W = a + b T^8 with parameters a, b",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]]
}], "Text"],

Cell[BoxData[
    \(\(model = 
        CalcModel[
          CreateModel[{T}, {t}, {\[Tau]}, \(-3\)\ \ Log[T + Conjugate[T]], 
            a + b\ T^8, Par \[Rule] {a, b}]];\)\)], "Input",
  CellLabel->"In[46]:="],

Cell[TextData[{
  StyleBox["Now, form ",
    FontFamily->"Arial"],
  StyleBox["ideal1", "Input",
    FontFamily->"Arial"],
  StyleBox[" which is generated by the numerator of the partial derivatives \
of the scalar potential, and the polynomial ",
    FontFamily->"Arial"],
  StyleBox["fterm1", "Input",
    FontFamily->"Arial"],
  StyleBox[" corresponding to the numerator of the real part of the T \
F-term.",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(ideal1 = IDPot /. model\), "\[IndentingNewLine]", 
    \(fterm1 = \(\(t /. Gen\) /. IFTerms\) /. model\)}], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \({Var \[Rule] {t, \[Tau]}, Par \[Rule] {a, b}, 
      Gen \[Rule] {t \[Rule] \(-4\)\ \((15\ a\ b\ t\^6 - 65\ b\^2\ t\^14 - 
                  189\ a\ b\ t\^4\ \[Tau]\^2 - 385\ b\^2\ t\^12\ \[Tau]\^2 + 
                  105\ a\ b\ t\^2\ \[Tau]\^4 - 945\ b\^2\ t\^10\ \[Tau]\^4 + 
                  21\ a\ b\ \[Tau]\^6 - 1225\ b\^2\ t\^8\ \[Tau]\^6 - 
                  875\ b\^2\ t\^6\ \[Tau]\^8 - 315\ b\^2\ t\^4\ \[Tau]\^10 - 
                  35\ b\^2\ t\^2\ \[Tau]\^12 + 
                  5\ b\^2\ \[Tau]\^14)\), \[Tau] \[Rule] 
            56\ \((9\ a\ b\ t\^4\ \[Tau] + 5\ b\^2\ t\^12\ \[Tau] - 
                  30\ a\ b\ t\^2\ \[Tau]\^3 + 30\ b\^2\ t\^10\ \[Tau]\^3 + 
                  9\ a\ b\ \[Tau]\^5 + 75\ b\^2\ t\^8\ \[Tau]\^5 + 
                  100\ b\^2\ t\^6\ \[Tau]\^7 + 75\ b\^2\ t\^4\ \[Tau]\^9 + 
                  30\ b\^2\ t\^2\ \[Tau]\^11 + 
                  5\ b\^2\ \[Tau]\^13)\)}}\)], "Output",
  CellLabel->"Out[50]="],

Cell[BoxData[
    \(\(-3\)\ a + 13\ b\ t\^8 - 252\ b\ t\^6\ \[Tau]\^2 + 
      350\ b\ t\^4\ \[Tau]\^4 - 28\ b\ t\^2\ \[Tau]\^6 - 
      3\ b\ \[Tau]\^8\)], "Output",
  CellLabel->"Out[51]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["We can perform the saturation of ",
    FontFamily->"Arial"],
  StyleBox["ideal1", "Input",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" with respect to ",
    FontFamily->"Arial"],
  StyleBox["fterm1", "Input",
    FontFamily->"Arial"],
  StyleBox[" as follows.",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Saturation[ideal1, Poly \[Rule] fterm1, 
      VarPar -> {a \[Rule] 1, b \[Rule] 1}]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \({Gen \[Rule] {3\ t\^4\ \[Tau] - 10\ t\^2\ \[Tau]\^3 + 3\ \[Tau]\^5, 
          t\^6 - 35\ t\^2\ \[Tau]\^4 + 14\ \[Tau]\^6, 
          7\ t\^2\ \[Tau]\^5 - 3\ \[Tau]\^7, \[Tau]\^9}, Dim \[Rule] {0}, 
      Var \[Rule] {t, \[Tau]}, Par \[Rule] {}}\)], "Output",
  CellLabel->"Out[49]="]
}, Open  ]],

Cell[TextData[StyleBox["Here the parameters (a,b) are set 1; alternatively, \
they can be left as they are, in which case (a,b) will be considered as two \
extensions of the base number field of the polynomial ring.",
  FontFamily->"Arial"]], "Text"]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 283, 10, 76, "Title"],
Cell[2013, 64, 574, 17, 72, "Subsection"],
Cell[2590, 83, 14653, 365, 322, "Subsection"],
Cell[17246, 450, 3793, 130, 211, "Subsection"]
}, Open  ]],
Cell[21054, 583, 115, 3, 31, "Subtitle"],
Cell[21172, 588, 313, 7, 103, "Text"],
Cell[21488, 597, 211, 5, 45, "Input"],
Cell[21702, 604, 448, 13, 46, "Text"],

Cell[CellGroupData[{
Cell[22175, 621, 162, 3, 65, "Input"],
Cell[22340, 626, 972, 15, 101, "Output"],
Cell[23315, 643, 191, 4, 44, "Output"]
}, Open  ]],
Cell[23521, 650, 347, 12, 27, "Text"],

Cell[CellGroupData[{
Cell[23893, 666, 142, 3, 45, "Input"],
Cell[24038, 671, 304, 5, 44, "Output"]
}, Open  ]],
Cell[24357, 679, 250, 3, 46, "Text"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

