(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     53974,       1577]*)
(*NotebookOutlinePosition[     54702,       1602]*)
(*  CellTagsIndexPosition[     54658,       1598]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["SaturationExpand",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Calculates the saturation expansion of a model with respect to \
the F-terms and primary decomposes every term. \nUsage: ",
    FontFamily->"Arial"],
  StyleBox["SaturationExpand[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["model<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n\n",
    FontFamily->"Helvetica"],
  StyleBox["model ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[": An model list of the form explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Model Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12],
  "\n",
  StyleBox["\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  Cell[BoxData[GridBox[{
          {
            StyleBox["VarPar",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ rules\ assigning\ parameters/
                  variables\ to\ values\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            RowBox[{
              StyleBox["Poly",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12]}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox[\((list\ of)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["polynomial",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[\((s)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["or",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["ideal",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["list",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[\((s)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["(",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["default",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[\({\ }\),
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontVariations->{"CompatibilityType"->0}]}], 
                StyleBox[")",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}]}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}]}]},
          {
            StyleBox["FTerms",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              RowBox[{
                RowBox[{
                  StyleBox["a",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["single",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["integer",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["n",
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontSlant->"Italic",
                    FontVariations->{"CompatibilityType"->0}]}], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[",",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[\(a\ real\ field\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[",",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                
                StyleBox[\(a\ list\ with\ a\ subset\ of\ the\ real\ fields\ \
or\ a\ list\),
                  FontFamily->"Helvetica",
                  FontSize->12]}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["\[IndentingNewLine]",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[
                RowBox[{
                "with", " ", "a", " ", "subset", " ", "of", " ", "the", " ", 
                  "real", " ", "fields", " ", "and", " ", "integers", " ", 
                  RowBox[{"(", 
                    RowBox[{
                      StyleBox["default",
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox[" ",
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox[\({\ }\),
                        FontFamily->"Helvetica",
                        FontSize->12,
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}], ")"}], 
                  " "}],
                FontFamily->"Helvetica",
                FontSize->12]}]},
          {
            StyleBox["Dim",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ complex\ dimension \((s)\)\ of\ ideals\
\ to\ be\ kept\ \((default\ 0)\)\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["RRoots",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["\"\<Probab\>\"",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[",",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["\"\<Determ\>\"",
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontVariations->{"CompatibilityType"->0}], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["or",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["\"\<Off\>\"",
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontVariations->{"CompatibilityType"->0}], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["(",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    RowBox[{
                      StyleBox["default",
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox[" ",
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox["\"\<Off\>\"",
                        FontFamily->"Helvetica",
                        FontSize->12,
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}], 
                    StyleBox[")",
                      FontFamily->"Helvetica",
                      FontSize->12]}]}]}]}]},
          {
            StyleBox["NRoots",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(minimal\ number\ of\ real\ roots\ for\ ideal\ to\ be\ \
kept\ \((default\ 0)\)\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["SturmPoly",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            
            StyleBox[\(\((list\ of)\)\ polynomial \((s)\)\ for\ Sturm\ query\ \
filtering\ of\ ideals\ \((default\ 1)\)\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["SturmSigns",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox[\((list\ of)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["sign",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[\((s)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["for",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["Sturm",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["query",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["filtering",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["of",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["ideals",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["(",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["default",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[\({\ }\),
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontVariations->{"CompatibilityType"->0}]}], 
                StyleBox[")",
                  FontFamily->"Helvetica",
                  FontSize->12]}]}]},
          {
            StyleBox["TestRational",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox[\((list\ of)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["rational",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["function",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[\((s)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["for",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["Sturm",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["query",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["(",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["default",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[\({1}\),
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontVariations->{"CompatibilityType"->0}]}], 
                StyleBox[")",
                  FontFamily->"Helvetica",
                  FontSize->12]}]}]},
          {
            RowBox[{
              StyleBox["TestMatDet",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12]}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox[\((list\ of)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["rational",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["matrix",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[\((ces)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["for",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["Sturm",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["query",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["on",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["determinant",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[\((s)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["(",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["default",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[\({{1}}\),
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}]}], 
                  StyleBox[")",
                    FontFamily->"Helvetica",
                    FontSize->12]}]}]}]},
          {
            StyleBox["TestMatEigen",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox[\((list\ of)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["rational",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["matrix",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[\((ces)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["for",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["Sturm",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["query",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["on",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["Eigenvalues",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["(",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["default",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[\({{1}}\),
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontVariations->{"CompatibilityType"->0}]}], 
                StyleBox[")",
                  FontFamily->"Helvetica",
                  FontSize->12]}]}]},
          {
            StyleBox["TestLambda",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox["\"\<yes\>\"",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["or",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["\"\<no\>\"",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[\((default)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["to",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["switch",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["Sturm",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["queries",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["on",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["the",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["scalar",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["potential",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["on",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["and",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["off",
                FontFamily->"Helvetica",
                FontSize->12]}]},
          {
            StyleBox[\(\(TestStability\)\(\ \)\),
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              RowBox[{
                StyleBox["\"\<Hesse\>\"",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["for",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["a",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["stability",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["test",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["based",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["on",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["the",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["Hesse",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["matrix",
                  FontFamily->"Helvetica",
                  FontSize->12]}], 
              StyleBox[",",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["\"\<no\>\"",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[\((default)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["to",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["switch",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["off",
                  FontFamily->"Helvetica",
                  FontSize->12]}]}]},
          {
            StyleBox["SturmLabel",
              FontFamily->"Arial",
              FontSize->12,
              FontWeight->"Plain"], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12], 
            RowBox[{
              StyleBox["\"\<Automatic\>\"",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[",",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["\"\<None\>\"",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[",",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox[
                  RowBox[{
                    StyleBox["o",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox["r",
                      ShowStringCharacters->True,
                      FontFamily->"Arial",
                      FontSize->12,
                      GridBoxOptions->{ColumnAlignments->{Left}}]}]], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["a",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["string",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["labeling",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["the",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["quantity",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["to",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["be",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox["tested",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], 
                StyleBox[" ",
                  ShowStringCharacters->True,
                  FontFamily->"Arial",
                  FontSize->12,
                  GridBoxOptions->{ColumnAlignments->{Left}}], "by", " ", 
                "Sturm", " ", "queries"}]}]}
          }]],
    ShowStringCharacters->True,
    GridBoxOptions->{ColumnAlignments->{Left}}],
  "\n",
  StyleBox["              \t \nGlobal options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" \nCalculate terms in the saturation expansion of ",
    FontSize->12],
  StyleBox["IDPot/.model", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" (i.e., dV, the ideal generated by the numerators of the \
derivatives of the scalar potential) which arise from adding a subset of \
F-terms to the ideal and saturating out the complementary set. Each term is \
then primary decomposed. The module will only consider those terms in the \
saturation expansion where the F terms specified (by their corresponding real \
fields) in the ",
    FontSize->12],
  StyleBox["FTerms",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option vanish and a number between ",
    FontSize->12],
  StyleBox["nmin",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["nmax ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["(the minimum and maximum integers specified in the same option) \
other F terms do not vanish. Additional polynomial constraints (polynomials \
which must evaluate to zero on the resulting varieties), which will be \
imposed on each term in the saturation expansion, can be provided with the ",
    FontSize->12],
  StyleBox["Poly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option. \n\nNote that terms with an empty ideal list are omitted \
from the result. To calculate the number of real roots, the ",
    FontSize->12],
  StyleBox["RRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option should be set to either ",
    FontSize->12],
  StyleBox["\"Probab\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12],
  StyleBox["\"Determ\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". Sturm queries can be performed on the resulting ideals using \
the same options as for the module ",
    FontSize->12],
  StyleBox[ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[". Setting the option ",
    FontSize->12],
  StyleBox["TestLambda ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["to ",
    FontSize->12],
  StyleBox["\"yes\" ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["will cause the module to return the sign of the cosmological \
constant in each isolated vacuum found. Similarly setting ",
    FontSize->12],
  StyleBox["TestStability",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" to ",
    FontSize->12],
  StyleBox["\"Hesse\"",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" will induce the package to return the signs of the eigenvalues \
of the Hessian matrix on each isolated vacuum.\n\n",
    FontSize->12],
  "Output:",
  StyleBox[" \nA list with entries of the form ",
    FontSize->12],
  StyleBox["{",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["f_1,...,f_k ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["} \[Rule] {",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["list of ideal lists ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["}",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" where ",
    FontSize->12],
  StyleBox["{",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["f_1,...,f_k ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["}",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" is a subset of the real fields indicating the term in the \
saturation expansion with non-zero F-terms ",
    FontSize->12],
  StyleBox["f_1,...,f_k",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["  and all other F terms zero.  In particular, the supersymmetric \
term is of the form ",
    FontSize->12],
  StyleBox["{",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["}\[Rule]{",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["list of ideal lists ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["}",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". Ideals with complex dimensions other than those specified in \
option ",
    FontSize->12],
  StyleBox["Dim",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are filtered out. If real root calculation is switched on, only \
ideals with at least as many real roots as specified in option ",
    FontSize->12],
  StyleBox["NRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are calculated. If options ",
    FontSize->12],
  StyleBox["SturmPoly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["SturmSigns ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["are provided only ideals with real roots on which the polynomials \
in ",
    FontSize->12],
  StyleBox["SturmPoly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" assume the specified patterns of signs given in ",
    FontSize->12],
  StyleBox["SturmSigns",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are kept.  For the option ",
    FontSize->12],
  StyleBox["SturmLabel",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", i",
    FontSize->12],
  StyleBox["f set to ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"Automatic\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", it will handle labelling of the test results from the ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["TestLambda",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["TestStability",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" options automatically in that it fills in the labels ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"Pot\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"Hessian\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" (or a list of the two) and makes this the first entry in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Sturm",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["\[Rule]...;  if any other tests are performed in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"Automatic\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" mode the output will be labelled by the full expressions for the \
quantities tested (as will be the case if ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["SturmLabel",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" is set to ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\"None\"",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["). If set to any other string, the ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["SturmLabel ",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["option will provide the label in the first entry in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Sturm",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["\[Rule]... keyword, replacing the full expression. \n",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["\n\n",
    FontSize->12],
  "Examples:",
  StyleBox[" \n",
    FontSize->12],
  StyleBox["SaturationExpand[model,VarPar->{a->1,b->1}]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" finds all terms in the saturation expansion for parameter values \
",
    FontSize->12],
  StyleBox["a=b=1",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". \n",
    FontSize->12],
  StyleBox["SaturationExpand[model,VarPar->{a->1,b->1},FTerms->1]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" finds all terms in the saturation expansion with ",
    FontSize->12],
  StyleBox["a=b=1",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and one F term non-zero. \n",
    FontSize->12],
  StyleBox["SaturationExpand[model,VarPar->{a->1,b->1},FTerms->t]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["finds all terms in the saturation expansion with ",
    FontSize->12],
  StyleBox["a=b=1",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and the F-term corresponding to the field ",
    FontSize->12],
  StyleBox["t",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" is set to zero.",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Examples",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\n",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox["Consider a theory given by \n",
    FontFamily->"Arial"],
  StyleBox["\t",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox["a single field T = t + i \[Tau] \n\tKahler potential K = -3 Log[ \
T + Conjugate[T] ]\n\tSuperpotential W = a + b T^8 with parameters a, b",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]]
}], "Text"],

Cell[BoxData[
    \(\(model = 
        CalcModel[
          CreateModel[{T}, {t}, {\[Tau]}, \(-3\)\ \ Log[T + Conjugate[T]], 
            a + b\ T^8, Par \[Rule] {a, b}]];\)\)], "Input",
  CellLabel->"In[52]:="],

Cell[TextData[{
  StyleBox["Now perform complete saturation expansion of the critical points \
of the scalar potential V.\nThere are 2 (real) fields t and \[Tau] so there \
are 4 terms in the saturation expansion (recall that ",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`F\_T = \[PartialD]\_T W\)]],
  StyleBox[" and ",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`F\_t = Re(F\_T); \ \ \ \ F\_\[Tau] = Im(F\_T)\)]],
  "):",
  StyleBox["\n",
    FontFamily->"Arial"],
  StyleBox["\n",
    FontFamily->"Arial",
    FontSize->14],
  Cell[BoxData[
      \(TraditionalForm\`dV\  = \ 0; \ F\_t = \(F\_\[Tau] = 0\)\)],
    FontSize->14],
  StyleBox[" (complete SUSY vacuum)\n",
    FontSize->14],
  Cell[BoxData[
      \(TraditionalForm\`dV\  = \ 0; \ F\_t = 0\)],
    FontSize->14],
  "only",
  StyleBox["\n",
    FontSize->14],
  Cell[BoxData[
      \(TraditionalForm\`dV\  = \ 0; \ F\_\[Tau] = 0\)],
    FontSize->14],
  "only",
  StyleBox["\n",
    FontSize->14],
  Cell[BoxData[
      \(TraditionalForm\`dV\  = \ 0; \ F\_t = F\_\[Tau] \[NotEqual] \ 0\)],
    FontSize->14],
  StyleBox["(completely non-F-flat vacuum)",
    FontSize->14]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(satex = 
      SaturationExpand[model, 
        VarPar -> {a \[Rule] 1, b \[Rule] 1}]\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \({{} \[Rule] {{Gen \[Rule] {\[Tau], \(-3\) + 13\ t\^8}, Dim \[Rule] {0}, 
            Var \[Rule] {t, \[Tau]}, 
            Par \[Rule] {}}, {Gen \[Rule] {11\ t\^6 - 49\ t\^4\ \[Tau]\^2 + 
                  21\ t\^2\ \[Tau]\^4 + \[Tau]\^6, 
                363 + 65268\ t\^4\ \[Tau]\^4 - 41304\ t\^2\ \[Tau]\^6 - 
                  1772\ \[Tau]\^8, 
                179487\ t\^2 - 685947\ \[Tau]\^2 + 
                  15564096\ t\^2\ \[Tau]\^8 + 414656\ \[Tau]\^10, 
                24075711\ t\^4 - 107887770\ t\^2\ \[Tau]\^2 + 
                  49067691\ \[Tau]\^4 + 20000000\ \[Tau]\^12}, 
            Dim \[Rule] {0}, Var \[Rule] {t, \[Tau]}, 
            Par \[Rule] {}}}, {t} \[Rule] {{Gen \[Rule] {\[Tau], t}, 
            Dim \[Rule] {0}, Var \[Rule] {t, \[Tau]}, 
            Par \[Rule] {}}}}\)], "Output",
  CellLabel->"Out[53]="]
}, Open  ]],

Cell[TextData[StyleBox["Thus, to obtain the SUSY vacuum, we take the \
substitution",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({} /. satex\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], \(-3\) + 13\ t\^8}, Dim \[Rule] {0}, 
        Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}, {Gen \[Rule] {11\ t\^6 - 49\ t\^4\ \[Tau]\^2 + 
              21\ t\^2\ \[Tau]\^4 + \[Tau]\^6, 
            363 + 65268\ t\^4\ \[Tau]\^4 - 41304\ t\^2\ \[Tau]\^6 - 
              1772\ \[Tau]\^8, 
            179487\ t\^2 - 685947\ \[Tau]\^2 + 15564096\ t\^2\ \[Tau]\^8 + 
              414656\ \[Tau]\^10, 
            24075711\ t\^4 - 107887770\ t\^2\ \[Tau]\^2 + 
              49067691\ \[Tau]\^4 + 20000000\ \[Tau]\^12}, Dim \[Rule] {0}, 
        Var \[Rule] {t, \[Tau]}, Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[54]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["And for the one with ",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`F\_t = Re(F\_T) \[NotEqual] \ 0\)]],
  StyleBox["and ",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`F\_\[Tau] = \(Im(F\_T) = \ 0\)\)]],
  StyleBox[", we take the substitution",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({t} /. satex\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], t}, Dim \[Rule] {0}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[55]="]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 289, 10, 76, "Title"],
Cell[2019, 64, 615, 17, 72, "Subsection"],
Cell[2637, 83, 36259, 1000, 463, "Subsection"],
Cell[38899, 1085, 10527, 345, 532, "Subsection"]
}, Open  ]],
Cell[49441, 1433, 116, 3, 31, "Subtitle"],
Cell[49560, 1438, 431, 13, 105, "Text"],
Cell[49994, 1453, 211, 5, 45, "Input"],
Cell[50208, 1460, 1193, 39, 144, "Text"],

Cell[CellGroupData[{
Cell[51426, 1503, 143, 4, 45, "Input"],
Cell[51572, 1509, 863, 15, 101, "Output"]
}, Open  ]],
Cell[52450, 1527, 118, 2, 27, "Text"],

Cell[CellGroupData[{
Cell[52593, 1533, 69, 2, 45, "Input"],
Cell[52665, 1537, 672, 12, 82, "Output"]
}, Open  ]],
Cell[53352, 1552, 355, 11, 27, "Text"],

Cell[CellGroupData[{
Cell[53732, 1567, 70, 2, 45, "Input"],
Cell[53805, 1571, 153, 3, 44, "Output"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

