(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     18483,        571]*)
(*NotebookOutlinePosition[     19189,        595]*)
(*  CellTagsIndexPosition[     19145,        591]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["SaturationPrimDec",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Performs a saturation followed by a primary decomposition on an \
ideal. \nUsage: ",
    FontFamily->"Arial"],
  StyleBox["SaturationPrimDec[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ideal<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  "\n",
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A (list of) ideal list(s) of the form explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[", combined into a single ideal by the module",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\n\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  Cell[BoxData[GridBox[{
          {
            StyleBox["Poly",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{
              "CompatibilityType"->0}], \(\(\[Rule]\)\(\ \)\), 
            RowBox[{
              StyleBox[\((list\ of)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["polynomial",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[\((s)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["or",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["ideal",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox["list",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[\((s)\),
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["(",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["default",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox[" ",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  StyleBox["1",
                    FontFamily->"Helvetica",
                    FontSize->12,
                    FontWeight->"Plain",
                    FontVariations->{"CompatibilityType"->0}]}], 
                StyleBox[")",
                  FontFamily->"Helvetica",
                  FontSize->12]}]}]},
          {
            StyleBox["VarPar",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{
              "CompatibilityType"->0}], \(\(\[Rule]\)\(\ \)\), 
            
            StyleBox[\(\((list\ of)\)\ rules\ assigning\ parameters/
                  variables\ to\ values\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["Dim",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{
              "CompatibilityType"->0}], \(\(\[Rule]\)\(\ \)\), 
            
            StyleBox[\(\((list\ of)\)\ complex\ dimension \((s)\)\ of\ ideals\
\ to\ be\ kept\ \((default\ 0)\)\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["RRoots",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{
              "CompatibilityType"->0}], \(\(\[Rule]\)\(\ \)\), 
            RowBox[{
              StyleBox["\"\<Probab\>\"",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox[",",
                FontFamily->"Helvetica",
                FontSize->12], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12], 
              RowBox[{
                StyleBox["\"\<Determ\>\"",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["or",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["\"\<Off\>\"",
                  FontFamily->"Helvetica",
                  FontSize->12,
                  FontWeight->"Plain",
                  FontVariations->{"CompatibilityType"->0}], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["(",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["default",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox["\"\<Off\>\"",
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}]}], 
                  StyleBox[")",
                    FontFamily->"Helvetica",
                    FontSize->12]}]}]}]},
          {
            StyleBox["NRoots",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{
              "CompatibilityType"->0}], \(\(\[Rule]\)\(\ \)\), 
            
            StyleBox[\(minimal\ number\ of\ real\ roots\ for\ ideal\ to\ be\ \
kept\ \((default\ 0)\)\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["SturmPoly",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{
              "CompatibilityType"->0}], \(\(\[Rule]\)\(\ \)\), 
            
            StyleBox[\(\((list\ of)\)\ polynomial \((s)\)\ for\ Sturm\ query\ \
filtering\ of\ ideals\),
              FontFamily->"Helvetica",
              FontSize->12]},
          {
            StyleBox["SturmSigns",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{
              "CompatibilityType"->0}], \(\(\[Rule]\)\(\ \)\), 
            
            StyleBox[\(\((list\ of)\)\ sign \((s)\)\ for\ Sturm\ query\ \
filtering\ of\ ideals\),
              FontFamily->"Helvetica",
              FontSize->12]}
          }]],
    ShowStringCharacters->True,
    GridBoxOptions->{ColumnAlignments->{Left}}],
  StyleBox["\n",
    FontFamily->"Helvetica"],
  StyleBox["        \nGlobal options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" Calculates the saturation of the ideal with respect to the \
polynomials specified in the ",
    FontSize->12],
  StyleBox["Poly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option and then performs a primary decomposition. To switch \
calculation of the number of real roots on, the ",
    FontSize->12],
  StyleBox["RRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option should be set to one of the two methods ",
    FontSize->12],
  StyleBox["\"Probab\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12],
  StyleBox["\"Determ\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". This combines the modules ",
    FontSize->12],
  StyleBox[ButtonBox["Saturation",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" and ",
    FontSize->12],
  StyleBox[ButtonBox["PrimDec",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" at the level of ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" for faster performance.\n\n",
    FontSize->12],
  "Output:",
  StyleBox["  A list of ideal lists, each corresponding to a primary ideal. \
Ideals with complex dimensions other than those specified in option ",
    FontSize->12],
  StyleBox["Dim",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are filtered out. If real root calculation is switched on, only \
ideals with at least as many real roots as specified in option ",
    FontSize->12],
  StyleBox["NRoots",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are calculated. If options ",
    FontSize->12],
  StyleBox["SturmPoly",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontSize->12],
  StyleBox["SturmSigns ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["are provided only ideals with real roots on which the polynomials \
in ",
    FontSize->12],
  StyleBox["SturmPoly ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["assume the specified patterns of signs given in ",
    FontSize->12],
  StyleBox["SturmSigns",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" are kept.\n\n",
    FontSize->12],
  "Example:",
  StyleBox[" ",
    FontSize->12],
  StyleBox["SaturationPrimDec[IDPot/.model,t/.Gen/.IFTerms/.model]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" saturates the ideal ",
    FontSize->12],
  StyleBox["IDPot/.model ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["with respect to the real part of the F\[Dash]term corresponding \
to the variable t and performs a primary decomposition.",
    FontSize->12]
}], "Subsection"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Detailed Examples",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\n",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox["Consider a theory given by \n\t",
    FontFamily->"Arial"],
  StyleBox["a single field T = t + i \[Tau] \n\tKahler potential K = -3 Log[ \
T + Conjugate[T] ]\n\tSuperpotential W = a + b T^8 with parameters a, b",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]]
}], "Text"],

Cell[BoxData[
    \(\(model = 
        CalcModel[
          CreateModel[{T}, {t}, {\[Tau]}, \(-3\)\ \ Log[T + Conjugate[T]], 
            a + b\ T^8, Par \[Rule] {a, b}]];\)\)], "Input",
  CellLabel->"In[56]:="],

Cell[TextData[{
  "Now, form ",
  StyleBox["ideal1", "Input"],
  " which is dV, generated by the numerators of the partial derivatives of \
the scalar potential, and the polynomial ",
  StyleBox["fterm1", "Input"],
  " corresponding to the F-term for t"
}], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(ideal1 = IDPot /. model\), "\[IndentingNewLine]", 
    \(fterm1 = \(\(t /. Gen\) /. IFTerms\) /. model\)}], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \({Var \[Rule] {t, \[Tau]}, Par \[Rule] {a, b}, 
      Gen \[Rule] {t \[Rule] \(-4\)\ \((15\ a\ b\ t\^6 - 65\ b\^2\ t\^14 - 
                  189\ a\ b\ t\^4\ \[Tau]\^2 - 385\ b\^2\ t\^12\ \[Tau]\^2 + 
                  105\ a\ b\ t\^2\ \[Tau]\^4 - 945\ b\^2\ t\^10\ \[Tau]\^4 + 
                  21\ a\ b\ \[Tau]\^6 - 1225\ b\^2\ t\^8\ \[Tau]\^6 - 
                  875\ b\^2\ t\^6\ \[Tau]\^8 - 315\ b\^2\ t\^4\ \[Tau]\^10 - 
                  35\ b\^2\ t\^2\ \[Tau]\^12 + 
                  5\ b\^2\ \[Tau]\^14)\), \[Tau] \[Rule] 
            56\ \((9\ a\ b\ t\^4\ \[Tau] + 5\ b\^2\ t\^12\ \[Tau] - 
                  30\ a\ b\ t\^2\ \[Tau]\^3 + 30\ b\^2\ t\^10\ \[Tau]\^3 + 
                  9\ a\ b\ \[Tau]\^5 + 75\ b\^2\ t\^8\ \[Tau]\^5 + 
                  100\ b\^2\ t\^6\ \[Tau]\^7 + 75\ b\^2\ t\^4\ \[Tau]\^9 + 
                  30\ b\^2\ t\^2\ \[Tau]\^11 + 
                  5\ b\^2\ \[Tau]\^13)\)}}\)], "Output",
  CellLabel->"Out[57]="],

Cell[BoxData[
    \(\(-3\)\ a + 13\ b\ t\^8 - 252\ b\ t\^6\ \[Tau]\^2 + 
      350\ b\ t\^4\ \[Tau]\^4 - 28\ b\ t\^2\ \[Tau]\^6 - 
      3\ b\ \[Tau]\^8\)], "Output",
  CellLabel->"Out[58]="]
}, Open  ]],

Cell[TextData[{
  "We can saturate ",
  StyleBox["ideal1", "Input"],
  " with respect to the t F\[Dash]term ",
  StyleBox["fterm1", "Input"],
  ", setting both parameters ",
  StyleBox["(a,b)",
    FontSlant->"Italic"],
  " to 1"
}], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(sat1 = 
      Saturation[ideal1, Poly \[Rule] fterm1, 
        VarPar -> {a \[Rule] 1, b \[Rule] 1}, 
        RRoots \[Rule] "\<Probab\>"]\)], "Input",
  CellLabel->"In[59]:="],

Cell[BoxData[
    \({Gen \[Rule] {3\ t\^4\ \[Tau] - 10\ t\^2\ \[Tau]\^3 + 3\ \[Tau]\^5, 
          t\^6 - 35\ t\^2\ \[Tau]\^4 + 14\ \[Tau]\^6, 
          7\ t\^2\ \[Tau]\^5 - 3\ \[Tau]\^7, \[Tau]\^9}, Dim \[Rule] {0}, 
      NRoots \[Rule] {1}, Var \[Rule] {t, \[Tau]}, 
      Par \[Rule] {}}\)], "Output",
  CellLabel->"Out[59]="]
}, Open  ]],

Cell["\<\
We can subsequently perform a primary decomposition, keeping all \
ideals of, say, dimension 0 to 2:\
\>", "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[sat1, Dim \[Rule] {0, 1, 2}]\)], "Input",
  CellLabel->"In[60]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], t}, Dim \[Rule] {0}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[60]="]
}, Open  ]],

Cell[TextData[{
  "Alternatively, we can perform the combined task of saturation and primary \
decomposition with the current module ",
  StyleBox["SaturationPrimDec[]:", "Input"]
}], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(SaturationPrimDec[ideal1, Poly -> fterm1, 
      VarPar -> {a \[Rule] 1, b \[Rule] 1}, Dim \[Rule] {0, 1, 2}]\)], "Input",\

  CellLabel->"In[61]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], t}, Dim \[Rule] {0}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[61]="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 290, 10, 76, "Title"],
Cell[2020, 64, 577, 17, 72, "Subsection"],
Cell[2600, 83, 8322, 241, 314, "Subsection"],
Cell[10925, 326, 3291, 105, 209, "Subsection"],

Cell[CellGroupData[{
Cell[14241, 435, 116, 3, 31, "Subtitle"],
Cell[14360, 440, 372, 10, 104, "Text"],
Cell[14735, 452, 211, 5, 45, "Input"],
Cell[14949, 459, 288, 8, 28, "Text"],

Cell[CellGroupData[{
Cell[15262, 471, 162, 3, 65, "Input"],
Cell[15427, 476, 972, 15, 101, "Output"],
Cell[16402, 493, 191, 4, 44, "Output"]
}, Open  ]],
Cell[16608, 500, 264, 10, 28, "Text"],

Cell[CellGroupData[{
Cell[16897, 514, 196, 5, 45, "Input"],
Cell[17096, 521, 331, 6, 44, "Output"]
}, Open  ]],
Cell[17442, 530, 147, 4, 27, "Text"],

Cell[CellGroupData[{
Cell[17614, 538, 94, 2, 45, "Input"],
Cell[17711, 542, 153, 3, 44, "Output"]
}, Open  ]],
Cell[17879, 548, 214, 5, 28, "Text"],

Cell[CellGroupData[{
Cell[18118, 557, 169, 4, 45, "Input"],
Cell[18290, 563, 153, 3, 44, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

