(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     23343,        765]*)
(*NotebookOutlinePosition[     24182,        795]*)
(*  CellTagsIndexPosition[     24104,        789]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[StyleBox["Stringvacua Installation",
  FontSize->24]], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[CellGroupData[{

Cell[TextData[StyleBox["System requirements",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nThe package is designed for systems with an underlying \
Unix-type operating system, including various flavours of Linux and Darwin \
(OSX), as well as Cygwin for Windows. It assumes that you have a working \
installation of ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" (version 5.0 or higher) and a working version of ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" (version 3.0.1 or higher). The latter is freely available from \
the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" website at ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["http://www.singular.uni-kl.de",
    FontColor->RGBColor[0, 0, 1]],
  " ",
  StyleBox["which also provides detailed installation instructions. For \
Windows users, we require that Cygwin be installed;  it is freely available \
from ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["http://www.cygwin.com",
    FontFamily->"Helvetica",
    FontSize->12,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" which has self-explanatory installation guidance. Luckily, ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" is now a standard sub-package within Cygwin.\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Cautionary notes for Cygwin",
    FontFamily->"Helvetica",
    FontSize->12,
    FontColor->RGBColor[0.784314, 0, 0]],
  StyleBox[": \n(1) During Cygwin setup/installation, be sure to select ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" in the maths category;\n(2) Make sure you have write permission \
to the directory ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["/tmp",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" (this is usually true);\n(3) Make sure Cygwin is in your Windows \
path. This is usually done with the cygwin installation. You can check by \
running ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["cmd ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["from Start\[Rule] Run... and typing path on the shell which pops \
up; make sure ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["DRIVENAME:\\cygwin\\bin",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" is in the path (where ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["DRIVENAME",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" is the drive on which you have installed Cygwin). Otherwise, go \
to Control Panel \[Rule] Performance and Maintenance \[Rule] System, in the ",
    
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Advanced",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" tab click on the button ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Environment Variables",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", add the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["DRIVENAME:\\cygwin\\bin",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" explicitly.\n\nFor a run-time ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" protocol the package uses a standard xterm, which will be \
available on most Unix systems. This protocol feature can be switched off in \
which case xterm will not be required. On the Cygwin system, the xterm option \
is currently disabled to avoid incompatibility with Windows.",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Installation procedure",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nFirst download the file ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Stringvacua.tar ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["and save it in the directory ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["$UserBaseDirectory/Applications",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". Execute ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["$UserBaseDirectory",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" to find the explicit path on your particular architecture. On \
Linux systems it is typically ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["~/.Mathematica/Applications, ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["on OSX it is ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["~/Library/Mathematica/Applications",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and on Windows/Cygwin, it is typically ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["/cygdrive/c/Documents\\ and\\ Settings/USERNAME/Application\\ \
Data/Mathematica/Applications ",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" \n(or ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["C:\\Documents/ and/ Settings\\USERNAME\\Application/ \
Data\\Mathematica",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" in Windows format).\nThen, in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["$UserBaseDirectory/Applications",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", untar the file using \n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["tar -xf Stringvacua.tar\n", "Input",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Then, the directory ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["$UserBaseDirectory/Applications",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" contains the file ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Stringvacua.m",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and the two sub-directories ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Stringvacua",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Stringvacualib",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". Note that the file structure has to be precisely of this form \
for the package to work properly. If you use the above shell tar command this \
will be automatic. \nBeware that other tar software (such as StuffIt and \
WinZip) often creates a sub-directory so that the file ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Stringvacua.m",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and the two package sub-directories end up in a folder which may \
be called ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["$UserBaseDirectory/Applications/Stringvacua Folder",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". In this case, the package content has to be moved back to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["$UserBaseDirectory/Applications",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". This completes the installation.",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Running for the first time",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nStart up ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[". In the case of Macintosh and unix systems, start up X11 if it \
is not already running. If you are using the notebook interface on ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" versions less than 6.0, open the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Help",
    FontFamily->"Helvetica",
    FontSize->12,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" pop-up menu and execute ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Help:Rebuild Help Index",
    FontFamily->"Helvetica",
    FontSize->12,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[". This will install the browser help for the package in the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["AddOns&Links ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["section of the help browser, which should now contain a new entry \
",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Stringvacua",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".   On ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" 6.0, there is no need to rebuild the help index. Of course,  if \
you are using the text interface, there is no such step.\nThen simply \
evaluate (note the backward apostrophe):\n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["<<", "Input",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Stringvacua`\n", "Input",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["to load the package. To get started follow the help system or the \
tutorial which is available on the package web page.",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Limitations",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nThe package is using a fixed set of files to handle the \
exchange of data between ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox["and ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[". This means there should be only one ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" call from the package at any given time. For example, running \
two notebooks and package modules in each at the same time will lead to \
problems. In addition, although the package is compatible with the use of ssh \
forwarding, problems will occur if the user employs the internal ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" option to run the kernel on a remote machine.",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Troubleshooting",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["When I load the package into ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontVariations->{"Underline"->True}],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic",
    FontVariations->{"Underline"->True}],
  StyleBox[" I am being told the xterm or ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontVariations->{"Underline"->True}],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic",
    FontVariations->{"Underline"->True}],
  StyleBox[" executables have not been found.\n",
    FontFamily->"Helvetica",
    FontSize->12,
    FontVariations->{"Underline"->True}],
  StyleBox["This means the executables for xterm or ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" have not been found (as you might have guessed). The package is \
looking for these executables in the standard places, so your system probably \
has a non-standard installation. This problem can be easily fixed by \
following the instructions in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["XtermPath",
    ButtonData:>"Stringvacua Options",
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" and",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->12,
    ButtonBoxOptions->{ButtonData:>"Stringvacua Options",
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData}],
  StyleBox[ButtonBox["SingularPath",
    ButtonData:>"Stringvacua Options",
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  CellTags->"missingpaths"],

Cell[TextData[{
  StyleBox["The system manager was too lazy to install ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontVariations->{"Underline"->True}],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic",
    FontVariations->{"Underline"->True}],
  StyleBox[". Can I still use the package?",
    FontFamily->"Helvetica",
    FontSize->12,
    FontVariations->{"Underline"->True}],
  StyleBox["\nYes! Just install ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" yourself somewhere in your home directory tree. It's easy and \
the details are described on the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" home page. If you install so that the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" executable (or a soft link to it) is in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["~/bin",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["~/usr/bin",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" it will be automatically found by the package. Otherwise, read ",
    
    FontFamily->"Helvetica",
    FontSize->12],
  ButtonBox["this",
    ButtonData:>"missingpaths",
    ButtonStyle->"Hyperlink"],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"],

Cell[TextData[{
  StyleBox["If I use a module which calls ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontVariations->{"Underline"->True}],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic",
    FontVariations->{"Underline"->True}],
  StyleBox[", ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontVariations->{"Underline"->True}],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic",
    FontVariations->{"Underline"->True}],
  StyleBox[" just hangs and nothing happens. ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontVariations->{"Underline"->True}],
  StyleBox["\nFirst check if both ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" and xterm actually work (for example by starting them from an \
xterm) and that X11 is running. Make sure that the package has found the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" and xterm executables. This is the case if no warning message is \
displayed when loading the package. Otherwise, read ",
    FontFamily->"Helvetica",
    FontSize->12],
  ButtonBox["this",
    ButtonData:>"missingpaths",
    ButtonStyle->"Hyperlink"],
  StyleBox[". If the problem persists there is likely to be a problem with \
the scripts which handle the interaction between ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" and ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[". Although these scripts have been tested on a number of \
different architectures, there is no guarantee for all flavours of unix. In \
this case you can first try to switch off the xterm protocol feature by \
executing ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["SetOptions[Stringvacua,SingularProtocol\[Rule]\"None\"]", "Input",
    
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". If this doesn't help try switching off the intermediate scripts \
completely by executing ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["SetOptions[Stringvacua,SingularTimer\[Rule]\"off\"]", "Input",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" will then be directly called from ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" and things should definitely work. However, in this mode, ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calls cannot be timed and there is no protocol for ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculations. (See ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" for details on the above option settings and how to configure \
start-up values for those options.)",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 741}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "missingpaths"->{
    Cell[15526, 506, 1990, 64, 65, "Text",
      CellTags->"missingpaths"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"missingpaths", 24002, 782}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 124, 3, 76, "Title"],

Cell[CellGroupData[{
Cell[1903, 58, 118, 3, 31, "Subtitle"],
Cell[2024, 63, 4295, 133, 295, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6356, 201, 121, 3, 31, "Subtitle"],
Cell[6480, 206, 4835, 152, 274, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11352, 363, 125, 3, 31, "Subtitle"],
Cell[11480, 368, 2482, 79, 198, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13999, 452, 110, 3, 31, "Subtitle"],
Cell[14112, 457, 1260, 39, 84, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15409, 501, 114, 3, 31, "Subtitle"],
Cell[15526, 506, 1990, 64, 65, "Text",
  CellTags->"missingpaths"],
Cell[17519, 572, 1881, 65, 67, "Text"],
Cell[19403, 639, 3924, 123, 181, "Text"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

