(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     10826,        353]*)
(*NotebookOutlinePosition[     11510,        376]*)
(*  CellTagsIndexPosition[     11466,        372]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Stringvacua Interface",
  FontSize->24]], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[StyleBox["Overview",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["\nThe package is designed to be used exclusively from ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" without any need to deal with ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" directly. \n\nIn ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Unix / Macintosh",
    FontFamily->"Helvetica",
    FontSize->12,
    FontColor->RGBColor[0.784314, 0, 0]],
  StyleBox[" operating systems, normally a protocol of the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculation is displayed in an xterm which is automatically \
opened (make sure in such cases an X-server is already running). Although \
this xterm protocol can be turned off by setting the global variable ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["SingularProtocol",
    ButtonData:>"Stringvacua Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\"None\"",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" it should normally be kept as it provides a way of monitoring \
and, if appropriate, interrupting long calculations; ctrl\[Dash]c in the \
xterm will interrupt the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculation and triggers the return to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[".\n\nThe package also provides a built-in timer with an upper \
time limit stored in the global variable ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["SingularMaxTime",
    ButtonData:>"Stringvacua Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".  When a ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculation exceeds this time limit it will be automatically \
stopped and the process returns to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[". Some modules, such as ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[",  perform multiple calls to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[".  Each such call is then timed individually. In such as case, a \
ctrl\[Dash]c signal in the xterm will stop the currently running ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" process but not subsequent ones. To terminate such a module \
completely, perform first a ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" interrupt and then a ctrl\[Dash]c in the ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" xterm. When interrupted or timed out, the data is not completely \
lost, any ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculations which have been already performed will be \
automatically returned to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[".\n\nIn ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Windows (Cygwin),",
    FontFamily->"Helvetica",
    FontSize->12,
    FontColor->RGBColor[0.784314, 0, 0]],
  StyleBox[" the xterm option is currently disabled to avoid issues with \
X-Cygwin. To avoid issues of popping out DOS shells, ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" is called silently and to abort a calculation use the standard \
\"Abort Evaluation\" or \"Interrupt Evaluation\" within ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" (the latter is preferred since it seems to be more responsive). \
Some modules, such as ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["SaturationExpand",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[",  perform multiple calls to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[".   In such a case, each abort will stop the currently running ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" process but not subsequent ones. Currently, no timer has been \
inplemented on Windows, i.e., the only way to terminate a ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" computation is by Abort. When interrupted, the data is not \
completely lost, any ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculations which have been already performed will be \
automatically returned to ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[".\n\n\nThe relation between ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Mathematica",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" and ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" is determined by a number of global options listed under ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonData:>"Stringvacua Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" and a number of global status variables listed under ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Status",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".\nVariables for fields or parameters must be of the form \n\t",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["_Symbol",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Subscript[_Symbol,_Integer]",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Subscript[_Symbol,_Symbol]",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["_Symbol[_Integer]",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" \nto be recognised by the package and be accurately converted \
for use in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Helvetica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
WindowSize->{1016, 666},
WindowMargins->{{1, Automatic}, {Automatic, 3}},
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 121, 3, 76, "Title"],
Cell[1851, 57, 85, 2, 31, "Subtitle"],
Cell[1939, 61, 8871, 289, 502, "Text"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

