(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     22771,        560]*)
(*NotebookOutlinePosition[     23455,        583]*)
(*  CellTagsIndexPosition[     23411,        579]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Stringvacua Modules",
  FontSize->24]], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[StyleBox["Overview",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["This page provides a brief summary of the modules provided by the \
Stringvacua package. For detailed help on any of the modules click the module \
name in the list below. Most modules have a fairly small number of fixed \
arguments and a set of ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["local",
    ButtonData:>"Module Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" and ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["global",
    ButtonData:>"Stringvacua Options",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" options. For more detailed information on the optional arguments \
for each module see its help page. ",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  CellDingbat->None,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"],

Cell[TextData[{
  StyleBox["List of Stringvacua modules ",
    FontFamily->"Arial",
    FontSize->18],
  "\n\n",
  Cell[BoxData[GridBox[{
          {
            ButtonBox[
              ButtonBox[
                ButtonBox["CalcModel",
                  ButtonStyle->"AddOnsLink"],
                ButtonStyle->"AddOnsLink"],
              ButtonFunction:>(FrontEndExecute[ {
                  FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
              ButtonData:>CalcModel,
              Active->True,
              ButtonStyle->"AddOnsLink",
              ButtonSource->ButtonData], GridBox[{
                {\(Calculates\ ideals\ in\ a\ model\ \(\(list\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["CalcModel",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(model\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["CallSingular",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {
                  RowBox[{"Low", "-", 
                    RowBox[{"level", " ", "call", " ", 
                      RowBox[{
                        StyleBox[
                          RowBox[{"of", 
                            StyleBox["Singular",
                              FontSlant->"Italic"]}]], "."}]}]}]},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["CallSingular",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      
                      StyleBox[\(script, 
                        varpara, \(\(repl\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            RowBox[{
              ButtonBox["CreateIdeal",
                ButtonStyle->"AddOnsLink"], " "}], GridBox[{
                {\(Creates\ an\ ideal\ \(\(list\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["CreateIdeal",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["CreateModel",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Creates\ a\ model\ list, \ 
                  calculates\ scalar\ potential\ \(\(etc\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["CreateModel",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      
                      StyleBox[\(fields, refields, 
                        i\[AlignmentMarker]mfields, 
                        K, \(\(W\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["DimIdeal",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Calculates\ the\ dimension\ of\ an\ \(\(ideal\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["DimIdeal",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["Elimination",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Calculates\ the\ ideal\ obtained\ by\ eliminating\ a\ set\ \
of\ \(\(variables\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["Elimination",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["FacGroebner",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Performs\ the\ factorizing\ Buchberger\ algorithm\ on\ a\ \
set\ of\ generators\ of\ an\ \(\(ideal\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["FacGroebner",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            RowBox[{
              ButtonBox["NumRoots",
                ButtonStyle->"AddOnsLink"], " "}], GridBox[{
                {\(Calculates\ the\ number\ of\ real\ roots\ of\ a\ zero - 
                    dimensional\ \(\(ideal\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["NumRoots",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            RowBox[{
              ButtonBox["PrimDec",
                ButtonStyle->"AddOnsLink"], " "}], GridBox[{
                {\(Primary\ decomposition\ of\ an\ \(\(ideal\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["PrimDec",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["RemoveEntry",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Removes\ an\ entry\ from\ a\ \(\(list\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["RemoveEntry",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(list, keyword\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["Saturation",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Form\ an\ element\ of\ the\ saturation\ expansion\ of\ an\ \
\(\(ideal\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["Saturation",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["SaturationExpand",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Computes\ the\ saturation\ expansion\ for\ a\ model\ and\ \
primary\ \(\(decomposes\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["SaturationExpand",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(model\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["SaturationPrimDec",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Form\ an\ element\ of\ the\ saturation\ expansion\ of\ an\ \
ideal\ and\ primary\ \(\(decomposes\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox["SaturationPrimDec",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["SimpleMinimise",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(A\ simple\ interactive\ minimisation\ module\ as\ a\ first\
\ trial\ of\ this\ \(\(package\)\(.\)\)\)},
                {
                  RowBox[{"Usage", ":", 
                    StyleBox[\(SimpleMinimise[]\),
                      "Input",
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}]}]}
                }]},
          {
            ButtonBox["SturmQuery",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {
                  RowBox[{
                  "Calculates", " ", "the", " ", "sign", " ", "of", " ", 
                    "various", " ", "objects", " ", "on", " ", "the", " ", 
                    "real", " ", "roots", " ", "of", " ", "a", " ", "zero", 
                    " ", "dimensional", " ", 
                    RowBox[{"ideal", ".", 
                      StyleBox[" ",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]},
                {
                  RowBox[{"Usage", ":", 
                    RowBox[{
                      StyleBox[\(Stur  m  Query\),
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox[\(\(\(ideal\)\(<\)\), \(\(options\)\(>\)\)\),
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic",
                        FontVariations->{"CompatibilityType"->0}], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain",
                        FontVariations->{"CompatibilityType"->0}]}]}]}
                }]},
          {
            ButtonBox["TF",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Displays\ model, \ 
                  ideal\ or\ other\ lists\ in\ table\ \(\(form\)\(.\)\)\)},
                {
                  RowBox[{"Usage", " ", ":", 
                    RowBox[{
                      StyleBox["TF",
                        "Input",
                        FontWeight->"Plain"], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain"], 
                      StyleBox["list",
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic"], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain"]}]}]}
                }]},
          {
            ButtonBox["ThomEncode",
              ButtonStyle->"AddOnsLink"], GridBox[{
                {\(Thom\ encodes\ the\ real\ roots\ of\ zero\ dimensional\ \(\
\(ideals\)\(.\)\)\)},
                {
                  RowBox[{"Usage", " ", ":", 
                    RowBox[{
                      StyleBox["ThomEncode",
                        "Input",
                        FontWeight->"Plain"], 
                      StyleBox["[",
                        "Input",
                        FontWeight->"Plain"], 
                      StyleBox["ideal",
                        "Input",
                        FontWeight->"Plain",
                        FontSlant->"Italic"], 
                      StyleBox["]",
                        "Input",
                        FontWeight->"Plain"]}]}]}
                }]}
          }]],
    FontFamily->"Arial",
    GridBoxOptions->{RowAlignments->Top,
    ColumnAlignments->{Left}}]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
WindowSize->{1144, 683},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 119, 3, 76, "Title"],
Cell[1849, 57, 85, 2, 31, "Subtitle"],
Cell[1937, 61, 1031, 32, 46, "Text"],
Cell[2971, 95, 19784, 462, 777, "Subsection"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

