(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     31385,        970]*)
(*NotebookOutlinePosition[     34015,       1053]*)
(*  CellTagsIndexPosition[     33510,       1034]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Stringvacua Options",
  FontSize->24]], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[StyleBox["Overview",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["The ",
    FontFamily->"Arial"],
  StyleBox["Stringvacua",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" package and its relation to ",
    FontFamily->"Arial"],
  StyleBox["Singular",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" is controlled by a number of global options whose current values \
can be listed with ",
    FontFamily->"Arial"],
  StyleBox["Options[Stringvacua]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" and changed with the ",
    FontFamily->"Arial"],
  StyleBox["Mathematica",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" ",
    FontFamily->"Arial"],
  StyleBox["SetOptions", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" command.  Sensible defaults are set by the system. Start-up \
values for these options can be defined in the file \"StringvacuaOptions.m\" \
which is included with the package, see ",
    FontFamily->"Arial"],
  StyleBox[ButtonBox["here",
    ButtonData:>"start-up",
    ButtonStyle->"Hyperlink"],
    FontFamily->"Arial"],
  StyleBox[" for details.",
    FontFamily->"Arial"]
}], "Text",
  CellDingbat->None,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"],

Cell[TextData[{
  StyleBox["List of global options ",
    FontFamily->"Arial",
    FontSize->18],
  StyleBox["\n\n",
    FontSize->18],
  Cell[BoxData[
      StyleBox[GridBox[{
            {
              StyleBox[
                RowBox[{
                  ButtonBox["FieldChar",
                    ButtonData:>"FieldChar",
                    ButtonExpandable->False,
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(Prime\ integer\ between\ 0\ and\ 2147483629, 
                default\ 0. \ \[IndentingNewLine]
                
                Characteristic\ of\ base\ number\ field\ of\ the\ polynomial\ \
\(\(ring\)\(.\)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["PackageProtocol",
                    ButtonData:>"PackageProtocol",
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(Integer\ from\ 0\ to\ 10, 
                level\ of\ protocol\ for\ package\ \(\(modules\)\(.\)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["PlanckInverse",
                    ButtonData:>"PlanckInverse",
                    ButtonStyle->"Hyperlink"], "     "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(Number\ or\ symbol, 
                value\ of\ inverse\ Planck\ \(\(mass\)\(.\)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                ButtonBox["SingularAccuracy",
                  ButtonData:>"SingularAccuracy",
                  ButtonStyle->"Hyperlink"],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(Real, 
                accuracy\ for\ converting\ real\ into\ rational\ \
\(\(numbers\)\(.\)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["SingularEcho",
                    ButtonData:>"SingularEcho",
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(Integer\ from\ 0\ to\ 10, 
                level\ of\ input\ echo\ of\ the\ Singular\ \
\(\(protocol\)\(.\)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["SingularMaxTime",
                    ButtonData:>"SingularMaxTime",
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(Integer, 
                time\ limit\ for\ a\ single\ call\ to\ Singular\ \(\(\((in\ \
seconds)\) . \ \[IndentingNewLine]\((This\ is\ currently\ not\ an\ option\ \
for\ Windows/Cygwin)\)\)\(.\)\(\ \)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["SingularPath",
                    ButtonData:>"SingularPath",
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox[\(Path\ to\ Singular\ \(\(executable\)\(.\)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["SingularProtocol",
                    ButtonData:>"SingularProtocol",
                    ButtonStyle->"Hyperlink"], "\[IndentingNewLine]"}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                
                ShowStringCharacters->
                  False], \("\<Xterm\>", \ "\<String\>"\ or\ "\<None\>", \ 
              determines\ form\ of\ the\ Singular\ \(\(protocol\)\(.\)\(\
\[IndentingNewLine]\)\(\ \ \ \)\((This\ is\ currently\ not\ an\ option\ for\ \
Windows/Cygwin)\)\(.\)\(\ \)\)\)},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["SingularTimer",
                    ButtonData:>"SingularTimer",
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              RowBox[{\("\<on\>"\ or\ "\<off\>"\), 
                StyleBox[",",
                  ShowSpecialCharacters->True,
                  ShowStringCharacters->False], 
                
                StyleBox[\(to\ switch\ timer\ on\ or\ \(\(off . \ \
\[IndentingNewLine]\((This\ is\ currently\ not\ an\ option\ for\ Windows/
                              Cygwin)\)\)\(.\)\(\ \)\)\),
                  ShowSpecialCharacters->True,
                  ShowStringCharacters->False]}]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["SingularTrace",
                    ButtonData:>"SingularTrace",
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(Integer\ from\ 0\ to\ 4, 
                level\ of\ detail\ of\ the\ Singular\ \(\(protocol\)\(.\)\)\),
                
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["XtermGeometry",
                    ButtonData:>"XtermGeometry",
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(String, 
                specifying\ the\ geometry\ of\ \(\(xterm . \ \
\[IndentingNewLine]\((This\ is\ currently\ not\ an\ option\ for\ Windows/
                            Cygwin)\)\)\(.\)\(\ \)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]},
            {
              StyleBox[
                RowBox[{
                  ButtonBox["XtermPath",
                    ButtonData:>"XtermPath",
                    ButtonStyle->"Hyperlink"], " "}],
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              StyleBox["\[Rule]",
                ShowSpecialCharacters->True,
                ShowStringCharacters->False], 
              
              StyleBox[\(Path\ to\ xterm\ \(\(executable . \ \
\[IndentingNewLine]\((This\ is\ currently\ not\ an\ option\ for\ Windows/
                            Cygwin)\)\)\(.\)\(\ \)\)\),
                ShowSpecialCharacters->True,
                ShowStringCharacters->False]}
            }],
        ShowSpecialCharacters->True,
        AutoStyleOptions->{"UnmatchedBracketStyle"->None},
        ShowStringCharacters->True]],
    AutoStyleOptions->{"UnmatchedBracketStyle"->None},
    FontFamily->"Arial",
    GridBoxOptions->{ColumnAlignments->{Left}}]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[StyleBox["Detailed description of global options",
  FontSize->18]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  "Global option ",
  StyleBox["FieldChar",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Characteristic of the number field underlying polynomial ring for \
",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculations. Must be a prime integer between 0 and 2147483629. \
Default is 0 which corresponds to the rationals. However, polynomial \
manipulations in finite fields of non-zero characteristic are usually \
significantly faster.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"FieldChar"],

Cell[TextData[{
  "Global option ",
  StyleBox["PackageProtocol ",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Ranges from 0 (no protocol) to 10 and determines the level of \
protocol for package modules which perform multiple ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calls. Useful to follow the progress of longer calculations. \
Default is 10.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"PackageProtocol"],

Cell[TextData[{
  "Global option ",
  StyleBox["PlanckInverse",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Value of inverse Planck mass. Default value is 1. Set to 0 in \
order to obtain the global SUSY potential.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"PlanckInverse"],

Cell[TextData[{
  "Global option ",
  StyleBox["SingularAccuracy",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Accuracy for converting real into rational numbers before a ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" call (default 10^(-5)).",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SingularAccuracy"],

Cell[TextData[{
  "Global option ",
  StyleBox["SingularEcho",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Takes values 0,..,10 which determine the level of input echo of \
the ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" protocol. Default is 0.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SingularEcho"],

Cell[TextData[{
  "Global option ",
  StyleBox["SingularMaxTime",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Time limit for a single call to ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" (in seconds). If the ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calculation exceeds this limit, the calculation is timed out and \
returns to ",
    FontSize->12],
  StyleBox["Mathematica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[". The default is 60 seconds.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SingularMaxTime"],

Cell[TextData[{
  "Global option ",
  StyleBox["SingularPath",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["This contains the path to the Singular executable. The package \
searches for this path at start-up relying on a list of standard paths. For \
Singular installed in a standard location everything should be automatic. If \
more than one version of Singular is found the user is prompted to make a \
choice. If the path is not found the package displays a warning at start-up. \
In case of problems, follow the steps described under option",
    FontSize->12],
  StyleBox[" ",
    FontSize->12],
  StyleBox[ButtonBox["XtermPath",
    ButtonData:>"XtermPath",
    ButtonStyle->"Hyperlink"],
    FontSize->12],
  StyleBox[".",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SingularPath"],

Cell[TextData[{
  "Global option ",
  StyleBox["SingularProtocol ",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Determines the protocol mode for ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" calls.  If running ",
    FontSize->12],
  StyleBox["Mathematica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" in Notebook form, this can be set to ",
    FontSize->12],
  StyleBox["\"Xterm\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", ",
    FontSize->12],
  StyleBox["\"String\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12],
  StyleBox["\"None\":",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\nIf set to ",
    FontSize->12],
  StyleBox["\"Xterm\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" the ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" protocol is shown in a separate xterm. \nIf set to ",
    FontSize->12],
  StyleBox["\"String\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" the protocol is stored in the",
    FontSize->12],
  StyleBox[" ",
    FontSize->12],
  StyleBox[ButtonBox["global variable",
    ButtonData:>"Stringvacua Status",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[" ",
    FontSize->12,
    ButtonBoxOptions->{ButtonData:>"Stringvacua Status",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData}],
  StyleBox["SingularProt",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". \nIf set to ",
    FontSize->12],
  StyleBox["\"None\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" no xterm is evoked. \nIf the xterm executable is found the \
default is ",
    FontSize->12],
  StyleBox["\"Xterm\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", otherwise it is ",
    FontSize->12],
  StyleBox["\"None\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".\nIf running ",
    FontSize->12],
  StyleBox["Mathematica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" in Text Mode, and ",
    FontSize->12],
  StyleBox["SingularProtocol ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["is set to ",
    FontSize->12],
  StyleBox["\"None\"",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[",",
    FontSize->12],
  StyleBox[" no xterm is evoked and the ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" protocol is piped direct to the screen inside the ",
    FontSize->12],
  StyleBox["math",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" session.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SingularProtocol"],

Cell[TextData[{
  "Global option ",
  StyleBox["SingularTimer",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Possible values are ",
    FontSize->12],
  StyleBox["\"on\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" or ",
    FontSize->12],
  StyleBox["\"off\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". If set to ",
    FontSize->12],
  StyleBox["\"on\" ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["scripts will handle the communication between ",
    FontSize->12],
  StyleBox["Mathematica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" and Singular. In this case the timing and protocol features \
controlled by the global options ",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        StyleBox[
          ButtonBox["SingularProtocol",
            ButtonData:>"SingularProtocol",
            ButtonStyle->"Hyperlink"],
          FontFamily->"Arial",
          FontSize->12], TraditionalForm]]],
  StyleBox[" and ",
    FontSize->12],
  Cell[BoxData[
      FormBox[
        StyleBox[
          ButtonBox["SingularMaxTime",
            ButtonData:>"SingularMaxTime",
            ButtonStyle->"Hyperlink"],
          FontFamily->"Arial",
          FontSize->12], TraditionalForm]]],
  StyleBox[" are available. If set to ",
    FontSize->12],
  StyleBox["\"off\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" Singular will be called directly from ",
    FontSize->12],
  StyleBox["Mathematica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[". Timing and interrupting the Singular call will then usually not \
be possible. This simple calling mode can be useful on machines where the \
intermediate handling scripts cause problems.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SingularTimer"],

Cell[TextData[{
  "Global option ",
  StyleBox["SingularTrace",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["Takes values 0,..,4 which determine the level of detail of the ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" protocol. Default is 0.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"SingularTrace"],

Cell[TextData[{
  "Global option ",
  StyleBox["XtermGeometry",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["A string specifying the geometry (i.e. size and position on the \
screen) of the xterm which is opened when the global option ",
    FontSize->12],
  StyleBox[ButtonBox["SingularProtocol",
    ButtonData:>"SingularProtocol",
    ButtonStyle->"Hyperlink"],
    FontSize->12],
  StyleBox[" is set to ",
    FontSize->12],
  StyleBox["\"Xterm\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". The format of the string exactly corresponds to the argument of \
the ",
    FontSize->12],
  StyleBox["-geometry",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option of the ",
    FontSize->12],
  StyleBox["xterm",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" command (see the xterm man pages). \nDefault is ",
    FontSize->12],
  StyleBox["\"50x10-50+0\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" which creates an xterm of length 50 and height 10 at horizontal \
position 50 from the right of the screen and vertical position 0 from the top \
of the screen.",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"XtermGeometry"],

Cell[TextData[{
  "Global option ",
  StyleBox["XtermPath",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "\n",
  StyleBox["This contains the path to the ",
    FontSize->12],
  StyleBox["xterm",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" executable. The package searches for this path at start-up \
relying on a list of standard paths. For ",
    FontSize->12],
  StyleBox["xterm",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" installed in a standard location everything should be automatic. \
\n\nIf the path is not found the package displays a warning at start-up. In \
this case there are two simple options to fix the problem which, however, \
require explicit knowledge of the path, ",
    FontSize->12],
  StyleBox["path/to/xterm",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", to the ",
    FontSize->12],
  StyleBox["xterm",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" executable:\n1) You can set the path manually within ",
    FontSize->12],
  StyleBox["Mathematica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" by executing the command ",
    FontSize->12],
  StyleBox["SetOptions[Stringvacua,XtermPath \[Rule] \"/path/to/xterm\"].", 
    "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["This setting will only last for the current ",
    FontSize->12],
  StyleBox["Mathematica",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" session. \n2) Add the entry ",
    FontSize->12],
  StyleBox[" \"/path/to/xterm\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" to the list of paths in the file \"StringvacuaPaths.m\" in ",
    FontSize->12],
  StyleBox["$UserBaseDirectory<>\"/Applications/Stringvacualib\"",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", to have it loaded automatically at start-up (see ",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"paths",
    ButtonStyle->"Hyperlink"],
    FontSize->12],
  StyleBox[" for details).",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  Background->GrayLevel[0.833326],
  CellTags->"XtermPath"],

Cell[TextData[StyleBox["Configuring start-up default values for global \
options",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  CellTags->"start-up"],

Cell[TextData[{
  "The Stringvacua package comes with a file \"StringvacuaOptions.m\" which, \
along with other package files, resides in the directory ",
  StyleBox["$UserBaseDirectory/Applications/Stringvacualib ",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "(Execute ",
  StyleBox["$UserBaseDirectory",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  " in ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "to find the explicit path for your particular architecture). It is a \
simple text file with default content"
}], "Text",
  CellDingbat->None,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    StyleBox[\(Options[Stringvacua]\),
      "Input"]], "Input",
  CellLabel->"In[17]:=",
  CellDingbat->None,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"],

Cell[BoxData[
    \({PlanckInverse \[Rule] 1, PackageProtocol \[Rule] 10, 
      FieldChar \[Rule] 0, SingularProtocol \[Rule] "Xterm", 
      SingularTimer \[Rule] "on", SingularMaxTime \[Rule] 60, 
      SingularTrace \[Rule] 0, SingularEcho \[Rule] 0, 
      SingularAccuracy \[Rule] 1\/100000, XtermGeometry \[Rule] "50x10-50+0", 
      XtermPath \[Rule] "/usr/X11R6/bin/xterm", 
      SingularPath \[Rule] "/sw/bin/Singular"}\)], "Output",
  CellLabel->"Out[17]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["\n",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "In order to change this default simply insert new values into the above \
assignments and save the file. For example, replacing ",
  StyleBox["SingularMaxTime \[Rule] 60",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  " by ",
  StyleBox["SingularMaxTime \[Rule] 10",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  " implies that the package starts up in a configuration where ",
  StyleBox["Singular",
    FontSlant->"Italic"],
  " calls are timed out after 10 rather than 60 seconds. Replacing ",
  StyleBox["XtermGeometry \[Rule] \"50x10-50+0\"",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  " by ",
  StyleBox["XtermGeometry \[Rule] \"80x50+0+0\"",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  " leads to a large default xterm in the upper left corner, rather than a \
small one in the upper right corner. "
}], "Text",
  CellDingbat->None,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"],

Cell[TextData[StyleBox["Configuring start-up default values for search paths",
  
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  CellTags->"paths"],

Cell[TextData[{
  "The Stringvacua package comes with a file \"StringvacuaPaths.m\" which, \
along with other package files, resides in the directory ",
  StyleBox["$UserBaseDirectory<>\"/Applications/Stringvacualib\" ",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  "(Execute ",
  StyleBox["$UserBaseDirectory",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  " in ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "to find the explicit path for your particular architecture). It is a \
simple text file with default content"
}], "Text",
  CellDingbat->None,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Get", "[", 
      StyleBox[\($UserBaseDirectory <> \
"\</Applications/Stringvacualib/StringvacuaPaths.m\>"\),
        "Input"], 
      StyleBox["]",
        "Input"]}]], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \({"/usr/local/sbin", "/usr/sbin", "/sbin", "/usr/local/bin", "/bin", 
      "/usr/bin", "/usr/X11R6/bin", "/sw/bin", "/Users/andre/bin", 
      "/Users/andre/usr/bin"}\)], "Output",
  CellLabel->"Out[20]="]
}, Open  ]],

Cell[TextData[{
  "At start-up, the package will be looking for the xterm and Singular \
executables in those directories and the paths it finds will be stored in the \
global options ",
  ButtonBox["XtermPath",
    ButtonData:>"XtermPath",
    ButtonStyle->"Hyperlink"],
  " and ",
  ButtonBox["SingularPath",
    ButtonData:>"SingularPath",
    ButtonStyle->"Hyperlink"],
  ". If the executables are not found in those directories a warning will be \
displayed at start-up. The problem can then be easily fixed by adding the \
appropriate paths to the above list."
}], "Text",
  CellDingbat->None,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold"]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{187, Automatic}, {Automatic, 8}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "FieldChar"->{
    Cell[12659, 339, 669, 20, 104, "Subsection",
      CellTags->"FieldChar"]},
  "PackageProtocol"->{
    Cell[13331, 361, 577, 18, 104, "Subsection",
      CellTags->"PackageProtocol"]},
  "PlanckInverse"->{
    Cell[13911, 381, 383, 12, 87, "Subsection",
      CellTags->"PlanckInverse"]},
  "SingularAccuracy"->{
    Cell[14297, 395, 468, 16, 87, "Subsection",
      CellTags->"SingularAccuracy"]},
  "SingularEcho"->{
    Cell[14768, 413, 470, 17, 87, "Subsection",
      CellTags->"SingularEcho"]},
  "SingularMaxTime"->{
    Cell[15241, 432, 748, 27, 104, "Subsection",
      CellTags->"SingularMaxTime"]},
  "SingularPath"->{
    Cell[15992, 461, 895, 24, 121, "Subsection",
      CellTags->"SingularPath"]},
  "SingularProtocol"->{
    Cell[16890, 487, 3314, 123, 189, "Subsection",
      CellTags->"SingularProtocol"]},
  "SingularTimer"->{
    Cell[20207, 612, 2067, 68, 140, "Subsection",
      CellTags->"SingularTimer"]},
  "SingularTrace"->{
    Cell[22277, 682, 465, 16, 87, "Subsection",
      CellTags->"SingularTrace"]},
  "XtermGeometry"->{
    Cell[22745, 700, 1465, 45, 138, "Subsection",
      CellTags->"XtermGeometry"]},
  "XtermPath"->{
    Cell[24213, 747, 2444, 74, 223, "Subsection",
      CellTags->"XtermPath"]},
  "start-up"->{
    Cell[26660, 823, 157, 4, 31, "Subtitle",
      CellTags->"start-up"]},
  "paths"->{
    Cell[29365, 905, 153, 4, 31, "Subtitle",
      CellTags->"paths"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"FieldChar", 32065, 988},
  {"PackageProtocol", 32168, 991},
  {"PlanckInverse", 32275, 994},
  {"SingularAccuracy", 32382, 997},
  {"SingularEcho", 32488, 1000},
  {"SingularMaxTime", 32593, 1003},
  {"SingularPath", 32699, 1006},
  {"SingularProtocol", 32806, 1009},
  {"SingularTimer", 32916, 1012},
  {"SingularTrace", 33022, 1015},
  {"XtermGeometry", 33126, 1018},
  {"XtermPath", 33228, 1021},
  {"start-up", 33325, 1024},
  {"paths", 33413, 1027}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 119, 3, 76, "Title"],
Cell[1849, 57, 85, 2, 31, "Subtitle"],
Cell[1937, 61, 1357, 43, 68, "Text"],
Cell[3297, 106, 9241, 227, 522, "Subsection"],
Cell[12541, 335, 115, 2, 31, "Subtitle"],
Cell[12659, 339, 669, 20, 104, "Subsection",
  CellTags->"FieldChar"],
Cell[13331, 361, 577, 18, 104, "Subsection",
  CellTags->"PackageProtocol"],
Cell[13911, 381, 383, 12, 87, "Subsection",
  CellTags->"PlanckInverse"],
Cell[14297, 395, 468, 16, 87, "Subsection",
  CellTags->"SingularAccuracy"],
Cell[14768, 413, 470, 17, 87, "Subsection",
  CellTags->"SingularEcho"],
Cell[15241, 432, 748, 27, 104, "Subsection",
  CellTags->"SingularMaxTime"],
Cell[15992, 461, 895, 24, 121, "Subsection",
  CellTags->"SingularPath"],
Cell[16890, 487, 3314, 123, 189, "Subsection",
  CellTags->"SingularProtocol"],
Cell[20207, 612, 2067, 68, 140, "Subsection",
  CellTags->"SingularTimer"],
Cell[22277, 682, 465, 16, 87, "Subsection",
  CellTags->"SingularTrace"],
Cell[22745, 700, 1465, 45, 138, "Subsection",
  CellTags->"XtermGeometry"],
Cell[24213, 747, 2444, 74, 223, "Subsection",
  CellTags->"XtermPath"],
Cell[26660, 823, 157, 4, 31, "Subtitle",
  CellTags->"start-up"],
Cell[26820, 829, 688, 19, 72, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27545, 853, 195, 7, 46, "Input"],
Cell[27743, 862, 469, 8, 107, "Output"]
}, Open  ]],
Cell[28227, 873, 1135, 30, 113, "Text"],
Cell[29365, 905, 153, 4, 31, "Subtitle",
  CellTags->"paths"],
Cell[29521, 911, 692, 19, 72, "Text"],

Cell[CellGroupData[{
Cell[30238, 934, 230, 7, 45, "Input"],
Cell[30471, 943, 225, 4, 44, "Output"]
}, Open  ]],
Cell[30711, 950, 670, 18, 71, "Text"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

