(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     34912,        998]*)
(*NotebookOutlinePosition[     35759,       1029]*)
(*  CellTagsIndexPosition[     35685,       1023]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["SturmQuery",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Performs Sturm queries on a zero-dimensional ideal.\nUsage: ",
    FontFamily->"Arial"],
  StyleBox["SturmQuery[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ideal<,options>", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  "\n",
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A (list of) ideal list(s) of the form explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[", combined into a single ideal by the module\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["\nOptional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)",
    FontFamily->"Helvetica",
    FontSize->12],
  "\n\n",
  Cell[BoxData[GridBox[{
          {
            StyleBox["TestRational",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0},
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12,
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[
              RowBox[{
                StyleBox[\((list\ of)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["rational",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["function",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[\((s)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["for",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["Sturm",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["query",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["(",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["default",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[\({1}\),
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}]}], 
                  StyleBox[")",
                    FontFamily->"Helvetica",
                    FontSize->12]}]}],
              GridBoxOptions->{ColumnAlignments->{Left}}]},
          {
            StyleBox["TestMatDet",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0},
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12,
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[
              RowBox[{
                StyleBox[\((list\ of)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["rational",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["matrix",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[\((ces)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["for",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["Sturm",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["query",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["on",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox["determinant",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[\((s)\),
                  FontFamily->"Helvetica",
                  FontSize->12], 
                StyleBox[" ",
                  FontFamily->"Helvetica",
                  FontSize->12], 
                RowBox[{
                  StyleBox["(",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["default",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[\({{1}}\),
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}]}], 
                  StyleBox[")",
                    FontFamily->"Helvetica",
                    FontSize->12]}]}],
              GridBoxOptions->{ColumnAlignments->{Left}}]},
          {
            StyleBox["TestMatEigen",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0},
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12,
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            RowBox[{
              StyleBox["A",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["rational",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["Hermitian",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["matrix",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["for",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["Sturm",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["query",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["on",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["Eigenvalues",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                FontFamily->"Helvetica",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[
                RowBox[{
                  StyleBox["(",
                    FontFamily->"Helvetica",
                    FontSize->12], 
                  RowBox[{
                    StyleBox["default",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[" ",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    StyleBox[\({{1}}\),
                      FontFamily->"Helvetica",
                      FontSize->12,
                      FontWeight->"Plain",
                      FontVariations->{"CompatibilityType"->0}]}], 
                  StyleBox[")",
                    FontFamily->"Helvetica",
                    FontSize->12]}],
                GridBoxOptions->{ColumnAlignments->{Left}}]}]},
          {
            StyleBox["VarPar",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0},
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12,
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            
            StyleBox[\(\((list\ of)\)\ rule \((s)\)\ assigning\ parameter \
\((s)\)/variable \((s)\)\ to\ value \((s)\)\ \((default\ {})\)\),
              FontFamily->"Helvetica",
              FontSize->12,
              GridBoxOptions->{ColumnAlignments->{Left}}]},
          {
            StyleBox["RRoots",
              FontFamily->"Helvetica",
              FontSize->12,
              FontWeight->"Plain",
              FontVariations->{"CompatibilityType"->0},
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12,
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox[
              RowBox[{
                RowBox[{
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True], 
                  StyleBox["Probab",
                    ShowStringCharacters->True,
                    FontWeight->"Plain"], 
                  StyleBox["\"\<\>",
                    ShowStringCharacters->True]}], 
                StyleBox[",",
                  ShowStringCharacters->True], 
                StyleBox[" ",
                  ShowStringCharacters->True], 
                RowBox[{
                  RowBox[{
                    StyleBox["\"\<\>",
                      ShowStringCharacters->True], 
                    StyleBox["Determ",
                      ShowStringCharacters->True,
                      FontWeight->"Plain"], 
                    StyleBox["\"\<\>",
                      ShowStringCharacters->True]}], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  StyleBox["or",
                    ShowStringCharacters->True], 
                  StyleBox[" ",
                    ShowStringCharacters->True], 
                  RowBox[{
                    StyleBox["\"\<\>",
                      ShowStringCharacters->True], 
                    StyleBox["Off",
                      ShowStringCharacters->True,
                      FontWeight->"Plain"], 
                    StyleBox["\"\<\>",
                      ShowStringCharacters->True]}], 
                  StyleBox[
                    RowBox[{
                      StyleBox[" ",
                        ShowStringCharacters->True], 
                      StyleBox[" ",
                        FontFamily->"Helvetica",
                        FontSize->12]}]], 
                  RowBox[{
                    StyleBox["(",
                      FontFamily->"Helvetica",
                      FontSize->12], 
                    RowBox[{
                      StyleBox["default",
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      StyleBox[" ",
                        FontFamily->"Helvetica",
                        FontSize->12], 
                      RowBox[{
                        StyleBox["\"\<\>",
                          ShowStringCharacters->True,
                          FontFamily->"Helvetica",
                          FontSize->12], 
                        StyleBox["Off",
                          ShowStringCharacters->True,
                          FontFamily->"Helvetica",
                          FontSize->12,
                          FontWeight->"Plain"], 
                        StyleBox["\"\<\>",
                          ShowStringCharacters->True,
                          FontFamily->"Helvetica",
                          FontSize->12]}]}], 
                    StyleBox[")",
                      FontFamily->"Helvetica",
                      FontSize->12]}]}]}],
              GridBoxOptions->{ColumnAlignments->{Left}}]},
          {
            StyleBox["SturmLabel",
              FontWeight->"Plain",
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            StyleBox["\[Rule]",
              FontFamily->"Helvetica",
              FontSize->12,
              GridBoxOptions->{ColumnAlignments->{Left}}], 
            RowBox[{
              StyleBox["\"\<\>",
                ShowStringCharacters->True,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["None",
                ShowStringCharacters->True,
                FontWeight->"Plain",
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["\"\<\>",
                ShowStringCharacters->True,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[
                RowBox[{
                  StyleBox["o",
                    ShowStringCharacters->True,
                    GridBoxOptions->{ColumnAlignments->{Left}}], 
                  StyleBox["r",
                    ShowStringCharacters->True,
                    FontFamily->"Arial",
                    FontSize->12,
                    GridBoxOptions->{ColumnAlignments->{Left}}]}]], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["a",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["string",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["labeling",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["the",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["quantity",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["to",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["be",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox["tested",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                FontSize->12,
                GridBoxOptions->{ColumnAlignments->{Left}}], 
              StyleBox[" ",
                ShowStringCharacters->True,
                FontFamily->"Arial",
                GridBoxOptions->{ColumnAlignments->{Left}}]}]}
          }]],
    GridBoxOptions->{ColumnAlignments->{Left}}],
  StyleBox["\n",
    FontFamily->"Helvetica"],
  StyleBox["\nGlobal options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox[" Given a (zero-dimensional) polynomial ideal ",
    FontSize->12],
  StyleBox["ideal,",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" ",
    FontSize->12],
  StyleBox["SturmQuery[ ]", "Input",
    FontSize->12],
  StyleBox[" finds, at the real roots of ",
    FontSize->12],
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", the signs of given rational functions provided in the ",
    FontSize->12],
  StyleBox["TestRational",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" option, the signs of the determinants of the matrices in the ",
    FontSize->12],
  StyleBox["TestMatDet ",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["option and the signs of the eigenvalues of the matrix in ",
    FontSize->12],
  StyleBox["TestMatEigen",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[". No number field extensions are allowed, so all parameters have \
to be replaced by values, for example, by using the option ",
    FontSize->12],
  StyleBox["VarPar",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".\nN.B. The eigenvalues of the matrix set in ",
    FontSize->12],
  StyleBox["TestMatEigen",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" should be real on the real roots of ",
    FontSize->12],
  StyleBox["ideal ",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic"],
  StyleBox["(one way to ensure this is by passing the module a Hermitian \
matrix). If this is not the case errors may result in the ",
    FontSize->12],
  StyleBox["Singular",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" call.",
    FontSize->12],
  StyleBox["\n\n",
    FontSize->12],
  "Output:",
  StyleBox["  An ideal list with a list assigned to the keyword ",
    FontSize->12],
  StyleBox["Sturm; ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["see",
    FontSize->12,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[ButtonBox["Ideal Keywords",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Arial",
    FontSize->12],
  StyleBox[". For the possible structures of ",
    FontSize->12],
  StyleBox["Sturm",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[", see the illustrative examples ",
    FontSize->12],
  StyleBox[ButtonBox["below",
    ButtonData:>"egSturm",
    ButtonStyle->"Hyperlink"],
    FontSize->12],
  StyleBox[". The sign of a rational function f here takes value {-1, 0, 1, \
2, 3} for f being, respectively, {negative, zero, positive, infinity, \
indeterminate}. The ",
    FontSize->12],
  StyleBox["SturmLabel ",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox["option may be used to replace the first entries of the keyword ",
    FontSize->12],
  StyleBox["Sturm",
    FontSize->12,
    FontWeight->"Plain"],
  StyleBox[" in the output, the full expressions for the quantities that were \
tested, with a user defined label.\n\n",
    FontSize->12],
  "Example:",
  StyleBox[" ",
    FontSize->12],
  StyleBox["SturmQuery[id ,TestRational->x]", "Input",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontSize->12,
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["finds the signs of the ideal variable ",
    FontSize->12],
  StyleBox["x",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" on the real roots of the ideal described by the ideal list ",
    FontSize->12],
  StyleBox["id",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[".",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[TextData[StyleBox["Detailed Examples",
  FontSize->18,
  FontWeight->"Bold"]], "Subtitle",
  CellDingbat->None],

Cell[BoxData[
    RowBox[{"\[IndentingNewLine]", 
      RowBox[{
      "In", " ", "the", " ", "following", " ", "we", " ", "see", " ", "that", 
        " ", "the", " ", "complex", " ", "dimension", " ", "of", " ", "the", 
        " ", "ideal", " ", "generated", " ", "by", " ", 
        StyleBox[
          FormBox[\(\((x\^3 + y\^3, \ x*\((y - 1)\))\)\(\ \)\),
            "TraditionalForm"],
          FontFamily->"Arial"], 
        StyleBox["in",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["two",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["complex",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["variables",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["x",
          FontSlant->"Italic"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["and",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["y",
          FontSlant->"Italic"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["is",
          FontFamily->"Arial"], 
        StyleBox[" ",
          FontFamily->"Arial"], 
        StyleBox["0.",
          FontFamily->"Arial"]}]}]], "Text",
  FontFamily->"Arial",
  CellTags->"egSturm"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(id = 
        CreateIdeal[{Var \[Rule] {x, y}, Par \[Rule] {}, 
            Gen \[Rule] {x\^3 + y\^3, 
                x \((\ y - 1)\)}}];\)\), "\[IndentingNewLine]", 
    \(Dim /. DimIdeal[id]\)}], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \({0}\)], "Output",
  CellLabel->"Out[63]="]
}, Open  ]],

Cell[BoxData[
    \(We\ see\ further\ that\ there\ are\ two\ real\ roots\ to\ this\ zero\
\[Dash]dimensional\ \(\(ideal\)\(.\)\)\)], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NumRoots[id]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \({Var \[Rule] {x, y}, Par \[Rule] {}, 
      Gen \[Rule] {x\^3 + y\^3, x\ \((\(-1\) + y)\)}, Dim \[Rule] {0}, 
      NRoots \[Rule] {2}}\)], "Output",
  CellLabel->"Out[64]="]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Indeed", ",", " ", 
      RowBox[{
        RowBox[{
        "we", " ", "can", " ", "see", " ", "by", " ", "inspection", " ", 
          "that", " ", "the", " ", "roots", " ", "are", " ", "at", "  ", 
          StyleBox[\((x, y)\),
            FontSlant->"Italic"]}], 
        StyleBox["=",
          FontSlant->"Italic"], 
        RowBox[{
          StyleBox[\((0, 0)\),
            FontSlant->"Italic"], " ", "and", " ", 
          StyleBox[\((\(-1\), 1)\),
            FontSlant->"Italic"]}]}], 
      StyleBox[",",
        FontSlant->"Italic"], 
      StyleBox[" ",
        FontSlant->
          "Italic"], \(though\ such\ explicit\ knowledge\ is\ neither\ needed\
\ nor\ determinable\ in\ general\ for\ the\ Sturm\ Queries\ which\ we\ may\ \
\(\(perform\)\(.\)\)\)}]], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sturm /. 
      SturmQuery[id, TestRational \[Rule] {1, x, y}, 
        RRoots \[Rule] "\<Probab\>"]\)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \({{1, x, y}, {1, 0, 0}, {1, \(-1\), 1}, {1, 1}}\)], "Output",
  CellLabel->"Out[65]="]
}, Open  ]],

Cell["\<\
We see that at the first root (0,0), the signs of {1,x,y} are \
(1,0,0) and at the root (-1, 1), the signs are (1,-1,1). The last entry is \
the multiplicity, i.e., {1,1} indicates that  the combination of signs \
{1,0,0} and {1,-1,1} each appear on a single root.\
\>", "Text",
  FontFamily->"Arial"],

Cell[BoxData[
    \(Now\ let\ us\ check\ the\ behaviour\ of\ matrices\ on\ the\ roots . \ 
        For\ example, \ define, \)], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(mm\  = \ {{x^2 + y^2, x*y}, {x + y, 5}}\)], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    \({{x\^2 + y\^2, x\ y}, {x + y, 5}}\)], "Output",
  CellLabel->"Out[66]="]
}, Open  ]],

Cell[BoxData[
    \(We\ can\ now\ test, \ 
    in\ addition\ to\ the\ above\ polynomials, \ \ the\ sign\ of\ the\ \
determinant\ of\ this\ matrix\ on\ the\ roots\ of\ our\ \
\(\(ideal\)\(.\)\)\)], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sturm /. 
      SturmQuery[id, TestRational \[Rule] {1, x, y}, TestMatDet \[Rule] mm, 
        RRoots \[Rule] "\<Probab\>"]\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \({{1, x, y}, {{{x\^2 + y\^2, x\ y}, {x + y, 5}}}, {1, 0, 0, 
        0}, {1, \(-1\), 1, 1}, {1, 1}}\)], "Output",
  CellLabel->"Out[67]="]
}, Open  ]],

Cell[BoxData[
    RowBox[{\(We\ see\ that\), ",", " ", \(for\ example\), ",", 
      "  ", \(in\ addition\ to\ the\ signs\ of\ the\ polynomials\ being\ {1, 
          0, 0}\ for\ the\ first\ root\), ",", " ", 
      RowBox[{"the", " ", "determinant", " ", "of", 
        StyleBox[" ",
          FontSlant->"Italic"], 
        StyleBox["mm",
          FontSlant->"Italic"], 
        StyleBox[" ",
          FontSlant->"Italic"], "is", " ", "0", " ", \(thereupon . \ Thus\), 
        " ", "we", " ", "have", " ", \({1, 0, 0, 0}\), " ", "in", " ", "the", 
        " ", "above", " ", \(\(output\)\(.\)\)}]}]], "Text",
  FontFamily->"Arial"],

Cell[BoxData[
    \(We\ can\ also\ test\ the\ signs\ of\ the\ eigenvalues\ of\ the\ matrix\ \
on\ the\ roots\ as\ \(\(follows\)\(.\)\)\)], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sturm /. 
      SturmQuery[id, TestRational \[Rule] {1, x, y}, TestMatEigen \[Rule] mm, 
        RRoots \[Rule] "\<Probab\>"]\)], "Input",
  CellLabel->"In[68]:="],

Cell[BoxData[
    \({{1, x, 
        y}, {{x\^2 + y\^2, x\ y}, {x + y, 5}}, {{1, 0, 0}, {0, 1}, {1, 
          1}}, {{1, \(-1\), 1}, {1}, {2}}}\)], "Output",
  CellLabel->"Out[68]="]
}, Open  ]],

Cell[BoxData[
    RowBox[{\(In\ the\ above\), ",", 
      " ", \(there\ is\ one\ list\ \ of\ signs\ and\ multiplicities\ for\ \
each\ root . \ \ For\ example\), ",", 
      "  ", \({{1, 0, 0}, {0, 1}, {1, 
            1}}\ indicates\ that\ on\ the\ first\ root\ \((which\ we\ recall\ \
is\ \((0, 0)\))\)\), ",", " ", 
      RowBox[{
      "the", " ", "rational", "  ", "functions", " ", \({1, x, y}\), " ", 
        "have", " ", "signs", " ", \({1, 0, 0}\), " ", "and", " ", "the", " ",
         "matrix", " ", 
        StyleBox["mm",
          FontSlant->"Italic"], " ", "has", " ", "eigenvalues", " ", "whose", 
        " ", "signs", " ", "are", " ", \((0, 1)\), " ", "with", " ", 
        "multiplicity", " ", \(\((1, 1)\) . \ \ On\), " ", "the", " ", 
        "second", " ", "root"}], ",", " ", \(the\ final\ entries\), ",", 
      " ", \({1}\ and {2}\), ",", " ", 
      RowBox[{"indicate", " ", "that", " ", 
        StyleBox["mm",
          FontSlant->"Italic"], " ", "has", " ", "two", " ", "positive", 
        " ", \(\(eigenvalues\)\(.\)\)}]}]], "Text",
  FontFamily->"Arial"]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 741}},
CellGrouping->Manual,
WindowSize->{1141, 683},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "egSturm"->{
    Cell[27889, 778, 1439, 45, 46, "Text",
      CellTags->"egSturm"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"egSturm", 35588, 1016}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 283, 10, 76, "Title"],
Cell[2062, 65, 548, 16, 72, "Subsection"],
Cell[2613, 83, 20694, 538, 298, "Subsection"],
Cell[23310, 623, 4445, 147, 227, "Subsection"]
}, Open  ]],
Cell[27770, 773, 116, 3, 31, "Subtitle"],
Cell[27889, 778, 1439, 45, 46, "Text",
  CellTags->"egSturm"],

Cell[CellGroupData[{
Cell[29353, 827, 255, 6, 66, "Input"],
Cell[29611, 835, 62, 2, 44, "Output"]
}, Open  ]],
Cell[29688, 840, 163, 3, 25, "Text"],

Cell[CellGroupData[{
Cell[29876, 847, 70, 2, 45, "Input"],
Cell[29949, 851, 194, 4, 44, "Output"]
}, Open  ]],
Cell[30158, 858, 830, 22, 25, "Text"],

Cell[CellGroupData[{
Cell[31013, 884, 158, 4, 45, "Input"],
Cell[31174, 890, 105, 2, 44, "Output"]
}, Open  ]],
Cell[31294, 895, 311, 6, 46, "Text"],
Cell[31608, 903, 158, 3, 25, "Text"],

Cell[CellGroupData[{
Cell[31791, 910, 97, 2, 45, "Input"],
Cell[31891, 914, 92, 2, 44, "Output"]
}, Open  ]],
Cell[31998, 919, 227, 5, 25, "Text"],

Cell[CellGroupData[{
Cell[32250, 928, 181, 4, 45, "Input"],
Cell[32434, 934, 157, 3, 44, "Output"]
}, Open  ]],
Cell[32606, 940, 636, 13, 25, "Text"],
Cell[33245, 955, 169, 3, 25, "Text"],

Cell[CellGroupData[{
Cell[33439, 962, 183, 4, 45, "Input"],
Cell[33625, 968, 182, 4, 44, "Output"]
}, Open  ]],
Cell[33822, 975, 1086, 21, 63, "Text"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

