(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7572,        245]*)
(*NotebookOutlinePosition[      8313,        270]*)
(*  CellTagsIndexPosition[      8269,        266]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Stringvacua Module ",
    FontSize->24],
  StyleBox["ThomEncode",
    FontSize->24,
    FontWeight->"Plain",
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[TextData[{
  StyleBox["Thom encoding of zero-dimensional ideals.\nUsage:",
    FontFamily->"Arial"],
  StyleBox[" ", "Subsubsection",
    FontFamily->"Arial"],
  StyleBox["ThomEncode[", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["list,keyword", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["]", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain",
    FontVariations->{"CompatibilityType"->0}]
}], "Subsection"],

Cell[TextData[{
  StyleBox["Arguments\n",
    FontFamily->"Helvetica"],
  StyleBox["\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["ideal",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Plain",
    FontSlant->"Italic",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" : A (list of) zero-dimensional ideal list(s) of the form \
explained ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["here",
    ButtonData:>"Ideal Keywords",
    Active->True,
    ButtonStyle->"AddOnsLink",
    ButtonSource->ButtonData],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[", combined into a single ideal by the module\n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox["Optional arguments ",
    FontFamily->"Helvetica"],
  StyleBox["(see ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Module Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[" for details)\n\n",
    FontFamily->"Helvetica",
    FontSize->12],
  Cell[BoxData[
      RowBox[{GridBox[{
            {
              StyleBox["VarPar",
                FontFamily->"Helvetica",
                FontSize->12,
                FontWeight->"Plain",
                FontVariations->{"CompatibilityType"->0}], 
              StyleBox["\[Rule]",
                FontFamily->"Helvetica",
                FontSize->12], 
              
              StyleBox[\(\((list\ of)\)\ rule \((s)\)\ assigning\ parameters/
                    variables\ to\ values\),
                FontFamily->"Helvetica",
                FontSize->12]}
            }], "."}]],
    ShowStringCharacters->True,
    GridBoxOptions->{ColumnAlignments->{Left}}],
  "\n\n",
  StyleBox["Global options in ",
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[ButtonBox["Stringvacua Options",
    ButtonStyle->"AddOnsLink"],
    FontFamily->"Helvetica",
    FontSize->12],
  StyleBox[".",
    FontFamily->"Helvetica",
    FontSize->12]
}], "Subsection",
  CellFrame->True,
  CellDingbat->"\[FilledSquare]",
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  Background->RGBColor[1, 1, 0.901961]],

Cell[TextData[{
  "Purpose:",
  StyleBox["  Takes a zero-dimensional ideal, breaks it up into triangular \
systems, and Thom encodes each of them in turn. This can be used in \
conjunction with the module ",
    FontSize->12],
  ButtonBox["SturmQuery",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" so as to uniquely identify the roots of a zero-dimensional ideal \
by correlating the signs of test functions with those roots.\n\n",
    FontSize->12],
  "Output:",
  StyleBox[" A series of zero-dimensional ideals and a set of polynomials, \
the signs of which uniquely determine the roots on each triangular system.",
    FontSize->12]
}], "Subsection"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Example",
  FontSize->14]], "Title"],

Cell[BoxData[
    RowBox[{
    "In", " ", "the", " ", "following", " ", "we", " ", "see", " ", "that", 
      " ", "the", " ", "complex", " ", "dimension", " ", "of", " ", "the", 
      " ", "ideal", " ", "generated", " ", "by", " ", 
      StyleBox[
        FormBox[\(\((x\^3 + y\^3, \ x*\((y - 1)\))\)\(\ \)\),
          "TraditionalForm"],
        FontFamily->"Arial"], "in", " ", "two", " ", "complex", " ", 
      "variables", " ", 
      StyleBox["x",
        FontSlant->"Italic"], " ", "and", " ", 
      StyleBox["y",
        FontSlant->"Italic"], " ", "is", " ", "0"}]], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(id = 
        CreateIdeal[{Var \[Rule] {x, y}, Par \[Rule] {}, 
            Gen \[Rule] {x\^3 + y\^3, 
                x \((\ y - 1)\)}}];\)\), "\[IndentingNewLine]", 
    \(Dim /. DimIdeal[id]\)}], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \({0}\)], "Output",
  CellLabel->"Out[72]="]
}, Open  ]],

Cell[BoxData[
    \(We\ now\ perform\ a\ Thom\ encoding\ on\ this\ \(\(ideal\)\(.\)\)\)], \
"Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ThomEncode[id]\)], "Input",
  CellLabel->"In[73]:="],

Cell[BoxData[
    \({{Var \[Rule] {x, y}, Par \[Rule] {}, Gen \[Rule] {y\^3, x}, 
        Thom \[Rule] {y\^3, 3\ y\^2, 6\ y, x}}, {Var \[Rule] {x, y}, 
        Par \[Rule] {}, Gen \[Rule] {\(-1\) + y, 1 + x\^3}, 
        Thom \[Rule] {\(-1\) + y, 1 + x\^3, 3\ x\^2, 6\ x}}}\)], "Output",
  CellLabel->"Out[73]="]
}, Open  ]],

Cell[BoxData[
    \(The\ two\ resulting\ triangular\ systems\ have\ generators\ \((x, 
        y\^3)\)\ and\ \((y - 1, x\^3 + 1)\) . \ 
        The\ Thom\ polynomials\ are, \ 
    respectively, {y\^3, 3\ y\^2, 6\ y, 
        x}\ and\ {\(-1\) + y, 1 + x\^3, 3\ x\^2, 6\ x} . \ 
        The\ signs\ of\ the\ Thom\ polynomials\ associated\ to\ each\ \
triangular\ system\ uniquely\ distinguish\ the\ roots\ of\ that\ \
\(\(ideal\)\(.\)\)\)], "Text",
  FontFamily->"Arial"]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 717}},
ScreenStyleEnvironment->"Working",
WindowSize->{1144, 683},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 283, 10, 76, "Title"],
Cell[2013, 64, 593, 18, 72, "Subsection"],
Cell[2609, 84, 2203, 74, 208, "Subsection"],
Cell[4815, 160, 653, 15, 107, "Subsection"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5505, 180, 60, 1, 61, "Title"],
Cell[5568, 183, 610, 14, 25, "Text"],

Cell[CellGroupData[{
Cell[6203, 201, 255, 6, 66, "Input"],
Cell[6461, 209, 62, 2, 44, "Output"]
}, Open  ]],
Cell[6538, 214, 122, 3, 25, "Text"],

Cell[CellGroupData[{
Cell[6685, 221, 72, 2, 45, "Input"],
Cell[6760, 225, 312, 5, 44, "Output"]
}, Open  ]],
Cell[7087, 233, 469, 9, 50, "Text"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

