(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     19027,        525]*)
(*NotebookOutlinePosition[     64092,       1956]*)
(*  CellTagsIndexPosition[     64048,       1952]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Example:  ",
    FontSize->24],
  StyleBox["Constraints on flux parameters",
    FontSize->24,
    FontColor->RGBColor[0.784314, 0, 0]]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[CellGroupData[{

Cell[TextData[StyleBox["1. Introduction",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["In this example we will take a class of supergravity models, \
parameterised by a certain set of flux parameters, and demonstrate how to \
derive constraints on those parameters which are necessary for certain types \
of vacua to exist in the models. This example is based on ",
    FontFamily->"Arial"],
  StyleBox["hep-th/0508133",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[". \n\n\t",
    FontFamily->"Arial"],
  StyleBox["Three superfields:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" \tT = t + \[ImaginaryI] \[Tau] , S = s + \[ImaginaryI] \[Sigma] \
, U = u + \[ImaginaryI] v\n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Kahler potential:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" \t\tK = -3 Log[ -\[ImaginaryI] (T - Conjugate[T])] - Log[ -\
\[ImaginaryI] (S - Conjugate[S])]\n\t\t\t\t\t\t-3 Log[-\[ImaginaryI] \
(U-Conjugate[U])]\n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Superpotential:\t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["W = a0 - 3a1T + 3a2T^2 - a3T^3\n\t\t\t\t\t\t+ S (-b0+3b1 T -3 b2 \
T^2+ b3 T^3)\n\t\t\t\t\t\t+3 U (c0 + (cc1+ch1-ct1)T- (cc2+ch2-ct2)T^2- c3T^3)\
\n        \t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Eleven parameters:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\ta0, a1, a2, a3, b0, b1, b2, b3, c0, cc1, ch1, ct1, cc2, ch2, \
ct2, c3\n        \t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Constraints:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\t\t\t18 further constraints on the flux parameters, detailed \
below.",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n        ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["2. Setting up the model",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["Load the package, ",
    FontFamily->"Arial"],
  StyleBox["      ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< Stringvacua`\)], "Input",
  CellLabel->"In[1]:="],

Cell["\<\
Stringvacua package, Version 1.0, December 2007
James Gray, Yang-Hui He, Anton Ilderton and Andre Lukas\
\>", "Text",
  CellFrame->True,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->13,
  FontWeight->"Bold",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[1, 1, 0.900008]],

Cell[BoxData["\<\"Execute \
\\*ButtonBox[StyleBox[\\\"SimpleMinimise[]\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonEvaluator->\\\"Automatic\\\
\",ButtonFunction:>(SimpleMinimise[]&)] for a simple interactive calculation \
or click \\*ButtonBox[StyleBox[\\\"here\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[\
 { FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Generalities\\\"]}]&)] to \
open the Help Browser, which contains \
\\*ButtonBox[StyleBox[\\\"examples\\\",FontColor->RGBColor[0,0,0.7],FontSize->\
12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { \
FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Examples\\\", \
\\\"Examples_Overview.nb\\\"]}]&)].\"\>"], "Output",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->12,
  FontColor->RGBColor[0.6, 0.300008, 0]]
}, Open  ]],

Cell[TextData[{
  StyleBox["Then create the model, using ",
    FontFamily->"Arial"],
  StyleBox["CreateModel ", "Input",
    FontFamily->"Arial"],
  StyleBox["and ",
    FontFamily->"Arial"],
  StyleBox["CalcModel", "Input",
    FontFamily->"Arial"],
  StyleBox[", as shown (note that it may take a moment or two to generate all \
of the information).",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    \(\(modelW = 
        CalcModel[
          CreateModel[{T, S, U}, {\[Tau], \[Sigma], \ \[Zeta]}, \ {t, \ s, \ 
              u}, \ \(-3\)\ Log[\(-I\)\ \((T\  - \ Conjugate[T])\)] - 
              Log[\(-I\)\ \((S - Conjugate[S])\)] - 
              3\ Log[\(-I\)\ \((U - Conjugate[U])\)], 
            a0 - 3\ a1\ T + 3\ a2\ T^2 - a3\ T^3 + 
              S\ \((\(-b0\) + 3\ b1\ T - 3\ b2\ T^2 + b3\ T^3)\) + 
              3\ U\ \((c0 + \((\ 
                        cc1 + ch1 - ct1)\)\ T - \((cc2 + ch2 - ct2)\)\ T^2 - 
                    c3\ T^3)\), 
            Par \[Rule] {a0, a1, a2, a3, b0, b1, b2, b3, c0, cc1, ch1, ct1, \ 
                cc2, \ ch2, \ ct2, c3}]];\)\)], "Input",
  CellLabel->"In[2]:="],

Cell[TextData[{
  StyleBox["The flux parameters of this model are known to obey a set of \
constraints coming from consideration of tadpoles and intergrability of \
Bianchi indentities. These are collected in the list ",
    FontFamily->"Arial"],
  StyleBox["Constraints",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", input as below.",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    \(\(Constraints = {\[IndentingNewLine]\[IndentingNewLine]a0\ b3 - 
            3\ a1\ b2 + 3\ a2\ b1 - a3\ b0 - 
            16, \[IndentingNewLine]\[IndentingNewLine]a0\ c3 + 
            a1\ \((\ cc2 + ch2 - ct2)\) - a2\ \((\ cc1 + ch1 - ct1)\) - 
            a3\ c0, \[IndentingNewLine]\[IndentingNewLine]c0\ b2 - ct1\ b1 + 
            ch1\ b1 - cc2\ b0, \[IndentingNewLine]cc1\ b3 - ch2\ b2 + 
            ct2\ b2 - c3\ b1, \[IndentingNewLine]c0\ b3 - ct1\ b2 + ch1\ b2 - 
            cc2\ b1, \[IndentingNewLine]cc1\ b2 - ch2\ b1 + ct2\ b1 - 
            c3\ b0, \[IndentingNewLine]\[IndentingNewLine]c0\ b2 - ct1\ b1 + 
            cc1\ b1 - ch2\ b0, \[IndentingNewLine]ch1\ b3 - cc2\ b2 + 
            ct2\ b2 - c3\ b1, \[IndentingNewLine]c0\ b3 - ct1\ b2 + cc1\ b2 - 
            ch2\ b1, \[IndentingNewLine]ch1\ b2 - cc2\ b1 + ct2\ b1 - 
            c3\ b0, \[IndentingNewLine]\[IndentingNewLine]c0\ ct2 - cc1^2 + 
            ct1\ ch1 - ch2\ c0, \[IndentingNewLine]c3\ ct1\  - cc2^2 + 
            ct2\ ch2 - ch1\ c3, \[IndentingNewLine]c3\ c0\  - cc2\ ch1\  + 
            ct2\ cc1 - ch1\ ct2, \[IndentingNewLine]ch2\ ct1 - ct1\ cc2 + 
            cc1\ ch2 - 
            c0\ c3, \[IndentingNewLine]\[IndentingNewLine]c0\ ct2 - ch1^2 + 
            ct1\ cc1 - cc2\ c0, \[IndentingNewLine]c3\ ct1\  - ch2^2 + 
            ct2\ cc2 - cc1\ c3, \[IndentingNewLine]c3\ c0\  - ch2\ cc1\  + 
            ct2\ ch1 - cc1\ ct2, \[IndentingNewLine]cc2\ ct1 - ct1\ ch2 + 
            ch1\ cc2 - c0\ c3\[IndentingNewLine]};\)\)], "Input",
  CellLabel->"In[3]:="]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["3. Finding constraints",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["We will address the question of whether any ",
    FontFamily->"Arial"],
  StyleBox["further ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["constraints are necessary in order for ",
    FontFamily->"Arial"],
  StyleBox["supersymmetic",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial"],
  StyleBox[" Minkowski",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" vacua to exist in such models. We first collect the objects we \
wish to vanish together, as follows:",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    \(\(MinkVac = 
        CreateIdeal[{IDSPot /. modelW, ISPot /. modelW, 
            Constraints}];\)\)], "Input",
  CellLabel->"In[4]:="],

Cell[TextData[{
  StyleBox["This contains all objects which are required to vanish -- the \
constraints must always hold, while the vanishing of ",
    FontFamily->"Arial"],
  StyleBox["ISPot/.model",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" and ",
    FontFamily->"Arial"],
  StyleBox["IDSPot/.model",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", being the superpotential and its derivatives in a suitable \
form, are sufficient to imply the vanishing of the derivatives of the scalar \
potential and give a zero cosmological constant. This collection of equations \
contains a great deal of information, on both the possible field values at \
the extrema and the possible parameter values.  It is the latter that we wish \
to extract. To do so we use the module ",
    FontFamily->"Arial"],
  StyleBox["Elimination[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", to eliminate the field variables from the equations. What are \
the field variables? They are contained in ",
    FontFamily->"Arial"],
  StyleBox["MinkVac",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" as the keyword ",
    FontFamily->"Arial"],
  StyleBox["Var",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", extracted as shown",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Var /. MinkVac\)], "Input",
  CellLabel->"In[5]:="],

Cell[BoxData[
    \({S, T, U}\)], "Output",
  CellLabel->"Out[5]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["As we need only consider the superpotential and its derivatives, \
which are defined using the superfields, we see only these three variables \
appearing. In the module ",
    FontFamily->"Arial"],
  StyleBox["Elimination[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" the variables to be eliminated are set using the module option ",
    
    FontFamily->"Arial"],
  StyleBox["VarPar",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", as shown below. The option ",
    FontFamily->"Arial"],
  StyleBox["Meth",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" specifies the method used to perform the elimination,",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ElimOut = 
      Elimination[MinkVac, VarPar \[Rule] {S, T, U}, 
        Meth \[Rule] "\<Prod\>"]\)], "Input",
  CellLabel->"In[6]:="],

Cell[BoxData[
    \({Var \[Rule] {a0, a1, a2, a3, b0, b1, b2, b3, c0, c3, cc1, cc2, ch1, 
          ch2, ct1, ct2}, Par \[Rule] {}, 
      Gen \[Rule] {ct2, ct1, ch2 + ct2, ch1 + ct1, cc2 - ch2, cc1 - ch1, c3, 
          c0, 16 + a3\ b0 - 3\ a2\ b1 + 3\ a1\ b2 - a0\ b3, 
          16\ a3\^2\ b0\^2 - 96\ a2\ a3\ b0\ b1 - 288\ a2\^2\ b1\^2 + 
            432\ a1\ a3\ b1\^2 + 54\ a2\^3\ b1\^3 - 81\ a1\ a2\ a3\ b1\^3 + 
            27\ a0\ a3\^2\ b1\^3 + 432\ a1\ a3\ b0\ b2 - 
            27\ a2\^2\ a3\ b0\^2\ b2 + 48\ a1\ a3\^2\ b0\^2\ b2 - 
            288\ a0\ a3\ b1\ b2 - 18\ a1\ a2\ a3\ b0\ b1\ b2 - 
            45\ a0\ a3\^2\ b0\ b1\ b2 - 54\ a1\ a2\^2\ b1\^2\ b2 + 
            81\ a1\^2\ a3\ b1\^2\ b2 - 27\ a0\ a2\ a3\ b1\^2\ b2 + 
            54\ a0\ a2\ a3\ b0\ b2\^2 + 27\ a0\ a1\ a3\ b1\ b2\^2 - 
            27\ a0\^2\ a3\ b2\^3 - 288\ a1\ a2\ b0\ b3 - 32\ a0\ a3\ b0\ b3 + 
            27\ a2\^3\ b0\^2\ b3 - 45\ a1\ a2\ a3\ b0\^2\ b3 + 
            432\ a0\ a2\ b1\ b3 - 27\ a1\ a2\^2\ b0\ b1\ b3 + 
            54\ a1\^2\ a3\ b0\ b1\ b3 + 48\ a0\ a2\ a3\ b0\ b1\ b3 + 
            18\ a0\ a2\^2\ b1\^2\ b3 - 81\ a0\ a1\ a3\ b1\^2\ b3 - 
            144\ a0\ a1\ b2\ b3 + 27\ a1\^2\ a2\ b0\ b2\ b3 - 
            54\ a0\ a2\^2\ b0\ b2\ b3 - 51\ a0\ a1\ a3\ b0\ b2\ b3 + 
            27\ a0\ a1\ a2\ b1\ b2\ b3 + 45\ a0\^2\ a3\ b1\ b2\ b3 - 
            27\ a0\ a1\^2\ b2\^2\ b3 + 27\ a0\^2\ a2\ b2\^2\ b3 + 
            16\ a0\^2\ b3\^2 - 27\ a1\^3\ b0\ b3\^2 + 
            45\ a0\ a1\ a2\ b0\ b3\^2 + 27\ a0\ a1\^2\ b1\ b3\^2 - 
            48\ a0\^2\ a2\ b1\ b3\^2 + 3\ a0\^2\ a1\ b2\ b3\^2}}\)], "Output",\

  CellLabel->"Out[6]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["The field variables have now been eliminated. The 10 polynomials \
contained in ",
    FontFamily->"Arial"],
  StyleBox["Gen/.ElimOut", "Input",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" depend only on the parameters. These polynomials are required to \
vanish for the existence of supersymmetric Minkowski vacua, as were those \
with which we began. They represent the ",
    FontFamily->"Arial"],
  StyleBox["necessary,",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["and in this case",
    FontFamily->"Arial"],
  StyleBox[" sufficient, constraints",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" which must be imposed on the flux parameters in order for \
supersymmetric Minkowski vacua to exist.",
    FontFamily->"Arial"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["4. Extension - simplifications",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["It is often possible to simplify the derived constraints using \
the module ",
    FontFamily->"Arial"],
  StyleBox["PrimDec[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", which breaks up sets of equations into smaller pieces. PrimDec \
may be used to separate the constraints into smaller or simpler groups of \
equations, the vanishing of any set implying that the original constraints \
are satisfied. Here, we find that our constraints can not be broken up but \
they can be simplified a little, ",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[ElimOut, 
      Dim \[Rule] {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 
          16}]\)], "Input",
  CellLabel->"In[7]:="],

Cell[BoxData[
    \({{Gen \[Rule] {ct2, ct1, ch2, ch1, cc2, cc1, c3, c0, 
            16 + a3\ b0 - 3\ a2\ b1 + 3\ a1\ b2 - a0\ b3, 
            4096 - 432\ a2\^2\ b1\^2 + 432\ a1\ a3\ b1\^2 + 
              54\ a2\^3\ b1\^3 - 81\ a1\ a2\ a3\ b1\^3 + 
              27\ a0\ a3\^2\ b1\^3 + 6912\ a1\ b2 + 432\ a2\^2\ b0\ b2 - 
              3024\ a1\ a2\ b1\ b2 + 432\ a0\ a3\ b1\ b2 - 
              81\ a2\^3\ b0\ b1\ b2 + 324\ a1\ a2\^2\ b1\^2\ b2 + 
              81\ a1\^2\ a3\ b1\^2\ b2 - 162\ a0\ a2\ a3\ b1\^2\ b2 + 
              3456\ a1\^2\ b2\^2 - 864\ a0\ a2\ b2\^2 + 
              81\ a1\ a2\^2\ b0\ b2\^2 - 810\ a1\^2\ a2\ b1\ b2\^2 + 
              162\ a0\ a2\^2\ b1\ b2\^2 + 162\ a0\ a1\ a3\ b1\ b2\^2 + 
              432\ a1\^3\ b2\^3 - 162\ a0\ a1\ a2\ b2\^3 - 
              27\ a0\^2\ a3\ b2\^3 + 432\ a1\ a2\ b0\ b3 + 
              27\ a2\^3\ b0\^2\ b3 - 864\ a1\^2\ b1\ b3 - 
              432\ a0\ a2\ b1\ b3 - 162\ a1\ a2\^2\ b0\ b1\ b3 + 
              162\ a1\^2\ a2\ b1\^2\ b3 + 162\ a0\ a2\^2\ b1\^2\ b3 - 
              81\ a0\ a1\ a3\ b1\^2\ b3 - 432\ a0\ a1\ b2\ b3 + 
              162\ a1\^2\ a2\ b0\ b2\ b3 - 81\ a0\ a2\^2\ b0\ b2\ b3 - 
              162\ a1\^3\ b1\ b2\ b3 - 162\ a0\ a1\^2\ b2\^2\ b3 + 
              81\ a0\^2\ a2\ b2\^2\ b3 - 27\ a1\^3\ b0\ b3\^2 + 
              81\ a0\ a1\^2\ b1\ b3\^2}, Dim \[Rule] {6}, 
        Var \[Rule] {a0, a1, a2, a3, b0, b1, b2, b3, c0, c3, cc1, cc2, ch1, 
            ch2, ct1, ct2}, Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[7]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["The option ",
    FontFamily->"Arial"],
  StyleBox["Dim",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" is now set to a list of integers from 0 to 16. Here,  ",
    FontFamily->"Arial"],
  StyleBox["Dim",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" refers to the number of flat directions that the constraints \
enjoy in parameter space, if they are to be kept in the output. We are \
interested in any and all constraints which we must impose, so we keep all \
possible dimensions of solution. Hence we instruct ",
    FontFamily->"Arial"],
  StyleBox["PrimDec",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" to retain solutions from all dimensions from 0, always a \
minimum, up to 16. As 16 is the number of parameters, it is also the maximum \
number of flat directions.",
    FontFamily->"Arial"]
}], "Text"]
}, Closed]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 777}},
ScreenStyleEnvironment->"Working",
CellGrouping->Manual,
WindowSize->{905, 657},
WindowMargins->{{17, Automatic}, {0, Automatic}},
ShowCellLabel->False,
StyleDefinitions -> Notebook[{
  
  Cell[CellGroupData[{
  Cell["Style Definitions", "Subtitle"],
  
  Cell["\<\
Modify the definitions below to change the default appearance of \
all cells in a given style. Make modifications to any definition using \
commands in the Format menu.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell["Style Environment Names", "Section"],
  
  Cell[StyleData[All, "Working"],
    ScriptMinSize->9],
  
  Cell[StyleData[All, "Presentation"],
    ScriptMinSize->9],
  
  Cell[StyleData[All, "SlideShow"],
    PageWidth->WindowWidth,
    ScrollingOptions->{"PagewiseDisplay"->True},
    ShowCellBracket->False,
    ScriptMinSize->9],
  
  Cell[StyleData[All, "Printout"],
    PageWidth->PaperWidth,
    ShowCellLabel->False,
    CellFrameLabelMargins->6,
    ImageSize->{200, 200},
    PrivateFontOptions->{"FontType"->"Outline"}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Notebook Options", "Section"],
  
  Cell["\<\
The options defined for the style below will be used at the \
Notebook level.\
\>", "Text"],
  
  Cell[StyleData["Notebook"],
    PageHeaders->{{Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"], None, Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"]}},
    PageHeaderLines->{True, True},
    PrintingOptions->{"FirstPageHeader"->False,
    "FacingPages"->True},
    CellLabelAutoDelete->False,
    CellFrameLabelMargins->6,
    StyleMenuListing->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headings", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Title"],
    CellFrame->{{0, 0}, {0, 0.5}},
    CellMargins->{{18, 10}, {4, 20}},
    CellGroupingRules->{"TitleGrouping", 0},
    PageBreakBelow->False,
    CellFrameMargins->9,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.35999999999999999, \
-0.10000000000000001}, {0, 0}}, BoxBaselineShift -> -0.20000000000000001], 
                FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "mma"->"Mathematica", "Mma"->"Mathematica", "MMA"->"Mathematica", 
        "gridMathematica"->FormBox[ 
            RowBox[ {"grid", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
        "webMathematica"->FormBox[ 
            RowBox[ {"web", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.17499999999999999, 0}, {0, 0}}]}], 
            TextForm], Inherited},
    LineSpacing->{0.95, 12},
    CounterIncrements->"Title",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->36,
    FontWeight->"Bold"],
  
  Cell[StyleData["Title", "Presentation"],
    CellMargins->{{25, 10}, {15, 30}},
    CellFrameMargins->{{0, 0}, {0, 15}},
    FontSize->54],
  
  Cell[StyleData["Title", "Printout"],
    CellMargins->{{18, 4}, {0, 0}},
    CellFrameMargins->4,
    FontSize->24]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subtitle"],
    CellMargins->{{18, 10}, {0, 10}},
    CellGroupingRules->{"TitleGrouping", 10},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.35999999999999999, \
-0.10000000000000001}, {0, 0}}, BoxBaselineShift -> -0.20000000000000001], 
                FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "mma"->"Mathematica", "Mma"->"Mathematica", "MMA"->"Mathematica", 
        "gridMathematica"->FormBox[ 
            RowBox[ {"grid", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
        "webMathematica"->FormBox[ 
            RowBox[ {"web", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.17499999999999999, 0}, {0, 0}}]}], 
            TextForm], Inherited},
    LineSpacing->{1, 0},
    CounterIncrements->"Subtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->24],
  
  Cell[StyleData["Subtitle", "Presentation"],
    CellMargins->{{25, 10}, {25, 10}},
    FontSize->36],
  
  Cell[StyleData["Subtitle", "Printout"],
    FontSize->18]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SectionFirst"],
    CellFrame->{{0, 0}, {0, 3}},
    CellMargins->{{18, 10}, {10, 30}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CellFrameMargins->5,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.35999999999999999, \
-0.10000000000000001}, {0, 0}}, BoxBaselineShift -> -0.20000000000000001], 
                FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "mma"->"Mathematica", "Mma"->"Mathematica", "MMA"->"Mathematica", 
        "gridMathematica"->FormBox[ 
            RowBox[ {"grid", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
        "webMathematica"->FormBox[ 
            RowBox[ {"web", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.17499999999999999, 0}, {0, 0}}]}], 
            TextForm], Inherited},
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->18,
    FontWeight->"Bold"],
  
  Cell[StyleData["SectionFirst", "Presentation"],
    CellFrame->{{0, 0}, {0, 5}},
    CellMargins->{{25, 10}, {40, 20}},
    FontSize->27],
  
  Cell[StyleData["SectionFirst", "Printout"],
    CellMargins->{{18, 4}, {10, 30}},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Section"],
    CellMargins->{{18, 10}, {10, 30}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", 
                BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.35999999999999999, \
-0.10000000000000001}, {0, 0}}, BoxBaselineShift -> -0.20000000000000001], 
                FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "mma"->"Mathematica", "Mma"->"Mathematica", "MMA"->"Mathematica", 
        "gridMathematica"->FormBox[ 
            RowBox[ {"grid", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
        "webMathematica"->FormBox[ 
            RowBox[ {"web", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.17499999999999999, 0}, {0, 0}}]}], 
            TextForm], Inherited},
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->18,
    FontWeight->"Bold"],
  
  Cell[StyleData["Section", "Presentation"],
    CellMargins->{{25, 10}, {30, 22}},
    FontSize->27],
  
  Cell[StyleData["Section", "Printout"],
    CellMargins->{{18, 4}, {10, 30}},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsection"],
    CellDingbat->"\[FilledSquare]",
    CellMargins->{{18, 50}, {10, 26}},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.35999999999999999, \
-0.10000000000000001}, {0, 0}}, BoxBaselineShift -> -0.20000000000000001], 
                FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "mma"->"Mathematica", "Mma"->"Mathematica", "MMA"->"Mathematica", 
        "gridMathematica"->FormBox[ 
            RowBox[ {"grid", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
        "webMathematica"->FormBox[ 
            RowBox[ {"web", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.17499999999999999, 0}, {0, 0}}]}], 
            TextForm], Inherited},
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsection", "Presentation"],
    CellMargins->{{25, 10}, {25, 25}},
    FontSize->21],
  
  Cell[StyleData["Subsection", "Printout"],
    FontSize->12]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubsection"],
    CellDingbat->"\[FilledSmallSquare]",
    CellMargins->{{18, 50}, {6, 20}},
    CellGroupingRules->{"SectionGrouping", 60},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.35999999999999999, \
-0.10000000000000001}, {0, 0}}, BoxBaselineShift -> -0.20000000000000001], 
                FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "mma"->"Mathematica", "Mma"->"Mathematica", "MMA"->"Mathematica", 
        "gridMathematica"->FormBox[ 
            RowBox[ {"grid", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
        "webMathematica"->FormBox[ 
            RowBox[ {"web", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.17499999999999999, 0}, {0, 0}}]}], 
            TextForm], Inherited},
    CounterIncrements->"Subsubsection",
    FontFamily->"Helvetica",
    FontSize->12,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsubsection", "Presentation"],
    CellMargins->{{25, 10}, {10, 20}},
    FontSize->18],
  
  Cell[StyleData["Subsubsection", "Printout"],
    FontSize->10]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Body Text", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Text"],
    CellMargins->{{18, 10}, {Inherited, 4}},
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.35999999999999999, \
-0.10000000000000001}, {0, 0}}, BoxBaselineShift -> -0.20000000000000001], 
                FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "mma"->"Mathematica", "Mma"->"Mathematica", "MMA"->"Mathematica", 
        "gridMathematica"->FormBox[ 
            RowBox[ {"grid", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
        "webMathematica"->FormBox[ 
            RowBox[ {"web", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.17499999999999999, 0}, {0, 0}}]}], 
            TextForm], Inherited},
    TextJustification->1,
    LineSpacing->{1, 3},
    CounterIncrements->"Text"],
  
  Cell[StyleData["Text", "Presentation"],
    CellMargins->{{25, 10}, {8, 8}},
    FontSize->18],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{18, 4}, {Inherited, 0}},
    Hyphenation->True,
    LineSpacing->{1, 3},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Caption"],
    CellMargins->{{55, 55}, {5, 4}},
    PageBreakAbove->False,
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["Caption", "Presentation"],
    CellMargins->{{75, 55}, {5, 5}},
    FontSize->14],
  
  Cell[StyleData["Caption", "Printout"],
    Hyphenation->True,
    FontSize->8]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Input/Output", "Section"],
  
  Cell["\<\
The cells in this section define styles used for input and output \
to the kernel.  Be careful when modifying, renaming, or removing these \
styles, because the front end associates special meanings with these style \
names.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Input"],
    CellFrame->{{1, 1}, {0, 1}},
    CellMargins->{{55, 10}, {0, 10}},
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultInputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    AutoItalicWords->{},
    LanguageCategory->"Formula",
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    LinebreakAdjustments->{0.85, 2, 10, 0, 1},
    CounterIncrements->"Input",
    FontWeight->"Bold",
    Background->GrayLevel[0.850004]],
  
  Cell[StyleData["Input", "Presentation"],
    CellFrame->{{2, 2}, {0, 2}},
    CellMargins->{{75, 10}, {0, 10}},
    CellFrameMargins->12,
    FontSize->18],
  
  Cell[StyleData["Input", "Printout"],
    CellMargins->{{55, 55}, {0, 10}},
    ShowCellLabel->False,
    LinebreakAdjustments->{0.85, 2, 10, 1, 1},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["InlineInput"],
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    DefaultFormatType->DefaultInputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    AutoItalicWords->{},
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold"],
  
  Cell[StyleData["InlineInput", "Presentation"],
    CellMargins->{{8, 8}, {4, 4}},
    FontSize->18]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Output"],
    CellFrame->{{1, 1}, {1, 0}},
    CellMargins->{{55, 10}, {10, 0}},
    CellEditDuplicate->True,
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    AutoItalicWords->{},
    LanguageCategory->"Formula",
    FormatType->InputForm,
    CounterIncrements->"Output",
    Background->GrayLevel[0.850004]],
  
  Cell[StyleData["Output", "Presentation"],
    CellFrame->{{2, 2}, {2, 0}},
    CellMargins->{{75, 10}, {15, 0}},
    CellFrameMargins->12,
    FontSize->18],
  
  Cell[StyleData["Output", "Printout"],
    CellMargins->{{55, 55}, {10, 0}},
    ShowCellLabel->False,
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["InputOnly"],
    CellFrame->1,
    CellMargins->{{55, 10}, {0, 10}},
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultInputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    AutoItalicWords->{},
    LanguageCategory->"Formula",
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    LinebreakAdjustments->{0.85, 2, 10, 0, 1},
    CounterIncrements->"Input",
    FontWeight->"Bold",
    Background->GrayLevel[0.850004]],
  
  Cell[StyleData["InputOnly", "Presentation"],
    CellFrame->2,
    CellMargins->{{75, 10}, {0, 15}},
    CellFrameMargins->12,
    FontSize->18],
  
  Cell[StyleData["InputOnly", "Printout"],
    CellMargins->{{55, 55}, {0, 10}},
    ShowCellLabel->False,
    LinebreakAdjustments->{0.85, 2, 10, 1, 1},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Message"],
    CellFrame->{{1, 1}, {0, 0}},
    CellDingbat->"\[LongDash]",
    CellMargins->{{55, 10}, {0, 0}},
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    AutoStyleOptions->{"UnmatchedBracketStyle"->None},
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    AutoItalicWords->{},
    FormatType->InputForm,
    CounterIncrements->"Message",
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->10,
    FontSlant->"Oblique"],
  
  Cell[StyleData["Message", "Presentation"],
    CellFrame->{{2, 2}, {0, 0}},
    CellMargins->{{75, 10}, {0, 0}},
    FontSize->15],
  
  Cell[StyleData["Message", "Printout"],
    CellMargins->{{55, 55}, {0, 0}},
    FontSize->8]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Print"],
    CellMargins->{{55, 26}, {1, 6}},
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    TextAlignment->Left,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    AutoItalicWords->{},
    FormatType->InputForm,
    CounterIncrements->"Print",
    StyleMenuListing->None,
    Background->GrayLevel[0.700008]],
  
  Cell[StyleData["Print", "Presentation"],
    CellMargins->{{75, 34}, {0, 8}},
    CellFrameMargins->12,
    FontSize->18],
  
  Cell[StyleData["Print", "Printout"],
    CellMargins->{{54, 72}, {2, 6}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Graphics"],
    CellFrame->{{1, 1}, {0, 0}},
    CellMargins->{{55, 10}, {0, 0}},
    CellGroupingRules->"GraphicsGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Graphics",
    ImageMargins->{{35, Inherited}, {Inherited, 0}},
    StyleMenuListing->None,
    Background->GrayLevel[0.850004]],
  
  Cell[StyleData["Graphics", "Presentation"],
    CellFrame->{{2, 2}, {0, 0}},
    CellMargins->{{75, 10}, {0, 0}},
    CellFrameMargins->12,
    FontSize->18],
  
  Cell[StyleData["Graphics", "Printout"],
    CellMargins->{{55, 55}, {0, 0}},
    ImageSize->{0.0625, 0.0625}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["CellLabel"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontSlant->"Oblique"],
  
  Cell[StyleData["CellLabel", "Presentation"],
    FontSize->14],
  
  Cell[StyleData["CellLabel", "Printout"],
    CellMargins->{{0, Inherited}, {Inherited, Inherited}},
    FontSize->8]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Unique Styles", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Author"],
    CellMargins->{{20, Inherited}, {2, 20}},
    CellGroupingRules->{"TitleGrouping", 20},
    PageBreakBelow->False,
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontSlant->"Italic"],
  
  Cell[StyleData["Author", "Presentation"],
    CellMargins->{{25, 10}, {4, 30}},
    FontSize->21],
  
  Cell[StyleData["Author", "Printout"],
    CellMargins->{{36, Inherited}, {2, 30}},
    FontSize->12]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Address"],
    CellMargins->{{18, Inherited}, {2, 2}},
    CellGroupingRules->{"TitleGrouping", 30},
    PageBreakBelow->False,
    LineSpacing->{1, 1},
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSlant->"Italic"],
  
  Cell[StyleData["Address", "Presentation"],
    CellMargins->{{25, 10}, {15, 4}},
    FontSize->18],
  
  Cell[StyleData["Address", "Printout"],
    CellMargins->{{36, Inherited}, {2, 2}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Abstract"],
    CellFrame->False,
    CellMargins->{{45, 75}, {Inherited, 30}},
    LineSpacing->{1, 1}],
  
  Cell[StyleData["Abstract", "Presentation"],
    CellMargins->{{60, 75}, {4, 35}},
    FontSize->18],
  
  Cell[StyleData["Abstract", "Printout"],
    CellMargins->{{36, 67}, {Inherited, 50}},
    Hyphenation->True,
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Reference"],
    CellMargins->{{18, 40}, {Inherited, 0}},
    TextJustification->1,
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Reference", "Presentation"],
    CellMargins->{{18, 40}, {2, 2}},
    FontSize->18],
  
  Cell[StyleData["Reference", "Printout"],
    Hyphenation->True,
    FontSize->8]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["PictureGroup"],
    CellFrame->{{1, 1}, {0, 0}},
    CellMargins->{{55, Inherited}, {0, 0}},
    CellGroupingRules->"GraphicsGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    ShowCellLabel->False,
    CounterIncrements->"Graphics",
    ImageMargins->{{35, Inherited}, {Inherited, 0}},
    StyleMenuListing->None,
    Background->GrayLevel[0.850004]],
  
  Cell[StyleData["PictureGroup", "Presentation"],
    CellFrame->{{2, 2}, {0, 0}},
    CellMargins->{{75, 10}, {0, 0}},
    CellFrameMargins->12,
    FontSize->18],
  
  Cell[StyleData["PictureGroup", "Printout"],
    CellMargins->{{55, Inherited}, {0, 0}},
    ImageSize->{0.0625, 0.0625}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Hyperlink Styles", "Section"],
  
  Cell["\<\
The cells below define styles useful for making hypertext \
ButtonBoxes.  The \"Hyperlink\" style is for links within the same Notebook, \
or between Notebooks.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Hyperlink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonNote->ButtonData}],
  
  Cell[StyleData["Hyperlink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell["\<\
The following styles are for linking automatically to the on-line \
help system.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MainBookLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["MainBookLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["AddOnsLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["AddOnsLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["RefGuideLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "RefGuide", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["RefGuideLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["GettingStartedLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["GettingStartedLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DemosLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "Demos", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["DemosLink", "Printout"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["DemosLink", "EnhancedPrintout"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["DemosLink", "EnhancedPrintoutNonGray"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["TourLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "Tour", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["TourLink", "Printout"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["TourLink", "EnhancedPrintout"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["TourLink", "EnhancedPrintoutNonGray"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MasterIndexLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "MasterIndex", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["MasterIndexLink", "Printout"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["MasterIndexLink", "EnhancedPrintout"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["MasterIndexLink", "EnhancedPrintoutNonGray"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["OtherInformationLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["OtherInformationLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Palette Styles", "Section"],
  
  Cell["\<\
The cells below define styles that define standard \
ButtonFunctions, for use in palette buttons.\
\>", "Text"],
  
  Cell[StyleData["Paste"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, After]}]&)}],
  
  Cell[StyleData["Evaluate"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, All], 
        SelectionEvaluate[ 
          FrontEnd`InputNotebook[ ], All]}]&)}],
  
  Cell[StyleData["EvaluateCell"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, All], 
        FrontEnd`SelectionMove[ 
          FrontEnd`InputNotebook[ ], All, Cell, 1], 
        FrontEnd`SelectionEvaluateCreateCell[ 
          FrontEnd`InputNotebook[ ], All]}]&)}],
  
  Cell[StyleData["CopyEvaluate"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`SelectionCreateCell[ 
          FrontEnd`InputNotebook[ ], All], 
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, All], 
        FrontEnd`SelectionEvaluate[ 
          FrontEnd`InputNotebook[ ], All]}]&)}],
  
  Cell[StyleData["CopyEvaluateCell"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`SelectionCreateCell[ 
          FrontEnd`InputNotebook[ ], All], 
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, All], 
        FrontEnd`SelectionEvaluateCreateCell[ 
          FrontEnd`InputNotebook[ ], All]}]&)}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Slide Show Styles", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SlideShowNavigationBar"],
    Editable->False,
    CellFrame->True,
    CellMargins->{{0, 0}, {3, 3}},
    CellElementSpacings->{"CellMinHeight"->0.8125},
    CellGroupingRules->{"SectionGrouping", 30},
    CellFrameMargins->False,
    CellFrameColor->GrayLevel[1],
    CellFrameLabelMargins->False,
    TextAlignment->Center,
    CounterIncrements->"SlideShowNavigationBar",
    StyleMenuListing->None,
    FontSize->10,
    Background->GrayLevel[0.8],
    Magnification->1,
    GridBoxOptions->{GridBaseline->Center,
    RowSpacings->0,
    ColumnSpacings->0,
    ColumnWidths->{3.5, 3.5, 3.5, 3.5, 13, 5, 4},
    RowAlignments->Baseline,
    ColumnAlignments->{
      Center, Center, Center, Center, Center, Center, Right, Center}}],
  
  Cell[StyleData["SlideShowNavigationBar", "Presentation"]],
  
  Cell[StyleData["SlideShowNavigationBar", "SlideShow"],
    Deletable->False,
    ShowCellBracket->False,
    CellMargins->{{-1, -1}, {-1, -1}},
    PageBreakAbove->True,
    CellFrameMargins->{{1, 1}, {0, 0}}],
  
  Cell[StyleData["SlideShowNavigationBar", "Printout"],
    CellMargins->{{18, 4}, {4, 4}},
    LineSpacing->{1, 3},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SlideShowSection"],
    CellFrame->{{0, 0}, {0, 0.5}},
    CellMargins->{{0, 0}, {10, 0}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CellFrameMargins->{{12, 4}, {6, 12}},
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.35999999999999999, \
-0.10000000000000001}, {0, 0}}, BoxBaselineShift -> -0.20000000000000001], 
                FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.074999999999999997, \
-0.085000000000000006}, {0, 0}}, BoxBaselineShift -> 0.5], "X"}]], 
        "mma"->"Mathematica", "Mma"->"Mathematica", "MMA"->"Mathematica", 
        "gridMathematica"->FormBox[ 
            RowBox[ {"grid", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.175, 0}, {0, 0}}]}], TextForm], 
        "webMathematica"->FormBox[ 
            RowBox[ {"web", 
              AdjustmentBox[ 
                StyleBox[ "Mathematica", FontSlant -> "Italic"], 
                BoxMargins -> {{-0.17499999999999999, 0}, {0, 0}}]}], 
            TextForm], Inherited},
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->18,
    FontWeight->"Plain",
    FontColor->GrayLevel[1],
    Background->RGBColor[0.408011, 0.440726, 0.8]],
  
  Cell[StyleData["SlideShowSection", "Presentation"],
    CellFrameMargins->{{18, 10}, {10, 18}},
    FontSize->27],
  
  Cell[StyleData["SlideShowSection", "SlideShow"],
    ShowCellBracket->False,
    PageBreakAbove->True],
  
  Cell[StyleData["SlideShowSection", "Printout"],
    CellMargins->{{18, 30}, {0, 30}},
    CellFrameMargins->5,
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SlideHyperlink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontSize->26,
    FontColor->GrayLevel[0.400015],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonMinHeight->0.85,
    ButtonMargins->0.5,
    ButtonNote->None}],
  
  Cell[StyleData["SlideHyperlink", "Presentation"],
    CellMargins->{{10, 10}, {10, 12}},
    FontSize->36],
  
  Cell[StyleData["SlideHyperlink", "SlideShow"]],
  
  Cell[StyleData["SlideHyperlink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SlideTOCLink"],
    CellMargins->{{24, Inherited}, {Inherited, Inherited}},
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Helvetica",
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonMargins->1.5,
    ButtonNote->ButtonData}],
  
  Cell[StyleData["SlideTOCLink", "Presentation"],
    CellMargins->{{35, 10}, {5, 10}},
    FontSize->18],
  
  Cell[StyleData["SlideTOCLink", "SlideShow"]],
  
  Cell[StyleData["SlideTOCLink", "Printout"],
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SlideTOC"],
    CellDingbat->"\[Bullet]",
    CellMargins->{{18, Inherited}, {Inherited, Inherited}},
    StyleMenuListing->None,
    FontFamily->"Helvetica"],
  
  Cell[StyleData["SlideTOC", "Presentation"],
    CellMargins->{{25, 10}, {10, 5}},
    FontSize->18],
  
  Cell[StyleData["SlideTOC", "SlideShow"],
    FontSize->14],
  
  Cell[StyleData["SlideTOC", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Automatic Numbering", "Section"],
  
  Cell["\<\
The following styles are useful for numbered equations, figures, \
etc.  They automatically give the cell a FrameLabel containing a reference to \
a particular counter, and also increment that counter.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedEquation"],
    CellMargins->{{55, 10}, {2, 10}},
    CellFrameLabels->{{None, Cell[ 
            TextData[ {"(", 
              CounterBox[ "NumberedEquation"], ")"}], FontFamily -> 
            "Helvetica", FontSize -> 10, FontWeight -> "Bold"]}, {
        None, None}},
    DefaultFormatType->DefaultInputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    CounterIncrements->"NumberedEquation",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedEquation", "Presentation"],
    CellMargins->{{80, 10}, {2, 10}},
    CellFrameLabels->{{None, Cell[ 
            TextData[ {"(", 
              CounterBox[ "NumberedEquation"], ")"}], FontFamily -> 
            "Helvetica", FontSize -> 16, FontWeight -> "Bold"]}, {
        None, None}},
    FontSize->18],
  
  Cell[StyleData["NumberedEquation", "Printout"],
    CellFrameLabels->{{None, Cell[ 
            TextData[ {"(", 
              CounterBox[ "NumberedEquation"], ")"}], FontFamily -> 
            "Helvetica", FontSize -> 8, FontWeight -> "Bold"]}, {
        None, None}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedFigure"],
    CellMargins->{{55, 145}, {2, 10}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Figure ", 
              CounterBox[ "NumberedFigure"]}], FontFamily -> "Helvetica", 
            FontSize -> 10, FontWeight -> "Bold"], None}},
    TextAlignment->Left,
    CounterIncrements->"NumberedFigure",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedFigure", "Presentation"],
    CellMargins->{{80, 145}, {2, 10}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Figure ", 
              CounterBox[ "NumberedFigure"]}], FontFamily -> "Helvetica", 
            FontSize -> 16, FontWeight -> "Bold"], None}},
    TextAlignment->Left,
    FontSize->18],
  
  Cell[StyleData["NumberedFigure", "Printout"],
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Figure ", 
              CounterBox[ "NumberedFigure"]}], FontFamily -> "Helvetica", 
            FontSize -> 8, FontWeight -> "Bold"], None}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedTable"],
    CellMargins->{{55, 145}, {2, 10}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Table ", 
              CounterBox[ "NumberedTable"]}], FontFamily -> "Helvetica", 
            FontSize -> 10, FontWeight -> "Bold"], None}},
    TextAlignment->Center,
    CounterIncrements->"NumberedTable",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedTable", "Presentation"],
    CellMargins->{{80, 145}, {10, 10}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Table ", 
              CounterBox[ "NumberedTable"]}], FontFamily -> "Helvetica", 
            FontSize -> 16, FontWeight -> "Bold"], None}},
    FontSize->18],
  
  Cell[StyleData["NumberedTable", "Printout"],
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Table ", 
              CounterBox[ "NumberedTable"]}], FontFamily -> "Helvetica", 
            FontSize -> 8, FontWeight -> "Bold"], None}}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Formulas and Programming", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DisplayFormula"],
    CellMargins->{{55, 10}, {2, 10}},
    CellHorizontalScrolling->True,
    DefaultFormatType->DefaultInputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    LanguageCategory->"Formula",
    ScriptLevel->0,
    SingleLetterItalics->True,
    UnderoverscriptBoxOptions->{LimitsPositioning->True}],
  
  Cell[StyleData["DisplayFormula", "Presentation"],
    CellMargins->{{80, 10}, {5, 20}},
    FontSize->18],
  
  Cell[StyleData["DisplayFormula", "Printout"],
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ChemicalFormula"],
    CellMargins->{{55, 10}, {2, 10}},
    DefaultFormatType->DefaultInputFormatType,
    "TwoByteSyntaxCharacterAutoReplacement"->True,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    LanguageCategory->"Formula",
    AutoSpacing->False,
    ScriptLevel->1,
    ScriptBaselineShifts->{0.6, Automatic},
    SingleLetterItalics->False,
    ZeroWidthTimes->True],
  
  Cell[StyleData["ChemicalFormula", "Presentation"],
    CellMargins->{{80, 10}, {10, 10}},
    FontSize->18],
  
  Cell[StyleData["ChemicalFormula", "Printout"],
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Program"],
    CellMargins->{{18, 10}, {Inherited, 6}},
    Hyphenation->False,
    LanguageCategory->"Formula",
    FontFamily->"Courier"],
  
  Cell[StyleData["Program", "Presentation"],
    CellMargins->{{25, 10}, {10, 6}},
    FontSize->18],
  
  Cell[StyleData["Program", "Printout"],
    CellMargins->{{18, 30}, {Inherited, 4}},
    FontSize->9.5]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headers and Footers", "Section"],
  
  Cell[StyleData["Header"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["PageNumber"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontWeight->"Bold"],
  
  Cell[StyleData["Footer"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->7]
  }, Closed]]
  }, Open  ]]
  }]
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 223, 8, 76, "Title"],

Cell[CellGroupData[{
Cell[2002, 63, 131, 3, 31, "Subtitle"],
Cell[2136, 68, 2593, 76, 245, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[4766, 149, 139, 3, 25, "Subtitle"],
Cell[4908, 154, 184, 7, 28, "Text"],

Cell[CellGroupData[{
Cell[5117, 165, 72, 2, 45, "Input"],
Cell[5192, 169, 361, 12, 62, "Text"],
Cell[5556, 183, 912, 15, 44, "Output"]
}, Open  ]],
Cell[6483, 201, 391, 12, 27, "Text"],
Cell[6877, 215, 732, 14, 125, "Input"],
Cell[7612, 231, 391, 10, 46, "Text"],
Cell[8006, 243, 1579, 23, 545, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[9622, 271, 138, 3, 25, "Subtitle"],
Cell[9763, 276, 636, 20, 46, "Text"],
Cell[10402, 298, 155, 4, 45, "Input"],
Cell[10560, 304, 1331, 35, 103, "Text"],

Cell[CellGroupData[{
Cell[11916, 343, 71, 2, 45, "Input"],
Cell[11990, 347, 67, 2, 44, "Output"]
}, Open  ]],
Cell[12072, 352, 730, 21, 65, "Text"],

Cell[CellGroupData[{
Cell[12827, 377, 154, 4, 45, "Input"],
Cell[12984, 383, 1655, 26, 196, "Output"]
}, Open  ]],
Cell[14654, 412, 955, 27, 65, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[15646, 444, 146, 3, 25, "Subtitle"],
Cell[15795, 449, 589, 13, 65, "Text"],

Cell[CellGroupData[{
Cell[16409, 466, 163, 4, 45, "Input"],
Cell[16575, 472, 1527, 24, 196, "Output"]
}, Open  ]],
Cell[18117, 499, 894, 23, 84, "Text"]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

