(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     31309,        979]*)
(*NotebookOutlinePosition[     32039,       1004]*)
(*  CellTagsIndexPosition[     31995,       1000]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Example: ",
    FontSize->24],
  StyleBox["Constraints in non-perturbative models",
    FontSize->24,
    FontColor->RGBColor[0.784314, 0, 0],
    FontVariations->{"CompatibilityType"->0}]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[CellGroupData[{

Cell[TextData[StyleBox["1. Introduction",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["This example demonstrates how one may identify constraints on the \
flux parameters of a class of  supergravity models which are necessary for \
certain vacua to exist. We will begin with a perturbative model, identify the \
constraints, then add non-perturbative corrections and study their effects on \
the constraints. The models we will use are based on ",
    FontFamily->"Arial"],
  StyleBox["hep-th/0507173",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[".",
    FontFamily->"Arial"],
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["The perturbative model,  \n\n",
    FontFamily->"Arial"],
  StyleBox["\t",
    FontFamily->"Helvetica",
    FontWeight->"Bold"],
  StyleBox["3 superfields: \t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + \[ImaginaryI] \[Tau] ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Z = z + \[ImaginaryI] \[Zeta]  ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[",  ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S = s + \[ImaginaryI] \[Sigma] \n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Kahler potential: \t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["-3 Log[T+Conjugate[T]]-3 Log[Z+Conjugate[Z]]-Log[S+Conjugate[S]]\n\
\t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Superpotential:   \t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\[ImaginaryI] ( \[Xi] + \[ImaginaryI] e T )+( \[Epsilon] + \
\[ImaginaryI] p T ) Z + \[ImaginaryI]  ( \[Mu] + \[ImaginaryI] q T ) ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  Cell[BoxData[
      FormBox[
        StyleBox[\(Z\^2/2\),
          FontFamily->"Arial",
          FontColor->RGBColor[0, 0, 1]], TraditionalForm]],
    FontFamily->"Arial",
    FontSize->14],
  StyleBox["\n\t",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox["(which is",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" independent ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["of",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S )",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\tFlux parameters:\t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\[Xi] , e , \[Epsilon] , p , \[Mu] , q . \n",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\tGiven constraints:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" \t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["q \[Epsilon] - p \[Mu] = 0",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["2. Setting up the perturbative model",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle",
  CellDingbat->None],

Cell[TextData[StyleBox["First, load the package,",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< Stringvacua`\)], "Input",
  CellLabel->"In[3]:="],

Cell["\<\
Stringvacua package, Version 1.0, December 2007
James Gray, Yang-Hui He, Anton Ilderton and Andre Lukas\
\>", "Text",
  CellFrame->True,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->13,
  FontWeight->"Bold",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[1, 1, 0.900008]],

Cell[BoxData["\<\"Execute \
\\*ButtonBox[StyleBox[\\\"SimpleMinimise[]\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonEvaluator->\\\"Automatic\\\
\",ButtonFunction:>(SimpleMinimise[]&)] for a simple interactive calculation \
or click \\*ButtonBox[StyleBox[\\\"here\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[\
 { FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Generalities\\\"]}]&)] to \
open the Help Browser, which contains \
\\*ButtonBox[StyleBox[\\\"examples\\\",FontColor->RGBColor[0,0,0.7],FontSize->\
12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { \
FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Examples\\\", \
\\\"Examples_Overview.nb\\\"]}]&)].\"\>"], "Output",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->12,
  FontColor->RGBColor[0.6, 0.300008, 0]]
}, Open  ]],

Cell[TextData[StyleBox["We now create the model. Note that the commands \
CalcModel and CreateModel can be nested as shown below,",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    \(\(\(Pmodel = 
        CalcModel[
          CreateModel[{T, Z, S}, {t, z, 
              s}, {\[Tau], \[Zeta], \[Sigma]}, \(-3\)\ Log[
                  T + Conjugate[T]]\  - \ 3\ Log[Z + Conjugate[Z]] - 
              Log[S + Conjugate[
                    S]], \[ImaginaryI] \((\[Xi] + \[ImaginaryI]\ e\ T)\)\  + \
\ \((\[Epsilon] + \[ImaginaryI]\ p\ T)\) 
                Z\  + \[ImaginaryI]\ \((\[Mu] + \[ImaginaryI]\ q\ T)\) 
                Z\^2/2, \ 
            Par -> {\[Xi], e, \[Epsilon], p, \[Mu], 
                q}]];\)\(\[IndentingNewLine]\)
    \)\)], "Input",
  CellLabel->"In[2]:="],

Cell[TextData[{
  StyleBox[" There is also a given ",
    FontFamily->"Arial",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["constraint",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" on the parameters of this model, ",
    FontFamily->"Arial",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["q \[Epsilon]-p \[Mu] = 0",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" e",
    FontFamily->"Arial",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["ntered using",
    FontFamily->"Arial",
    FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Constraint = 
      q\ \[Epsilon]\  - \(\(p\)\(\ \)\(\[Mu]\)\(\ \)\)\)], "Input",
  CellLabel->"In[3]:="],

Cell[BoxData[
    \(q\ \[Epsilon] - p\ \[Mu]\)], "Output",
  CellLabel->"Out[3]="]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["2. Constraints on flux parameters",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["Consider looking for SUSY vacua at a ",
    FontFamily->"Arial"],
  StyleBox["specific point in field space,",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" say",
    FontFamily->"Arial"],
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox["t=1, s=24, ",
    FontFamily->"Arial",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox["\[Sigma]",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox["=0.",
    FontFamily->"Arial",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox["This may be ",
    FontFamily->"Arial"],
  StyleBox["motivated by properties of the desired vacua such as vanishing \
theta terms, etc. The question is, which sets of flux parameters allow the \
presence of SUSY vacua at this location?\n\nFirst, we combine the objects we \
wish to vanish using the module ",
    FontFamily->"Arial"],
  StyleBox["CreateIdeal[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    \(\(MyIdeal = 
        CreateIdeal[{IFTerms /. Pmodel, \ Constraint, t - 1, 
            s - 24, \[Sigma]}];\)\)], "Input",
  CellLabel->"In[4]:="],

Cell[TextData[{
  StyleBox["where the braces contain all of the objects which we wish to \
vanish. We include the F-terms, in a suitable form indicated by prefixing an \
\"",
    FontFamily->"Arial"],
  StyleBox["I",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["\" to the relevant keyword, the known constraint on the flux \
parameters, and the polynomials which correspond to the field values we have \
chosen -- i.e. we are looking for vacua at ",
    FontFamily->"Arial"],
  StyleBox["t = 1",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[", so we include the polynomial ",
    FontFamily->"Arial"],
  StyleBox["t - 1",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[".  We now eliminate all of the field variables from the \
equations, which leaves equations depending only on the parameters -- these \
are the necessary constraints.  This step is performed with the module ",
    FontFamily->"Arial"],
  StyleBox["Elimination[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", where the option ",
    FontFamily->"Arial"],
  StyleBox["VarPar ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["selects the variables to be eliminated. In our case we want to \
eliminate the field variables present in  ",
    FontFamily->"Arial"],
  StyleBox["MyIdeal",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Var /. MyIdeal\)], "Input",
  CellLabel->"In[5]:="],

Cell[BoxData[
    \({s, t, z, \[Zeta], \[Sigma], \[Tau]}\)], "Output",
  CellLabel->"Out[5]="]
}, Open  ]],

Cell[TextData[{
  "which are just the real and imaginary parts of the superfields.  We now \
call ",
  StyleBox["Elimination[ ]",
    FontWeight->"Bold"],
  " and present the results in table form using ",
  StyleBox["//TF",
    FontWeight->"Bold"],
  ","
}], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Elimination[MyIdeal, 
        VarPar \[Rule] {t, z, \[Zeta], \[Tau], s, \[Sigma]}] // TF\)], "Input",\

  CellLabel->"In[6]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \(\*"\<\"\"\>"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({e, p, q, \[Epsilon], \[Mu], \[Xi]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({e\ \[Mu] - q\ \[Xi], q\ \[Epsilon] - p\ \[Mu], 
                  e\ \[Epsilon] - p\ \[Xi]}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["The list ",
    FontFamily->"Arial"],
  StyleBox["Gen",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" contains the constraints -- the second in this list is the \
original constraint, but the first,  ",
    FontFamily->"Arial"],
  StyleBox["e \[Mu]- q \[Xi] ",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["= 0",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[",",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox[" ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold"],
  StyleBox["is a ",
    FontFamily->"Arial"],
  StyleBox["new constraint",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" which the flux parameters must satisfy in order for SUSY vacua \
to exist at the chosen point in field space. The third constraint in the list \
is redundant, given the other two, if none of the parameters vanish.",
    FontFamily->"Arial"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["3. Non-perturbative corrections",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["We now add a non-perturbative correction to the superpotential of \
our model, \n\n",
    FontFamily->"Arial"],
  StyleBox["\t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold"],
  StyleBox["3 complex fields: \t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + \[ImaginaryI] \[Tau] ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Z = z + \[ImaginaryI] \[Zeta]  ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[",  ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S = s + \[ImaginaryI] \[Sigma]\n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Kahler potential:\t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["-3 Log[T+Conjugate[T]]-3 Log[Z+Conjugate[Z]]-Log[S+Conjugate[S]]\n\
\t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Superpotential:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\t    ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Plain",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\[ImaginaryI] ( \[Xi] + \[ImaginaryI] e T )+( \[Epsilon] + \
\[ImaginaryI] p T ) Z + \[ImaginaryI] ( \[Mu] + \[ImaginaryI] q T ) ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  Cell[BoxData[
      \(TraditionalForm\`Z\^2\)],
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["/2 + \[Lambda] Exp[-c S]\n\t(which now includes a",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" non-perturbative correction ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["given by the exponential)",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\tFlux parameters:\t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" \[Xi] , e , \[Epsilon] , p , \[Mu] , q , \[Lambda] , c . \n",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\tGiven constraints:\t",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["q \[Epsilon] - p \[Mu] = 0\n",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\nand input the supergravity model into the package using ",
    FontFamily->"Arial"],
  StyleBox["CreateModel",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[". To add the keywords necessary for our methods, using ",
    FontFamily->"Arial"],
  StyleBox["CalcModel",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial"],
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["we must include dummy variables which replace the exponential \
function (for more information, see the example on finding vacua in \
non-perturbative models). Note that the calculation below may take a few \
minutes, depending on the speed of your machine,",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    \(\(NONPmodel = 
        CalcModel[
          CreateModel[{T, Z, S}, {t, z, 
              s}, {\[Tau], \[Zeta], \[Sigma]}, \(-3\)\ Log[
                  T + Conjugate[T]]\  - \ 3\ Log[Z + Conjugate[Z]] - 
              Log[S + Conjugate[
                    S]], \[ImaginaryI] \((\[Xi] + \[ImaginaryI]\ e\ T)\)\  + \
\ \((\[Epsilon] + \[ImaginaryI]\ p\ T)\) 
                Z\  + \[ImaginaryI]\ \((\[Mu] + \[ImaginaryI]\ q\ T)\) 
                Z\^2/2 + \[Lambda]\ Exp[\(-c\)\ S], \ 
            Par -> {\[Xi], e, \[Epsilon], p, \[Mu], q, \[Lambda], c}], 
          CDummies \[Rule] {Exp[\(-c\)\ S] \[Rule] X, 
              Exp[\(-c\)\ Conjugate[S]] \[Rule] Y}, 
          Dummies \[Rule] {Exp[\(-c\)\ s] \[Rule] x, 
              Exp[\(-\[ImaginaryI]\)\ c\ \[Sigma]] \[Rule] y}];\)\)], "Input",\

  CellLabel->"In[7]:="],

Cell[TextData[{
  StyleBox["The  ",
    FontFamily->"Arial",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["constraint",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" we initially imposed on the flux parameters also applies here,",
    FontFamily->"Arial",
    FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(Constraint\)\(\ \)\)\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(q\ \[Epsilon] - p\ \[Mu]\)], "Output",
  CellLabel->"Out[8]="]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["4. Further constraints on flux parameters",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle",
  CellDingbat->None],

Cell[TextData[{
  StyleBox["Again we look for ",
    FontFamily->"Arial"],
  StyleBox["SUSY ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["vacua, at  ",
    FontFamily->"Arial"],
  StyleBox["t = 1, s = 24, \[Sigma] = 0",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[", but now in our non-perturbative model.  This will allow us to \
observe how non-perturbative corrections effect the constraints we earlier \
derived.\n\nAs before,  we collect the objects we wish to vanish, this time \
into an object  ",
    FontFamily->"Arial"],
  StyleBox["NPIdeal",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    \(\(NPIdeal = 
        CreateIdeal[{IFTerms /. NONPmodel, \ Constraint, \ t - 1, 
            s - 24, \[Sigma], \ y - 1}];\)\)], "Input",
  CellLabel->"In[9]:="],

Cell[TextData[{
  StyleBox["Note that setting the value \[Sigma] = 0",
    FontFamily->"Arial"],
  StyleBox["  ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["implies a fixed value of the dummy variable ",
    FontFamily->"Arial"],
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["y",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" = Exp[ -\[ImaginaryI] c \[Sigma] ] = 1.  In the above, we have \
specified this by including the polynomial ",
    FontFamily->"Arial"],
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["y - ",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox["1",
    FontFamily->"Arial"],
  StyleBox[" in our list.  However, even though we have similarly fixed the \
value of ",
    FontFamily->"Arial"],
  StyleBox["s",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" at 24, ",
    FontFamily->"Arial"],
  StyleBox["the dummy variable ",
    FontFamily->"Arial"],
  StyleBox["x",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" still carries a dependence on the parameter ",
    FontFamily->"Arial"],
  StyleBox["c",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[". As such, we cannot fix its value.\n\nTo find our constraints, \
we now eliminate all the field variables from the equations -- again, we use \
the module ",
    FontFamily->"Arial"],
  StyleBox["Elimination[ ] ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" with the option ",
    FontFamily->"Arial"],
  StyleBox["VarPar ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["set to the list of field variables -- note that ",
    FontFamily->"Arial"],
  StyleBox["x",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" is not included in this list as it depends on the parameter ",
    FontFamily->"Arial"],
  StyleBox["c",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[",",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NoFields = 
      Elimination[NPIdeal, 
        VarPar \[Rule] {s, t, y, z, \[Zeta], \[Tau], \[Sigma]}]\)], "Input",
  CellLabel->"In[10]:="],

Cell[BoxData[
    \({Var \[Rule] {c, e, p, q, x, \[Epsilon], \[Lambda], \[Mu], \[Xi]}, 
      Par \[Rule] {}, 
      Gen \[Rule] {q\ \[Epsilon] - p\ \[Mu], e\ q\ \[Mu] - q\^2\ \[Xi], 
          e\ p\ \[Mu] - p\ q\ \[Xi], 
          e\^2\ \[Mu] - e\ q\ \[Xi], \(-e\)\ \[Mu] + 96\ c\ e\ \[Mu] + 
            q\ \[Xi] - 96\ c\ q\ \[Xi], 
          e\ p\ \[Epsilon] - p\^2\ \[Xi], \(-e\)\ \[Epsilon] + 
            96\ c\ e\ \[Epsilon] + p\ \[Xi] - 
            96\ c\ p\ \[Xi], \(-4\)\ e\^2\ \[Mu] + 
            3\ e\ x\ \[Lambda]\ \[Mu] + 4\ e\ q\ \[Xi] - 
            3\ q\ x\ \[Lambda]\ \[Xi], 
          e\ \[Mu] - x\ \[Lambda]\ \[Mu] + 24\ c\ x\ \[Lambda]\ \[Mu] - 
            q\ \[Xi], \(-4\)\ e\^2\ \[Epsilon] + 
            3\ e\ x\ \[Epsilon]\ \[Lambda] + 4\ e\ p\ \[Xi] - 
            3\ p\ x\ \[Lambda]\ \[Xi], 
          e\ \[Epsilon] - x\ \[Epsilon]\ \[Lambda] + 
            24\ c\ x\ \[Epsilon]\ \[Lambda] - 
            p\ \[Xi], \(-q\)\ x\ \[Lambda] + 
            24\ c\ q\ x\ \[Lambda], \(-p\)\ x\ \[Lambda] + 
            24\ c\ p\ x\ \[Lambda], 
          x\ \[Lambda]\ \[Xi] - 120\ c\ x\ \[Lambda]\ \[Xi] + 
            2304\ c\^2\ x\ \[Lambda]\ \[Xi], 
          2\ e\ p\ \[Epsilon] - 3\ p\ x\ \[Epsilon]\ \[Lambda] - 
            12\ e\ x\ \[Lambda]\ \[Mu] + 9\ x\^2\ \[Lambda]\^2\ \[Mu] - 
            2\ p\^2\ \[Xi] + 
            6\ q\ x\ \[Lambda]\ \[Xi], \(-p\^2\)\ x\ \[Lambda] - 
            2\ e\ q\ x\ \[Lambda] + 3\ q\ x\^2\ \[Lambda]\^2, 
          4\ e\ x\ \[Lambda] - 96\ c\ e\ x\ \[Lambda] - 
            3\ x\^2\ \[Lambda]\^2 + 72\ c\ x\^2\ \[Lambda]\^2, 
          2\ e\ x\ \[Lambda] - 112\ c\ e\ x\ \[Lambda] + 
            1536\ c\^2\ e\ x\ \[Lambda] - x\^2\ \[Lambda]\^2 + 
            24\ c\ x\^2\ \[Lambda]\^2}}\)], "Output",
  CellLabel->"Out[10]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["These are our new set of constraints! There are still \
simplifications which we can make. One option would be to use ",
    FontFamily->"Arial"],
  StyleBox["PrimDec[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" to split the equations into smaller groups. Another option is to \
ask that none of the parameters vanish (essentially ensuring that we are \
indeed studying the model we proposed, and not some simpler model).\n\nWe use \
the module ",
    FontFamily->"Arial"],
  StyleBox["Saturation[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" to obtain constraints on non-vanishing parameters.  The option ",
    
    FontFamily->"Arial"],
  StyleBox["Poly",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" is used to specify a list of polynomials which we wish to be \
non-zero. We set this polynomial to the product of the parameters -- this \
polynomial is non-zero if and only if all parameters are non-zero. The result \
is shown below,",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Saturation[NoFields, 
        Poly \[Rule] {x\ \[Xi]\ \[Mu]\ \[Lambda]\ p\ q\ e\ \[Epsilon]\ c}] // 
      TF\)], "Input",
  CellLabel->"In[11]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \(\*"\<\"\"\>"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({\(-1\) + 24\ c, e\ \[Mu] - q\ \[Xi], 
                  q\ \[Epsilon] - p\ \[Mu], 
                  e\ \[Epsilon] - p\ \[Xi], \(-p\)\ \[Epsilon] + 
                    3\ x\ \[Lambda]\ \[Mu] - 2\ q\ \[Xi], \(-p\^2\) - 
                    2\ e\ q + 3\ q\ x\ \[Lambda]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({5}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({c, e, p, q, x, \[Epsilon], \[Lambda], \[Mu], \[Xi]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["The output contains a list of constraints, in ",
    FontFamily->"Arial"],
  StyleBox["Gen",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", which are required to be zero in order for SUSY vacua of the \
type of interest to exist. In this list we see our original constraint and we \
recover the constraint we found in the perturbative case, which therefore \
still applies.\n\nAdditionally, we see that when non-perturbative effects are \
present ( \[Lambda] \[NotEqual] 0, ",
    FontFamily->"Arial"],
  StyleBox["c",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" \[NotEqual] 0 ) there are two  further constraints, e.g.  ",
    FontFamily->"Arial"],
  StyleBox["-p \[Epsilon] - 2 q \[Xi] + 3 ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["x",
    FontWeight->"Bold"],
  " ",
  StyleBox["\[Lambda] \[Mu] = 0",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",  along with a constraint  fixing the value of  ",
    FontFamily->"Arial"],
  StyleBox["c.",
    FontFamily->"Arial",
    FontSlant->"Italic"]
}], "Text"]
}, Closed]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 777}},
CellGrouping->Manual,
WindowSize->{761, 651},
WindowMargins->{{92, Automatic}, {Automatic, -55}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 276, 9, 76, "Title"],

Cell[CellGroupData[{
Cell[2055, 64, 131, 3, 31, "Subtitle"],
Cell[2189, 69, 4557, 145, 223, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6783, 219, 152, 3, 25, "Subtitle"],
Cell[6938, 224, 83, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[7046, 229, 72, 2, 45, "Input"],
Cell[7121, 233, 361, 12, 62, "Text"],
Cell[7485, 247, 912, 15, 44, "Output"]
}, Open  ]],
Cell[8412, 265, 164, 2, 27, "Text"],
Cell[8579, 269, 620, 14, 107, "Input"],
Cell[9202, 285, 753, 23, 28, "Text"],

Cell[CellGroupData[{
Cell[9980, 312, 125, 3, 45, "Input"],
Cell[10108, 317, 82, 2, 44, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[10239, 325, 149, 3, 25, "Subtitle"],
Cell[10391, 330, 1146, 39, 104, "Text"],
Cell[11540, 371, 165, 4, 45, "Input"],
Cell[11708, 377, 1438, 40, 122, "Text"],

Cell[CellGroupData[{
Cell[13171, 421, 71, 2, 45, "Input"],
Cell[13245, 425, 94, 2, 44, "Output"]
}, Open  ]],
Cell[13354, 430, 290, 10, 46, "Text"],

Cell[CellGroupData[{
Cell[13669, 444, 167, 5, 37, "Input"],
Cell[13839, 451, 1265, 35, 82, "Output"]
}, Open  ]],
Cell[15119, 489, 1071, 35, 67, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[16227, 529, 147, 3, 25, "Subtitle"],
Cell[16377, 534, 4313, 132, 261, "Text"],
Cell[20693, 668, 844, 17, 148, "Input"],
Cell[21540, 687, 389, 11, 27, "Text"],

Cell[CellGroupData[{
Cell[21954, 702, 81, 2, 45, "Input"],
Cell[22038, 706, 82, 2, 44, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[22169, 714, 157, 3, 25, "Subtitle"],
Cell[22329, 719, 692, 21, 84, "Text"],
Cell[23024, 742, 179, 4, 45, "Input"],
Cell[23206, 748, 2006, 67, 122, "Text"],

Cell[CellGroupData[{
Cell[25237, 819, 161, 4, 45, "Input"],
Cell[25401, 825, 1794, 33, 158, "Output"]
}, Open  ]],
Cell[27210, 861, 1068, 26, 141, "Text"],

Cell[CellGroupData[{
Cell[28303, 891, 187, 5, 37, "Input"],
Cell[28493, 898, 1675, 44, 98, "Output"]
}, Open  ]],
Cell[30183, 945, 1110, 31, 122, "Text"]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

