(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     37263,       1142]*)
(*NotebookOutlinePosition[     37992,       1167]*)
(*  CellTagsIndexPosition[     37948,       1163]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Example:  ",
    FontSize->24],
  StyleBox["A non-perturbative example",
    FontSize->24,
    FontColor->RGBColor[0.784314, 0, 0]]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[CellGroupData[{

Cell[TextData[StyleBox["1. Introduction",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["Some effects, such as non-perturbative corrections to the \
superpotential, may appear as non-rational functions of the fields. The \
STRINGVACUA package is ideally suited to systems in which the scalar \
potential, the F-terms, etc, can be written as rational functions. However, \
it can also deal with non-rational functions, as we now describe. We will \
consider the model below, which contains two fields, one of which enters the \
superpotential through an exponential,\n",
    FontFamily->"Arial"],
  StyleBox["\n",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Superfields:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["\t\t\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + \[ImaginaryI] \[Tau], S = s + \[ImaginaryI] \[Sigma]\n\t",
    
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Kahler potential:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\t\tK = ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  Cell[BoxData[
      FormBox[
        RowBox[{"-", 
          RowBox[{"Log", " ", "[", 
            RowBox[{
              FormBox[
                StyleBox["T",
                  FontSlant->"Plain"],
                "TraditionalForm"], "+", 
              RowBox[{"Conjugate", "[", 
                StyleBox["T",
                  FontSlant->"Plain"], "]"}]}], "]"}]}], TraditionalForm]],
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  Cell[BoxData[
      FormBox[
        RowBox[{\(-3\), 
          RowBox[{"Log", " ", "[", 
            RowBox[{"S", "+", 
              RowBox[{"Conjugate", "[", 
                StyleBox["S",
                  FontSlant->"Plain"], "]"}]}], "]"}]}], TraditionalForm]],
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Superpotential:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\t\tW = T + Exp[",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["-",
    FontFamily->"Arial",
    FontSize->14,
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["S]",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["2. Setting up the model",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[StyleBox["Load the package,",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< Stringvacua`\)], "Input",
  CellLabel->"In[2]:="],

Cell["\<\
Stringvacua package, Version 1.0, December 2007
James Gray, Yang-Hui He, Anton Ilderton and Andre Lukas\
\>", "Text",
  CellFrame->True,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->13,
  FontWeight->"Bold",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[1, 1, 0.900008]],

Cell[BoxData["\<\"Execute \
\\*ButtonBox[StyleBox[\\\"SimpleMinimise[]\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonEvaluator->\\\"Automatic\\\
\",ButtonFunction:>(SimpleMinimise[]&)] for a simple interactive calculation \
or click \\*ButtonBox[StyleBox[\\\"here\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[\
 { FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Generalities\\\"]}]&)] to \
open the Help Browser, which contains \
\\*ButtonBox[StyleBox[\\\"examples\\\",FontColor->RGBColor[0,0,0.7],FontSize->\
12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { \
FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Examples\\\", \
\\\"Examples_Overview.nb\\\"]}]&)].\"\>"], "Output",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->12,
  FontColor->RGBColor[0.6, 0.300008, 0]]
}, Open  ]],

Cell[TextData[StyleBox["and create the supergravity model in the usual way",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    \(\(model = 
        CreateModel[{T, S}, {t, 
            s}, {\[Tau], \[Sigma]}, \(-\ \ Log[T + Conjugate[T]]\) - 
            3  Log[S + Conjugate[S]], T + Exp[\(-S\)]];\)\)], "Input",
  CellLabel->"In[2]:="],

Cell[TextData[{
  StyleBox["This calculates the scalar potential etc, as normal. In order to \
apply our methods, the system must be transformed into a set of polynomials. \
To do this we must include dummy variables which eliminate any non-rational \
functions (here the exponential function) from all of the expressions. This \
is achieved through the options ",
    FontFamily->"Arial"],
  StyleBox["Dummies, CDummies",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" and ",
    FontFamily->"Arial"],
  StyleBox["RDummies",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" to give a set of replacements for non-rational functions of real \
fields, superfields, (respectively) and a set of any additional rules for any \
other replacements. These options should be included in the call to ",
    FontFamily->"Arial"],
  StyleBox["CalcModel[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", as shown below.",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    \(\(Fullmodel = 
        CalcModel[model, 
          Dummies \[Rule] {Exp[\(-s\)] \[Rule] x, 
              Exp[\(-I\) \[Sigma]] \[Rule] y}, 
          CDummies \[Rule] {Exp[\(-S\)] \[Rule] X, 
              Exp[\(-Conjugate[S]\)] \[Rule] Y}];\)\)], "Input",
  CellLabel->"In[3]:="],

Cell[TextData[{
  StyleBox["These options correspond to choosing the following set of dummy \
variables,\n\t\t",
    FontFamily->"Arial"],
  StyleBox["x  = Exp ( -s ), \ty = Exp (-\[ImaginaryI] \[Sigma] )\n\t\tX = \
Exp ( -S ),\t\tY = Exp (-Conjugate[S] )",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["\nTo see the effect of these options, compare the scalar \
potential of the supergravity model,",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Pot /. Fullmodel\)], "Input",
  CellLabel->"In[4]:="],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\(-2\)\ s\)\ \((3 + 12\ s + 4\ s\^2 + 3\ \
\[ExponentialE]\^\(2\ s\)\ t\^2 + 3\ \[ExponentialE]\^\(2\ s\)\ \[Tau]\^2 + 6\
\ \[ExponentialE]\^s\ \((\(-1\) + 2\ s)\)\ t\ Cos[\[Sigma]] - 6\ \
\[ExponentialE]\^s\ \((1 + 2\ s)\)\ \[Tau]\ Sin[\[Sigma]])\)\)\/\(48\ s\^3\ t\
\)\)], "Output",
  CellLabel->"Out[4]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["with the corresponding expression ",
    FontFamily->"Arial"],
  StyleBox["IPot",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" in the extended model list ",
    FontFamily->"Arial"],
  StyleBox["Fullmodel",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IPot /. Fullmodel\)], "Input",
  CellLabel->"In[5]:="],

Cell[BoxData[
    \({Var \[Rule] {t, s, \[Tau], \[Sigma], x, y}, Par \[Rule] {}, 
      Gen \[Rule] {\(-3\)\ t\ x + 6\ s\ t\ x + 3\ t\^2\ y + 3\ x\^2\ y + 
            12\ s\ x\^2\ y + 4\ s\^2\ x\^2\ y - 3\ t\ x\ y\^2 + 
            6\ s\ t\ x\ y\^2 + 3\ \[ImaginaryI]\ x\ \[Tau] + 
            6\ \[ImaginaryI]\ s\ x\ \[Tau] - 
            3\ \[ImaginaryI]\ x\ y\^2\ \[Tau] - 
            6\ \[ImaginaryI]\ s\ x\ y\^2\ \[Tau] + 
            3\ y\ \[Tau]\^2}}\)], "Output",
  CellLabel->"Out[5]="]
}, Open  ]],

Cell[TextData[StyleBox["This is polynomial in an enlarged field space -- the \
variables {x, y} have been added to the variables {t , s , \[Tau], \[Sigma]}. \
 Below, we will use our methods to find and classify all of the vacua of this \
model. ",
  FontFamily->"Arial"]], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["3. Finding and classifying vacua",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["We can use ",
    FontFamily->"Arial"],
  StyleBox["SaturationExpand[ ] ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["to find and classify all vacua of this model. The call below \
categorizes the vacua with respect to supersymmetry breaking and dimension, \
and stores them in a list called ",
    FontFamily->"Arial"],
  StyleBox["SatEx",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[". ",
    FontFamily->"Arial"],
  StyleBox["It is important",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]],
  StyleBox[" when using dummy variables to set the option ",
    FontFamily->"Arial"],
  StyleBox["Dim",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" to a list of numbers from 0 up to the number of pieces of \
information withheld from the model -- here we have withheld the two \
relations x = Exp( -s ),  y = Exp(-\[ImaginaryI] \[Sigma] ), so the list of \
dimensions takes values 0, 1 and 2, as shown:",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(SatEx = SaturationExpand[Fullmodel, Dim \[Rule] {0, 1, 2}]\)], "Input",
  CellLabel->"In[6]:="],

Cell[BoxData[
    \({}\)], "Output",
  CellLabel->"Out[6]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["From the above list, we may extract, for example, the equations \
describing SUSY vacua using ",
    FontFamily->"Arial"],
  StyleBox["{ } /. SatEx", "Input",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[". However, an inspection of these equations reveals that their \
solutions require unphysical field values (see also the ",
    FontFamily->"Arial"],
  StyleBox["Extensions",
    FontFamily->"Arial",
    FontVariations->{"Underline"->True}],
  StyleBox[" section, below ).  Consider instead the vacua in which  the \
\"s\" and \"t\" F-terms are non-zero, while the \"",
    FontFamily->"Arial"],
  StyleBox["\[Sigma]\"",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox[" and \"",
    FontFamily->"Arial"],
  StyleBox["\[Tau]\"",
    FontFamily->"Arial",
    FontSize->14],
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["F-terms vanish. This is extracted from our decomposition ",
    FontFamily->"Arial"],
  StyleBox["SatEx",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" using",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({s, t} /. SatEx\)], "Input",
  CellLabel->"In[6]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], 
            1 + y, \(-3\)\ t\^2 + 7\ t\ x + 2\ s\ t\ x + 2\ x\^2 + 
              4\ s\ x\^2, \(-7\)\ t - 2\ s\ t + x + 8\ s\ x + 
              4\ s\^2\ x, \(-11\)\ t\^2 + 2\ s\ t\^2 + 11\ t\ x + 6\ x\^2 + 
              16\ s\ x\^2, \(-15\)\ t - 2\ s\ t + 2\ s\^2\ t - 3\ x - 
              4\ s\ x, 
            3\ t\^3 - 24\ t\^2\ x + 23\ t\ x\^2 + 10\ x\^3 + 24\ s\ x\^3, 
            3\ t\^4 - 18\ t\^3\ x + 11\ t\^2\ x\^2 - 28\ t\ x\^3 - 
              4\ x\^4, \(-18\) - 78\ s - 41\ s\^2 + 4\ s\^3 + 4\ s\^4}, 
        Dim \[Rule] {2}, Var \[Rule] {s, t, x, y, \[Sigma], \[Tau]}, 
        Par \[Rule] {}}, {Gen \[Rule] {\[Tau], \(-1\) + y, 
            3\ t\^2 + 7\ t\ x + 2\ s\ t\ x - 2\ x\^2 - 4\ s\ x\^2, 
            7\ t + 2\ s\ t + x + 8\ s\ x + 4\ s\^2\ x, \(-11\)\ t\^2 + 
              2\ s\ t\^2 - 11\ t\ x + 6\ x\^2 + 16\ s\ x\^2, \(-15\)\ t - 
              2\ s\ t + 2\ s\^2\ t + 3\ x + 4\ s\ x, \(-3\)\ t\^3 - 
              24\ t\^2\ x - 23\ t\ x\^2 + 10\ x\^3 + 24\ s\ x\^3, 
            3\ t\^4 + 18\ t\^3\ x + 11\ t\^2\ x\^2 + 28\ t\ x\^3 - 
              4\ x\^4, \(-18\) - 78\ s - 41\ s\^2 + 4\ s\^3 + 4\ s\^4}, 
        Dim \[Rule] {2}, Var \[Rule] {s, t, x, y, \[Sigma], \[Tau]}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[6]="]
}, Open  ]],

Cell[TextData[StyleBox["There are two pieces here,",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[{s, t} /. SatEx]\)], "Input",
  CellLabel->"In[6]:="],

Cell[BoxData[
    \(2\)], "Output",
  CellLabel->"Out[6]="]
}, Open  ]],

Cell[TextData[StyleBox["Let us look at the first of these , ",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(({s, t} /. SatEx)\)[\([1]\)] // TF\)], "Input",
  CellLabel->"In[7]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \(\*"\<\"\"\>"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({\[Tau], 
                  1 + y, \(-3\)\ t\^2 + 7\ t\ x + 2\ s\ t\ x + 2\ x\^2 + 
                    4\ s\ x\^2, \(-7\)\ t - 2\ s\ t + x + 8\ s\ x + 
                    4\ s\^2\ x, \(-11\)\ t\^2 + 2\ s\ t\^2 + 11\ t\ x + 
                    6\ x\^2 + 16\ s\ x\^2, \(-15\)\ t - 2\ s\ t + 
                    2\ s\^2\ t - 3\ x - 4\ s\ x, 
                  3\ t\^3 - 24\ t\^2\ x + 23\ t\ x\^2 + 10\ x\^3 + 
                    24\ s\ x\^3, 
                  3\ t\^4 - 18\ t\^3\ x + 11\ t\^2\ x\^2 - 28\ t\ x\^3 - 
                    4\ x\^4, \(-18\) - 78\ s - 41\ s\^2 + 4\ s\^3 + 
                    4\ s\^4}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["The second and final polynomials in the list of generators ",
    FontFamily->"Arial"],
  StyleBox["Gen",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" identify, respectively,  the values of  the dummy variable ",
    FontFamily->"Arial"],
  StyleBox["y",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" (and therefore \[Sigma]) and the value of ",
    FontFamily->"Arial"],
  StyleBox["s",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" (and therefore the dummy variable ",
    FontFamily->"Arial"],
  StyleBox["x ",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox["). In this way, the fields entering the system non-algebraically \
may be identified, i.e. the non-perturbative part of the system have been \
solved. For more details, and the algorithm which allows such systems to be \
completely solved, see ",
    FontFamily->"Arial"],
  StyleBox["hep-th/0703249.",
    FontSize->14,
    FontWeight->"Bold"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["4. Extension - physical properties and saturations",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["Here and below we illustrate some further applications of the \
package modules. Continuing with the model above, we first provide an \
alternative method for analysing some physical properties of vacua. Consider \
the supersymmetric vacua, which although stored in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["{ }/.SatEx",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" are also described by the vanishing of the F-terms. In fact, \
removing unphysical solutions given by infinite field values, the SUSY vacua \
are described by the set of equations contained in the keyword ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["IFTerms",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[", in the model list ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Fullmodel",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
    \(IFTerms /. Fullmodel // TF\)], "Input",
  CellLabel->"In[8]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \(\*"\<\"\"\>"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({t, s, \[Tau], \[Sigma], x, y}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({t \[Rule] \(-x\) + 2\ t\ y - x\ y\^2, 
                  s \[Rule] \(-3\)\ x - 2\ s\ x - 6\ t\ y - 3\ x\ y\^2 - 
                      2\ s\ x\ y\^2, \[Tau] \[Rule] \(-x\) + x\ y\^2 + 
                      2\ \[ImaginaryI]\ y\ \[Tau], \[Sigma] \[Rule] \(-3\)\ x \
- 2\ s\ x + 3\ x\ y\^2 + 2\ s\ x\ y\^2 + 6\ \[ImaginaryI]\ y\ \[Tau]}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["As earlier, these polynomials live in an enlarged field space -- \
the variables {",
    FontFamily->"Arial"],
  StyleBox["x",
    FontFamily->"Arial"],
  StyleBox[", ",
    FontFamily->"Arial"],
  StyleBox["y",
    FontFamily->"Arial"],
  StyleBox["} have been added to the variables {",
    FontFamily->"Arial"],
  StyleBox["t",
    FontFamily->"Arial"],
  StyleBox[", ",
    FontFamily->"Arial"],
  StyleBox["s",
    FontFamily->"Arial"],
  StyleBox[", ",
    FontFamily->"Arial"],
  StyleBox["\[Tau]",
    FontFamily->"Arial"],
  StyleBox[", ",
    FontFamily->"Arial"],
  StyleBox["\[Sigma]",
    FontFamily->"Arial"],
  StyleBox["}.\n\nThe module ",
    FontFamily->"Arial"],
  StyleBox["PrimDec[ ] ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["may be used to break equations up into simpler pieces categorised \
by their number of flat directions. As before, we must add the option ",
    FontFamily->"Arial"],
  StyleBox["Dim ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  "\[Rule]",
  StyleBox[" {0,1,2} ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["which retains all vacua in the enlarged field space which may be \
true isolated vacua. The output of this operation is shown below,",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[IFTerms /. Fullmodel, Dim \[Rule] {0, 1, 2}] // TF\)], "Input",
  CellLabel->"In[9]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \(1\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\(-\[ImaginaryI]\) + y, x + \[Tau], t, s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(4\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], y, x, t}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(2\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\[ImaginaryI] + y, x - \[Tau], t, s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(6\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], x - \[Tau], t, s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(3\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], \(-1\) + y, t - x, 3 + s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(5\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], 1 + y, t + x, 3 + s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(7\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], x + \[Tau], t, s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["This is the equivalent to the list obtained from ",
    FontFamily->"Arial"],
  StyleBox["{ }/.SatEx",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[". It is common for these lists to contain unphysical solutions \
(several can be seen immediately in the list above). Many of these can be \
removed from the outset by using the module ",
    FontFamily->"Arial"],
  StyleBox["Saturation[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[". Consider the polynomial  \"",
    FontFamily->"Arial"],
  StyleBox["s t x y",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox["\" which vanishes if and only if one or more of the field \
variables vanishes.  Any of these factors vanishing corresponds to an \
unphysical field value. Using ",
    FontFamily->"Arial"],
  StyleBox["Saturation",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" we can remove such vacua. ",
    FontFamily->"Arial"],
  StyleBox["Saturation",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" is called with the option ",
    FontFamily->"Arial"],
  StyleBox["Poly ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  "\[Rule]",
  StyleBox[" ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", which specifies ",
    FontFamily->"Arial"],
  StyleBox["a list of polynomials we desire to be non-zero. Here, the \
following calculation is of interest,",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[Saturation[IFTerms /. Fullmodel, Poly \[Rule] {s\ t\ x\ y}], 
        Dim \[Rule] {0, 1, 2}] // TF\)], "Input",
  CellLabel->"In[10]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \(1\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], \(-1\) + y, t - x, 3 + s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(2\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({2}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], 1 + y, t + x, 3 + s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, x, y, \[Sigma], \[Tau]}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["After calling ",
    FontFamily->"Arial"],
  StyleBox["Saturation[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" we have added a ",
    FontFamily->"Arial"],
  StyleBox["PrimDec[ ] ",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox["command to split up the equations, again keep dimensions of up to \
2.  The result sets of equations are much simpler than both ",
    FontFamily->"Arial"],
  StyleBox["IFTerms/.Fullmodel",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[".\n\nIt is now clear that, despite removing many unphysical \
solutions using ",
    FontFamily->"Arial"],
  StyleBox["Saturation[ ]",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", the remaining equations do not describe physical vacua -- the \
equations require ",
    FontFamily->"Arial"],
  StyleBox["3+s=0",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" which implies an unphysical value for the real part of a \
superfield. Hence, there are no physical SUSY vacua in this model.",
    FontFamily->"Arial"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["5. Extension - identifying field values",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["We now return to the non-supersymmetric vacua contained in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["{s,t}/. SatEx",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[". We stated above that the value of the field ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["s",
    FontFamily->"Arial",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" was specified by the equations contained in this list. Let us \
explore this in a little more detail, again taking one of the vacua in ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["(",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox["{s , t}/. SatEx)[[1]]",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" to illustrate. ",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
    \(MyIdeal = \(({s, t} /. SatEx)\)[\([1]\)]\)], "Input",
  CellLabel->"In[11]:="],

Cell[BoxData[
    \({Gen \[Rule] {\[Tau], 
          1 + y, \(-3\)\ t\^2 + 7\ t\ x + 2\ s\ t\ x + 2\ x\^2 + 
            4\ s\ x\^2, \(-7\)\ t - 2\ s\ t + x + 8\ s\ x + 
            4\ s\^2\ x, \(-11\)\ t\^2 + 2\ s\ t\^2 + 11\ t\ x + 6\ x\^2 + 
            16\ s\ x\^2, \(-15\)\ t - 2\ s\ t + 2\ s\^2\ t - 3\ x - 4\ s\ x, 
          3\ t\^3 - 24\ t\^2\ x + 23\ t\ x\^2 + 10\ x\^3 + 24\ s\ x\^3, 
          3\ t\^4 - 18\ t\^3\ x + 11\ t\^2\ x\^2 - 28\ t\ x\^3 - 
            4\ x\^4, \(-18\) - 78\ s - 41\ s\^2 + 4\ s\^3 + 4\ s\^4}, 
      Dim \[Rule] {2}, Var \[Rule] {s, t, x, y, \[Sigma], \[Tau]}, 
      Par \[Rule] {}}\)], "Output",
  CellLabel->"Out[11]="]
}, Open  ]],

Cell[TextData[{
  "In general, we may identify necessary constraints on a particular variable \
by eliminating all other variables from the equations -- indeed the algorithm \
described in ",
  StyleBox["hep-th/0703249",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  " requires us to first identify the values of s and \[Sigma],",
  StyleBox[" ",
    FontWeight->"Bold"],
  "which enter non-algebraically through x",
  StyleBox[" ",
    FontWeight->"Bold"],
  "and y respectively, in order to completely solve the minimisation problem. \
We refer the user to this paper for more information.\n\nTo illustrate, we \
now eliminate all variables other than s and x  from the ideal above in order \
to identify necessary constraints on s, using the module ",
  StyleBox["Elimination[ ]. ",
    FontWeight->"Bold"],
  "This is called with the option ",
  StyleBox["VarPar",
    FontWeight->"Bold"],
  " which specifies a list of variables to be eliminated, as shown below -- \
note that we should not eliminate x",
  StyleBox[" ",
    FontSlant->"Italic"],
  "because it depends on s,"
}], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ElimId = 
      Elimination[MyIdeal, 
        VarPar \[Rule] {t, \[Tau], \[Sigma], y}]\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \({Var \[Rule] {s, x}, Par \[Rule] {}, 
      Gen \[Rule] {\(-18\) - 78\ s - 41\ s\^2 + 4\ s\^3 + 
            4\ s\^4}}\)], "Output",
  CellLabel->"Out[12]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["Here we have a single generator, which in this case was already \
present in the equations we extracted from ",
    FontFamily->"Arial"],
  StyleBox["{s,t}/.SatEx.",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" In general, we would expect equations depending on both s and x",
    
    FontFamily->"Arial"],
  StyleBox[" -- ",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox["here we are lucky, in that there is a single polynomial \
(contained in ",
    FontFamily->"Arial"],
  StyleBox["Gen/.ElimId",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" ) which depends on s only. We can solve this using ",
    FontFamily->"Arial"],
  StyleBox["Mathematica",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox["'s built in functions ",
    FontFamily->"Arial"],
  StyleBox["--",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" the solution is exact,",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Solve[\ \((Gen /. ElimId)\) \[Equal] 0, s]\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \({{s \[Rule] \(-\(1\/4\)\) - \@57\/4 - 
            1\/2\ \[ImaginaryI]\ \@\(\(-7\) + \@57\)}, {s \[Rule] \(-\(1\/4\)\
\) - \@57\/4 + 
            1\/2\ \[ImaginaryI]\ \@\(\(-7\) + \@57\)}, {s \[Rule] \(-\(1\/4\)\
\) + \@57\/4 - \@\(7 + \@57\)\/2}, {s \[Rule] \(-\(1\/4\)\) + \@57\/4 + \@\(7 \
+ \@57\)\/2}}\)], "Output",
  CellLabel->"Out[13]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["Only one root in the above is in the allowed region of field \
space (s ",
    FontFamily->"Arial"],
  StyleBox[">",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" 1), ",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \({{s \[Rule] \(-2.1374586088176875`\) - 
            0.3707541082950902`\ \[ImaginaryI]}, {s \[Rule] \
\(-2.1374586088176875`\) + 
            0.3707541082950902`\ \[ImaginaryI]}, {s \[Rule] \
\(-0.26975365226330905`\)}, {s \[Rule] 3.5446708698986837`}}\)], "Output",
  CellLabel->"Out[14]="]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
  "so there is only one possible value of s which satisfies the equations in \
",
  StyleBox["MyIdeal",
    FontWeight->"Bold"],
  "."
}], "Text",
  FontFamily->"Arial"]
}, Closed]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 777}},
CellGrouping->Manual,
WindowSize->{747, 666},
WindowMargins->{{140, Automatic}, {0, Automatic}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 219, 8, 76, "Title"],

Cell[CellGroupData[{
Cell[1998, 63, 131, 3, 31, "Subtitle"],
Cell[2132, 68, 3112, 95, 163, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[5281, 168, 139, 3, 25, "Subtitle"],
Cell[5423, 173, 76, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[5524, 178, 72, 2, 45, "Input"],
Cell[5599, 182, 361, 12, 62, "Text"],
Cell[5963, 196, 912, 15, 44, "Output"]
}, Open  ]],
Cell[6890, 214, 109, 1, 27, "Text"],
Cell[7002, 217, 228, 5, 65, "Input"],
Cell[7233, 224, 994, 24, 103, "Text"],
Cell[8230, 250, 300, 7, 65, "Input"],
Cell[8533, 259, 482, 12, 86, "Text"],

Cell[CellGroupData[{
Cell[9040, 275, 73, 2, 45, "Input"],
Cell[9116, 279, 349, 6, 61, "Output"]
}, Open  ]],
Cell[9480, 288, 355, 13, 27, "Text"],

Cell[CellGroupData[{
Cell[9860, 305, 74, 2, 45, "Input"],
Cell[9937, 309, 497, 9, 82, "Output"]
}, Open  ]],
Cell[10449, 321, 281, 4, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[10767, 330, 148, 3, 25, "Subtitle"],
Cell[10918, 335, 1056, 29, 103, "Text"],

Cell[CellGroupData[{
Cell[11999, 368, 115, 2, 45, "Input"],
Cell[12117, 372, 60, 2, 44, "Output"]
}, Open  ]],
Cell[12192, 377, 1186, 37, 85, "Text"],

Cell[CellGroupData[{
Cell[13403, 418, 72, 2, 45, "Input"],
Cell[13478, 422, 1317, 21, 177, "Output"]
}, Open  ]],
Cell[14810, 446, 85, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[14920, 451, 80, 2, 45, "Input"],
Cell[15003, 455, 59, 2, 44, "Output"]
}, Open  ]],
Cell[15077, 460, 95, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[15197, 465, 113, 3, 37, "Input"],
Cell[15313, 470, 1999, 50, 154, "Output"]
}, Open  ]],
Cell[17327, 523, 1015, 29, 86, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[18379, 557, 166, 3, 25, "Subtitle"],
Cell[18548, 562, 1046, 31, 103, "Text"],

Cell[CellGroupData[{
Cell[19619, 597, 103, 3, 37, "Input"],
Cell[19725, 602, 1503, 38, 100, "Output"]
}, Open  ]],
Cell[21243, 643, 1304, 44, 103, "Text"],

Cell[CellGroupData[{
Cell[22572, 691, 135, 3, 37, "Input"],
Cell[22710, 696, 4121, 109, 146, "Output"]
}, Open  ]],
Cell[26846, 808, 1473, 44, 122, "Text"],

Cell[CellGroupData[{
Cell[28344, 856, 184, 4, 37, "Input"],
Cell[28531, 862, 1828, 49, 66, "Output"]
}, Open  ]],
Cell[30374, 914, 1104, 32, 122, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[31515, 951, 155, 3, 25, "Subtitle"],

Cell[CellGroupData[{
Cell[31695, 958, 904, 31, 65, "Text"],

Cell[CellGroupData[{
Cell[32624, 993, 98, 2, 45, "Input"],
Cell[32725, 997, 661, 11, 120, "Output"]
}, Open  ]],
Cell[33401, 1011, 1121, 28, 160, "Text"],

Cell[CellGroupData[{
Cell[34547, 1043, 144, 4, 45, "Input"],
Cell[34694, 1049, 177, 4, 44, "Output"]
}, Open  ]],
Cell[34886, 1056, 988, 31, 84, "Text"],

Cell[CellGroupData[{
Cell[35899, 1091, 100, 2, 45, "Input"],
Cell[36002, 1095, 365, 7, 108, "Output"]
}, Open  ]],
Cell[36382, 1105, 252, 9, 27, "Text"],

Cell[CellGroupData[{
Cell[36659, 1118, 62, 2, 45, "Input"],
Cell[36724, 1122, 311, 6, 44, "Output"]
}, Open  ]]
}, Open  ]],
Cell[37062, 1132, 185, 7, 27, "Text"]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

