(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     32615,       1042]*)
(*NotebookOutlinePosition[     33344,       1067]*)
(*  CellTagsIndexPosition[     33300,       1063]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Example:  ",
    FontSize->24],
  StyleBox["Num",
    FontSize->24,
    FontColor->RGBColor[0.784314, 0, 0]],
  StyleBox["erical appr",
    FontSize->24,
    FontColor->RGBColor[0.784314, 0, 0]],
  StyleBox["oximations and number fields",
    FontSize->24,
    FontColor->RGBColor[0.784314, 0, 0]]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[CellGroupData[{

Cell[TextData[StyleBox["1. Introduction",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["Many of the methods which STRINGVACUA gives access to, through ",
    FontFamily->"Arial"],
  StyleBox["Singular",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[", are implemented for systems (of polynomials) where the \
underlying number field is either the rational numbers or a finite field. The \
finite field can have characteristic ",
    FontFamily->"Arial"],
  StyleBox["p",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" for any prime ",
    FontFamily->"Arial"],
  StyleBox["p",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" which is at most ",
    FontFamily->"Arial"],
  StyleBox["2147483629",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[".\n\nBy default, the package performs ",
    FontFamily->"Arial"],
  StyleBox["Singular",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" calculations using the field of rational numbers. To illustrate \
how the package accommodates this, consider the following toy model, where \
the coefficients in the superpotential are given as either floating point \
numbers or irrationals,\n",
    FontFamily->"Arial"],
  StyleBox["\n",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Superfields:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["\t\t\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["T = t + \[ImaginaryI] \[Tau], S = s + \[ImaginaryI] \[Sigma]\n\t",
    
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Kahler potential:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\t\tK = ",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  Cell[BoxData[
      FormBox[
        RowBox[{"-", 
          RowBox[{"Log", " ", "[", 
            RowBox[{
              FormBox[
                StyleBox["T",
                  FontSlant->"Plain"],
                "TraditionalForm"], "+", 
              RowBox[{"Conjugate", "[", 
                StyleBox["T",
                  FontSlant->"Plain"], "]"}]}], "]"}]}], TraditionalForm]],
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  Cell[BoxData[
      FormBox[
        RowBox[{\(-3\), 
          RowBox[{"Log", " ", "[", 
            RowBox[{"S", "+", 
              RowBox[{"Conjugate", "[", 
                StyleBox["S",
                  FontSlant->"Plain"], "]"}]}], "]"}]}], TraditionalForm]],
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Superpotential:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\t\tW = 0.5 T + \[Pi] S +T^2",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["2. Setting up the model",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[StyleBox["Load the package,",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< Stringvacua`\)], "Input",
  CellLabel->"In[4]:="],

Cell["\<\
Stringvacua package, Version 1.0, December 2007
James Gray, Yang-Hui He, Anton Ilderton and Andre Lukas\
\>", "Text",
  CellFrame->True,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->13,
  FontWeight->"Bold",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[1, 1, 0.900008]],

Cell[BoxData["\<\"Execute \
\\*ButtonBox[StyleBox[\\\"SimpleMinimise[]\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonEvaluator->\\\"Automatic\\\
\",ButtonFunction:>(SimpleMinimise[]&)] for a simple interactive calculation \
or click \\*ButtonBox[StyleBox[\\\"here\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[\
 { FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Generalities\\\"]}]&)] to \
open the Help Browser, which contains \
\\*ButtonBox[StyleBox[\\\"examples\\\",FontColor->RGBColor[0,0,0.7],FontSize->\
12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { \
FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Examples\\\", \
\\\"Examples_Overview.nb\\\"]}]&)].\"\>"], "Output",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->12,
  FontColor->RGBColor[0.6, 0.300008, 0]]
}, Open  ]],

Cell[TextData[StyleBox["and create the supergravity model in the usual way, \
entering the coefficients as they are given,",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    \(\(RRmodel = 
        CalcModel[
          CreateModel[{T, S}, {t, 
              s}, {\[Tau], \[Sigma]}, \(-\ \ Log[T + Conjugate[T]]\) - 
              3  Log[S + Conjugate[S]], 
            0.5\ T\  + \ Pi\ S + T^2]];\)\)], "Input",
  CellLabel->"In[2]:="],

Cell[TextData[{
  StyleBox["This calculates the scalar potential etc, as normal. The \
supergravity keywords (generated by ",
    FontFamily->"Arial"],
  StyleBox["CreateModel",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[") ",
    FontFamily->"Arial"],
  StyleBox["do not contain any approximations, for example the F-terms are",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FTerms /. RRmodel\)], "Input",
  CellLabel->"In[3]:="],

Cell[BoxData[
    \({T \[Rule] \(\(0.5`\)\(\[InvisibleSpace]\)\) + 
          2\ T - \(\[Pi]\ S + T\ \((\(\(0.5`\)\(\[InvisibleSpace]\)\) + \
T)\)\)\/\(2\ t\), 
      S \[Rule] \[Pi] - \(3\ \((\[Pi]\ S + T\ \
\((\(\(0.5`\)\(\[InvisibleSpace]\)\) + T)\))\)\)\/\(2\ s\)}\)], "Output",
  CellLabel->"Out[3]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["The keywords in the extended model list (generated by ",
    FontFamily->"Arial"],
  StyleBox["CalcModel",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[") also contain",
    FontFamily->"Arial"],
  StyleBox[" no approximations, e.g. the F-terms are written, in a language ",
    
    FontFamily->"Arial"],
  StyleBox["Singular",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" can understand, as",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IFTerms /. RRmodel\)], "Input",
  CellLabel->"In[4]:="],

Cell[BoxData[
    \({Var \[Rule] {t, s, \[Tau], \[Sigma]}, Par \[Rule] {}, 
      Gen \[Rule] {t \[Rule] \(-\[Pi]\)\ s + 0.5`\ t + 3\ t\^2 + \[Tau]\^2, 
          s \[Rule] \(-\[Pi]\)\ s - 1.5`\ t - 3\ t\^2 + 
              3\ \[Tau]\^2, \[Tau] \[Rule] \(-\(\(\[Pi]\ \[Sigma]\)\/2\)\) - 
              0.25`\ \[Tau] + 
              t\ \[Tau], \[Sigma] \[Rule] \(-3\)\ \[Pi]\ \[Sigma] - 
              1.5`\ \[Tau] - 6\ t\ \[Tau]}}\)], "Output",
  CellLabel->"Out[4]="]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["3. Classifying vacua",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["When one comes to performing calculations in ",
    FontFamily->"Arial"],
  StyleBox["Singular",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[", floating point expressions (e.g. the coefficient of ",
    FontFamily->"Arial"],
  StyleBox["0.5",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" in the above model) and irrational numbers (e.g. the coefficient \
of ",
    FontFamily->"Arial"],
  StyleBox["\[Pi]",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold"],
  StyleBox[" in the above) are automatically approximated by rational \
numbers. We can use the module ",
    FontFamily->"Arial"],
  StyleBox["SaturationExpand",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" to quickly classify the vacua of our simple model, collecting \
the equations into a list ",
    FontFamily->"Arial"],
  StyleBox["SatEx",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial"],
  StyleBox[" as shown",
    FontFamily->"Arial"]
}], "Text"],

Cell[BoxData[
    \(\(SatEx = SaturationExpand[RRmodel];\)\)], "Input",
  CellLabel->"In[5]:="],

Cell[TextData[{
  StyleBox["Inspecting the output of ",
    FontFamily->"Arial"],
  StyleBox["SaturationExpand",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[",",
    FontFamily->"Arial"],
  StyleBox[" we see that the supersymmetric vacua are described by",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({} /. SatEx // TF\)], "Input",
  CellLabel->"In[6]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \(1\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], \[Sigma], t, s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(2\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], \[Sigma], 1 + 3\ t, \(-113\) + 2130\ s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, \[Sigma], \[Tau]}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[StyleBox["and one of the non-supersymmetric vacua is described \
by",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(({s, t} /. SatEx)\)[\([1]\)] // TF\)], "Input",
  CellLabel->"In[7]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \(\*"\<\"\"\>"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({\[Tau], \[Sigma], 
                  305148521 + 3594945840\ s + 4920519915\ t + 
                    10298762334\ t\^2, \(-5557428945641410\) + 
                    261887676646185969\ s - 79193362980778920\ t + 
                    1964157447828319821\ s\ t, \
\(-797179620774490884463519550933\) - 110015304141077919355141632603480\ s + 
                    1264475165981326009677005035999464\ s\^2 - 
                    9224507401385917213276143975198\ t}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({s, t, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12]},
          {Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["(there are other solutions contained in ",
    FontFamily->"Arial"],
  StyleBox["{s}/.SatEx",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" and ",
    FontFamily->"Arial"],
  StyleBox["{s, t, \[Sigma], \[Tau]}/.SatEx",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[", but the equations for the latter are rather lengthy, so we do \
not explicitly present them here). We see firstly that all results are given \
with integer coefficients -- this is because the package has converted all \
coefficients to rationals before calling ",
    FontFamily->"Arial"],
  StyleBox["Singular.",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" Secondly, note that the coefficients in the given equations are \
somewhat large -- this is a ",
    FontFamily->"Arial"],
  StyleBox["common feature",
    FontFamily->"Arial",
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" of results when rational approximations are used. In practice, \
while the coefficients may appear unpleasant, the functionality of the \
package is not lost by making these approximations.",
    FontFamily->"Arial"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["4. Controlling SingularAccuracy",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["The accuracy with which the package converts numbers to rationals \
is controlled by the global  STRINGVACUA option ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["SingularAccuracy",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[". The current accuracy is contained in the list",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[Stringvacua]\)], "Input",
  CellLabel->"In[11]:="],

Cell[BoxData[
    \({PlanckInverse \[Rule] 1, PackageProtocol \[Rule] 10, 
      FieldChar \[Rule] 0, SingularProtocol \[Rule] "Xterm", 
      SingularTimer \[Rule] "on", SingularMaxTime \[Rule] 60, 
      SingularTrace \[Rule] 0, SingularEcho \[Rule] 0, 
      SingularAccuracy \[Rule] 1\/100000, XtermGeometry \[Rule] "50x10-50+0", 
      XtermPath \[Rule] Stringvacua`Private`xtermpath, 
      SingularPath \[Rule] "/usr/bin/Singular"}\)], "Output",
  CellLabel->"Out[11]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["We see that the default accuracy is 10^(-5). To change the \
default accuracy, first open the ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["StringvacuaOptions.m",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" file, which resides in \
$UserBaseDirectory/Applications/Stringvacualib on your machine. This is a \
simple text file containing the above list. Edit the text directly, for \
example replacing \"SingularAccuracy ",
    FontFamily->"Arial",
    FontSize->12],
  "\[Rule]",
  StyleBox[" 1/100000\" with \"SingularAccuracy ",
    FontFamily->"Arial",
    FontSize->12],
  "\[Rule]",
  StyleBox[" 1/1000000\" to increase the default accuracy by a factor of 10. \
This will be applied the next time the package is loaded.",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["5. Extension - controlling SingularAccuracy locally",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["One may also set the accuracy locally, i.e. it can be temporarily \
set to a chosen value for a particular calculation. For example, we can call \
",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["SaturationExpand",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" as above, but now with a reduced accuracy,",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  FontSize->14],

Cell[BoxData[
    \(\(SatEx2\  = \ 
        SaturationExpand[RRmodel, 
          SingularAccuracy \[Rule] 1/100];\)\)], "Input",
  CellLabel->"In[8]:="],

Cell[TextData[StyleBox["Compare the equations describing SUSY vacua in SatEx \
(or original saturation expansion) with those in the new list SatEx2,",
  FontFamily->"Arial",
  FontSize->12]], "Text",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[{
    \({} /. SatEx // TF\), "\[IndentingNewLine]", 
    \({} /. SatEx2 // TF\)}], "Input",
  CellLabel->"In[10]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \(1\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], \[Sigma], t, s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(2\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], \[Sigma], 1 + 3\ t, \(-113\) + 2130\ s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, \[Sigma], \[Tau]}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \(1\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], \[Sigma], t, s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, \[Sigma], \[Tau]}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(2\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({\[Tau], \[Sigma], 1 + 3\ t, \(-7\) + 132\ s}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({s, t, \[Sigma], \[Tau]}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell["\<\
we see, in the second sets of equations, at first sight seemingly different \
solutions for the field \"s\". However, checking the value of this field we \
find,\
\>", "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(N[113/2130]\), "\[IndentingNewLine]", 
    \(N[7/132]\)}], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(0.053051643192488264`\)], "Output",
  CellLabel->"Out[12]="],

Cell[BoxData[
    \(0.05303030303030303`\)], "Output",
  CellLabel->"Out[13]="]
}, Open  ]],

Cell[TextData[StyleBox["The two sets of solutions describe the same vacua, \
the coefficients differing because of the different accuracies used to \
convert the coefficients of the model to rationals. Here, the second \"s\" \
value, 7/132, was arrived at using a less accurate rational approximation to \
the initial equations.",
  FontFamily->"Arial"]], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["6. Extension - finite fields",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["As mentioned above, calculations may also be performed over \
finite fields. The field characteristic is a global option set by default to \
0 (so that the default number field is the rationals). The default is \
contained in the option ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["FieldChar",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" in the ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["StringvacuOptions.m",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" file, ",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[Stringvacua]\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \({PlanckInverse \[Rule] 1, PackageProtocol \[Rule] 10, 
      FieldChar \[Rule] 0, SingularProtocol \[Rule] "Xterm", 
      SingularTimer \[Rule] "on", SingularMaxTime \[Rule] 60, 
      SingularTrace \[Rule] 0, SingularEcho \[Rule] 0, 
      SingularAccuracy \[Rule] 1\/100000, XtermGeometry \[Rule] "50x10-50+0", 
      XtermPath \[Rule] Stringvacua`Private`xtermpath, 
      SingularPath \[Rule] "/usr/bin/Singular"}\)], "Output",
  CellLabel->"Out[14]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["The default field may be given characteristic ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["p",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[", where ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["p",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" is any prime number up to ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["2147483629",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[", by editing the above the ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["StringvacuaOptions.m",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" file directly, and setting ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["FieldChar \[Rule] p",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[".\n\nAgain, the field characteristic may also be set locally. For \
example, beginning with the default ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["0",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[", consider the `vacuum space' described by the simple equation \
x^2-18=0. In a form suitable for interpretation by ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Singular",
    FontFamily->"Arial",
    FontSize->12,
    FontSlant->"Italic"],
  StyleBox[" , we store this in an object ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["Id",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[" using ",
    FontFamily->"Arial",
    FontSize->12],
  StyleBox["CreateIdeal",
    FontFamily->"Arial",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox[", ",
    FontFamily->"Arial",
    FontSize->12]
}], "Text",
  FontSize->14],

Cell[CellGroupData[{

Cell[BoxData[
    \(Id = CreateIdeal[{x^2 - 18}, Var \[Rule] x]\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \({Var \[Rule] {x}, Par \[Rule] {}, 
      Gen \[Rule] {\(-18\) + x\^2}}\)], "Output",
  CellLabel->"Out[15]="]
}, Open  ]],

Cell[TextData[{
  "Using ",
  StyleBox["PrimDec",
    FontWeight->"Bold"],
  " we can try to split up these equations into simpler pieces describing \
different vacua,"
}], "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[Id] // TF\)], "Input",
  CellLabel->"In[16]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \(1\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({\(-18\) + x\^2}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({x}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["Of course, since we are working over the rationals, these \
equations cannot be split up any further -- the square root of 18 is three \
times the square root of 2, which is not a rational number. However, if we \
wanted to work over a finite field of characteristic 17, we could add the \
following option to our ",
    FontFamily->"Arial"],
  StyleBox["PrimDec",
    FontFamily->"Arial",
    FontWeight->"Bold"],
  StyleBox[" call,",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[Id, FieldChar \[Rule] 17] // TF\)], "Input",
  CellLabel->"In[17]:=",
  Background->None],

Cell[BoxData[
    TagBox[GridBox[{
          {Cell[BoxData[
                \(" "\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Dim(compl.dim.)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Gen(Generators)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Par(Parameters)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \("Var(Variables)"\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]]},
          {Cell[BoxData[
                \(1\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({\(-1\) + x}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({x}\)],
              FontSize->12]},
          {Cell[BoxData[
                \(2\)],
              FontSize->10,
              FontColor->RGBColor[0.500008, 0, 0]], Cell[BoxData[
                \({0}\)],
              FontSize->12], Cell[BoxData[
                \({1 + x}\)],
              FontSize->12], Cell[BoxData[
                \({}\)],
              FontSize->12], Cell[BoxData[
                \({x}\)],
              FontSize->12]}
          },
        RowSpacings->0,
        ColumnWidths->Automatic,
        ColumnAlignments->{Left},
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  Background->None]
}, Open  ]],

Cell[TextData[{
  StyleBox["Here we have been able to simplify a quadratic into two linear \
equations, because in the finite field 18 is congruent to 1, and the equation \
x^2-1=0 has rational roots. It should be noted that calculations in ",
    FontFamily->"Arial"],
  StyleBox["Singular",
    FontFamily->"Arial",
    FontSlant->"Italic"],
  StyleBox[" are often much quicker when working over a finite field.",
    FontFamily->"Arial"]
}], "Text"]
}, Closed]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 777}},
CellGrouping->Manual,
WindowSize->{815, 707},
WindowMargins->{{162, Automatic}, {Automatic, 0}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 385, 14, 76, "Title"],

Cell[CellGroupData[{
Cell[2164, 69, 131, 3, 31, "Subtitle"],
Cell[2298, 74, 3514, 110, 201, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[5849, 189, 139, 3, 25, "Subtitle"],
Cell[5991, 194, 76, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[6092, 199, 72, 2, 45, "Input"],
Cell[6167, 203, 361, 12, 62, "Text"],
Cell[6531, 217, 912, 15, 44, "Output"]
}, Open  ]],
Cell[7458, 235, 157, 2, 27, "Text"],
Cell[7618, 239, 278, 7, 65, "Input"],
Cell[7899, 248, 384, 11, 46, "Text"],

Cell[CellGroupData[{
Cell[8308, 263, 74, 2, 45, "Input"],
Cell[8385, 267, 306, 6, 58, "Output"]
}, Open  ]],
Cell[8706, 276, 494, 16, 46, "Text"],

Cell[CellGroupData[{
Cell[9225, 296, 75, 2, 45, "Input"],
Cell[9303, 300, 469, 8, 80, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[9821, 314, 136, 3, 25, "Subtitle"],
Cell[9960, 319, 1056, 34, 66, "Text"],
Cell[11019, 355, 95, 2, 45, "Input"],
Cell[11117, 359, 310, 10, 27, "Text"],

Cell[CellGroupData[{
Cell[11452, 373, 94, 3, 37, "Input"],
Cell[11549, 378, 1825, 49, 66, "Output"]
}, Open  ]],
Cell[13389, 430, 116, 2, 27, "Text"],

Cell[CellGroupData[{
Cell[13530, 436, 113, 3, 37, "Input"],
Cell[13646, 441, 1838, 47, 154, "Output"]
}, Open  ]],
Cell[15499, 491, 1183, 30, 103, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[16719, 526, 147, 3, 25, "Subtitle"],
Cell[16869, 531, 422, 13, 46, "Text"],

Cell[CellGroupData[{
Cell[17316, 548, 78, 2, 45, "Input"],
Cell[17397, 552, 477, 8, 126, "Output"]
}, Open  ]],
Cell[17889, 563, 874, 25, 85, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[18800, 593, 167, 3, 25, "Subtitle"],
Cell[18970, 598, 449, 14, 46, "Text"],
Cell[19422, 614, 152, 4, 45, "Input"],
Cell[19577, 620, 215, 4, 27, "Text"],

Cell[CellGroupData[{
Cell[19817, 628, 149, 4, 57, "Input"],
Cell[19969, 634, 1825, 49, 66, "Output"],
Cell[21797, 685, 1822, 49, 66, "Output"]
}, Open  ]],
Cell[23634, 737, 208, 5, 46, "Text"],

Cell[CellGroupData[{
Cell[23867, 746, 113, 3, 65, "Input"],
Cell[23983, 751, 80, 2, 44, "Output"],
Cell[24066, 755, 79, 2, 44, "Output"]
}, Open  ]],
Cell[24160, 760, 363, 5, 65, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[24560, 770, 144, 3, 25, "Subtitle"],
Cell[24707, 775, 665, 22, 46, "Text"],

Cell[CellGroupData[{
Cell[25397, 801, 78, 2, 45, "Input"],
Cell[25478, 805, 477, 8, 126, "Output"]
}, Open  ]],
Cell[25970, 816, 1848, 70, 103, "Text"],

Cell[CellGroupData[{
Cell[27843, 890, 101, 2, 45, "Input"],
Cell[27947, 894, 129, 3, 44, "Output"]
}, Open  ]],
Cell[28091, 900, 203, 7, 27, "Text"],

Cell[CellGroupData[{
Cell[28319, 911, 95, 3, 37, "Input"],
Cell[28417, 916, 1325, 37, 50, "Output"]
}, Open  ]],
Cell[29757, 956, 500, 12, 65, "Text"],

Cell[CellGroupData[{
Cell[30282, 972, 117, 3, 37, "Input"],
Cell[30402, 977, 1730, 49, 66, "Output"]
}, Open  ]],
Cell[32147, 1029, 452, 10, 46, "Text"]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

