(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     12491,        431]*)
(*NotebookOutlinePosition[     13268,        458]*)
(*  CellTagsIndexPosition[     13224,        454]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Example:  ",
    FontSize->24],
  StyleBox["A simple single-modulus example",
    FontSize->24,
    FontColor->RGBColor[0.784314, 0, 0]]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[CellGroupData[{

Cell[TextData[StyleBox["1. Introduction",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["In this example we will take a very simple supergravity model, \
whose field content consists of a single modulus, and use this to illustrate \
some of the basic features of the package.\n\n\t",
    FontFamily->"Arial"],
  StyleBox["One superfield:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" \tT = t + \[ImaginaryI] \[Tau] \n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Kahler potential:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" \tK = -3 Log[ (T + Conjugate[T])] \n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Superpotential:      ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["W = a - bT\n        \t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["Two parameters:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["\ta,b",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["2. Setting up the model",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Load the package",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< Stringvacua`\)], "Input",
  CellLabel->"In[5]:="],

Cell["\<\
Stringvacua package, Version 1.0, December 2007
James Gray, Yang-Hui He, Anton Ilderton and Andre Lukas\
\>", "Text",
  CellFrame->True,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->13,
  FontWeight->"Bold",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[1, 1, 0.900008]],

Cell[BoxData["\<\"Execute \
\\*ButtonBox[StyleBox[\\\"SimpleMinimise[]\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonEvaluator->\\\"Automatic\\\
\",ButtonFunction:>(SimpleMinimise[]&)] for a simple interactive calculation \
or click \\*ButtonBox[StyleBox[\\\"here\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[\
 { FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Generalities\\\"]}]&)] to \
open the Help Browser, which contains \
\\*ButtonBox[StyleBox[\\\"examples\\\",FontColor->RGBColor[0,0,0.7],FontSize->\
12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { \
FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Examples\\\", \
\\\"Examples_Overview.nb\\\"]}]&)].\"\>"], "Output",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->12,
  FontColor->RGBColor[0.6, 0.300008, 0]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Create simple model with one modulus T, standard \
Kahler potential and flux superpotential",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    \(\(model = 
        CreateModel[{T}, {t}, {\[Tau]}, \(-3\)\ \ Log[T + Conjugate[T]], 
          a + b\ T^8, Par \[Rule] {a, b}];\)\)], "Input",
  CellLabel->"In[8]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Calculate the extended model list",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    \(\(model = CalcModel[model];\)\)], "Input",
  CellLabel->"In[9]:="]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["3. Extracting information from the model list",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Display scalar potential for this model",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Pot /. model\)], "Input",
  CellLabel->"In[10]:="],

Cell[BoxData[
    \(\(4\ b\ \((5\ b\ \((t\^2 + \[Tau]\^2)\)\^7 - 3\ a\ \((t\^6 - 21\ t\^4\ \
\[Tau]\^2 + 35\ t\^2\ \[Tau]\^4 - 7\ \[Tau]\^6)\))\)\)\/\(3\ t\)\)], "Output",\

  CellLabel->"Out[10]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Display a generating polynomial for Numerator[",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`\[PartialD]\_t V\)],
    FontFamily->"Arial"],
  StyleBox["] for the ideal dV",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(t /. Gen\) /. IDPot\) /. model\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(\(-4\)\ \((15\ a\ b\ t\^6 - 65\ b\^2\ t\^14 - 
          189\ a\ b\ t\^4\ \[Tau]\^2 - 385\ b\^2\ t\^12\ \[Tau]\^2 + 
          105\ a\ b\ t\^2\ \[Tau]\^4 - 945\ b\^2\ t\^10\ \[Tau]\^4 + 
          21\ a\ b\ \[Tau]\^6 - 1225\ b\^2\ t\^8\ \[Tau]\^6 - 
          875\ b\^2\ t\^6\ \[Tau]\^8 - 315\ b\^2\ t\^4\ \[Tau]\^10 - 
          35\ b\^2\ t\^2\ \[Tau]\^12 + 5\ b\^2\ \[Tau]\^14)\)\)], "Output",
  CellLabel->"Out[12]="]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["4. Simple examples of using the Stringvacua modules",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Calculate constraints on parameters for existence of \
vacuum with zero cosmological constant",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Elimination[{IDPot, IPot} /. model, Meth -> "\<Basic\>", 
      VarPar \[Rule] {t, \[Tau]}]\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \({Var \[Rule] {a, b}, Par \[Rule] {}, Gen \[Rule] {0}}\)], "Output",
  CellLabel->"Out[13]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Calculate primary decomposition of ideal dV. \
Parameters can be set to numerical values, either by explicit substitution \
into\nthe ideal, or by use of the optional VarPar argument of the PrimDec \
module.",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[IDPot /. model, VarPar \[Rule] {a \[Rule] 1, b \[Rule] 1}, 
      NRoots \[Rule] 1, RRoots \[Rule] "\<Probab\>"]\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], \(-3\) + 13\ t\^8}, Dim \[Rule] {0}, 
        NRoots \[Rule] {2}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}, {Gen \[Rule] {\[Tau], t}, Dim \[Rule] {0}, 
        NRoots \[Rule] {1}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}, {Gen \[Rule] {11\ t\^6 - 49\ t\^4\ \[Tau]\^2 + 
              21\ t\^2\ \[Tau]\^4 + \[Tau]\^6, 
            363 + 65268\ t\^4\ \[Tau]\^4 - 41304\ t\^2\ \[Tau]\^6 - 
              1772\ \[Tau]\^8, 
            179487\ t\^2 - 685947\ \[Tau]\^2 + 15564096\ t\^2\ \[Tau]\^8 + 
              414656\ \[Tau]\^10, 
            24075711\ t\^4 - 107887770\ t\^2\ \[Tau]\^2 + 
              49067691\ \[Tau]\^4 + 20000000\ \[Tau]\^12}, Dim \[Rule] {0}, 
        NRoots \[Rule] {4}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[14]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Calculate saturation of ideal dV w.r.t. real part of \
F_T. ",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Saturation[
      IDPot /. model, \(\(Poly -> t /. Gen\) /. IFTerms\) /. model, 
      VarPar -> {a \[Rule] 1, b \[Rule] 1}]\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \({Gen \[Rule] {3\ t\^4\ \[Tau] - 10\ t\^2\ \[Tau]\^3 + 3\ \[Tau]\^5, 
          t\^6 - 35\ t\^2\ \[Tau]\^4 + 14\ \[Tau]\^6, 
          7\ t\^2\ \[Tau]\^5 - 3\ \[Tau]\^7, \[Tau]\^9}, Dim \[Rule] {0}, 
      Var \[Rule] {t, \[Tau]}, Par \[Rule] {}}\)], "Output",
  CellLabel->"Out[15]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compute dimension of ideal", "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dim /. DimIdeal[IDPot /. model]\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \({0}\)], "Output",
  CellLabel->"Out[16]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Combine saturation and primary decomposition", "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(SaturationPrimDec[
      IDPot /. model, \(\(Poly -> t /. Gen\) /. IFTerms\) /. model, 
      VarPar -> {a \[Rule] 1, b \[Rule] 1}]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], t}, Dim \[Rule] {0}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[17]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Now perform a complete saturation expansion for a=b=1", "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \(satex = 
      SaturationExpand[model, 
        VarPar -> {a \[Rule] 1, b \[Rule] 1}]\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \({}\)], "Output",
  CellLabel->"Out[22]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["And here is the supersymmetric term...", "Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \({} /. satex\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], \(-3\) + 13\ t\^8}, Dim \[Rule] {0}, 
        Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}, {Gen \[Rule] {11\ t\^6 - 49\ t\^4\ \[Tau]\^2 + 
              21\ t\^2\ \[Tau]\^4 + \[Tau]\^6, 
            363 + 65268\ t\^4\ \[Tau]\^4 - 41304\ t\^2\ \[Tau]\^6 - 
              1772\ \[Tau]\^8, 
            179487\ t\^2 - 685947\ \[Tau]\^2 + 15564096\ t\^2\ \[Tau]\^8 + 
              414656\ \[Tau]\^10, 
            24075711\ t\^4 - 107887770\ t\^2\ \[Tau]\^2 + 
              49067691\ \[Tau]\^4 + 20000000\ \[Tau]\^12}, Dim \[Rule] {0}, 
        Var \[Rule] {t, \[Tau]}, Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[14]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
And here is the one with Im(F_T) zero and Re(F_T) non-zero...\
\>", \
"Text",
  FontFamily->"Arial"],

Cell[CellGroupData[{

Cell[BoxData[
    \({t} /. satex\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau], t}, Dim \[Rule] {0}, Var \[Rule] {t, \[Tau]}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[15]="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 777}},
WindowToolbars->"EditBar",
CellGrouping->Manual,
WindowSize->{1007, 683},
WindowMargins->{{Automatic, 10}, {Automatic, 0}},
ShowSelection->True,
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 224, 8, 76, "Title"],

Cell[CellGroupData[{
Cell[2003, 63, 131, 3, 31, "Subtitle"],
Cell[2137, 68, 1407, 44, 145, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[3581, 117, 118, 2, 25, "Subtitle"],

Cell[CellGroupData[{
Cell[3724, 123, 75, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[3824, 128, 72, 2, 45, "Input"],
Cell[3899, 132, 361, 12, 62, "Text"],
Cell[4263, 146, 912, 15, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5212, 166, 150, 2, 27, "Text"],
Cell[5365, 170, 186, 4, 45, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5588, 179, 92, 1, 27, "Text"],
Cell[5683, 182, 86, 2, 45, "Input"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[5830, 191, 140, 2, 25, "Subtitle"],

Cell[CellGroupData[{
Cell[5995, 197, 98, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[6118, 202, 70, 2, 45, "Input"],
Cell[6191, 206, 198, 4, 64, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6438, 216, 260, 8, 27, "Text"],

Cell[CellGroupData[{
Cell[6723, 228, 92, 2, 45, "Input"],
Cell[6818, 232, 440, 7, 63, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[7319, 246, 146, 2, 25, "Subtitle"],

Cell[CellGroupData[{
Cell[7490, 252, 152, 2, 27, "Text"],

Cell[CellGroupData[{
Cell[7667, 258, 149, 3, 45, "Input"],
Cell[7819, 263, 112, 2, 44, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7980, 271, 266, 4, 46, "Text"],

Cell[CellGroupData[{
Cell[8271, 279, 178, 3, 45, "Input"],
Cell[8452, 284, 844, 15, 82, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9345, 305, 119, 2, 27, "Text"],

Cell[CellGroupData[{
Cell[9489, 311, 182, 4, 45, "Input"],
Cell[9674, 317, 304, 5, 44, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10027, 328, 65, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[10117, 333, 89, 2, 45, "Input"],
Cell[10209, 337, 62, 2, 44, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10320, 345, 83, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[10428, 350, 189, 4, 45, "Input"],
Cell[10620, 356, 153, 3, 44, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10822, 365, 92, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[10939, 370, 143, 4, 45, "Input"],
Cell[11085, 376, 61, 2, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11183, 383, 77, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[11285, 388, 69, 2, 45, "Input"],
Cell[11357, 392, 672, 12, 82, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[12078, 410, 110, 4, 27, "Text"],

Cell[CellGroupData[{
Cell[12213, 418, 70, 2, 45, "Input"],
Cell[12286, 422, 153, 3, 44, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

