(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     16641,        470]*)
(*NotebookOutlinePosition[     17370,        495]*)
(*  CellTagsIndexPosition[     17326,        491]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Example:  ",
    FontSize->24],
  StyleBox["A simple two-modulus example",
    FontSize->24,
    FontColor->RGBColor[0.784314, 0, 0]]
}], "Title",
  CellFrame->{{0, 0}, {2, 0}},
  FontSize->18],

Cell[CellGroupData[{

Cell[TextData[StyleBox["1. Introduction",
  FontSize->18]], "Subtitle",
  CellDingbat->None,
  FontVariations->{"Underline"->True}],

Cell[TextData[{
  StyleBox["In this example we will take a very simple supergravity model, \
whose field content consists of two moduli, and use this to illustrate some \
of the basic features of the package. This example is taken from Section 4.1 \
of Gray-He-Lukas, hep-th/06060122.\n\n\t",
    FontFamily->"Arial"],
  StyleBox["Two superfields:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" \t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  Cell[BoxData[
      FormBox[
        StyleBox[\(T\_i = t\_i\  + \ \[Iota]\ \[Tau]\_i\),
          FontSize->14], TraditionalForm]],
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" for i=1,2;",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["\n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Kahler potential:",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox[" \t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["K = ",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  Cell[BoxData[
      FormBox[
        RowBox[{\(-3\), 
          RowBox[{"Log", " ", "[", 
            RowBox[{
              FormBox[\(T\_1\),
                "TraditionalForm"], "+", \(Conjugate[T\_1]\)}], "]"}]}], 
        TraditionalForm]],
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  Cell[BoxData[
      FormBox[
        RowBox[{\(-3\), 
          RowBox[{"Log", " ", "[", 
            RowBox[{
              FormBox[\(T\_2\),
                "TraditionalForm"], "+", \(Conjugate[T\_2]\)}], "]"}]}], 
        TraditionalForm]],
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["\n\t",
    FontFamily->"Arial",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["Superpotential:      ",
    FontFamily->"Arial",
    FontSize->14,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["W= ",
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]],
  Cell[BoxData[
      \(TraditionalForm\`\(-T\_1\^2\) - \(T\_1\) T\_2\  - \ T\_2\^2 + 
        10  T\_1 + 10  T\_2 - 100\)],
    FontFamily->"Arial",
    FontColor->RGBColor[0, 0, 1]]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["2. Setting up the model",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle"],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< Stringvacua`\)], "Input",
  CellLabel->"In[6]:="],

Cell["\<\
Stringvacua package, Version 1.0, December 2007
James Gray, Yang-Hui He, Anton Ilderton and Andre Lukas\
\>", "Text",
  CellFrame->True,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->13,
  FontWeight->"Bold",
  FontColor->RGBColor[0.500008, 0, 0],
  Background->RGBColor[1, 1, 0.900008]],

Cell[BoxData["\<\"Execute \
\\*ButtonBox[StyleBox[\\\"SimpleMinimise[]\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonEvaluator->\\\"Automatic\\\
\",ButtonFunction:>(SimpleMinimise[]&)] for a simple interactive calculation \
or click \\*ButtonBox[StyleBox[\\\"here\\\",FontColor->RGBColor[0,0,0.7],\
FontSize->12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[\
 { FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Generalities\\\"]}]&)] to \
open the Help Browser, which contains \
\\*ButtonBox[StyleBox[\\\"examples\\\",FontColor->RGBColor[0,0,0.7],FontSize->\
12],Active->True,ButtonFrame->None,ButtonFunction:>(FrontEndExecute[ { \
FrontEnd`HelpBrowserLookup[ \\\"AddOns\\\", \\\"Examples\\\", \
\\\"Examples_Overview.nb\\\"]}]&)].\"\>"], "Output",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->12,
  FontColor->RGBColor[0.6, 0.300008, 0]]
}, Open  ]],

Cell[TextData[StyleBox["Input the model:",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    \(\(model = 
        CreateModel[{T\_1, T\_2}, {t\_1, 
            t\_2}, {\[Tau]\_1, \[Tau]\_2}, \(-3\)\ \ Log[
                T\_1 + Conjugate[T\_1]] - 
            3  Log[T\_2 + Conjugate[T\_2]], \(-T\_1\^2\) - T\_1\ T\_2 - 
            T\_2\^2 + 10 \((T\_1 + T\_2)\) - 100];\)\)], "Input",
  CellLabel->"In[35]:="],

Cell[TextData[StyleBox["Calculate the ideals for the model:",
  FontFamily->"Arial"]], "Text"],

Cell[BoxData[
    \(\(model = CalcModel[model];\)\)], "Input",
  CellLabel->"In[36]:="]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["3. Extracting information from the model list",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle"],

Cell[TextData[StyleBox["Display the scalar potential",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Pot /. model\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(\(\(1\/\(192\ t\_1\%3\ t\_2\%3\)\)\((t\_1\%4 + 
        t\_1\%3\ \((20 - 14\ t\_2)\) + 20\ t\_2\%3 + t\_2\%4 - 
        60\ t\_2\ \((100 + 2\ \[Tau]\_1\ \[Tau]\_2 + \[Tau]\_2\%2)\) + 
        t\_1\%2\ \((\(-500\) + 280\ t\_2 - 37\ t\_2\%2 + 10\ \[Tau]\_1\%2 + 
              10\ \[Tau]\_1\ \[Tau]\_2 + 7\ \[Tau]\_2\%2)\) + 
        t\_2\%2\ \((7\ \[Tau]\_1\%2 + 10\ \[Tau]\_1\ \[Tau]\_2 + 
              10\ \((\(-50\) + \[Tau]\_2\%2)\))\) - 
        2\ t\_1\ \((\(-140\)\ t\_2\%2 + 7\ t\_2\%3 + 
              30\ \((100 + \[Tau]\_1\%2 + 2\ \[Tau]\_1\ \[Tau]\_2)\) + 
              3\ t\_2\ \((200 + \[Tau]\_1\%2 + 
                    4\ \[Tau]\_1\ \[Tau]\_2 + \[Tau]\_2\%2)\))\) + 
        9\ \((10000 + \[Tau]\_1\%4 + 2\ \[Tau]\_1\%3\ \[Tau]\_2 - 
              100\ \[Tau]\_2\%2 + 
              2\ \[Tau]\_1\ \[Tau]\_2\%3 + \[Tau]\_2\%4 + \[Tau]\_1\%2\ \
\((\(-100\) + 3\ \[Tau]\_2\%2)\))\))\)\)\)], "Output",
  CellLabel->"Out[37]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["Now, any quantity can be recast into the format of a polynomial \
ideal (q.v. ",
    FontFamily->"Arial"],
  ButtonBox["Ideal Keywords",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" ",
    FontSize->12],
  StyleBox[") by adding a capital \"I\". Thus, associated to the keyword ",
    FontFamily->"Arial"],
  StyleBox["Pot", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[" should now be ",
    FontFamily->"Arial"],
  StyleBox["IPot", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[" where only the numerator is kept (we do no consider the runaway \
situation where the denominator, and whence the field values, tend to \
infinity) :",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IPot /. model\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \({Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, Par \[Rule] {}, 
      Gen \[Rule] {90000 - 6000\ t\_1 - 500\ t\_1\%2 + 20\ t\_1\%3 + 
            t\_1\%4 - 6000\ t\_2 - 1200\ t\_1\ t\_2 + 280\ t\_1\%2\ t\_2 - 
            14\ t\_1\%3\ t\_2 - 500\ t\_2\%2 + 280\ t\_1\ t\_2\%2 - 
            37\ t\_1\%2\ t\_2\%2 + 20\ t\_2\%3 - 14\ t\_1\ t\_2\%3 + 
            t\_2\%4 - 900\ \[Tau]\_1\%2 - 60\ t\_1\ \[Tau]\_1\%2 + 
            10\ t\_1\%2\ \[Tau]\_1\%2 - 6\ t\_1\ t\_2\ \[Tau]\_1\%2 + 
            7\ t\_2\%2\ \[Tau]\_1\%2 + 9\ \[Tau]\_1\%4 - 
            120\ t\_1\ \[Tau]\_1\ \[Tau]\_2 + 
            10\ t\_1\%2\ \[Tau]\_1\ \[Tau]\_2 - 
            120\ t\_2\ \[Tau]\_1\ \[Tau]\_2 - 
            24\ t\_1\ t\_2\ \[Tau]\_1\ \[Tau]\_2 + 
            10\ t\_2\%2\ \[Tau]\_1\ \[Tau]\_2 + 18\ \[Tau]\_1\%3\ \[Tau]\_2 - 
            900\ \[Tau]\_2\%2 + 7\ t\_1\%2\ \[Tau]\_2\%2 - 
            60\ t\_2\ \[Tau]\_2\%2 - 6\ t\_1\ t\_2\ \[Tau]\_2\%2 + 
            10\ t\_2\%2\ \[Tau]\_2\%2 + 27\ \[Tau]\_1\%2\ \[Tau]\_2\%2 + 
            18\ \[Tau]\_1\ \[Tau]\_2\%3 + 9\ \[Tau]\_2\%4}}\)], "Output",
  CellLabel->"Out[38]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["Similarly, we can display dV, generated by the partial \
derivatives of the scalar potential, by ",
    FontFamily->"Arial"],
  StyleBox["DPot", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[" and ",
    FontFamily->"Arial"],
  StyleBox["IDPot", "Input",
    FontFamily->"Arial",
    FontWeight->"Plain"],
  StyleBox[". To obtain explicitly the ideal corresponding to the partial \
derivative with respect to, say, field ",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`t\_1\)]],
  ",",
  StyleBox[" we substitute",
    FontFamily->"Arial"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(t\_1 /. Gen\) /. IDPot\) /. model\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(\(-270000\) + 12000\ t\_1 + 500\ t\_1\%2 + t\_1\%4 + 18000\ t\_2 + 
      2400\ t\_1\ t\_2 - 280\ t\_1\%2\ t\_2 + 1500\ t\_2\%2 - 
      560\ t\_1\ t\_2\%2 + 37\ t\_1\%2\ t\_2\%2 - 60\ t\_2\%3 + 
      28\ t\_1\ t\_2\%3 - 3\ t\_2\%4 + 2700\ \[Tau]\_1\%2 + 
      120\ t\_1\ \[Tau]\_1\%2 - 10\ t\_1\%2\ \[Tau]\_1\%2 + 
      12\ t\_1\ t\_2\ \[Tau]\_1\%2 - 21\ t\_2\%2\ \[Tau]\_1\%2 - 
      27\ \[Tau]\_1\%4 + 240\ t\_1\ \[Tau]\_1\ \[Tau]\_2 - 
      10\ t\_1\%2\ \[Tau]\_1\ \[Tau]\_2 + 360\ t\_2\ \[Tau]\_1\ \[Tau]\_2 + 
      48\ t\_1\ t\_2\ \[Tau]\_1\ \[Tau]\_2 - 
      30\ t\_2\%2\ \[Tau]\_1\ \[Tau]\_2 - 54\ \[Tau]\_1\%3\ \[Tau]\_2 + 
      2700\ \[Tau]\_2\%2 - 7\ t\_1\%2\ \[Tau]\_2\%2 + 
      180\ t\_2\ \[Tau]\_2\%2 + 12\ t\_1\ t\_2\ \[Tau]\_2\%2 - 
      30\ t\_2\%2\ \[Tau]\_2\%2 - 81\ \[Tau]\_1\%2\ \[Tau]\_2\%2 - 
      54\ \[Tau]\_1\ \[Tau]\_2\%3 - 27\ \[Tau]\_2\%4\)], "Output",
  CellLabel->"Out[39]="]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["4. Simple examples of using the Stringvacua modules",
  FontSize->18,
  FontVariations->{"Underline"->True}]], "Subtitle"],

Cell[TextData[StyleBox["We determine that dV is one complex dimensional",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dim /. DimIdeal[IDPot /. model]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \({1}\)], "Output",
  CellLabel->"Out[40]="]
}, Open  ]],

Cell[TextData[{
  StyleBox["The supersymmetric vacuum is simply the given by the vanishing \
locus of all of the F-terms. In the notation of Section 4.1 in \
hep-th/06060122, this is the vacuum defined by",
    FontFamily->"Arial"],
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\[LeftAngleBracket]f\_1, f\_2, f\_3, 
        f\_4\[RightAngleBracket]\)\)\)]],
  ".  ",
  StyleBox["Note that the last component, one of zero-dimension and 2 real \
roots,  is the one presented in Eq(27) of ",
    FontFamily->"Arial"],
  StyleBox["Cit. Ibid.",
    FontFamily->"Arial",
    FontSlant->"Italic"]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(susy = 
      PrimDec[IFTerms /. model, RRoots \[Rule] "\<Probab\>"]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \({{Gen \[Rule] {\[Tau]\_2, \[Tau]\_1, t\_1 - t\_2, 
            300 - 40\ t\_2 + 3\ t\_2\%2}, Dim \[Rule] {0}, 
        NRoots \[Rule] {0}, Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, 
        Par \[Rule] {}}, {Gen \[Rule] {\[Tau]\_2, \[Tau]\_1, \(-5\) + t\_1 + 
              t\_2, 225 - 5\ t\_2 + t\_2\%2}, Dim \[Rule] {0}, 
        NRoots \[Rule] {0}, Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, 
        Par \[Rule] {}}, {Gen \[Rule] {\(-5\) + t\_1 + t\_2, \(-2975\) + 
              510\ t\_1 + 18\ \[Tau]\_2\%2, 
            2125 + 18\ \[Tau]\_1\ \[Tau]\_2, \(-25\)\ \[Tau]\_1 - 
              35\ \[Tau]\_2 + 6\ t\_2\ \[Tau]\_2, \(-2975\) + 510\ t\_2 + 
              18\ \[Tau]\_1\%2, 
            5\ \[Tau]\_1 + 6\ t\_2\ \[Tau]\_1 + 25\ \[Tau]\_2, \(-2975\) - 
              3060\ t\_2 + 612\ t\_2\%2 - 90\ \[Tau]\_1\ \[Tau]\_2}, 
        Dim \[Rule] {0}, NRoots \[Rule] {0}, 
        Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, 
        Par \[Rule] {}}, {Gen \[Rule] {10 + \[Tau]\_2, \(-10\) + \[Tau]\_1, 
            t\_2, t\_1}, Dim \[Rule] {0}, NRoots \[Rule] {1}, 
        Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, 
        Par \[Rule] {}}, {Gen \[Rule] {\(-10\) + \[Tau]\_2, 10 + \[Tau]\_1, 
            t\_2, t\_1}, Dim \[Rule] {0}, NRoots \[Rule] {1}, 
        Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, 
        Par \[Rule] {}}, {Gen \[Rule] {\[Tau]\_1 - \[Tau]\_2, \(-5\) + 
              t\_2, \(-5\) + t\_1, \(-175\) + 9\ \[Tau]\_2\%2}, 
        Dim \[Rule] {0}, NRoots \[Rule] {2}, 
        Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[41]="]
}, Open  ]],

Cell[TextData[StyleBox["Here is a non-supersymmetric vacuum defined by (dV = \
0, f2 := Re(F_T2) = 0 and f4 := Im(F_T2) = 0) but f1 := Re(F_T1) \[NotEqual] \
0 and f3 := Im(F_T1) \[NotEqual] 0.",
  FontFamily->"Arial"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(dVf2f4 = 
        CreateIdeal[{IDPot /. model, \(\(t\_2 /. Gen\) /. IFTerms\) /. 
              model, \(\(\[Tau]\_2 /. Gen\) /. IFTerms\) /. 
              model}];\)\), "\[IndentingNewLine]", 
    \(f1 = \(\(t\_1 /. Gen\) /. IFTerms\) /. model; 
    f3 = \(\(\[Tau]\_1 /. Gen\) /. IFTerms\) /. 
        model;\), "\[IndentingNewLine]", 
    \(Saturation[Saturation[dVf2f4, Poly \[Rule] f3], 
      Poly \[Rule] f1]\)}], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \({Gen \[Rule] {t\_1, \(-30\)\ \[Tau]\_1 + t\_2\ \[Tau]\_1 - 
            30\ \[Tau]\_2 + 2\ t\_2\ \[Tau]\_2, \(-300\) + 10\ t\_2 + 
            t\_2\%2 + 3\ \[Tau]\_1\%2 + 3\ \[Tau]\_1\ \[Tau]\_2 + 
            3\ \[Tau]\_2\%2, 
          300\ \[Tau]\_1 + \[Tau]\_1\%3 + 200\ \[Tau]\_2 - 
            10\ t\_2\ \[Tau]\_2 + 3\ \[Tau]\_1\%2\ \[Tau]\_2 + 
            3\ \[Tau]\_1\ \[Tau]\_2\%2 + 2\ \[Tau]\_2\%3}, Dim \[Rule] {1}, 
      Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, 
      Par \[Rule] {}}\)], "Output",
  CellLabel->"Out[44]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimDec[%, Dim \[Rule] {1}]\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \({{Gen \[Rule] {t\_1, \(-30\)\ \[Tau]\_1 + t\_2\ \[Tau]\_1 - 
              30\ \[Tau]\_2 + 2\ t\_2\ \[Tau]\_2, \(-300\) + 10\ t\_2 + 
              t\_2\%2 + 3\ \[Tau]\_1\%2 + 3\ \[Tau]\_1\ \[Tau]\_2 + 
              3\ \[Tau]\_2\%2, 
            300\ \[Tau]\_1 + \[Tau]\_1\%3 + 200\ \[Tau]\_2 - 
              10\ t\_2\ \[Tau]\_2 + 3\ \[Tau]\_1\%2\ \[Tau]\_2 + 
              3\ \[Tau]\_1\ \[Tau]\_2\%2 + 2\ \[Tau]\_2\%3}, Dim \[Rule] {1}, 
        Var \[Rule] {t\_1, t\_2, \[Tau]\_1, \[Tau]\_2}, 
        Par \[Rule] {}}}\)], "Output",
  CellLabel->"Out[45]="]
}, Open  ]]
}, Closed]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1152}, {0, 777}},
CellGrouping->Manual,
WindowSize->{1008, 664},
WindowMargins->{{Automatic, 46}, {0, Automatic}},
ShowCellLabel->False,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 221, 8, 76, "Title"],

Cell[CellGroupData[{
Cell[2000, 63, 131, 3, 31, "Subtitle"],
Cell[2134, 68, 2354, 77, 125, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[4525, 150, 118, 2, 25, "Subtitle"],

Cell[CellGroupData[{
Cell[4668, 156, 72, 2, 45, "Input"],
Cell[4743, 160, 361, 12, 62, "Text"],
Cell[5107, 174, 912, 15, 44, "Output"]
}, Open  ]],
Cell[6034, 192, 75, 1, 27, "Text"],
Cell[6112, 195, 337, 7, 67, "Input"],
Cell[6452, 204, 94, 1, 27, "Text"],
Cell[6549, 207, 87, 2, 45, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6673, 214, 140, 2, 25, "Subtitle"],
Cell[6816, 218, 87, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[6928, 223, 70, 2, 45, "Input"],
Cell[7001, 227, 960, 16, 100, "Output"]
}, Open  ]],
Cell[7976, 246, 748, 22, 46, "Text"],

Cell[CellGroupData[{
Cell[8749, 272, 71, 2, 45, "Input"],
Cell[8823, 276, 1142, 18, 101, "Output"]
}, Open  ]],
Cell[9980, 297, 626, 20, 46, "Text"],

Cell[CellGroupData[{
Cell[10631, 321, 95, 2, 45, "Input"],
Cell[10729, 325, 938, 15, 82, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[11716, 346, 146, 2, 25, "Subtitle"],
Cell[11865, 350, 106, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[11996, 355, 89, 2, 45, "Input"],
Cell[12088, 359, 62, 2, 44, "Output"]
}, Open  ]],
Cell[12165, 364, 607, 15, 46, "Text"],

Cell[CellGroupData[{
Cell[12797, 383, 126, 3, 45, "Input"],
Cell[12926, 388, 1667, 27, 158, "Output"]
}, Open  ]],
Cell[14608, 418, 228, 3, 27, "Text"],

Cell[CellGroupData[{
Cell[14861, 425, 478, 10, 85, "Input"],
Cell[15342, 437, 564, 10, 63, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15943, 452, 85, 2, 45, "Input"],
Cell[16031, 456, 582, 10, 63, "Output"]
}, Open  ]]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

